@@ -45,12 +45,84 @@ static char rcsid[] = "$Header: /ufs/repository/magic/calma/CalmaRdcl.c,v 1.4 20
 #include "misc/styles.h"
 #include "textio/textio.h"
 #include "calma/calmaInt.h"
+#include "calma/calma.h"
 
 int calmaNonManhattan;
 
 extern void CIFPaintCurrent();
 
 extern HashTable calmaDefInitHash;
+
+/* forward declarations */
+bool calmaElementSref();
+bool calmaParseElement();
+
+/* Structure used when flattening the GDS hierarchy on read-in */
+
+typedef struct {
+    Plane *plane;
+    Transform *trans;
+} GDSCopyRec;
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * calmaExact --
+ *
+ * When dictated to flatten small cells (e.g., vias which have been placed
+ * into their own cells to make use of GDS arrays), instead of calling
+ * CIFPaintCurrent(), we just transfer the planes of cifCurReadPlanes to
+ * the current cell, and swap the planes from the current cell into the
+ * pointer array for cifCurReadPlanes, so we don't have to free and
+ * reallocate any memory for the operation.  Flag the cell as being a
+ * CIF cell, so that it can be destroyed after read-in and flattening.
+ *
+ * Results:
+ *	None.
+ *
+ * Side Effects:
+ *	Swaps the planes of cifReadCellDef with cifCurReadPlanes.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+void
+calmaExact()
+{
+    int pNum;
+    Plane *newplane;
+    Plane **parray;
+    int gdsCopyPaintFunc();	/* Forward reference */
+
+    parray = (Plane **)mallocMagic(MAXCIFRLAYERS * sizeof(Plane *));
+
+    for (pNum = 0; pNum < MAXCIFRLAYERS; pNum++)
+    {
+	if (cifCurReadPlanes[pNum] != NULL)
+	{
+	    GDSCopyRec gdsCopyRec;
+
+	    newplane = DBNewPlane((ClientData) TT_SPACE);
+	    DBClearPaintPlane(newplane);
+
+	    gdsCopyRec.plane = newplane;
+	    gdsCopyRec.trans = NULL;
+	    DBSrPaintArea((Tile *)NULL, cifCurReadPlanes[pNum], &TiPlaneRect,
+		&DBAllButSpaceBits, gdsCopyPaintFunc, &gdsCopyRec);
+	    parray[pNum] = newplane;
+	}
+	else
+	    parray[pNum] = NULL;
+    }
+    cifReadCellDef->cd_client = (ClientData)parray;
+    cifReadCellDef->cd_flags |= CDFLATGDS;
+
+    /* Clear out the current paint planes for the next cell */
+    for (pNum = 0; pNum < MAXCIFRLAYERS; pNum++)
+	DBClearPaintPlane(cifCurReadPlanes[pNum]);
+}
+
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -73,13 +145,20 @@ extern HashTable calmaDefInitHash;
  */
 
 bool
-calmaParseStructure()
+calmaParseStructure(filename)
+    char *filename;		/* Name of the GDS file read */
 {
     static int structs[] = { CALMA_STRCLASS, CALMA_STRTYPE, -1 };
-    int nbytes, rtype, nsrefs, osrefs;
+    int nbytes, rtype, nsrefs, osrefs, npaths;
     char *strname = NULL, newname[CALMANAMELENGTH*2];
     HashEntry *he;
     int suffix;
+    int mfactor;
+    off_t filepos;
+    bool was_called;
+
+    if (CalmaReadOnly)
+	filepos = ftell(calmaInputFile);
 
     /* Make sure this is a structure; if not, let the caller know we're done */
     PEEKRH(nbytes, rtype);
@@ -104,7 +183,7 @@ calmaParseStructure()
 	calmaReadError("Giving this cell a new name: %s\n", newname);
 	(void) strncpy(strname, newname, CALMANAMELENGTH*2);
     }
-    cifReadCellDef = calmaFindCell(strname);
+    cifReadCellDef = calmaFindCell(strname, &was_called);
     DBCellClearDef(cifReadCellDef);
     DBCellSetAvail(cifReadCellDef);
     HashSetValue(he, cifReadCellDef);
@@ -121,8 +200,9 @@ calmaParseStructure()
 
     /* Body of structure: a sequence of elements */
     osrefs = nsrefs = 0;
+    npaths = 0;
     calmaNonManhattan = 0;
-    while (calmaParseElement(&nsrefs))
+    while (calmaParseElement(&nsrefs, &npaths))
     {
 	if (SigInterruptPending)
 	    goto done;
@@ -142,16 +222,54 @@ calmaParseStructure()
     /* Make sure it ends with an ENDSTR record */
     if (!calmaSkipExact(CALMA_ENDSTR)) goto syntaxerror;
 
+    if (CalmaReadOnly)
+    {
+	/* Writing the file position into a string is slow, but */
+	/* it prevents requiring special handling when printing	*/
+	/* out the properties.					*/
+
+	char *fpcopy = (char *)mallocMagic(20);
+	char *fncopy = StrDup(NULL, filename);
+	sprintf(fpcopy, "%d", filepos);
+	DBPropPut(cifReadCellDef, "GDS_BEGIN", (ClientData)fpcopy);
+
+	fpcopy = (char *)mallocMagic(20);
+	filepos = ftell(calmaInputFile);
+	sprintf(fpcopy, "%d", filepos);
+	DBPropPut(cifReadCellDef, "GDS_END", (ClientData)fpcopy);
+
+	DBPropPut(cifReadCellDef, "GDS_FILE", (ClientData)fncopy);
+	cifReadCellDef->cd_flags |= CDNOEDIT;
+    }
+
     /*
-     * Do the geometrical processing and paint this material back into
-     * the appropriate cell of the database.
+     * Don't paint now, just keep the CIF planes, and flatten the
+     * cell by painting when instanced.  But---if this cell was
+     * instanced before it was defined, then it can't be flattened.
      */
+    if (CalmaFlattenUses && (!was_called) && (npaths < 10) && (nsrefs == 0))
+    {
+	TxPrintf("Flattening cell %s\n", cifReadCellDef->cd_name);
+	calmaExact();
+    }
+    else
+    {
+	/*
+	 * Do the geometrical processing and paint this material back into
+	 * the appropriate cell of the database.
+	 */
+
+	CIFPaintCurrent();
+    }
 
-    CIFPaintCurrent();
     DBAdjustLabelsNew(cifReadCellDef, &TiPlaneRect,
 	      (cifCurReadStyle->crs_flags & CRF_NO_RECONNECT_LABELS) ? 1 : 0);
     DBReComputeBbox(cifReadCellDef);
-    DRCCheckThis(cifReadCellDef, TT_CHECKPAINT, &cifReadCellDef->cd_bbox);
+
+    /* Don't bother to register with DRC if we're going to delete the cell */
+    if (!CalmaFlattenUses || (npaths >= 10) || (nsrefs != 0))
+	DRCCheckThis(cifReadCellDef, TT_CHECKPAINT, &cifReadCellDef->cd_bbox);
+
     DBWAreaChanged(cifReadCellDef, &cifReadCellDef->cd_bbox,
 	DBW_ALLWINDOWS, &DBAllButSpaceBits);
     DBCellSetModified(cifReadCellDef, TRUE);
@@ -172,6 +290,7 @@ syntaxerror:
     HashKill(&calmaLayerHash);
     return (calmaSkipTo(CALMA_ENDSTR));
 }
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -196,8 +315,8 @@ syntaxerror:
  */
 
 bool
-calmaParseElement(pnsrefs)
-    int *pnsrefs;
+calmaParseElement(pnsrefs, pnpaths)
+    int *pnsrefs, *pnpaths;
 {
     static int node[] = { CALMA_ELFLAGS, CALMA_PLEX, CALMA_LAYER,
 			  CALMA_NODETYPE, CALMA_XY, -1 };
@@ -219,12 +338,15 @@ calmaParseElement(pnsrefs)
 	    break;
 	case CALMA_BOUNDARY:	
 	    calmaElementBoundary();
+	    (*pnpaths)++;
 	    break;
 	case CALMA_BOX:
 	    calmaElementBox();
+	    (*pnpaths)++;
 	    break;
 	case CALMA_PATH:	
 	    calmaElementPath();
+	    (*pnpaths)++;
 	    break;
 	case CALMA_TEXT:	
 	    calmaElementText();
@@ -240,7 +362,7 @@ calmaParseElement(pnsrefs)
 
     return (calmaSkipTo(CALMA_ENDEL));
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -269,6 +391,10 @@ calmaElementSref()
     Point refarray[3], p;
     CellUse *use;
     CellDef *def;
+    int gdsCopyPaintFunc();	/* Forward reference */
+    /* Added by NP */
+    char *useid = NULL, *arraystr = NULL;
+    int propAttrType;
 
     /* Skip CALMA_ELFLAGS, CALMA_PLEX */
     calmaSkipSet(calmaElementIgnore);
@@ -341,7 +467,7 @@ calmaElementSref()
     for (n = 0; n < nref; n++)
     {
 	savescale = cifCurReadStyle->crs_scaleFactor;
-	READPOINT(&refarray[n]);
+	calmaReadPoint(&refarray[n], 1);
 	refarray[n].p_x = CIFScaleCoord(refarray[n].p_x, COORD_EXACT);
 	if (savescale != cifCurReadStyle->crs_scaleFactor)
 	{
@@ -416,13 +542,58 @@ calmaElementSref()
 	}
     }
 
+    /* Added by NP --- parse PROPATTR and PROPVALUE record types */
+    /* The PROPATTR types 98 and 99 are defined internally to	 */
+    /* magic and are used to retain information that cannot be	 */
+    /* saved to the GDS file in any other manner.		 */
+
+    while (1)
+    {
+	READRH(nbytes, rtype);
+	if (nbytes < 0) return;
+	if (rtype == CALMA_PROPATTR)
+	{
+	    READI2(propAttrType);
+	    if (propAttrType == CALMA_PROP_USENAME)
+	    {
+		if (!calmaReadStringRecord(CALMA_PROPVALUE, &useid))
+		    return;
+	    }
+	    else if (propAttrType == CALMA_PROP_ARRAY_LIMITS)
+	    {
+		if (!calmaReadStringRecord(CALMA_PROPVALUE, &arraystr))
+		    return;
+		else if (arraystr)
+		{
+		    int xl, xh, yl, yh;
+		    if (sscanf(arraystr, "%d_%d_%d_%d", &xl, &xh, &yl, &yh)
+				!= 4)
+			TxError("Malformed \"array\" property ignored: %s",
+				arraystr);
+		    else
+		    {
+			xlo = xl;
+			ylo = yl;
+			xhi = xh;
+			yhi = yh;
+		    }
+		}
+	    }
+	}
+	else
+	{
+	    UNREADRH(nbytes, rtype);
+	    break;
+	}
+    }
+
     /*
      * Create a new cell use with this transform.  If the
      * cell being referenced doesn't exist, create it.
      * Don't give it a use-id; we will do that only after
      * we've placed all cells.
      */
-    def = calmaFindCell(sname);
+    def = calmaFindCell(sname, NULL);
     if (DBIsAncestor(def, cifReadCellDef))
     {
 	calmaReadError("Cell %s is an ancestor of %s",
@@ -431,17 +602,91 @@ calmaElementSref()
 	calmaReadError("(Use skipped)\n");
 	return;
     }
-    use = DBCellNewUse(calmaFindCell(sname), (char *) NULL);
-    if (isArray)
-	DBMakeArray(use, &GeoIdentityTransform, xlo, ylo, xhi, yhi, xsep, ysep);
-    DBSetTrans(use, &trans);
-    DBPlaceCell(use, cifReadCellDef);
 
-    /* Done with sname */
+    /*
+     * Check for cells which have been marked for flattening
+     */
+    if (def->cd_flags & CDFLATGDS)
+    {
+	Plane **gdsplanes = (Plane **)def->cd_client;
+	GDSCopyRec gdsCopyRec;
+	int pNum;
+
+	/* To do:  Deal with arrays by modifying trans and	*/
+	/* looping over X and Y					*/
+
+	for (pNum = 0; pNum < MAXCIFRLAYERS; pNum++)
+	{
+	    if (gdsplanes[pNum] != NULL)
+	    {
+		gdsCopyRec.plane = cifCurReadPlanes[pNum];
+		if (isArray)
+		{
+		    Transform artrans;
+		    int x, y;
+
+		    gdsCopyRec.trans = &artrans;
+
+		    for (x = 0; x < cols; x++)
+			for (y = 0; y < rows; y++)
+			{
+			    GeoTransTranslate((x * xsep), (y * ysep), &trans,
+					&artrans);
+			    DBSrPaintArea((Tile *)NULL, gdsplanes[pNum],
+					&TiPlaneRect, &DBAllButSpaceBits,
+					gdsCopyPaintFunc, &gdsCopyRec);
+			}
+		}
+		else
+		{
+		    gdsCopyRec.trans = &trans;
+		    DBSrPaintArea((Tile *)NULL, gdsplanes[pNum], &TiPlaneRect,
+			&DBAllButSpaceBits, gdsCopyPaintFunc, &gdsCopyRec);
+		}
+	    }
+	}
+    }
+    else
+    {
+	use = DBCellNewUse(def, (useid) ? useid : (char *) NULL);
+	if (isArray)
+	    DBMakeArray(use, &GeoIdentityTransform, xlo, ylo, xhi, yhi, xsep, ysep);
+	DBSetTrans(use, &trans);
+	DBPlaceCell(use, cifReadCellDef);
+    }
+
+    /* Done with allocated variables */
     if (sname != NULL) freeMagic(sname);
+    if (useid != NULL) freeMagic(useid);
+    if (arraystr != NULL) freeMagic(arraystr);
 }
 
 
+/* Callback function for copying paint from one CIF cell into another */
+
+int
+gdsCopyPaintFunc(tile, gdsCopyRec)
+    Tile *tile;
+    GDSCopyRec *gdsCopyRec;
+{
+    int pNum;
+    Rect sourceRect, targetRect;
+    Transform *trans = gdsCopyRec->trans;
+    Plane *plane = gdsCopyRec->plane;
+
+    if (trans)
+    {
+	TiToRect(tile, &sourceRect);
+	GeoTransRect(trans, &sourceRect, &targetRect);
+    }
+    else
+	TiToRect(tile, &targetRect);
+
+    DBPaintPlane(plane, &targetRect, CIFPaintTable, (PaintUndoInfo *)NULL);
+    
+    return 0;
+}
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -462,8 +707,14 @@ calmaElementSref()
  */
 
 CellDef *
-calmaFindCell(name)
+calmaFindCell(name, was_called)
     char *name;		/* Name of desired cell */
+    bool *was_called;	/* If this cell is in the hash table, then it
+			 * was instanced before it was defined.  We
+			 * need to know this so as to avoid flattening
+			 * the cell if requested.
+			 */
+
 {
     HashEntry *h;
     CellDef *def;
@@ -485,6 +736,9 @@ calmaFindCell(name)
 	     DBReComputeBbox(def);
 	}
 	HashSetValue(h, def);
+	if (was_called) *was_called = FALSE;
     }
+    else
+	if (was_called) *was_called = TRUE;
     return (CellDef *) HashGetValue(h);
 }
