@@ -54,6 +54,133 @@ extern HashTable calmaDefInitHash;
 #endif
 
 extern void calmaLayerError();
+bool calmaReadPath();
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * calmaInputRescale ---
+ *
+ *	This routine does the same thing as CIFInputRescale().  However,
+ *	the "gds flatten" option allows us to retain GDS layout
+ *	information in the cd_client record of a cell def.  If we
+ *	change the GDS input scale factor, then all of these saved
+ *	layouts need to be rescaled.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	Reallocates memory for layout planes in each of the cells in
+ *	the database that have the CDFLATGDS flag set.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+void
+calmaInputRescale(n, d)
+    int n, d;
+{
+    HashEntry *h;
+    HashSearch hs;
+    CellDef *def;
+
+    HashStartSearch(&hs);
+    while (TRUE)
+    {
+	h = HashNext(&CifCellTable, &hs);
+	if (h == NULL) break;
+
+	def = (CellDef *) HashGetValue(h);
+	if (def == NULL) continue;	 /* shouldn't happen */
+	if (def->cd_flags & CDFLATGDS)
+	{
+	    /* Scale the GDS planes in this cell's cd_client record */
+	    Plane **gdsplanes = (Plane **)def->cd_client;
+	    CIFScalePlanes(n, d, gdsplanes);
+	}
+    }
+
+    CIFInputRescale(n, d);
+}
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * calmaReadPoint ---
+ *
+ * Read a point from the input.
+ * We take care of scaling by calmaReadScale1/calmaReadScale2.
+ * Also take care of noting when the scaling results in a sub-integer
+ * value, and rescaling everything appropriately.  "iscale" is an
+ * integer scaling value used to return values at, for instance, double
+ * the scale, as for a path centerline.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	The Point pointed to by parameter "p" is filled with the
+ *	coordinates of the point.  If a fractional integer is
+ *	encountered, then everything in the GDS planes is rescaled
+ *	to match.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+void
+calmaReadPoint(p, iscale)
+    Point *p;
+    int iscale;
+{
+    int rescale;
+
+    READI4((p)->p_x);
+    p->p_x *= (calmaReadScale1 * iscale);
+    if ((iscale != 0) && (p->p_x % calmaReadScale2 != 0))
+    {
+	rescale = calmaReadScale2 / FindGCF(calmaReadScale2, abs(p->p_x));
+	if ((calmaReadScale1 * rescale) > CIFRescaleLimit)
+	{
+	    calmaReadError("Warning:  calma units at max scale; value rounded\n");
+	    if (p->p_x < 0)
+		p->p_x -= ((calmaReadScale2 - 1) >> 1);
+	    else
+		p->p_x += (calmaReadScale2 >> 1);
+	}
+	else
+	{
+	    calmaReadScale1 *= rescale;
+	    calmaInputRescale(rescale, 1);
+	    p->p_x *= rescale;
+	}
+    }
+    p->p_x /= calmaReadScale2;
+
+    READI4((p)->p_y);
+    p->p_y *= (calmaReadScale1 * iscale);
+    if ((iscale != 0) && (p->p_y % calmaReadScale2 != 0))
+    {
+	rescale = calmaReadScale2 / FindGCF(calmaReadScale2, abs(p->p_y));
+	if ((calmaReadScale1 * rescale) > CIFRescaleLimit)
+	{
+	    calmaReadError("Warning:  calma units at max scale; value rounded\n");
+	    if (p->p_y < 0)
+		p->p_y -= ((calmaReadScale2 - 1) >> 1);
+	    else
+		p->p_y += (calmaReadScale2 >> 1);
+	}
+	else
+	{
+	    calmaReadScale1 *= rescale;
+	    calmaInputRescale(rescale, 1);
+	    p->p_x *= rescale;
+	    p->p_y *= rescale;
+	}
+    }
+    p->p_y /= calmaReadScale2;
+}
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -100,15 +227,19 @@ calmaElementBoundary()
 	plane = NULL;
 	calmaLayerError("Unknown layer/datatype in boundary", layer, dt);
     }
-    else plane = cifCurReadPlanes[ciftype];
 
     /* Read the path itself, building up a path structure */
-    if (!calmaReadPath(&pathheadp)) 
+    if (!calmaReadPath(&pathheadp, (plane == NULL) ? 0 : 1)) 
     {
-	calmaReadError("Error while reading path for boundary/box; ignored.\n");
+	if (plane != NULL)
+	    calmaReadError("Error while reading path for boundary/box; ignored.\n");
 	return;
     }
 
+    /* Note that calmaReadPath() may reallocate planes of cifCurReadPlanes */
+
+    plane = cifCurReadPlanes[ciftype];
+
     /* Convert the polygon to rectangles. */
 
 #ifdef NONMANHATTAN
@@ -126,7 +257,7 @@ calmaElementBoundary()
 	he = HashFind(&calmaDefInitHash, newname);
 	if (!HashGetValue(he))
 	{
-	    newdef = calmaFindCell(newname);
+	    newdef = calmaFindCell(newname, NULL);
 	    cifReadCellDef = newdef;
 	    DBCellClearDef(cifReadCellDef);
 	    DBCellSetAvail(cifReadCellDef);
@@ -159,7 +290,7 @@ calmaElementBoundary()
     {
 	if (plane)
 	    DBPaintPlane(plane, &rp->r_r, CIFPaintTable, (PaintUndoInfo *)NULL);
-	FREE((char *) rp);
+	freeMagic((char *) rp);
     }
 
 #ifdef NONMANHATTAN
@@ -209,7 +340,7 @@ calmaElementBoundary()
 void
 calmaElementBox()
 {
-    int nbytes, rtype, npoints;
+    int nbytes, rtype, npoints, savescale;
     int dt, layer, ciftype;
     Plane *plane;
     Point p;
@@ -265,7 +396,16 @@ calmaElementBox()
     }
     while (npoints-- > 0)
     {
-	READPOINT(&p);
+	savescale = calmaReadScale1;
+	calmaReadPoint(&p, 1);
+	if (savescale != calmaReadScale1)
+	{
+	    int newscale = calmaReadScale1 / savescale;
+	    r.r_xbot *= newscale;
+	    r.r_xtop *= newscale;
+	    r.r_ybot *= newscale;
+	    r.r_ytop *= newscale;
+	}
 	if (p.p_x < r.r_xbot) r.r_xbot = p.p_x;
 	if (p.p_y < r.r_ybot) r.r_ybot = p.p_y;
 	if (p.p_x > r.r_xtop) r.r_xtop = p.p_x;
@@ -275,7 +415,7 @@ calmaElementBox()
     /* Paint the rectangle */
     DBPaintPlane(plane, &r, CIFPaintTable, (PaintUndoInfo *)NULL);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -297,7 +437,7 @@ calmaElementPath()
 {
     static int ignore[] = { CALMA_BGNEXTN, CALMA_ENDEXTN, -1 };
     int nbytes, rtype;
-    int layer, dt, width, halfwidth, pathtype, ciftype;
+    int layer, dt, width, pathtype, ciftype, savescale;
     int xmin, ymin, xmax, ymax, temp;
     CIFPath *pathheadp, *pathp, *previousp;
     Rect segment;
@@ -334,130 +474,48 @@ calmaElementPath()
 	    return;
 	}
     }
-    halfwidth = (width * calmaReadScale1)/(calmaReadScale2 * 2);
+    width *= calmaReadScale1;
+    if (width % calmaReadScale2 != 0)
+	calmaReadError("Wire width snapped to nearest integer boundary.\n");
+
+    width /= calmaReadScale2;
 
     /* Skip BGNEXTN, ENDEXTN */
     calmaSkipSet(ignore);
 
     /* Read the points in the path */
-    if (!calmaReadPath(&pathheadp)) 
+    savescale = calmaReadScale1;
+    if (!calmaReadPath(&pathheadp, 2)) 
     {
 	calmaReadError("Improper path; ignored.\n");
 	return;
     }
+    if (savescale != calmaReadScale1)
+	width *= (calmaReadScale1 / savescale);
 
     /* Don't process zero-width paths any further */
     if (width <= 0)
-	goto freeit;
+    {
+	CIFFreePath(pathheadp);
+	return;
+    }
 
     /* Make sure we know about this type */
     ciftype = CIFCalmaLayerToCifLayer(layer, dt, cifCurReadStyle);
     if (ciftype < 0)
     {
 	calmaLayerError("Unknown layer/datatype in path", layer, dt);
-	goto freeit;
+	CIFFreePath(pathheadp);
     }
-
-    plane = cifCurReadPlanes[ciftype];
-    for (previousp = pathheadp,pathp=pathheadp->cifp_next,first=TRUE,last=FALSE;
-	    pathp != NULL; pathp = pathp->cifp_next)
+    else
     {
-	if (pathp->cifp_next == NULL) last = TRUE;
-	xmin = previousp->cifp_x;
-	xmax = pathp->cifp_x;
-	ymin = previousp->cifp_y;
-	ymax = pathp->cifp_y;
-
-	if (xmin != xmax && ymin != ymax)
-	{
-	     calmaReadError("Non-orthogonal path; ignored.\n");
-	     return;
-	}
-	if (first == FALSE && last == FALSE)
-	{
-	     /* middle segment; expand the endpoints */
-	     if (xmax < xmin) temp = xmin, xmin = xmax, xmax = temp;
-	     if (ymax < ymin) temp = ymin, ymin = ymax, ymax = temp;
-	     segment.r_xbot = xmin-halfwidth;
-	     segment.r_ybot = ymin-halfwidth;
-	     segment.r_xtop = xmax+halfwidth;
-	     segment.r_ytop = ymax+halfwidth;
-	}
-	else 
-	{
-	     if (xmin == xmax) /* vertical wire */
-	     {
-	          segment.r_xbot = xmin-halfwidth;
-	          segment.r_xtop = xmax+halfwidth;
-		  if (ymax < ymin)
-		  {
-		       segment.r_ybot = ymax;
-		       segment.r_ytop = ymin;
-		       if (first == FALSE|| pathtype != CALMAPATH_SQUAREFLUSH)
-		       {
-		       	    segment.r_ytop += halfwidth;
-		       }
-		       if (last == FALSE || pathtype != CALMAPATH_SQUAREFLUSH)
-		       {
-		       	    segment.r_ybot -= halfwidth;
-		       }
-		  }
-		  else
-		  {
-		       segment.r_ytop = ymax;
-		       segment.r_ybot = ymin;
-		       if (first == FALSE|| pathtype != CALMAPATH_SQUAREFLUSH)
-		       {
-		       	    segment.r_ybot -= halfwidth;
-		       }
-		       if (last  == FALSE|| pathtype != CALMAPATH_SQUAREFLUSH)
-		       {
-		       	    segment.r_ytop += halfwidth;
-		       }
-		  }
-	     }
-	     if (ymin == ymax) /* horiz wire */
-	     {
-	          segment.r_ybot = ymin-halfwidth;
-	          segment.r_ytop = ymax+halfwidth;
-		  if (xmax < xmin)
-		  {
-		       segment.r_xbot = xmax;
-		       segment.r_xtop = xmin;
-		       if (first == FALSE|| pathtype != CALMAPATH_SQUAREFLUSH)
-		       {
-		       	    segment.r_xtop += halfwidth;
-		       }
-		       if (last  == FALSE|| pathtype != CALMAPATH_SQUAREFLUSH)
-		       {
-		       	    segment.r_xbot -= halfwidth;
-		       }
-		  }
-		  else
-		  {
-		       segment.r_xtop = xmax;
-		       segment.r_xbot = xmin;
-		       if (first == FALSE|| pathtype != CALMAPATH_SQUAREFLUSH)
-		       {
-		       	    segment.r_xbot -= halfwidth;
-		       }
-		       if (last == FALSE || pathtype != CALMAPATH_SQUAREFLUSH)
-		       {
-		       	    segment.r_xtop += halfwidth;
-		       }
-		  }
-	     }
-	}
-	DBPaintPlane(plane, &segment, CIFPaintTable, (PaintUndoInfo *)NULL);
-	previousp = pathp;
-	first = FALSE;
+	plane = cifCurReadPlanes[ciftype];
+	CIFPaintWirePath(pathheadp, width,
+		(pathtype == CALMAPATH_SQUAREFLUSH) ? FALSE : TRUE,
+		plane, CIFPaintTable, (PaintUndoInfo *)NULL);
     }
-
-    /* All done */
-freeit:
-    CIFFreePath(pathheadp);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -523,7 +581,7 @@ calmaElementText()
     }
     else
     {
-	READPOINT(&r.r_ll);
+	calmaReadPoint(&r.r_ll, 1);
 	nbytes -= 8;
     }
     if (!calmaSkipBytes(nbytes)) return;
@@ -582,7 +640,9 @@ calmaElementText()
  * calmaReadPath --
  *
  * This procedure parses a Calma path, which is an XY record
- * containing one or more points.
+ * containing one or more points.  "iscale" is an internal
+ * scaling, usually 1, but is 2 for paths specified on a
+ * centerline, to avoid roundoff errors.
  *
  * Results:
  *	TRUE is returned if the path was parsed successfully,
@@ -596,11 +656,12 @@ calmaElementText()
  */
 
 bool
-calmaReadPath(pathheadpp)
+calmaReadPath(pathheadpp, iscale)
     CIFPath **pathheadpp;
+    int iscale;
 {
     CIFPath path, *pathtailp, *newpathp;
-    int nbytes, rtype, npoints;
+    int nbytes, rtype, npoints, savescale;
     bool nonManhattan = FALSE;
 
     *pathheadpp = (CIFPath *) NULL;
@@ -624,8 +685,20 @@ calmaReadPath(pathheadpp)
     npoints = (nbytes - CALMAHEADERLENGTH) / 8;
     while (npoints--)
     {
-	READPOINT(&path.cifp_point);
-	if (ABS(path.cifp_x) > 5000000 || ABS(path.cifp_y) > 5000000) {
+	savescale = calmaReadScale1;
+	calmaReadPoint(&path.cifp_point, iscale);
+	if (savescale != calmaReadScale1)
+	{
+	    CIFPath *phead = *pathheadpp;
+	    int newscale = calmaReadScale1 / savescale;
+	    while (phead != NULL)
+	    {
+		phead->cifp_x *= newscale;
+		phead->cifp_y *= newscale;
+		phead = phead->cifp_next;
+	    }
+	}
+	if (ABS(path.cifp_x) > 50000000 || ABS(path.cifp_y) > 50000000) {
 	    calmaReadError("Warning:  Very large point in path:  (%d, %d)\n",
 		path.cifp_x, path.cifp_y);
 	}
@@ -635,35 +708,39 @@ calmaReadPath(pathheadpp)
 	    return (FALSE);
 	}
 
-	MALLOC(CIFPath *, newpathp, sizeof (CIFPath));
-	*newpathp = path;
-	if (*pathheadpp)
+	if (iscale != 0)
 	{
-	    /*
-	     * Check that this segment is Manhattan.  If not, remember the
-	     * fact and later introduce extra stair-steps to make the path
-	     * Manhattan.  We don't do the stair-step introduction here for
-	     * two reasons: first, the same code is also used by the Calma
-	     * module, and second, it is important to know which side of
-	     * the polygon is the outside when generating the stair steps.
-	     */
-	    if (pathtailp->cifp_x != newpathp->cifp_x
-		    && pathtailp->cifp_y != (newpathp->cifp_y))
+	    newpathp = (CIFPath *) mallocMagic((unsigned) (sizeof (CIFPath)));
+	    *newpathp = path;
+
+	    if (*pathheadpp)
 	    {
-		if (!nonManhattan)
+		/*
+		 * Check that this segment is Manhattan.  If not, remember the
+		 * fact and later introduce extra stair-steps to make the path
+		 * Manhattan.  We don't do the stair-step introduction here for
+		 * two reasons: first, the same code is also used by the Calma
+		 * module, and second, it is important to know which side of
+		 * the polygon is the outside when generating the stair steps.
+		 */
+		if (pathtailp->cifp_x != newpathp->cifp_x
+			&& pathtailp->cifp_y != (newpathp->cifp_y))
 		{
-		    calmaNonManhattan++;
-		    nonManhattan = TRUE;
+		    if (!nonManhattan)
+		    {
+			calmaNonManhattan++;
+			nonManhattan = TRUE;
+		    }
 		}
+		pathtailp->cifp_next = newpathp;
 	    }
-	    pathtailp->cifp_next = newpathp;
+	    else *pathheadpp = newpathp;
+	    pathtailp = newpathp;
 	}
-	else *pathheadpp = newpathp;
-	pathtailp = newpathp;
     }
 
 #ifndef NONMANHATTAN
-    if (nonManhattan)
+    if (nonManhattan && (iscale != 0))
 	CIFMakeManhattanPath(*pathheadpp);
 #endif
 
