@@ -20,8 +20,9 @@
 static char rcsid[] = "$Header: /ufs/repository/magic/calma/CalmaRead.c,v 1.5 2001/07/23 17:49:49 tim Exp $";
 #endif  /* not lint */
 
-#include <sys/types.h>
 #include <stdio.h>
+#include <sys/types.h>
+#include <unistd.h>
 
 #include <netinet/in.h>
 
@@ -54,9 +55,20 @@ bool CalmaSubcellPolygons = FALSE;	/* Put non-Manhattan polygons
 					 */
 int CalmaPolygonCount;
 #endif
-
+bool CalmaFlattenUses = FALSE;		/* If TRUE, small cells in the input
+					 * stream are flattened when encountered
+					 * as uses.  This improves magic's
+					 * performance when handling contacts
+					 * saved as subcell arrays.
+					 */
+bool CalmaReadOnly = FALSE;		/* Set files to read-only and
+					 * retain file position information
+					 * so cells can be written verbatim.
+					 */
 extern void calmaUnexpected();
 
+bool calmaParseUnits();
+
 /*
  * Scaling.
  * Multiply all coordinates by calmaReadScale1, then divide them
@@ -110,8 +122,9 @@ int calmaElementIgnore[] = { CALMA_ELFLAGS, CALMA_PLEX, -1 };
  */
 
 void
-CalmaReadFile(file)
+CalmaReadFile(file, filename)
     FILE *file;			/* File from which to read Calma */
+    char *filename;		/* The real name of the file read */
 {
     int version;
     char *libname = NULL;
@@ -174,7 +187,7 @@ CalmaReadFile(file)
     if (!calmaParseUnits()) goto done;
 
     /* Main body of GDS-II input */
-    while (calmaParseStructure())
+    while (calmaParseStructure(filename))
 	if (SigInterruptPending)
 	    goto done;
     (void) calmaSkipExact(CALMA_ENDLIB);
