@@ -219,6 +219,16 @@ CalmaWrite(rootDef, f)
     bool good;
     CellUse dummy;
 
+    /*
+     * Do not attempt to write anything if a CIF/GDS output style
+     * has not been specified in the technology file.
+     */
+    if (!CIFCurStyle)
+    {
+	TxError("No CIF/GDS output style set!\n");
+	return FALSE;
+    }
+
     /*
      * Make sure that the entire hierarchy rooted at rootDef is
      * read into memory and that timestamp mismatches are resolved
@@ -322,6 +332,9 @@ calmaProcessDef(def, outf)
     CellDef *def;	/* Output this def's children, then the def itself */
     FILE *outf;		/* Stream file */
 {
+    char *filename;
+    bool isReadOnly;
+
     /* Skip if already output */
     if ((int) def->cd_client > 0)
 	return (0);
@@ -345,11 +358,68 @@ calmaProcessDef(def, outf)
      */
     (void) DBCellEnum(def, calmaProcessUse, (ClientData) outf);
 
+    /*
+     * Check if this is a read-only file that is supposed to be copied
+     * verbatim from input to output.  If so, do the direct copy.  If
+     * not, or if there is any problem obtaining the original cell
+     * definition, resort to writing out magic's version of the def,
+     * and print a warning message.
+     */
+
+    filename = (char *)DBPropGet(def, "GDS_FILE", &isReadOnly);
+    if (isReadOnly)
+    {
+	char *buffer, *offptr;
+	size_t defsize, numbytes;
+	off_t cellstart, cellend;
+	FILE *fi;
+
+	fi = fopen(filename, "r");
+	if (fi == NULL)
+	{
+	    TxError("Calma output error:  Can't find GDS file for vendor cell."
+	    		"  Using magic's internal definition\n");
+	    isReadOnly = FALSE;
+	}
+	else
+	{
+	    offptr = (char *)DBPropGet(def, "GDS_BEGIN", NULL);
+	    sscanf(offptr, "%d", &cellstart);
+	    fseek(fi, cellstart, SEEK_SET);
+	    offptr = (char *)DBPropGet(def, "GDS_END", NULL);
+	    sscanf(offptr, "%d", &cellend);
+	    defsize = (size_t)(cellend - cellstart);
+
+	    buffer = (char *)mallocMagic(defsize);
+	    numbytes = fread(buffer, sizeof(char), (size_t)defsize, fi);
+	    if (numbytes == defsize)
+	    {
+		numbytes = fwrite(buffer, sizeof(char), (size_t)defsize, outf);
+		if (numbytes <= 0)
+		{
+		    TxError("Calma output error:  Can't write cell from vendor GDS."
+			"  Using magic's internal definition\n");
+		    isReadOnly = FALSE;
+		}
+	    }
+	    else
+	    {
+		TxError("Calma output error:  Can't read cell from vendor GDS."
+			"  Using magic's internal definition\n");
+		isReadOnly = FALSE;
+	    }
+	    freeMagic(buffer);
+	    fclose(fi);
+	}
+    }
+
     /* Output this cell */
-    calmaOutFunc(def, outf);
+    if (!isReadOnly)
+	calmaOutFunc(def, outf);
 
     return (0);
 }
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -443,7 +513,48 @@ calmaOutFunc(def, f)
     /* End of structure */
     calmaOutRH(4, CALMA_ENDSTR, CALMA_NODATA, f);
 }
-
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * calmaIsUseNameDefault --
+ *
+ * Determine if this use name is not default; that is, it is not the name
+ * of the cell def followed by an underscore and a use index number.  If
+ * it is not default, then we want to write out the use name as a property
+ * in the GDS stream file so that we can recover the name when the file is
+ * read back into magic.
+ *
+ * Results:
+ *	TRUE if the cell use ID is a default name; FALSE if not.
+ *
+ * Side Effects:
+ *	None.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+bool
+calmaIsUseNameDefault(defName, useName)
+    char *defName;
+    char *useName;
+{
+    int idx, slen;
+    char *sptr;
+
+    if (useName == NULL) return TRUE;
+    slen = strlen(defName);
+    if (!strncmp(defName, useName, slen))
+    {
+	sptr = useName + slen;
+	if (*sptr != '_') return FALSE;
+	else sptr++;
+	if (sscanf(sptr, "%d", &idx) != 1) return FALSE;
+	else return TRUE;
+    }
+    return FALSE;
+}
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -631,6 +742,30 @@ calmaWriteUseFunc(use, f)
 	    calmaOutI4(p2.p_y, f);
 	}
 
+	/* By NP */
+	/* Property attributes/value pairs. */
+	/* Add a CellUse ID property, if the CellUse has a non-default name */
+
+	if (!calmaIsUseNameDefault(use->cu_def->cd_name, use->cu_id))
+	{
+	    calmaOutRH(6, CALMA_PROPATTR, CALMA_I2, f);
+	    calmaOutI2(CALMA_PROP_USENAME, f);
+	    calmaOutStringRecord(CALMA_PROPVALUE, use->cu_id, f);
+	}
+
+	/* Add an array limits property, if the CellUse is an array and */
+	/* limits of the array (xlo, ylo) are not zero (the default).	*/
+
+	if ((use->cu_xlo != 0) || (use->cu_ylo != 0))
+	{
+	    char arraystr[128];
+	    sprintf(arraystr, "%d_%d_%d_%d", use->cu_xlo, use->cu_xhi,
+			use->cu_ylo, use->cu_yhi);
+	    calmaOutRH(6, CALMA_PROPATTR, CALMA_I2, f);
+	    calmaOutI2(CALMA_PROP_ARRAY_LIMITS, f);
+	    calmaOutStringRecord(CALMA_PROPVALUE, arraystr, f);
+	}
+
 	/* End of element */
 	calmaOutRH(4, CALMA_ENDEL, CALMA_NODATA, f);
     }
