@@ -23,7 +23,7 @@ static char rcsid[] = "$Header: /ufs/repository/magic/cif/CIFmain.c,v 1.2 2001/0
 #include <stdio.h>
 #include <string.h>
 
-#include "magic/tclmagic.h"
+#include "tcltk/tclmagic.h"
 #include "misc/magic.h"
 #include "utils/geometry.h"
 #include "tiles/tile.h"
