@@ -24,10 +24,12 @@ static char rcsid[] = "$Header: /ufs/repository/magic/cif/CIFrdcl.c,v 1.4 2001/0
 
 #include <stdio.h>
 #include <ctype.h>
+
 #include "misc/magic.h"
 #include "utils/geometry.h"
 #include "tiles/tile.h"
 #include "utils/hash.h"
+#include "undo/undo.h"
 #include "database/database.h"
 #include "cif/CIFint.h"
 #include "cif/CIFread.h"
@@ -130,7 +132,7 @@ CIFReadCellInit(ptrkeys)
 	    cifSubcellPlanes[i] = DBNewPlane((ClientData) TT_SPACE);
     }
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -180,7 +182,6 @@ cifFindCell(cifNum)
     }
     return (CellDef *) HashGetValue(h);
 }
-
 
 /*
  * ----------------------------------------------------------------------------
@@ -193,35 +194,99 @@ cifFindCell(cifNum)
  */
 
 void
-CIFScalePlanes(scalen, scaled)
+CIFScalePlanes(scalen, scaled, planearray)
     int scalen;
     int scaled;
+    Plane **planearray;
 {
     int pNum;
     Plane *newplane;
 
     for (pNum = 0; pNum < MAXCIFRLAYERS; pNum++)
     {
-	if (cifEditCellPlanes[pNum] != NULL)
+	if (planearray[pNum] != NULL)
 	{
 	    newplane = DBNewPlane((ClientData) TT_SPACE);
 	    DBClearPaintPlane(newplane);
-	    dbScalePlane(cifEditCellPlanes[pNum], newplane, pNum, scalen, scaled);
-	    DBFreePaintPlane(cifEditCellPlanes[pNum]);
-	    TiFreePlane(cifEditCellPlanes[pNum]);
-	    cifEditCellPlanes[pNum] = newplane;
+	    dbScalePlane(planearray[pNum], newplane, pNum,
+			scalen, scaled, TRUE);
+	    DBFreePaintPlane(planearray[pNum]);
+	    TiFreePlane(planearray[pNum]);
+	    planearray[pNum] = newplane;
 	}
-	if (cifSubcellPlanes[pNum] != NULL)
+    }
+}
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * CIFInputRescale --
+ *
+ *	Scale all CIF distances by n / d.  Normally, rescaling is done in
+ *	the upward direction (n > 1, d = 1) in response to a value in the
+ *	CIF input that does not divide evenly into the database units of
+ *	the CIF planes.  Currently there is no call to CIFInputRescale
+ *	with d > 1, but it is left for the possibility that scalefactors
+ *	could be restored after finishing a subcell definition, possibly
+ *	preventing integer overflow in large designs.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	Many.  Adjusts the CIF input style scalefactor (CIF vs. magic
+ *	units), multiplier (CIF units vs. centimicrons), distances of all
+ *	CIF boolean operations, and the scale of all existing planes in
+ *	the current CIF database.
+ *
+ * ----------------------------------------------------------------------------
+ */
+ 
+void
+CIFInputRescale(n, d)
+    int n, d;
+{
+    CIFReadStyle *istyle = cifCurReadStyle;
+    CIFReadLayer *cl;
+    CIFOp *op;
+    int i;
+
+    /* 2-step process for efficiency */
+
+    if (n > 1)
+    {
+	istyle->crs_scaleFactor *= n;
+	istyle->crs_multiplier *= n;
+
+	for (i = 0; i < istyle->crs_nLayers; i++)
 	{
-	    newplane = DBNewPlane((ClientData) TT_SPACE);
-	    DBClearPaintPlane(newplane);
-	    dbScalePlane(cifSubcellPlanes[pNum], newplane, pNum, scalen, scaled);
-	    DBFreePaintPlane(cifSubcellPlanes[pNum]);
-	    TiFreePlane(cifSubcellPlanes[pNum]);
-	    cifSubcellPlanes[pNum] = newplane;
+	    cl = istyle->crs_layers[i];
+	    for (op = cl->crl_ops; op != NULL; op = op->co_next)
+		if (op->co_distance)
+		    op->co_distance *= n;
 	}
     }
 
+    if (d > 1)
+    {
+	istyle->crs_scaleFactor /= d;
+	istyle->crs_multiplier /= d;
+
+	for (i = 0; i < istyle->crs_nLayers; i++)
+	{
+	    cl = istyle->crs_layers[i];
+	    for (op = cl->crl_ops; op != NULL; op = op->co_next)
+		if (op->co_distance)
+		    op->co_distance /= d;
+	}
+    }
+
+    CIFScalePlanes(n, d, cifEditCellPlanes);
+    if (cifEditCellPlanes != cifSubcellPlanes)
+	CIFScalePlanes(n, d, cifSubcellPlanes);
+
+    CIFReadWarning("CIF style %s: units rescaled by factor of %d / %d\n",
+		istyle->crs_name, n, d);
 }
 
 /*
@@ -951,10 +1016,45 @@ CIFReadCellCleanup()
 	{
 	    CIFReadError("cell %s was used but not defined.\n", def->cd_name);
         }
+	if (def->cd_flags & CDFLATGDS)
+	{
+	    /* These cells have been flattened and are no longer needed. */
 
-	DRCCheckThis(def, TT_CHECKPAINT, &def->cd_bbox);
-	DBWAreaChanged(def, &def->cd_bbox, DBW_ALLWINDOWS, &DBAllButSpaceBits);
-	DBCellSetModified(def, TRUE);
+	    char *savename = StrDup((char **)NULL, def->cd_name);
+	    int pNum;
+	    Plane **gdsplanes = (Plane **)def->cd_client;
+
+	    UndoDisable();
+	    for (pNum = 0; pNum < MAXCIFRLAYERS; pNum++)
+	    {
+	        if (gdsplanes[pNum] != NULL)
+		{
+		    DBFreePaintPlane(gdsplanes[pNum]);
+		    TiFreePlane(gdsplanes[pNum]);
+		}
+	    }
+	    freeMagic(def->cd_client);
+
+	    if (def->cd_parents != (CellUse *)NULL)
+	    {
+		CellUse *cu_p = def->cd_parents;
+		TxError("GDS read warning:  Cell %s has parent %s\n", savename,
+				cu_p->cu_id);
+		def->cd_parents = (CellUse *)NULL;
+	    }
+	    if (DBCellDeleteDef(def) == FALSE)
+		TxError("GDS read error:  Unable to delete cell %s\n", savename);
+	    else
+		TxPrintf("GDS read:  Removed flattened cell %s\n", savename);
+	    UndoEnable();
+	    freeMagic(savename);
+	}
+	else
+	{
+	    DRCCheckThis(def, TT_CHECKPAINT, &def->cd_bbox);
+	    DBWAreaChanged(def, &def->cd_bbox, DBW_ALLWINDOWS, &DBAllButSpaceBits);
+	    DBCellSetModified(def, TRUE);
+	}
     }
 
     HashKill(&CifCellTable);
