@@ -251,7 +251,7 @@ CIFPolyToRects(path)
 
     if ((tail->cifp_x != path->cifp_x) || (tail->cifp_y != path->cifp_y))
     {
-	MALLOC(CIFPath *, p, sizeof (CIFPath));
+	p = (CIFPath *) mallocMagic ((unsigned) sizeof (CIFPath));
 	p->cifp_x = path->cifp_x;
 	p->cifp_y = path->cifp_y;
 	p->cifp_next = (CIFPath *) 0;
