@@ -28,7 +28,7 @@ static char rcsid[] = "$Header: /ufs/repository/magic/cif/CIFrdtech.c,v 1.5 2001
 #include <string.h>
 #endif
 
-#include "magic/tclmagic.h"
+#include "tcltk/tclmagic.h"
 #include "misc/magic.h"
 #include "utils/geometry.h"
 #include "tiles/tile.h"
@@ -409,6 +409,7 @@ CIFReadTechLine(sectionName, argc, argv)
     int nCalmaLayers, nCalmaTypes, l, t;
 
     if (argc <= 0) return TRUE;
+    else if (argc >= 2) l = strlen(argv[1]);
 
     /* See if we're starting a new style.  If so, create it.  If not,
      * make sure there's already a style around, and create one if
@@ -419,32 +420,78 @@ CIFReadTechLine(sectionName, argc, argv)
     {
 	if (argc != 2)
 	{
-	    wrongNumArgs:
-	    TechError("Wrong number of arguments in %s statement.\n",
-		    argv[0]);
-	    errorReturn:
-	    if (newOp != NULL)
-		freeMagic((char *) newOp);
-	    return TRUE;
+	    if ((argc != 4) || (strncmp(argv[2], "variant", 7)))
+	    {
+		wrongNumArgs:
+		TechError("Wrong number of arguments in %s statement.\n",
+				argv[0]);
+		errorReturn:
+		if (newOp != NULL)
+		    freeMagic((char *) newOp);
+		return TRUE;
+	    }
 	}
 	for (newStyle = cifReadStyleList; newStyle != NULL; 
 		newStyle = newStyle->crs_next)
 	{
-	    if (!strcmp(newStyle->crs_name, argv[1])) break;
+	    if (!strcmp(newStyle->crs_name, argv[1]))
+		break;
 	}
 	if (newStyle == NULL)
 	{
-	    newStyle = (CIFReadKeep *)mallocMagic(sizeof(CIFReadKeep));
-	    newStyle->crs_next = NULL;
-	    newStyle->crs_name = StrDup((char **) NULL, argv[1]);
-
-	    if (cifReadStyleList == NULL)
-		cifReadStyleList = newStyle;
-	    else
+	    if (argc == 2)
 	    {
+		newStyle = (CIFReadKeep *)mallocMagic(sizeof(CIFReadKeep));
+		newStyle->crs_next = NULL;
+		newStyle->crs_name = StrDup((char **) NULL, argv[1]);
+
 		/* Append to end of style list */
-		for (p = cifReadStyleList; p->crs_next; p = p->crs_next);
-		p->crs_next = newStyle;
+		if (cifReadStyleList == NULL)
+		    cifReadStyleList = newStyle;
+		else
+		{
+		    for (p = cifReadStyleList; p->crs_next; p = p->crs_next);
+		    p->crs_next = newStyle;
+		}
+	    }
+	    else	/* Handle style variants */
+	    {
+		CIFReadKeep *saveStyle = NULL;
+		char *tptr, *cptr;
+
+		/* 4th argument is a comma-separated list of variants.	*/
+		/* In addition to the default name recorded above,	*/
+		/* record each of the variants.				*/
+
+		tptr = argv[3];
+		while (*tptr != '\0')
+		{
+		    cptr = strchr(tptr, ',');
+		    if (cptr != NULL) *cptr = '\0';
+		    newStyle = (CIFReadKeep *)mallocMagic(sizeof(CIFReadKeep));
+		    newStyle->crs_next = NULL;
+		    newStyle->crs_name = (char *)mallocMagic(strlen(argv[1])
+				+ strlen(tptr) + 1);
+		    sprintf(newStyle->crs_name, "%s%s", argv[1], tptr);
+
+		    /* Remember the first variant as the default */
+		    if (saveStyle == NULL) saveStyle= newStyle;
+
+		    /* Append to end of style list */
+		    if (cifReadStyleList == NULL)
+			cifReadStyleList = newStyle;
+		    else
+		    {
+			for (p = cifReadStyleList; p->crs_next; p = p->crs_next);
+			p->crs_next = newStyle;
+		    }
+		    
+		    if (cptr == NULL)
+			break;
+		    else
+			tptr = cptr + 1;
+		}
+		newStyle = saveStyle;
 	    }
 	}
 	
@@ -454,16 +501,42 @@ CIFReadTechLine(sectionName, argc, argv)
 	    cifCurReadStyle->crs_name = newStyle->crs_name;
 	    cifCurReadStyle->crs_status = TECH_PENDING;
 	}
-	else if (cifCurReadStyle->crs_status == TECH_PENDING)
+	else if ((cifCurReadStyle->crs_status == TECH_PENDING) ||
+			(cifCurReadStyle->crs_status == TECH_SUSPENDED))
 	    cifCurReadStyle->crs_status = TECH_LOADED;
 	else if (cifCurReadStyle->crs_status == TECH_NOT_LOADED)
 	{
 	    if (cifCurReadStyle->crs_name == NULL)
 		return (FALSE);
-	    else if (!strcmp(argv[1], cifCurReadStyle->crs_name))
+	    else if (argc == 2)
 	    {
-		TxPrintf("Now reading CIF input style %s\n", cifCurReadStyle->crs_name);
-		cifCurReadStyle->crs_status = TECH_PENDING;
+		if (!strcmp(argv[1], cifCurReadStyle->crs_name))
+		    cifCurReadStyle->crs_status = TECH_PENDING;
+	    }
+	    else if (argc == 4)
+	    {
+		/* Verify that the style matches one variant */
+
+		char *tptr, *cptr;
+
+		if (!strncmp(cifCurReadStyle->crs_name, argv[1], l))
+		{
+		    tptr = argv[3];
+		    while (*tptr != '\0')
+		    {
+			cptr = strchr(tptr, ',');
+			if (cptr != NULL) *cptr = '\0';
+			if (!strcmp(cifCurReadStyle->crs_name + l, tptr))
+			{
+			    cifCurReadStyle->crs_status = TECH_PENDING;
+			    return TRUE;
+			}
+			if (cptr == NULL)
+			    return TRUE;
+			else
+			    tptr = cptr + 1;
+		    }
+		}
 	    }
 	}
 	return (TRUE);
@@ -471,7 +544,8 @@ CIFReadTechLine(sectionName, argc, argv)
 
     /* Only continue past this point if we are loading the cif input style */
     if (cifCurReadStyle == NULL) return FALSE;
-    if (cifCurReadStyle->crs_status != TECH_PENDING) return TRUE;
+    if ((cifCurReadStyle->crs_status != TECH_PENDING) &&
+		(cifCurReadStyle->crs_status != TECH_SUSPENDED)) return TRUE;
     
     /* Process scalefactor lines next. */
 
@@ -501,7 +575,52 @@ CIFReadTechLine(sectionName, argc, argv)
 	return TRUE;
     }
 
-    /* Process layer lines next. */
+    /* Process "variant" lines next. */
+
+    if (strncmp(argv[0], "variant", 7) == 0)
+    {
+	int l;
+	char *cptr, *tptr;
+
+	/* If our style variant is not one of the ones declared */
+	/* on the line, then we ignore all input until we 	*/
+	/* either reach the end of the style, the end of the	*/
+	/* section, or another "variant" line.			*/
+
+	if (argc != 2) goto wrongNumArgs;
+	tptr = argv[1];
+	while (*tptr != '\0')
+	{
+	    cptr = strchr(tptr, ',');
+	    if (cptr != NULL) *cptr = '\0';
+
+	    if (*tptr == '*')
+	    {
+		cifCurReadStyle->crs_status = TECH_PENDING;
+		return TRUE;
+	    }
+	    else
+	    {
+		l = strlen(cifCurReadStyle->crs_name) - strlen(tptr);
+		if (!strcmp(tptr, cifCurReadStyle->crs_name + l))
+		{
+		    cifCurReadStyle->crs_status = TECH_PENDING;
+		    return TRUE;
+		}
+	    }
+	
+	    if (cptr == NULL)
+		break;
+	    else
+		tptr = cptr + 1;
+	}
+	cifCurReadStyle->crs_status = TECH_SUSPENDED;
+    }
+
+    /* Anything below this line is not parsed if we're in TECH_SUSPENDED mode */
+    if (cifCurReadStyle->crs_status != TECH_PENDING) return TRUE;
+
+    /* Process layer lines next.	*/
 
     if (strcmp(argv[0], "layer") == 0)
     {
@@ -630,7 +749,9 @@ CIFReadTechLine(sectionName, argc, argv)
 	}
 	return TRUE;
     }
+
     /* miscellaneous cif-reading boolean options */
+
     if(strcmp(argv[0], "options") == 0) {
 	int i;
 	if (argc < 2) goto wrongNumArgs;
@@ -754,7 +875,7 @@ CIFReadTechFinal()
 	 cifCurReadStyle->crs_scaleFactor = 1;
     }
 
-    CIFTechInputScale(1, 1);
+    CIFTechInputScale(1, 1, TRUE);
 
     /* debug info --- Tim, 1/3/02 */
     TxPrintf("Input style %s: scaleFactor=%d, multiplier=%d\n",
@@ -793,7 +914,7 @@ CIFReadLoadStyle(stylename)
     TechLoad(NULL, invcifr);
 
     CIFReadTechFinal();
-    CIFTechInputScale(DBLambda[0], DBLambda[1]);
+    CIFTechInputScale(DBLambda[0], DBLambda[1], TRUE);
 }
     
 /*
@@ -1033,12 +1154,19 @@ cifParseCalmaNums(str, numArray, numNums)
  *   Scale all CIF input scale factors to make them equivalent
  *   to reducing the magic internal unit spacing by a factor of n/d.
  *
+ *   After scaling, we attempt to reduce the ratio scaleFactor : multiplier
+ *   if there is a common factor.  If "opt" is TRUE, we reduce both values
+ *   by this factor.  If FALSE, we do not reduce multiplier below its
+ *   original value.  This is important if we are still in the process of
+ *   reading CIF or GDS input.
+ *
  * ----------------------------------------------------------------------------
  */
 
 int
-CIFTechInputScale(n, d)
+CIFTechInputScale(n, d, opt)
     int n, d;
+    bool opt;
 {
     CIFReadStyle *istyle = cifCurReadStyle;
     CIFReadLayer *cl;
@@ -1075,16 +1203,27 @@ CIFTechInputScale(n, d)
     /* fprintf(stderr, "Multiplier goes from %d to %d\n", istyle->crs_multiplier,
 		istyle->crs_multiplier / lmult); */
 
-    istyle->crs_scaleFactor /= lmult;
-    istyle->crs_multiplier /= lmult;
-
-    for (i = 0; i < istyle->crs_nLayers; i++)
+    if (!opt)
     {
-	cl = istyle->crs_layers[i];
-	for (op = cl->crl_ops; op != NULL; op = op->co_next)
-	    if (op->co_distance)
-		op->co_distance /= lmult;
+	if ((lmult % d) == 0)
+	    lmult = d;
+	else
+	    lmult = 1;
     }
-    /* fflush(stderr); */
+
+    if (lmult > 1)
+    {
+	istyle->crs_scaleFactor /= lmult;
+	istyle->crs_multiplier /= lmult;
+
+	for (i = 0; i < istyle->crs_nLayers; i++)
+	{
+	    cl = istyle->crs_layers[i];
+	    for (op = cl->crl_ops; op != NULL; op = op->co_next)
+		if (op->co_distance)
+		    op->co_distance /= lmult;
+	}
+    }
+    return lmult;
 }
 
