@@ -71,7 +71,7 @@ int CIFRescaleLimit = CIFMAXRESCALE;	/* Don't increase cifReadScale1 by more
 					 * than this amount;  internal units
 					 * finer than this will be rounded.
 					 */
-int CIFRescaleAllow = TRUE;		/* Don't subdivide the magic internal
+bool CIFRescaleAllow = TRUE;		/* Don't subdivide the magic internal
 					 * grid if this is FALSE.
 					 */
 char *CIFErrorFilename;			/* Name of file for error redirection */
@@ -186,6 +186,7 @@ CIFScaleCoord(cifCoord, snap_type)
     int snap_type;			/* How to deal with fractional results */
 {
     int result, scale, remain, denom;
+    int mult, mfactor;
 
     /* If internal grid subdivision is disallowed, always round to the	*/
     /* nearest grid unit.						*/
@@ -194,6 +195,7 @@ CIFScaleCoord(cifCoord, snap_type)
 	snap_type = COORD_ANY;
 
     scale = cifCurReadStyle->crs_scaleFactor;
+    mult = cifCurReadStyle->crs_multiplier;
 
     /* Check for non-integer result and warn of fractional-lambda violation */
 
@@ -210,7 +212,7 @@ CIFScaleCoord(cifCoord, snap_type)
 		CIFReadWarning("Input off lambda grid by %d/%d; grid redefined.\n",
 			remain, denom);
 
-		CIFTechInputScale(1, denom);
+		CIFTechInputScale(1, denom, FALSE);
 		CIFTechOutputScale(1, denom);
 		DRCTechScale(1, denom);
 		ExtTechScale(1, denom);
@@ -236,7 +238,7 @@ CIFScaleCoord(cifCoord, snap_type)
 		    /* scale to nearest half-lambda */
 		    if (!(denom & 0x1)) denom >>= 1;
 
-		    CIFTechInputScale(1, denom);
+		    CIFTechInputScale(1, denom, FALSE);
 		    CIFTechOutputScale(1, denom);
 		    DRCTechScale(1, denom);
 		    ExtTechScale(1, denom);
@@ -512,77 +514,6 @@ CIFParseInteger(valuep)
     return TRUE;
 }
 
-/*
- * ----------------------------------------------------------------------------
- *
- * CIFInputRescale --
- *
- *	Scale all CIF distances by n / d.  Normally, rescaling is done in
- *	the upward direction (n > 1, d = 1) in response to a value in the
- *	CIF input that does not divide evenly into the database units of
- *	the CIF planes.  Currently there is no call to CIFInputRescale
- *	with d > 1, but it is left for the possibility that scalefactors
- *	could be restored after finishing a subcell definition, possibly
- *	preventing integer overflow in large designs.
- *
- * Results:
- *	None.
- *
- * Side effects:
- *	Many.  Adjusts the CIF input style scalefactor (CIF vs. magic
- *	units), multiplier (CIF units vs. centimicrons), distances of all
- *	CIF boolean operations, and the scale of all existing planes in
- *	the current CIF database.
- *
- * ----------------------------------------------------------------------------
- */
- 
-void
-CIFInputRescale(n, d)
-    int n, d;
-{
-    CIFReadStyle *istyle = cifCurReadStyle;
-    CIFReadLayer *cl;
-    CIFOp *op;
-    int i;
-
-    /* 2-step process for efficiency */
-
-    if (n > 1)
-    {
-	istyle->crs_scaleFactor *= n;
-	istyle->crs_multiplier *= n;
-
-	for (i = 0; i < istyle->crs_nLayers; i++)
-	{
-	    cl = istyle->crs_layers[i];
-	    for (op = cl->crl_ops; op != NULL; op = op->co_next)
-		if (op->co_distance)
-		    op->co_distance *= n;
-	}
-    }
-
-    if (d > 1)
-    {
-	istyle->crs_scaleFactor /= d;
-	istyle->crs_multiplier /= d;
-
-	for (i = 0; i < istyle->crs_nLayers; i++)
-	{
-	    cl = istyle->crs_layers[i];
-	    for (op = cl->crl_ops; op != NULL; op = op->co_next)
-		if (op->co_distance)
-		    op->co_distance /= d;
-	}
-    }
-
-    CIFScalePlanes(n, d);
-    CIFReadWarning("CIF style %s: units rescaled by factor of %d / %d\n",
-		istyle->crs_name, n, d);
-}
-
-
-
 /*
  * ----------------------------------------------------------------------------
  *
@@ -680,10 +611,6 @@ CIFParsePoint(pointp, iscale)
  * 	This procedure parses a CIF path, which is sequence of
  *	one or more points.
  *
- *	If the path is non-Manhattan, we introduce additional
- *	stair-steps a minimum of cifCurReadStyle->crs_scaleFactor
- *	high and wide along each non-Manhattan segment.
- *
  * Results:
  *	TRUE is returned if the path was parsed successfully,
  *	FALSE otherwise.
@@ -708,7 +635,7 @@ CIFParsePath(pathheadpp, iscale)
     int iscale;
 {
     CIFPath *pathtailp, *newpathp;
-    bool nonManhattan = FALSE;
+    bool nonManhattan = FALSE;		/* diagnostic only */
     CIFPath path;
     int savescale;
 
@@ -737,13 +664,11 @@ CIFParsePath(pathheadpp, iscale)
 		phead->cifp_y *= newscale;
 		phead = phead->cifp_next;
 	    }
-	    savescale = cifReadScale1;
 	}
-	MALLOC(CIFPath *, newpathp, sizeof (CIFPath));
+	newpathp = (CIFPath *) mallocMagic((unsigned) (sizeof (CIFPath)));
 	*newpathp = path;
 	if (*pathheadpp)
 	{
-#ifndef NONMANHATTAN
 	    /*
 	     * Check that this segment is Manhattan.  If not, remember the
 	     * fact and later introduce extra stair-steps to make the path
@@ -755,22 +680,17 @@ CIFParsePath(pathheadpp, iscale)
 	    if (pathtailp->cifp_x != newpathp->cifp_x
 		    && pathtailp->cifp_y != (newpathp->cifp_y))
 	    {
+#ifndef NONMANHATTAN
 		if (!nonManhattan)
 		    CIFReadWarning("non-Manhattan path; using stairstep.\n");
+#endif
 		nonManhattan = TRUE;
 	    }
-#endif
 	    pathtailp->cifp_next = newpathp;
 	}
 	else *pathheadpp = newpathp;
 	pathtailp = newpathp;
     }
-
-#ifndef NONMANHATTAN
-    if (nonManhattan)
-	CIFMakeManhattanPath(*pathheadpp);
-#endif
-
     return (*pathheadpp != NULL);
 }
 
@@ -960,7 +880,7 @@ path_intersect(pathHead, start, respt)
 
     if (does_cross && segcrossed)
     {
-	MALLOC(CIFPath *, new, sizeof (CIFPath));
+	new = (CIFPath *) mallocMagic((unsigned) (sizeof (CIFPath)));
 	new->cifp_next = segcrossed->cifp_next;
 	segcrossed->cifp_next = new;
  	new->cifp_x = respt->p_x;
@@ -1106,7 +1026,7 @@ CIFMakeManhattanPath(pathHead, plane, resultTbl, ui)
 
 	/* Otherwise, break out the triangle, then adjust as necessary */
 
-	MALLOC(CIFPath *, new, sizeof (CIFPath));
+	new = (CIFPath *) mallocMagic((unsigned) (sizeof (CIFPath)));
 	path->cifp_next = new;
 	new->cifp_next = next;
 
@@ -1142,7 +1062,7 @@ CIFMakeManhattanPath(pathHead, plane, resultTbl, ui)
 	    new->cifp_x = clipbase.p_x;
 	    new->cifp_y = clipbase.p_y;
 
-	    MALLOC(CIFPath *, new2, sizeof (CIFPath));
+	    new2 = (CIFPath *) mallocMagic((unsigned) (sizeof (CIFPath)));
 	    new->cifp_next = new2;
 	    new2->cifp_next = next;
 
@@ -1172,11 +1092,9 @@ CIFMakeManhattanPath(pathHead, plane, resultTbl, ui)
 	tt.r_ur = path->cifp_next->cifp_next->cifp_point;
 	GeoCanonicalRect(&tt, &tr);
 
-/*
-        TxPrintf("CIF read: Triangle %s %c at (%d, %d) plane %x\n",
-                (type & TT_SIDE) ? "right" : "left", (type & TT_DIRECTION)
-		? '\\' : '/', tt.r_xbot, tt.r_ybot, plane);
-*/
+//	TxPrintf("CIF read: Triangle %s %c at (%d, %d) plane %x\n",
+//		(type & TT_SIDE) ? "right" : "left", (type & TT_DIRECTION)
+//		? '\\' : '/', tt.r_xbot, tt.r_ybot, plane);
 
         if (plane)
             DBNMPaintPlane(plane, type, &tr, resultTbl, ui);
@@ -1245,7 +1163,7 @@ CIFMakeManhattanPath(pathHead)
 		    y += yincr)
 	    {
 		/* Move by one in y first */
-		MALLOC(CIFPath *, new, sizeof (CIFPath));
+		new = (CIFPath *) mallocMagic((unsigned) (sizeof (CIFPath)));
 		new->cifp_x = x;
 		new->cifp_y = y;
 		path->cifp_next = new;
@@ -1259,7 +1177,7 @@ CIFMakeManhattanPath(pathHead)
 		 * path->cifp_x up to but not reaching next->cifp_x.
 		 */
 		x = xinit + (xdiff * (y - yinit)) / ydiff;
-		MALLOC(CIFPath *, new, sizeof (CIFPath));
+		new = (CIFPath *) mallocMagic ((unsigned) (sizeof (CIFPath)));
 		new->cifp_x = x;
 		new->cifp_y = y;
 		new->cifp_next = next;
@@ -1275,7 +1193,7 @@ CIFMakeManhattanPath(pathHead)
 	     */
 	    if (x != next->cifp_x)
 	    {
-		MALLOC(CIFPath *, new, sizeof (CIFPath));
+		new = (CIFPath *) mallocMagic((unsigned) (sizeof (CIFPath)));
 		new->cifp_x = next->cifp_x;
 		new->cifp_y = ylast;;
 		new->cifp_next = next;
@@ -1295,7 +1213,7 @@ CIFMakeManhattanPath(pathHead)
 		x += xincr)
 	    {
 		/* Move by one in x first */
-		MALLOC(CIFPath *, new, sizeof (CIFPath));
+		new = (CIFPath *) mallocMagic((unsigned) (sizeof (CIFPath)));
 		new->cifp_x = x;
 		new->cifp_y = y;
 		path->cifp_next = new;
@@ -1309,7 +1227,7 @@ CIFMakeManhattanPath(pathHead)
 		 * path->cifp_y up to but not reaching next->cifp_y.
 		 */
 		y = yinit + (ydiff * (x - xinit)) / xdiff;
-		MALLOC(CIFPath *, new, sizeof (CIFPath));
+		new = (CIFPath *) mallocMagic((unsigned) (sizeof (CIFPath)));
 		new->cifp_x = x;
 		new->cifp_y = y;
 		new->cifp_next = next;
@@ -1325,7 +1243,7 @@ CIFMakeManhattanPath(pathHead)
 	     */
 	    if (y != next->cifp_y)
 	    {
-		MALLOC(CIFPath *, new, sizeof (CIFPath));
+		new = (CIFPath *) mallocMagic((unsigned) (sizeof (CIFPath)));
 		new->cifp_x = xlast;
 		new->cifp_y = next->cifp_y;
 		new->cifp_next = next;
@@ -1465,7 +1383,7 @@ path_inc:
 	if ((pathHead->cifp_x != path->cifp_x) ||	
 	    (pathHead->cifp_y != path->cifp_y))
 	{
-	    MALLOC(CIFPath *, next, sizeof (CIFPath));
+	    next = (CIFPath *) mallocMagic((unsigned) (sizeof (CIFPath)));
 	    next->cifp_x = pathHead->cifp_x;
 	    next->cifp_y = pathHead->cifp_y;
 	    next->cifp_next = (CIFPath *) 0;
