@@ -65,6 +65,7 @@ TileTypeBitMask CIFSolidBits;
 /* Forward Declarations */
 
 void cifTechStyleInit();
+bool cifCheckCalmaNum();
 
 
 /*
@@ -419,7 +420,7 @@ CIFTechLine(sectionName, argc, argv)
     char *argv[];		/* Values of fields. */
 {
     TileTypeBitMask mask, tempMask, bloatLayers;
-    int i, j, distance;
+    int i, j, l, distance;
     CIFLayer *newLayer;
     CIFOp *newOp = NULL;
     CIFKeep *newStyle, *p;
@@ -427,6 +428,7 @@ CIFTechLine(sectionName, argc, argv)
     int local_bloats[TT_MAXTYPES];
 
     if (argc <= 0) return TRUE;
+    else if (argc >= 2) l = strlen(argv[1]);
 
     /* See if we're starting a new CIF style.  If not, make
      * sure that the current (maybe default?) CIF style has
@@ -435,26 +437,83 @@ CIFTechLine(sectionName, argc, argv)
     
     if (strcmp(argv[0], "style") == 0)
     {
-	if (argc != 2) goto wrongNumArgs;
-
-        for (newStyle = CIFStyleList; newStyle != NULL;
+	if (argc != 2)
+	{
+	    if ((argc != 4) || (strncmp(argv[2], "variant", 7)))
+	    {
+		wrongNumArgs:
+		TechError("Wrong number of arguments in %s statement.\n",
+				argv[0]);
+		errorReturn:
+		if (newOp != NULL)
+		    freeMagic((char *) newOp);
+		return TRUE;
+	    }
+	}
+	for (newStyle = CIFStyleList; newStyle != NULL; 
 		newStyle = newStyle->cs_next)
 	{
-	    if (!strcmp(newStyle->cs_name, argv[1])) break;
+	    /* Here we're only establishing existance;		*/
+	    /* break on the first variant found.		*/
+
+	    if (!strcmp(newStyle->cs_name, argv[1]))
+		break;
 	}
 	if (newStyle == NULL)
 	{
-	    newStyle = (CIFKeep *)mallocMagic(sizeof(CIFKeep));
-	    newStyle->cs_next = NULL;
-	    newStyle->cs_name = StrDup((char **) NULL, argv[1]);
-
-	    if (CIFStyleList == NULL)
-	        CIFStyleList = newStyle;
-	    else
+	    if (argc == 2)
 	    {
-	        /* Append to end of style list */
-	        for (p = CIFStyleList; p->cs_next; p = p->cs_next);
-	        p->cs_next = newStyle;
+		newStyle = (CIFKeep *)mallocMagic(sizeof(CIFKeep));
+		newStyle->cs_next = NULL;
+		newStyle->cs_name = StrDup((char **) NULL, argv[1]);
+
+		/* Append to end of style list */
+		if (CIFStyleList == NULL)
+		    CIFStyleList = newStyle;
+		else
+		{
+		    for (p = CIFStyleList; p->cs_next; p = p->cs_next);
+		    p->cs_next = newStyle;
+		}
+	    }
+	    else	/* Handle style variants */
+	    {
+		CIFKeep *saveStyle = NULL;
+		char *tptr, *cptr;
+
+		/* 4th argument is a comma-separated list of variants.	*/
+		/* In addition to the default name recorded above,	*/
+		/* record each of the variants.				*/
+
+		tptr = argv[3];
+		while (*tptr != '\0')
+		{
+		    cptr = strchr(tptr, ',');
+		    if (cptr != NULL) *cptr = '\0';
+		    newStyle = (CIFKeep *)mallocMagic(sizeof(CIFKeep));
+		    newStyle->cs_next = NULL;
+		    newStyle->cs_name = (char *)mallocMagic(l
+				+ strlen(tptr) + 1);
+		    sprintf(newStyle->cs_name, "%s%s", argv[1], tptr);
+
+		    /* Remember the first variant as the default */
+		    if (saveStyle == NULL) saveStyle= newStyle;
+
+		    /* Append to end of style list */
+		    if (CIFStyleList == NULL)
+			CIFStyleList = newStyle;
+		    else
+		    {
+			for (p = CIFStyleList; p->cs_next; p = p->cs_next);
+			p->cs_next = newStyle;
+		    }
+		    
+		    if (cptr == NULL)
+			break;
+		    else
+			tptr = cptr + 1;
+		}
+		newStyle = saveStyle;
 	    }
 	}
 
@@ -464,37 +523,60 @@ CIFTechLine(sectionName, argc, argv)
 	    CIFCurStyle->cs_name = newStyle->cs_name;
 	    CIFCurStyle->cs_status = TECH_PENDING;
 	}
-	else if (CIFCurStyle->cs_status == TECH_PENDING)
+	else if ((CIFCurStyle->cs_status == TECH_PENDING) ||
+			(CIFCurStyle->cs_status == TECH_SUSPENDED))
 	    CIFCurStyle->cs_status = TECH_LOADED;
 	else if (CIFCurStyle->cs_status == TECH_NOT_LOADED)
 	{
 	    if (CIFCurStyle->cs_name == NULL)
 		return (FALSE);
-	    else if (!strcmp(argv[1], CIFCurStyle->cs_name))
-		CIFCurStyle->cs_status = TECH_PENDING;
+	    else if (argc == 2)
+	    {
+		if (!strcmp(argv[1], CIFCurStyle->cs_name))
+		    CIFCurStyle->cs_status = TECH_PENDING;
+	    }
+	    else if (argc == 4)
+	    {
+		/* Verify that the style matches one variant */
+
+		char *tptr, *cptr;
+
+		if (!strncmp(CIFCurStyle->cs_name, argv[1], l))
+		{
+		    tptr = argv[3];
+		    while (*tptr != '\0')
+		    {
+			cptr = strchr(tptr, ',');
+			if (cptr != NULL) *cptr = '\0';
+			if (!strcmp(CIFCurStyle->cs_name + l, tptr))
+			{
+			    CIFCurStyle->cs_status = TECH_PENDING;
+			    return TRUE;
+			}
+			if (cptr == NULL)
+			    return TRUE;
+			else
+			    tptr = cptr + 1;
+		    }
+		}
+	    }
 	}
 	return (TRUE);
     }
 
     /* Only continue past this point if we are loading the cif output style */
     if (CIFCurStyle == NULL) return FALSE;
-    if (CIFCurStyle->cs_status != TECH_PENDING) return TRUE;
+    if ((CIFCurStyle->cs_status != TECH_PENDING) &&
+		(CIFCurStyle->cs_status != TECH_SUSPENDED))
+	return TRUE;
 
     /* Process scalefactor lines next. */
 
     if (strcmp(argv[0], "scalefactor") == 0)
     {
-	if ((argc < 2) || (argc > 4))
-	{
-wrongNumArgs:
-	    TechError("Wrong number of arguments in cif %s statement\n"
-			"Do you have a space in a list of layers?\n\n",
-			argv[0]);
-	    errorReturn:
-	    if (newOp != NULL) freeMagic((char *) newOp);
-	    return TRUE;
-	}
-	CIFCurStyle->cs_scaleFactor = CIFParseScale(argv[1], &CIFCurStyle->cs_expander);
+	if ((argc < 2) || (argc > 4)) goto wrongNumArgs;
+	CIFCurStyle->cs_scaleFactor = CIFParseScale(argv[1],
+		&CIFCurStyle->cs_expander);
 
 	/*
 	 * The "nanometers" keyword multiplies the expander by 10.
@@ -528,6 +610,51 @@ wrongNumArgs:
 	return TRUE;
     }
 
+    /* Process "variant" lines next. */
+
+    if (strncmp(argv[0], "variant", 7) == 0)
+    {
+	int l;
+	char *cptr, *tptr;
+
+	/* If our style variant is not one of the ones declared */
+	/* on the line, then we ignore all input until we 	*/
+	/* either reach the end of the style, the end of the	*/
+	/* section, or another "variant" line.			*/
+
+	if (argc != 2) goto wrongNumArgs;
+	tptr = argv[1];
+	while (*tptr != '\0')
+	{
+	    cptr = strchr(tptr, ',');
+	    if (cptr != NULL) *cptr = '\0';
+
+	    if (*tptr == '*')
+	    {
+		CIFCurStyle->cs_status = TECH_PENDING;
+		return TRUE;
+	    }
+	    else
+	    {
+		l = strlen(CIFCurStyle->cs_name) - strlen(tptr);
+		if (!strcmp(tptr, CIFCurStyle->cs_name + l))
+		{
+		    CIFCurStyle->cs_status = TECH_PENDING;
+		    return TRUE;
+		}
+	    }
+	
+	    if (cptr == NULL)
+		break;
+	    else
+		tptr = cptr + 1;
+	}
+	CIFCurStyle->cs_status = TECH_SUSPENDED;
+    }
+
+    /* Anything below this line is not parsed if we're in TECH_SUSPENDED mode */
+    if (CIFCurStyle->cs_status != TECH_PENDING) return TRUE;
+
     newLayer = NULL;
     if ((strcmp(argv[0], "templayer") == 0)
 	|| (strcmp(argv[0], "layer") == 0))
@@ -829,6 +956,7 @@ wrongNumArgs:
 	
 	case CIFOP_SQUARES_G:
 	    local_bloats[3] = 1; /* set default grid and fall through */
+	    local_bloats[4] = 1;
 	case CIFOP_SQUARES:
 	    if (argc == 2)
 	    {
@@ -844,8 +972,8 @@ wrongNumArgs:
 		local_bloats[2] = i;
 		local_bloats[3] = 0;	/* placeholder; not used by SQUARES */
 	    }
-	    else if (argc == 4 || 
-		      (argc==5 && newOp->co_opcode==CIFOP_SQUARES_G))
+	    else if (argc == 4 || ((argc == 5 || argc == 6)
+			&& newOp->co_opcode==CIFOP_SQUARES_G))
 	    {
 		local_bloats[0] = atoi(argv[1]);
 		if (local_bloats[0] < 0)
@@ -865,14 +993,25 @@ wrongNumArgs:
 		    TechError("Square separation must be positive.\n");
 		    goto errorReturn;
 		}
-		if ( argc == 5 ) {
-		    local_bloats[3] = atoi(argv[4]);
+		if (argc >= 5)
+		{
+		    local_bloats[3] = local_bloats[4] = atoi(argv[4]);
 		    if (local_bloats[3] <= 0)
 		    {
-		        TechError("Square grid must be positive.\n");
+		        TechError("Square grid must be strictly positive.\n");
+			local_bloats[3] = 1;
+		        goto errorReturn;
+		    }
+		}
+		if (argc == 6)
+		{
+		    local_bloats[4] = atoi(argv[5]);
+		    if (local_bloats[4] <= 0)
+		    {
+		        TechError("Square y-grid must be strictly positive.\n");
+			local_bloats[4] = 1;
 		        goto errorReturn;
 		    }
-		 
 		}
 	    }
 	    else goto wrongNumArgs;
@@ -886,11 +1025,15 @@ wrongNumArgs:
     switch (newOp->co_opcode)
     {
 	case CIFOP_SQUARES:
-	case CIFOP_SQUARES_G:
 	    newOp->co_bloats = (int *) mallocMagic(4 * sizeof(int));
 	    for (i = 0; i < 4; i++)
 		newOp->co_bloats[i] = local_bloats[i];
 	    break;
+	case CIFOP_SQUARES_G:
+	    newOp->co_bloats = (int *) mallocMagic(5 * sizeof(int));
+	    for (i = 0; i < 5; i++)
+		newOp->co_bloats[i] = local_bloats[i];
+	    break;
 	case CIFOP_BLOAT:
 	case CIFOP_BLOATMAX:
 	case CIFOP_BLOATMIN:
@@ -1140,7 +1283,7 @@ CIFTechFinal()
 	    }
 	    if (op->co_bloats)
 	    {
-		if (op->co_opcode == CIFOP_SQUARES || op->co_opcode == CIFOP_SQUARES_G)
+		if (op->co_opcode == CIFOP_SQUARES)
 		{
 		    for (j = 0; j < 4; j++)
 		    {
@@ -1161,7 +1304,8 @@ CIFTechFinal()
 		}
 		else
 		{
-		    for (j = 0; j < TT_MAXTYPES; j++)
+		    for (j = 0; j < ((op->co_opcode == CIFOP_SQUARES_G)
+				? 5 : TT_MAXTYPES); j++)
 		    {
 			if (op->co_bloats[j] != 0)
 			{
@@ -1395,8 +1539,7 @@ CIFTechOutputScale(n, d)
 	{
 	    cl = ostyle->cs_layers[i];
 	    for (op = cl->cl_ops; op != NULL; op = op->co_next)
-		if ((op->co_opcode == CIFOP_SQUARES)
-				|| (op->co_opcode == CIFOP_SQUARES_G))
+		if (op->co_opcode == CIFOP_SQUARES)
 		    if ((op->co_bloats[1] & 0x01) || (op->co_bloats[2] & 0x01))
 		    {
 			has_odd_space = TRUE;
@@ -1434,8 +1577,7 @@ CIFTechOutputScale(n, d)
 	    if (op->co_bloats)
 	    {
 		int bvalue;
-		if ((op->co_opcode == CIFOP_SQUARES) || (op->co_opcode ==
-				CIFOP_SQUARES_G))
+		if (op->co_opcode == CIFOP_SQUARES)
 		{
 		    for (j = 0; j < 4; j++)
 		    {
@@ -1451,7 +1593,8 @@ CIFTechOutputScale(n, d)
 		}
 		else
 		{
-		    for (j = 0; j < TT_MAXTYPES; j++)
+		    for (j = 0; j < ((op->co_opcode == CIFOP_SQUARES_G)
+				? 5 : TT_MAXTYPES); j++)
 		    {
 			if (op->co_bloats[j] != 0)
 			{
@@ -1495,9 +1638,19 @@ CIFTechOutputScale(n, d)
 
 	    if (op->co_bloats)
 	    {
-		int nlayers = TT_MAXTYPES;
-		if (op->co_opcode == CIFOP_SQUARES || op->co_opcode ==
-				CIFOP_SQUARES_G) nlayers = 4;
+		int nlayers;
+		switch (op->co_opcode)
+		{
+		    case CIFOP_SQUARES:
+			nlayers = 4;
+			break;
+		    case CIFOP_SQUARES_G:
+			nlayers = 5;
+			break;
+		    default:
+			nlayers = TT_MAXTYPES;
+			break;
+		}
 		for (j = 0; j < nlayers; j++)
 		    if (op->co_bloats[j] != 0)
 			op->co_bloats[j] /= lexpand;
