@@ -34,6 +34,7 @@ static char rcsid[] = "$Header: /ufs/repository/magic/cif/CIFwrite.c,v 1.4 2001/
 #include "database/databaseInt.h"
 #include "tech/tech.h"
 #include "utils/stack.h"
+#include "undo/undo.h"
 #include "cif/cif.h"
 #include "cif/CIFint.h"
 #include "signals/signals.h"
@@ -172,7 +173,7 @@ CIFWrite(rootDef, f)
      * Now we are almost done.
      * Just output a call on the root cell
      */
-    (void) fprintf(f, "C %d;\nEnd\n", (int) rootDef->cd_client);
+    fprintf(f, "C %d;\nEnd\n", (int) rootDef->cd_client);
     good = !ferror(f);
     return (good);
 }
@@ -355,19 +356,19 @@ cifOutFunc(def, f)
     int type;
     CIFLayer *layer;
 
-    (void) fprintf(f, "DS %d %d %d;\n", (int) def->cd_client,
+    fprintf(f, "DS %d %d %d;\n", (int) def->cd_client,
 	CIFCurStyle->cs_reducer, 2 * CIFCurStyle->cs_expander);
 
     if (def->cd_name != (char *) NULL)
 	if (def->cd_name[0] != '\0')
 	{
 	    if (strcmp(def->cd_name, "(UNNAMED)") == 0)
-		(void) fprintf(f, "9 UNNAMED;\n");
+		fprintf(f, "9 UNNAMED;\n");
 	    else {
 	        if (CIFPathPrefix && CIFPathPrefix[0])
-		  (void) fprintf(f, "9 %s/%s;\n", CIFPathPrefix, def->cd_name);
+		  fprintf(f, "9 %s/%s;\n", CIFPathPrefix, def->cd_name);
 		else
-		  (void) fprintf(f, "9 %s;\n", def->cd_name);
+		  fprintf(f, "9 %s;\n", def->cd_name);
 	    }
 	}
 
@@ -415,13 +416,13 @@ cifOutFunc(def, f)
 	    size.p_y /= CIFCurStyle->cs_reducer;
 	    if (type >= 0)
 	    {
-		(void) fprintf(f, "95 %s %d %d %d %d %s;\n",
+		fprintf(f, "95 %s %d %d %d %d %s;\n",
 		    lab->lab_text, size.p_x, size.p_y, center.p_x, center.p_y,
 		    CIFCurStyle->cs_layers[type]->cl_name);
 	    }
 	    else
 	    {
-		(void) fprintf(f, "95 %s %d %d %d %d;\n",
+		fprintf(f, "95 %s %d %d %d %d;\n",
 		    lab->lab_text, size.p_x, size.p_y, center.p_x, center.p_y);
 	    }
 	}
@@ -429,13 +430,13 @@ cifOutFunc(def, f)
 	{
 	    if (type >= 0)
 	    {
-		(void) fprintf(f, "94 %s %d %d %s;\n",
+		fprintf(f, "94 %s %d %d %s;\n",
 		    lab->lab_text, center.p_x, center.p_y,
 		    CIFCurStyle->cs_layers[type]->cl_name);
 	    }
 	    else
 	    {
-		(void) fprintf(f, "94 %s %d %d;\n",
+		fprintf(f, "94 %s %d %d;\n",
 		    lab->lab_text, center.p_x, center.p_y);
 	    }
 	}
@@ -446,7 +447,7 @@ cifOutFunc(def, f)
      * arrays it is necessary to output one call for each instance.
      */
     (void) DBCellEnum(def, cifWriteUseFunc, (ClientData) f);
-    (void) fprintf(f, "DF;\n");
+    fprintf(f, "DF;\n");
 }
 
 /*
@@ -498,7 +499,7 @@ cifWriteUseFunc(use, f)
 	     * Insert a 91 user command to label the next cell
 	     */
 	    if (CIFDoCellIdLabels && use->cu_id && use->cu_id[0]) {
-	      (void) fprintf (f, "91 %s", use->cu_id);
+	      fprintf (f, "91 %s", use->cu_id);
 	      if (topx > 0 || topy > 0) {
 		if (topx > 0 && topy > 0)
 		  fprintf (f, "(%d,%d)", realy, realx);
@@ -509,9 +510,9 @@ cifWriteUseFunc(use, f)
 		    fprintf (f, "(%d)", realy);
 		}
 	      }
-	      (void) fprintf (f, ";\n");
+	      fprintf (f, ";\n");
 	    }
-	    (void) fprintf(f, "C %d", cifnum);
+	    fprintf(f, "C %d", cifnum);
 
 	    /*
 	     * The following translates from the abcdef transforms
@@ -523,11 +524,11 @@ cifWriteUseFunc(use, f)
 
 	    t = &use->cu_transform;
 	    if ((t->t_a != t->t_e) || ((t->t_a == 0) && (t->t_b == t->t_d)))
-		(void) fprintf(f, " MX R %d %d", -(t->t_a), -(t->t_d));
+		fprintf(f, " MX R %d %d", -(t->t_a), -(t->t_d));
 	    else
-		(void) fprintf(f, " R %d %d", t->t_a, t->t_d);
+		fprintf(f, " R %d %d", t->t_a, t->t_d);
 
-	    (void) fprintf(f, " T %d %d;\n",
+	    fprintf(f, " T %d %d;\n",
 		((t->t_c + t->t_a*(use->cu_xsep)*x + t->t_b*(use->cu_ysep)*y)
 		    * 2 * CIFCurStyle->cs_scaleFactor) / CIFCurStyle->cs_reducer,
 		((t->t_f + t->t_d*(use->cu_xsep)*x + t->t_e*(use->cu_ysep)*y)
@@ -573,7 +574,7 @@ cifWritePaintFunc(tile, f)
 
     if (cifPaintLayerName != NULL)
     {
-	(void) fprintf(f, "L %s;\n", cifPaintLayerName);
+	fprintf(f, "L %s;\n", cifPaintLayerName);
 	cifPaintLayerName = NULL;
     }
 
@@ -585,19 +586,18 @@ cifWritePaintFunc(tile, f)
 	Point points[5];
 	int i, np;
 
-	GrClipTriangle(&r, NULL, FALSE, SplitDirection(tile),
-	SplitSide(tile), points, &np);
+	GrClipTriangle(&r, NULL, FALSE, TiGetTypeExact(tile), points, &np);
 
 	/* Write triangle as a CIF polygon */
 
-	(void) fprintf(f, "    P");
+	fprintf(f, "    P");
 	for (i = 0; i < np; i++)
 	{
-	    (void) fprintf(f, " %d %d",
+	    fprintf(f, " %d %d",
 		(2*cifPaintScale*(points[i].p_x))/CIFCurStyle->cs_reducer,
 		(2*cifPaintScale*(points[i].p_y))/CIFCurStyle->cs_reducer);
 	}
-	(void) fprintf(f, ";\n");
+	fprintf(f, ";\n");
     }
     else
 #endif
@@ -608,7 +608,7 @@ cifWritePaintFunc(tile, f)
      * by shrinking by a factor of two in the "DS" statement.
      */
 
-    (void) fprintf(f, "    B %d %d %d %d;\n",
+    fprintf(f, "    B %d %d %d %d;\n",
 	(2*cifPaintScale*(r.r_xtop - r.r_xbot))/CIFCurStyle->cs_reducer,
 	(2*cifPaintScale*(r.r_ytop - r.r_ybot))/CIFCurStyle->cs_reducer,
 	(cifPaintScale*(r.r_xtop + r.r_xbot))/CIFCurStyle->cs_reducer,
@@ -698,7 +698,7 @@ CIFWriteFlat(rootDef, f)
      * Just output a call on the root cell
      */
 
-    (void) fprintf(f, "C %d;\nEnd\n", (int) CIFComponentDef->cd_client);
+    fprintf(f, "C %d;\nEnd\n", (int) CIFComponentDef->cd_client);
     DBCellClearDef(CIFComponentDef);
     good = !ferror(f);
     
