@@ -19,18 +19,10 @@
  * rcsid $Header: /ufs/repository/magic/cmwind/cmwind.h,v 1.2 2001/01/12 22:12:16 jsolomon Exp $
  */
 
-#define _CMWIND
-
-#ifndef _MAGIC
-    err0 = Need_to_include_misc/magic_header;
-#endif	/* _MAGIC */
-#ifndef	_GEOMETRY
-    err1 = Need_to_include_utils/geometry_header;
-#endif	/* _GEOMETRY */
-#ifndef _WINDOWS
-    err2 = Need_to_include_windows/windows_header;
-#endif	/* _WINDOWS */
+#ifndef _CMWIND_H
+#define _CMWIND_H
 
+#include "windows/windows.h"
 
 /* data structures */
 
@@ -83,3 +75,5 @@ extern ColorPump colorPumps[];
 extern Rect cmwCurrentColorArea;
 extern void cmwUndoColor();
 extern bool CMWCheckWritten();
+
+#endif /* _CMWIND_H */
