@@ -26,7 +26,7 @@ static char rcsid[] = "$Header: /ufs/repository/magic/commands/CmdCD.c,v 1.5 200
 #include <string.h>
 #endif
 
-#include "magic/tclmagic.h"
+#include "tcltk/tclmagic.h"
 #include "misc/magic.h"
 #include "utils/geometry.h"
 #include "tiles/tile.h"
@@ -46,6 +46,7 @@ static char rcsid[] = "$Header: /ufs/repository/magic/commands/CmdCD.c,v 1.5 200
 #include "misc/styles.h"
 #include "router/rtrDcmpose.h"
 #include "select/select.h"
+#include "signals/signals.h"
 #include "utils/malloc.h"
 #include "cif/CIFint.h"
 
@@ -66,6 +67,7 @@ struct cmdCornerArea
 
 /* Forward declarations */
 int cmdDumpFunc();
+bool cmdDumpParseArgs();
 #ifdef CALMA_MODULE
 
 /*
@@ -91,18 +93,17 @@ int cmdDumpFunc();
  */
 
 #define CALMA_HELP	0
-#define	CALMA_FLATTEN	1
-#define	CALMA_LABELS	2
-#define	CALMA_LOWER	3
-#define	CALMA_NOFLATTEN	4
-#define	CALMA_NOLABELS	5
-#define	CALMA_NOLOWER	6
-#define CALMA_READ	7
-#define CALMA_RESCALE	8
-#define CALMA_WARNING	9
-#define CALMA_WRITE	10
+#define CALMA_ARRAYS	1
+#define	CALMA_FLATTEN	2
+#define	CALMA_LABELS	3
+#define	CALMA_LOWER	4
+#define CALMA_READ	5
+#define CALMA_READONLY	6
+#define CALMA_RESCALE	7
+#define CALMA_WARNING	8
+#define CALMA_WRITE	9
 #ifdef NONMANHATTAN
-#define CALMA_POLYS	11
+#define CALMA_POLYS	10
 #endif
 
 #define CALMA_WARN_HELP CIF_WARN_END	/* undefined by CIF module */
@@ -118,20 +119,19 @@ CmdCalma(w, cmd)
     FILE *f;
 
     static char *gdsExts[] = {".gds", ".gds2", ".strm", "", NULL};
-    static char *cmdCalmaYesNo[] = { "no", "yes", 0 };
+    static char *cmdCalmaYesNo[] = { "no", "false", "off", "yes", "true", "on", 0 };
     static char *cmdCalmaWarnOptions[] = { "default", "none", "align",
 		"limit", "redirect", "help", 0 };
     static char *cmdCalmaOption[] =
     {	
 	"help		print this help information",
-	"flatten		output arrays as individual subuses (like in CIF)",
-	"labels		cause labels to be output when writing GDS-II",
-	"lower		allow both upper and lower case in labels",
-	"noflatten		output arrays intact; don't flatten",
-	"nolabels		labels won't be output with GDS-II",
-	"nolower	convert all labels to upper case",
+	"arrays [yes|no]	output arrays as individual subuses (like in CIF)",
+	"flatten [yes|no]	flatten simple cells (e.g., contacts) on input",
+	"labels [yes|no]	cause labels to be output when writing GDS-II",
+	"lower [yes|no]		allow both upper and lower case in labels",
 	"read file		read Calma GDS-II format from \"file\"\n"
 	"		into edit cell",
+	"readonly [yes|no]	set cell as read-only and generate output from GDS file",
 	"rescale [yes|no]	allow or disallow internal grid subdivision",
 	"warning [option]	set warning information level",
 	"write file		output Calma GDS-II format to \"file\"\n"
@@ -198,44 +198,90 @@ CmdCalma(w, cmd)
 	    return;
 
 	case CALMA_LABELS:
-	    if (cmd->tx_argc != 2)
+	    if (cmd->tx_argc == 2)
+	    {
+#ifdef MAGIC_WRAPPER
+		Tcl_SetObjResult(magicinterp, Tcl_NewBooleanObj(CalmaDoLabels));
+#else
+		TxPrintf("Labels will %sbe output to the GDS file.\n",
+			(CalmaDoLabels) ?  "" : "not ");
+#endif
+		return;
+	    }
+	    else if (cmd->tx_argc != 3)
 	    {
 		wrongNumArgs:
 		TxError("Wrong number of arguments in \"gds\" command.");
 		TxError("  Try \":gds help\" for help.\n");
 		return;
 	    }
-	    CalmaDoLabels = TRUE;
-	    TxPrintf("Labels will be output in .gds files\n");
-	    return;
 
-	case CALMA_NOLABELS:
-	    if (cmd->tx_argc != 2) goto wrongNumArgs;
-	    CalmaDoLabels = FALSE;
-	    TxPrintf("Labels won't be output in .gds files\n");
+	    option = Lookup(cmd->tx_argv[2], cmdCalmaYesNo);
+	    if (option < 0)
+		goto wrongNumArgs;
+	    CalmaDoLabels = (option < 3) ? FALSE : TRUE;
 	    return;
 
 	case CALMA_FLATTEN:
-	    if (cmd->tx_argc != 2) goto wrongNumArgs;
-	    CalmaFlattenArrays = TRUE;
-	    TxPrintf("Each element of an array will be output individually\n");
-	    TxPrintf("in .gds files\n");
+	    if (cmd->tx_argc == 2)
+	    {
+#ifdef MAGIC_WRAPPER
+		Tcl_SetObjResult(magicinterp, Tcl_NewBooleanObj(CalmaFlattenUses));
+#else
+		if (CalmaFlattenUses)
+		    TxPrintf("Small cells in input are flattened.\n");
+		else
+		    TxPrintf("Cells are never flattened on input.\n");
+#endif
+		return;
+	    }
+	    else if (cmd->tx_argc != 3)
+		goto wrongNumArgs;
+
+	    option = Lookup(cmd->tx_argv[2], cmdCalmaYesNo);
+	    if (option < 0)
+		goto wrongNumArgs;
+	    CalmaFlattenUses = (option < 3) ? FALSE : TRUE;
 	    return;
 
-	case CALMA_NOFLATTEN:
-	    if (cmd->tx_argc != 2) goto wrongNumArgs;
-	    CalmaFlattenArrays = FALSE;
-	    TxPrintf("Arrays will be output intact in .gds files\n");
+	case CALMA_ARRAYS:
+	    if (cmd->tx_argc == 2)
+	    {
+#ifdef MAGIC_WRAPPER
+		Tcl_SetObjResult(magicinterp, Tcl_NewBooleanObj(CalmaFlattenArrays));
+#else
+		TxPrintf("GDS are %s.\n",
+			(CalmaFlattenArrays) ?  "flattened" : "hierarchical");
+#endif
+		return;
+	    }
+	    else if (cmd->tx_argc != 3)
+		goto wrongNumArgs;
+
+	    option = Lookup(cmd->tx_argv[2], cmdCalmaYesNo);
+	    if (option < 0)
+		goto wrongNumArgs;
+	    CalmaFlattenArrays = (option < 3) ? FALSE : TRUE;
 	    return;
 
 	case CALMA_LOWER:
-	    TxPrintf("Labels in .gds file will be output as upper and lower case\n");
-	    CalmaDoLower = TRUE;
-	    return;
+	    if (cmd->tx_argc == 2)
+	    {
+#ifdef MAGIC_WRAPPER
+		Tcl_SetObjResult(magicinterp, Tcl_NewBooleanObj(CalmaDoLower));
+#else
+		TxPrintf("GDS labels are %s.\n",
+			(CalmaDoLower) ?  "lowercase only" : "mixed-case");
+#endif
+		return;
+	    }
+	    else if (cmd->tx_argc != 3)
+		goto wrongNumArgs;
 
-	case CALMA_NOLOWER:
-	    TxPrintf("Labels in .gds file will be converted to upper case\n");
-	    CalmaDoLower = FALSE;
+	    option = Lookup(cmd->tx_argv[2], cmdCalmaYesNo);
+	    if (option < 0)
+		goto wrongNumArgs;
+	    CalmaDoLower = (option < 3) ? FALSE : TRUE;
 	    return;
 
 #ifdef NONMANHATTAN
@@ -258,10 +304,30 @@ CmdCalma(w, cmd)
 	    option = Lookup(cmd->tx_argv[3], cmdCalmaYesNo);
 	    if (option < 0)
 		goto wrongNumArgs;
-	    CalmaSubcellPolygons = option;
+	    CalmaSubcellPolygons = (option < 3) ? FALSE : TRUE;
 	    return;
 #endif  /* NONMANHATTAN */
 
+	case CALMA_READONLY:
+	    if (cmd->tx_argc == 2)
+	    {
+#ifdef MAGIC_WRAPPER
+		Tcl_SetObjResult(magicinterp, Tcl_NewBooleanObj(CalmaReadOnly));
+#else
+		TxPrintf("GDS cells read from input file are set read-%s.\n",
+			(CalmaReadOnly) ?  "only" : "write");
+#endif
+		return;
+	    }
+	    else if (cmd->tx_argc != 3)
+		goto wrongNumArgs;
+
+	    option = Lookup(cmd->tx_argv[2], cmdCalmaYesNo);
+	    if (option < 0)
+		goto wrongNumArgs;
+	    CalmaReadOnly = (option < 3) ? FALSE : TRUE;
+	    return;
+
 	case CALMA_RESCALE:
 	    if (cmd->tx_argc == 2)
 	    {
@@ -279,7 +345,7 @@ CmdCalma(w, cmd)
 	    option = Lookup(cmd->tx_argv[2], cmdCalmaYesNo);
 	    if (option < 0)
 		goto wrongNumArgs;
-	    CIFRescaleAllow = option;
+	    CIFRescaleAllow = (option < 3) ? FALSE : TRUE;
 	    if (!CIFRescaleAllow)
 		CIFWarningLevel = CIF_WARN_LIMIT;
 	    return;
@@ -327,7 +393,7 @@ CmdCalma(w, cmd)
 
 	    for (ext = 0; gdsExts[ext] != NULL; ext++)
 		if ((f = PaOpen(cmd->tx_argv[2], "r", gdsExts[ext], Path,
-		    	(char *) NULL, (char **) NULL)) != (FILE *)NULL)
+		    	(char *) NULL, &namep)) != (FILE *)NULL)
 		    break;
 
 	    if (f == (FILE *) NULL)
@@ -337,7 +403,7 @@ CmdCalma(w, cmd)
 			cmd->tx_argv[2], cmd->tx_argv[2], cmd->tx_argv[2]);
 	        return;
 	    }
-	    CalmaReadFile(f);
+	    CalmaReadFile(f, namep);
 	    (void) fclose(f);
 	    return;
     }
@@ -407,6 +473,7 @@ CmdCellname(w, cmd)
     int locargc = cmd->tx_argc;
     char *cellname = NULL;
     void (*func)();
+    CellDef *newDef;
 
     static char *cmdCellOption[] =
     {	
@@ -420,14 +487,15 @@ CmdCellname(w, cmd)
 	"allcells	list all cells",
 	"topcells	list top-level cells",
 	"window		list top-level cell of a layout window",
+	"create		create a new cell definition",
 	"delete		delete the named cell definition",
 	"rename		rename the indicated cell",
 	NULL
     };
     typedef enum { IDX_CHILDREN, IDX_PARENTS, IDX_EXISTS, IDX_SELF,
 		   IDX_INSTANCE, IDX_CHILDINST, IDX_CELLDEF, IDX_ALLCELLS,
-		   IDX_TOPCELLS, IDX_IN_WINDOW, IDX_DELETE,
-		   IDX_RENAME } optionType;
+		   IDX_TOPCELLS, IDX_IN_WINDOW, IDX_CREATE,
+		   IDX_DELETE, IDX_RENAME } optionType;
 
     if (strstr(cmd->tx_argv[0], "in"))
 	func = DBUsePrint;
@@ -459,21 +527,38 @@ CmdCellname(w, cmd)
 	}
     }
 
+    /* These functions only work width instances */
+
+    if (func == DBUsePrint) 
+    {
+	switch (option) {
+	    case IDX_TOPCELLS:
+		TxError("Instances do not have a top level.  Use \"cellname\"?\n");
+		return;
+	    case IDX_IN_WINDOW:
+		TxError("Function unimplemented for instances.\n");
+		return;
+	    case IDX_DELETE:
+		TxError("Function unimplemented for instances.  Use \"delete\"\n");
+		return;
+	    case IDX_CREATE:
+		TxError("Function unimplemented for instances.  Use \"getcell\"\n");
+		return;
+	    case IDX_RENAME:
+		TxError("Function unimplemented for instances.  Use \"identify\"\n");
+		return;
+	}
+    }
+
     switch (option) {
 	case IDX_ALLCELLS:
 	    (*func)(NULL, ALLCELLS, dolist);
 	    break;
 	case IDX_TOPCELLS:
-	    if (func == DBUsePrint) 
-		TxError("Instances do not have a top level.  Use \"cellname\"\n");
-	    else
-		(*func)(NULL, TOPCELLS, dolist);
+	    (*func)(NULL, TOPCELLS, dolist);
 	    break;
 	case IDX_IN_WINDOW:
-	    if (func == DBUsePrint) 
-		TxError("Function unimplemented for instances.  Use \"cellname\"\n");
-	    else
-		DBTopPrint(w, dolist);
+	    DBTopPrint(w, dolist);
 	    break;
 	case IDX_SELF:
 	case IDX_EXISTS:
@@ -502,6 +587,14 @@ CmdCellname(w, cmd)
 	    /* Rename the cell and mark as modified.  Do not write to disk. */
 	    DBCellRename(cellname, cmd->tx_argv[3 + ((dolist) ? 1 : 0)]);
 	    break;
+	case IDX_CREATE:
+	    newDef = DBCellLookDef(cellname);
+	    if (newDef == (CellDef *) NULL)
+	    {
+		newDef = DBCellNewDef(cellname, (char *) NULL);
+		DBCellSetAvail(newDef);
+	    }
+	    break;
     }
     return;
 
@@ -509,10 +602,161 @@ badusage:
     TxError("Usage: %s [list] children|parents|self|exists|"
 		"instances|celldef|delete [name]\n", cmd->tx_argv[0]);
     TxError("or:    %s [list] allcells|topcells|window\n", cmd->tx_argv[0]);
+    TxError("or:    %s create name\n", cmd->tx_argv[0]);
     TxError("or:    %s rename name newname\n", cmd->tx_argv[0]);
     return;
 }
 
+#ifdef MAGIC_WRAPPER
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * CmdCellsearch --
+ *
+ *	This is a low-level search command that allows a hierarchical cell
+ *	search to execute a TCL procedure on each cell definition or cell
+ *	use found.
+ *
+ *	Use: cellsearch [instances] <tcl_procedure>
+ *
+ *	If "instances" is present, then the cell search is over all cell
+ *	uses, and <tcl_procedure> takes a list containing 4 integer values
+ *	representing the bounding box (llx, lly, urx, ury) of the cell,
+ *	and two string values containing the name of the cell use and the
+ *	cell definition, respectively.
+ *
+ *	If "instances" is *not* present, then the cell search is over each
+ *	cell definition, and <tcl_procedure> takes one value, which is the
+ *	name of the cell.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+void
+CmdCellsearch(w, cmd)
+    MagWindow *w;
+    TxCommand *cmd;
+{
+    SearchContext scx;
+    int xMask, windowMask;
+    char *tclproc, *pptr;
+    int cmdTclProcSrCellFunc();     /* Fwd declaration */
+    int cmdTclProcCellDefFunc();     /* Fwd declaration */
+    CellDef *rootBoxDef;
+    MagWindow *window;
+
+    bzero(&scx, sizeof(SearchContext));
+    if (!ToolGetBox (&rootBoxDef, NULL))
+    {
+	TxError ("Box tool must be present\n");
+	return;
+    }
+    window = ToolGetBoxWindow (&scx.scx_area, &windowMask);
+    if (!window)
+    {
+	TxError ("Box tool does not exist in any window\n");
+	return;
+    }
+    xMask = ((DBWclientRec *)window->w_clientData)->dbw_bitmask;
+    if ((windowMask & ~xMask) != 0)
+    {
+	window = CmdGetRootPoint((Point *) NULL, (Rect *) NULL);
+	xMask = ((DBWclientRec *) window->w_clientData)->dbw_bitmask;
+	if ((windowMask & xMask) == 0)
+	{
+	    TxPrintf("The box is in more than one window;  use the cursor\n");
+	    TxPrintf("to select the one you want to select from.\n");
+	    return;
+	}
+    }
+    scx.scx_use = (CellUse *) window->w_surfaceID;
+    scx.scx_trans = GeoIdentityTransform;
+
+    if ((cmd->tx_argc == 1) || (cmd->tx_argc > 3))
+    {
+	TxError("Usage: cellsearch [instances] <tcl_procedure>\n");
+	return;
+    }
+
+    pptr = cmd->tx_argv[cmd->tx_argc - 1];
+    if (*pptr == '{')           /* in-line procedure */
+    {
+	tclproc = StrDup(NULL, pptr + 1);
+	pptr = tclproc + strlen(tclproc) - 1;
+	if (*pptr == '}') *pptr = '\0';
+    }
+    else
+	tclproc = StrDup(NULL, cmd->tx_argv[cmd->tx_argc - 1]);
+
+    if ((cmd->tx_argc == 3) && (!strncmp(cmd->tx_argv[1], "inst", 4)))
+    {
+	DBTreeSrTiles(&scx, xMask, cmdTclProcSrCellFunc,
+		(ClientData) tclproc);
+    }
+    else
+    {
+	DBCellSrDefs(0, cmdTclProcCellDefFunc, (ClientData) tclproc);
+    }
+    freeMagic(tclproc);
+}   
+
+/*
+ * cmdTclProcSrCellFunc --
+ *  
+ * Filter procedure applied to cell instances by cmdCellsearch() above.
+ */ 
+  
+int
+cmdTclProcSrCellFunc(scx, tclproc)
+    SearchContext *scx;
+    char *tclproc;
+{
+    Rect rootRect;
+    char evalstr[512];
+    int rval;
+    CellDef *def;
+
+    def = scx->scx_use->cu_def;
+    GeoTransRect(&scx->scx_trans, &def->cd_bbox, &rootRect);
+
+    sprintf(evalstr, "%s %d %d %d %d %s %s", tclproc, rootRect.r_xbot,
+		rootRect.r_ybot, rootRect.r_xtop, rootRect.r_ytop,
+		def->cd_name, scx->scx_use->cu_id);
+
+    rval = Tcl_EvalEx(magicinterp, evalstr, -1, 0);
+    if (rval != TCL_OK) return 1;
+
+    /* Allow interrupts in case something goes wrong with the Tcl procedure */
+    if (SigInterruptPending) return 1;
+    return 0;
+}   
+
+/*
+ * cmdTclProcCellDefFunc --
+ *  
+ * Filter procedure applied to celldefs by cmdCellsearch() above.
+ */ 
+  
+int
+cmdTclProcCellDefFunc(def, tclproc)
+    CellDef *def;
+    char *tclproc;
+{
+    char evalstr[512];
+    int rval;
+
+    sprintf(evalstr, "%s %s", tclproc, def->cd_name);
+
+    rval = Tcl_EvalEx(magicinterp, evalstr, -1, 0);
+    if (rval != TCL_OK) return 1;
+
+    /* Allow interrupts in case something goes wrong with the Tcl procedure */
+    if (SigInterruptPending) return 1;
+    return 0;
+}   
+
+#endif
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1105,15 +1349,15 @@ CmdCopy(w, cmd)
 
     if (cmd->tx_argc > 1)
     {
-	int indx, amount;
+	int indx, amountx, amounty;
 	int xdelta, ydelta;
 
 	if (strcmp(cmd->tx_argv[1], "to") == 0)
 	{
 	    if (cmd->tx_argc != 4)
 		goto badUsage;
-	    editPoint.p_x = cmdParseCoord(cmd->tx_argv[2]);
-	    editPoint.p_y = cmdParseCoord(cmd->tx_argv[3]);
+	    editPoint.p_x = cmdParseCoord(w, cmd->tx_argv[2], FALSE, TRUE);
+	    editPoint.p_y = cmdParseCoord(w, cmd->tx_argv[3], FALSE, FALSE);
 	    GeoTransPoint(&EditToRootTransform, &editPoint, &rootPoint);
 	    goto copyToPoint;
 	}
@@ -1123,43 +1367,65 @@ CmdCopy(w, cmd)
 	    return;
 	if (cmd->tx_argc == 3)
 	{
-	    amount = cmdParseCoord(cmd->tx_argv[2]);
+	    if (DBWSnapToGrid != DBW_SNAP_USER)
+	    {
+		amountx = cmdParseCoord(w, cmd->tx_argv[2], TRUE, FALSE);
+		amounty = amountx;
+	    }
+	    else
+	    {
+		switch (indx)
+		{
+		    case GEO_EAST: case GEO_WEST:
+			amountx = cmdParseCoord(w, cmd->tx_argv[2], TRUE, FALSE);
+			amounty = amountx;
+			break;
+		    case GEO_NORTH: case GEO_SOUTH:
+			amounty = cmdParseCoord(w, cmd->tx_argv[2], TRUE, TRUE);
+			amountx = amounty;
+			break;
+		    default:
+			amountx = cmdParseCoord(w, cmd->tx_argv[2], TRUE, FALSE);
+			amounty = cmdParseCoord(w, cmd->tx_argv[2], TRUE, TRUE);
+			break;
+		}
+	    }
 	}
-	else amount = 1;
+	else amountx = amounty = 1;
 
 	switch (indx)
 	{
 	    case GEO_NORTH:
 		xdelta = 0;
-		ydelta = amount;
+		ydelta = amounty;
 		break;
 	    case GEO_SOUTH:
 		xdelta = 0;
-		ydelta = -amount;
+		ydelta = -amounty;
 		break;
 	    case GEO_EAST:
-		xdelta = amount;
+		xdelta = amountx;
 		ydelta = 0;
 		break;
 	    case GEO_WEST:
-		xdelta = -amount;
+		xdelta = -amountx;
 		ydelta = 0;
 		break;
 	    case GEO_NORTHWEST:
-		xdelta = -amount;
-		ydelta = amount;
+		xdelta = -amountx;
+		ydelta = amounty;
 		break;
 	    case GEO_NORTHEAST:
-		xdelta = amount;
-		ydelta = amount;
+		xdelta = amountx;
+		ydelta = amounty;
 		break;
 	    case GEO_SOUTHWEST:
-		xdelta = -amount;
-		ydelta = -amount;
+		xdelta = -amountx;
+		ydelta = -amounty;
 		break;
 	    case GEO_SOUTHEAST:
-		xdelta = amount;
-		ydelta = -amount;
+		xdelta = amountx;
+		ydelta = -amounty;
 		break;
 	    case GEO_CENTER:
 		xdelta = 0;
@@ -2236,7 +2502,7 @@ CmdDown(w, cmd)
     GeoTransRect(&EditToRootTransform, &(EditCellUse->cu_def->cd_bbox), &area);
     (void) WindSearch(DBWclientID, (ClientData) NULL,
 	    (Rect *) NULL, cmdEditRedisplayFunc, (ClientData) &area);
-    DBWloadWindow(w, EditCellUse->cu_def->cd_name);
+    DBWloadWindow(w, EditCellUse->cu_def->cd_name, TRUE, FALSE);
 }
 
 /* Search function to find the new edit cell:  look for a cell use
@@ -2306,15 +2572,16 @@ cmdDownEnumFunc(selUse, use, transform, area)
 #define CATCHUP		2
 #define CHECK		3
 #define COUNT		4
-#define FIND		5
+#define EUCLIDEAN	5
+#define FIND		6
 #undef HELP
-#define HELP		6
-#define OFF		7
-#define ON		8
-#define PRINTRULES	9
-#define RULESTATS	10
-#define STATISTICS	11
-#define WHY		12
+#define HELP		7
+#define OFF		8
+#define ON		9
+#define PRINTRULES	10
+#define RULESTATS	11
+#define STATISTICS	12
+#define WHY		13
 
 void
 CmdDrc(w, cmd)
@@ -2338,6 +2605,7 @@ CmdDrc(w, cmd)
 	"catchup                run checker and wait for it to complete",
 	"check                  recheck area under box in all cells",
 	"count                  count error tiles in each cell under box",
+	"euclidean on|off	enable/disable Euclidean geometry checking",
 	"find [nth]             locate next (or nth) error in current cell",
 	"help                   print this help information",
 	"off                    turn off background checker",
@@ -2364,7 +2632,7 @@ CmdDrc(w, cmd)
 	    cmd->tx_argc = 2;
 	}
 	if ((cmd->tx_argc > 2) && (option != PRINTRULES) && (option != FIND)
-	    && (option != SHOWINT) && (option != HELP))
+	    && (option != SHOWINT) && (option != HELP) && (option != EUCLIDEAN))
 	{
 	    badusage:
 	    TxError("Wrong arguments in \"drc %s\" command:\n",
@@ -2385,7 +2653,7 @@ CmdDrc(w, cmd)
 	
 	case SHOWINT:
 	    if (cmd->tx_argc != 3) goto badusage;
-	    radius = cmdParseCoord(cmd->tx_argv[2]);
+	    radius = cmdParseCoord(w, cmd->tx_argv[2], TRUE, FALSE);
 	    if (radius < 0)
 	    {
 		TxPrintf("Radius must not be negative\n");
@@ -2421,6 +2689,13 @@ CmdDrc(w, cmd)
 	    rootUse = (CellUse *) window->w_surfaceID;
 	    DRCCount(rootUse, &rootArea);
 	    break;
+
+	case EUCLIDEAN:
+	    if (!strcmp(cmd->tx_argv[2], "on"))
+		DRCEuclidean = TRUE;
+	    else
+		DRCEuclidean = FALSE;
+	    break;
 	
 	case FIND:
 	    if (cmd->tx_argc > 2)
