@@ -34,12 +34,11 @@ static char rcsid[] = "$Header: /ufs/repository/magic/commands/CmdFI.c,v 1.4 200
 #endif
 #endif
 
-#ifdef MAGIC_WRAPPER
-#include "magic/tclmagic.h"
-#endif
+#include "tcltk/tclmagic.h"
 #include "misc/magic.h"
 #include "utils/geometry.h"
 #include "utils/utils.h"
+#include "undo/undo.h"
 #include "tiles/tile.h"
 #include "utils/hash.h"
 #include "database/database.h"
@@ -240,9 +239,9 @@ CmdFeedback(w, cmd)
 	    {
 		int j, style;
 		text = DBWFeedbackNth(i, &box, (CellDef **) NULL, &style);
-		(void) fprintf(f, "box %d %d %d %d\n", box.r_xbot, box.r_ybot,
+		fprintf(f, "box %d %d %d %d\n", box.r_xbot, box.r_ybot,
 		    box.r_xtop, box.r_ytop);
-		(void) fprintf(f, "feedback add \"");
+		fprintf(f, "feedback add \"");
 
 		/* Be careful to backslash any quotes in the text! */
 
@@ -256,11 +255,11 @@ CmdFeedback(w, cmd)
 		{
 		    if (cmdFeedbackStyles[j] == style)
 		    {
-			(void) fprintf(f, " %s", cmdFeedbackStyleNames[j]);
+			fprintf(f, " %s", cmdFeedbackStyleNames[j]);
 			break;
 		    }
 		}
-		(void) fprintf(f, "\n");
+		fprintf(f, "\n");
 	    }
 	    (void) fclose(f);
 	    break;
@@ -548,26 +547,26 @@ usage:
  * Find a label and set the box to it.
  *
  * Usage:
- *	findlabel name
+ *	findlabel [-glob] name
  *
  * Results:
- *	None.
+ *	None.  In TCL, the -glob option will generate a list of matching
+ *	nodes returned in the interpreter result.
  *
  * Side effects:
- *	The box may be moved.
+ *	The box may be moved.  If "-glob" is specified, in the non-Tcl
+ *	version, matching label names are printed.
  *
  * ----------------------------------------------------------------------------
  */
 
-Rect cmdFindLabelRect;
-
 int cmdFindLabelFunc(rect, name, label, cdarg) 
     Rect *rect;
     char *name;
     Label *label;
-    ClientData cdarg;
+    Rect *cdarg;
 {
-    cmdFindLabelRect = *rect;
+    *cdarg = *rect;
     return 1;
 }
 
@@ -577,11 +576,16 @@ CmdFindLabel(w, cmd)
     TxCommand *cmd;
 {
     CellDef *boxDef;
-    Rect box;
+    Rect box, cmdFindLabelRect;
     char *labname;
     int found;
+    bool doglob = FALSE;   /* csh-style glob matching (see utils/match.c) */
+    int dbListLabels();	   /* forward declaration */
 
-    if (cmd->tx_argc != 2) goto usage;
+    if ((cmd->tx_argc == 3) && !strncmp(cmd->tx_argv[1], "-glob", 5))
+	doglob = TRUE;
+    else if (cmd->tx_argc != 2)
+	goto usage;
 
     if (w == NULL)
     {
@@ -602,25 +606,69 @@ CmdFindLabel(w, cmd)
 	return;
     };
 
-    labname = cmd->tx_argv[1];
-    found = DBSrLabelLoc(EditCellUse, labname, cmdFindLabelFunc, (ClientData) 0);
-    if (found) {
-	if (cmdFindLabelRect.r_xbot == cmdFindLabelRect.r_xtop) 
-	    cmdFindLabelRect.r_xtop++;
-	if (cmdFindLabelRect.r_ybot == cmdFindLabelRect.r_ytop) 
-	    cmdFindLabelRect.r_ytop++;
-	ToolMoveBox(TOOL_BL,&cmdFindLabelRect.r_ll,FALSE,EditCellUse->cu_def);
-	ToolMoveCorner(TOOL_TR,&cmdFindLabelRect.r_ur,FALSE,EditCellUse->cu_def);
-    } else {
-	TxError("Couldn't find label %s\n", labname);
+    labname = (cmd->tx_argc == 3) ? cmd->tx_argv[2] : cmd->tx_argv[1];
+
+    if (doglob)
+    {
+	/* Pattern-matching label search */
+
+	SearchContext scx;
+
+	scx.scx_use = EditCellUse;
+	scx.scx_area = EditCellUse->cu_def->cd_bbox;
+	scx.scx_trans = GeoIdentityTransform;
+
+	DBSearchLabel(&scx, &DBAllButSpaceAndDRCBits, 0, labname,
+		dbListLabels, (ClientData) 0);
+    }
+    else
+    {
+	/* Exact-match label search */
+
+	found = DBSrLabelLoc(EditCellUse, labname, cmdFindLabelFunc, (ClientData) 0);
+	if (found) {
+	    if (cmdFindLabelRect.r_xbot == cmdFindLabelRect.r_xtop) 
+		cmdFindLabelRect.r_xtop++;
+	    if (cmdFindLabelRect.r_ybot == cmdFindLabelRect.r_ytop) 
+		cmdFindLabelRect.r_ytop++;
+	    ToolMoveBox(TOOL_BL,&cmdFindLabelRect.r_ll,FALSE,EditCellUse->cu_def);
+	    ToolMoveCorner(TOOL_TR,&cmdFindLabelRect.r_ur,FALSE,EditCellUse->cu_def);
+	} else {
+	    TxError("Couldn't find label %s\n", labname);
+	}
     }
 
     return;
 
 
 usage:
-    TxError("Usage: findlabel label_name\n");
+    TxError("Usage: findlabel [-glob] label_name\n");
 }
+
+/*
+ * Callback routine for listing pattern-matched labels.
+ * Always return zero to keep the search going.
+ */
+
+int
+dbListLabels(scx, label, tpath, cdarg)
+    SearchContext *scx;
+    Label *label;			/* Pointer to label structure	*/
+    TerminalPath *tpath;		/* Full pathname of terminal	*/
+    ClientData cdarg;			/* (unused)			*/
+{
+    char *n = tpath->tp_next;
+    char c = *n;
+    strcpy(n, label->lab_text);
+#ifdef MAGIC_WRAPPER
+    Tcl_AppendElement(magicinterp, tpath->tp_first);
+#else
+    TxPrintf("%s\n", tpath->tp_first);
+#endif
+    *n = c;
+    return 0;
+}
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -765,6 +813,14 @@ CmdGetcell(w, cmd)
     DBWAreaChanged(EditCellUse->cu_def, &newUse->cu_bbox,
 	DBW_ALLWINDOWS, &DBAllButSpaceBits);
     DRCCheckThis(EditCellUse->cu_def, TT_CHECKSUBCELL, &newUse->cu_bbox);
+
+#ifdef MAGIC_WRAPPER
+    /* If using the TCL wrapper, set the TCL return value to the	*/
+    /* name of the new use.						*/
+    if (newUse->cu_id)
+	Tcl_SetResult(magicinterp, newUse->cu_id, TCL_VOLATILE);
+#endif
+
 }
 #ifndef NO_SIM_MODULE
 
@@ -778,10 +834,9 @@ CmdGetcell(w, cmd)
  *
  * Usage:
  *  	getnode
- *	getnode abort
- *	getnode abort string
- *      getnode alias on
- *      getnode alias off
+ *	getnode abort [string]
+ *      getnode alias [on | off]
+ *      getnode globals [on | off]
  *      getnode fast
  *
  * Results:
@@ -821,7 +876,17 @@ CmdGetnode(w, cmd)
 		is_fast = TRUE;
 	    }
 	    else {
-		goto badusage;
+		if (!strcmp("alias", cmd->tx_argv[1])) {
+		    TxPrintf("Aliases %s\n", (SimGetnodeAlias) ?  "on" : "off");
+		    return;
+		}
+		else if (strncmp("global", cmd->tx_argv[1], 6) == 0) {
+		    TxPrintf("Node names ending in ! are %s\n",
+				(SimIgnoreGlobals) ? "local (off)" : "global (on)");
+		    return;
+		}
+		else
+		    goto badusage;
 	    }
 	    break;
 
@@ -844,6 +909,18 @@ CmdGetnode(w, cmd)
 		else
 		    goto badusage;
 	    }
+	    else if (strncmp("global", cmd->tx_argv[1], 6) == 0) {
+		if (strcmp("off", cmd->tx_argv[2]) == 0) {
+		    SimIgnoreGlobals = TRUE;
+		    return;
+		}
+		else if (strcmp("on", cmd->tx_argv[2]) == 0) {
+		    SimIgnoreGlobals = FALSE;
+		    return;
+		}
+		else
+		    goto badusage;
+	    }
 	    else if (strcmp("abort", cmd->tx_argv[1]) == 0) {
 		if (SimInitGetnode) {
 		    HashInit(&SimGetnodeTbl, TBLSIZE, STRINGS);
@@ -885,8 +962,8 @@ CmdGetnode(w, cmd)
 
 badusage:
     TxError("Usage: getnode [abort [str]]\n");
-    TxError("   or: getnode alias on\n");
-    TxError("   or: getnode alias off\n");
+    TxError("   or: getnode alias [on | off]\n");
+    TxError("   or: getnode globals [on | off]\n");
     TxError("   or: getnode fast\n");
 }
 #endif
@@ -937,7 +1014,7 @@ CmdGrid(w, cmd)
 	crec->dbw_flags &= ~DBW_GRID;
     else if (cmd->tx_argc >= 2)
     {
-	xSpacing = cmdParseCoord(cmd->tx_argv[1]);
+	xSpacing = cmdParseCoord(w, cmd->tx_argv[1], TRUE, TRUE);
 	if (xSpacing <= 0)
 	{
 	    TxError("Grid spacing must be greater than zero.\n");
@@ -948,7 +1025,7 @@ CmdGrid(w, cmd)
 
 	if (cmd->tx_argc >= 3)
 	{
-	    ySpacing = cmdParseCoord(cmd->tx_argv[2]);
+	    ySpacing = cmdParseCoord(w, cmd->tx_argv[2], TRUE, FALSE);
 	    if (ySpacing <= 0)
 	    {
 		TxError("Grid spacing must be greater than zero.\n");
@@ -957,8 +1034,8 @@ CmdGrid(w, cmd)
 
 	    if (cmd->tx_argc == 5)
 	    {
-		xOrig = cmdParseCoord(cmd->tx_argv[3]);
-		yOrig = cmdParseCoord(cmd->tx_argv[4]);
+		xOrig = cmdParseCoord(w, cmd->tx_argv[3], FALSE, TRUE);
+		yOrig = cmdParseCoord(w, cmd->tx_argv[4], FALSE, FALSE);
 	    }
 	}
 
@@ -1124,11 +1201,26 @@ cmdIdFunc(selUse, use, transform, newId)
  * Implements goto command
  *
  * Usage:
- *	goto nodename
+ *	goto nodename [-nocomplain]
  *
  * Results:
+ *	None.  In TCL version, returns the material type
+ *	of the node, so that the node can be uniquely determined
+ *	by an automated script.   If the node cannot be found, a
+ *	null list is returned.
+ *
+ * Side Effects:
  *	changes box location.
  *
+ * Notes:
+ *	Due to the way global node names are handled, "getnode" and
+ *	"goto" are not necessarily reversible if "getnode globals on"
+ *	is set!  In this case, use the glob option for findlabel
+ *	to determine any valid local label belonging to the global
+ *	node (a node cannot be global unless it is labeled).  This
+ *	method, although awkward, can be much faster for large layouts
+ *	due to the time required for "getnode" to uniquely determine
+ *	the name of a large network such as power or ground.
  *
  * ----------------------------------------------------------------------------
  */
@@ -1146,9 +1238,10 @@ CmdGoto(w, cmd)
     Label		*label;
     Rect		localrect, rect;
     CellDef		*rootdef;
-    int			pnum, xpos, ypos;
+    int			pnum, xpos, ypos, locargc;
     char 		*xstr, *ystr;
     bool		locvalid;
+    bool		nocomplain = FALSE;
     TileType		ttype;
      
     windCheckOnlyWindow(&w, DBWclientID);
@@ -1160,9 +1253,19 @@ CmdGoto(w, cmd)
     scx.scx_use = (CellUse  *) w->w_surfaceID;
     rootdef = scx.scx_use->cu_def;
     scx.scx_trans = GeoIdentityTransform;
-    if (cmd->tx_argc != 2)
+
+    locargc = cmd->tx_argc;
+    if (locargc == 3)
     {
-     	TxError("usage: goto nodename\n");
+	if (!strncmp(cmd->tx_argv[2], "-nocom", 5))
+	{
+	    nocomplain = TRUE;
+	    locargc--;
+	}
+    }
+    if (locargc != 2)
+    {
+     	TxError("usage: goto nodename [-nocomplain]\n");
 	return;
     }
     s = nodename;
@@ -1285,7 +1388,8 @@ CmdGoto(w, cmd)
 	}
 	else
 	{
-     	    TxError("Couldn't find label %s\n",s);
+	    if (!nocomplain)
+     		TxError("Couldn't find label %s\n",s);
 	    return;
 	}
     }
@@ -1316,12 +1420,82 @@ findTile(tile, rtype)
 {
     TileType ttype;
 
-    ttype = TiGetTypeExact(tile);
+#ifdef NONMANHATTAN
+    if (IsSplit(tile))  
+    {   
+	if (SplitSide(tile))
+	    ttype = SplitRightType(tile);
+	else
+	    ttype = SplitLeftType(tile);
+    }
+    else
+#endif  
+	ttype = TiGetTypeExact(tile);
     *rtype = ttype;
     return 1;			/* stop search */
 }
 
-
+/*
+ * The following are from DBcellcopy.c; slightly modified for present 
+ * purposes.
+ */
+
+#define FLATTERMSIZE	1024
+
+void
+FlatCopyAllLabels(scx, mask, xMask, targetUse)
+    SearchContext *scx;
+    TileTypeBitMask *mask;
+    int xMask;		
+    CellUse *targetUse;
+{
+    int flatCopyAllLabels();
+    char pathstring[FLATTERMSIZE];
+    TerminalPath	tpath;
+    
+    tpath.tp_first = tpath.tp_next = pathstring;
+    tpath.tp_last = pathstring + FLATTERMSIZE;
+
+    DBTreeSrLabels(scx, mask, xMask, &tpath, flatCopyAllLabels,
+			(ClientData) targetUse);
+}
+
+int
+flatCopyAllLabels(scx, lab, tpath, targetUse)
+    SearchContext *scx;
+    Label *lab;
+    TerminalPath *tpath;
+    CellUse	*targetUse;
+{
+    Rect labTargetRect;
+    int targetPos;
+    CellDef *def;
+    char	labelname[1024];
+
+    def = targetUse->cu_def;
+    if (!GEO_LABEL_IN_AREA(&lab->lab_rect, &(scx->scx_area))) return 0;
+    GeoTransRect(&scx->scx_trans, &lab->lab_rect, &labTargetRect);
+    targetPos = GeoTransPos(&scx->scx_trans, lab->lab_pos);
+
+    /* Eliminate duplicate labels.  Don't pay any attention to layers
+     * in deciding on duplicates:  if text and position match, it's a
+     * duplicate.
+     */
+
+    (void) DBEraseLabelsByContent(def, &labTargetRect, targetPos,
+        -1, lab->lab_text);
+    {
+    	 char *n = tpath->tp_next;
+    	 char *f = tpath->tp_first;
+	 char c = *n;
+	 strcpy(n,lab->lab_text);
+         (void) DBPutLabel(def, &labTargetRect, targetPos,
+			f, lab->lab_type, lab->lab_flags);
+	 *n = c;
+    }
+    return 0;
+}
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1381,60 +1555,3 @@ CmdFlatten(w, cmd)
     UndoEnable();
 }
 
-/* the following are from DBcellcopy.c; slightly modified for present 
-   purposes.
-*/
-FlatCopyAllLabels(scx, mask, xMask, targetUse)
-    SearchContext *scx;
-    TileTypeBitMask *mask;
-    int xMask;		
-    CellUse *targetUse;
-{
-#define FLATTERMSIZE	1024
-    int flatCopyAllLabels();
-    char pathstring[FLATTERMSIZE];
-    TerminalPath	tpath;
-    
-    tpath.tp_first = tpath.tp_next = pathstring;
-    tpath.tp_last = pathstring+FLATTERMSIZE;
-
-    (void) DBTreeSrLabels(scx, mask, xMask, &tpath,
-			flatCopyAllLabels, (ClientData) targetUse);
-}
-
-    /*ARGSUSED*/
-int
-flatCopyAllLabels(scx, lab, tpath, targetUse)
-    SearchContext *scx;
-    Label *lab;
-    TerminalPath *tpath;
-    CellUse	*targetUse;
-{
-    Rect labTargetRect;
-    int targetPos;
-    CellDef *def;
-    char	labelname[1024];
-
-    def = targetUse->cu_def;
-    if (!GEO_LABEL_IN_AREA(&lab->lab_rect, &(scx->scx_area))) return 0;
-    GeoTransRect(&scx->scx_trans, &lab->lab_rect, &labTargetRect);
-    targetPos = GeoTransPos(&scx->scx_trans, lab->lab_pos);
-
-    /* Eliminate duplicate labels.  Don't pay any attention to layers
-     * in deciding on duplicates:  if text and position match, it's a
-     * duplicate.
-     */
-
-    (void) DBEraseLabelsByContent(def, &labTargetRect, targetPos,
-        -1, lab->lab_text);
-    {
-    	 char *n = tpath->tp_next;
-    	 char *f = tpath->tp_first;
-	 char c = *n;
-	 strcpy(n,lab->lab_text);
-         (void) DBPutLabel(def, &labTargetRect, targetPos,
-			f, lab->lab_type, lab->lab_flags);
-	 *n = c;
-    }
-    return 0;
-}
