@@ -27,7 +27,7 @@ static char rcsid[] = "$Header: /ufs/repository/magic/commands/CmdLQ.c,v 1.11 20
 #endif
 #include <ctype.h>
 
-#include "magic/tclmagic.h"
+#include "tcltk/tclmagic.h"
 #include "misc/magic.h"
 #include "utils/geometry.h"
 #include "utils/utils.h"
@@ -198,7 +198,7 @@ CmdLabel(w, cmd)
  * Implement the "load" command.
  *
  * Usage:
- *	load [name [scaled n [d]]]
+ *	load [name [scaled n [d]]] [-force]
  *
  * If name is supplied, then the window containing the point tool is
  * remapped so as to edit the cell with the given name.
@@ -227,6 +227,9 @@ CmdLoad(w, cmd)
 {
     int n = 1;
     int d = 1;
+    int locargc = cmd->tx_argc;
+    bool ignoreTech = FALSE;
+    int keepGoing();			/* forward declaration */
 
     windCheckOnlyWindow(&w, DBWclientID);
     if (w == (MagWindow *) NULL)
@@ -235,15 +238,20 @@ CmdLoad(w, cmd)
 	return;
     }
 
-    if (cmd->tx_argc > 2)
+    if (locargc > 2)
     {
-	if (cmd->tx_argc >= 4 && !strncmp(cmd->tx_argv[2], "scale", 5) &&
+	if (!strncmp(cmd->tx_argv[locargc - 1], "-force", 6))
+	{
+	    locargc--;
+	    ignoreTech = TRUE;
+	}
+	if (locargc >= 4 && !strncmp(cmd->tx_argv[2], "scale", 5) &&
 		StrIsInt(cmd->tx_argv[3]))
 	{
 	    n = atoi(cmd->tx_argv[3]);
  	    if (cmd->tx_argc == 5 && StrIsInt(cmd->tx_argv[4]))
 	        d = atoi(cmd->tx_argv[4]);
-	    else
+	    else if (locargc != 4)
 	    {
 		TxError("Usage: %s name scaled n [d]\n", cmd->tx_argv[0]);
 		return;
@@ -252,21 +260,41 @@ CmdLoad(w, cmd)
 	    DBLambda[1] *= n;
 	    ReduceFraction(&DBLambda[0], &DBLambda[1]);
 	}
-	else
+	else if (!ignoreTech)
 	{
 	    TxError("Usage: %s [name [scaled n [d]]]\n", cmd->tx_argv[0]);
 	    return;
 	}
     }
 
-    if (cmd->tx_argc > 1)
+    if (locargc > 1)
     {
 	if (CmdIllegalChars(cmd->tx_argv[1], "[],", "Cell name"))
 	    return;
-	DBWloadWindow(w, cmd->tx_argv[1]);
+	DBWloadWindow(w, cmd->tx_argv[1], ignoreTech, FALSE);
 
 	if ((n > 1) || (d > 1))
 	{
+            CellUse *topuse = (CellUse *)w->w_surfaceID;
+
+	    /* To ensure that all subcells are also scaled, we need to	*/
+	    /* expand the entire cell hierarchy to force a load of the	*/
+	    /* cell.  After this, we return all the expansion flags	*/
+	    /* back to the way they were originally.			*/
+
+	    TxPrintf("Recursively reading all cells at new scale.\n");
+
+	    DBExpandAll(topuse, &(topuse->cu_bbox),
+			((DBWclientRec *)w->w_clientData)->dbw_bitmask,
+			TRUE, keepGoing, NULL);
+
+	    DBExpandAll(topuse, &(topuse->cu_bbox),
+			((DBWclientRec *)w->w_clientData)->dbw_bitmask,
+			FALSE, keepGoing, NULL);
+	    DBExpand(topuse,
+			((DBWclientRec *)w->w_clientData)->dbw_bitmask,
+			TRUE);
+
 	    /* We don't want to save and restore DBLambda, because      */
 	    /* loading the file may change their values.  Instead, we   */
 	    /* invert the scaling factor and reapply.                   */
@@ -276,8 +304,22 @@ CmdLoad(w, cmd)
 	    ReduceFraction(&DBLambda[0], &DBLambda[1]);
 	}
     }
-    else DBWloadWindow(w, (char *) NULL);
+    else DBWloadWindow(w, (char *) NULL, TRUE, FALSE);
 }
+
+/*
+ * Function callback which continues the search through all cells for
+ * expansion/unexpansion.
+ */
+
+int
+keepGoing(use, clientdata)
+    CellUse *use;
+    ClientData clientdata;
+{
+    return 0;	/* keep the search going */
+}
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -319,15 +361,15 @@ CmdMove(w, cmd)
 
     if (cmd->tx_argc > 1)
     {
-	int indx, amount;
+	int indx, amountx, amounty;
 	int xdelta, ydelta;
 
 	if (strcmp(cmd->tx_argv[1], "to") == 0)
 	{
 	    if (cmd->tx_argc != 4)
 		goto badUsage;
-	    editPoint.p_x = cmdParseCoord(cmd->tx_argv[2]);
-	    editPoint.p_y = cmdParseCoord(cmd->tx_argv[3]);
+	    editPoint.p_x = cmdParseCoord(w, cmd->tx_argv[2], FALSE, TRUE);
+	    editPoint.p_y = cmdParseCoord(w, cmd->tx_argv[3], FALSE, FALSE);
 	    GeoTransPoint(&EditToRootTransform, &editPoint, &rootPoint);
 	    goto moveToPoint;
 	}
@@ -337,43 +379,64 @@ CmdMove(w, cmd)
 	    return;
 	if (cmd->tx_argc == 3)
 	{
-	    amount = cmdParseCoord(cmd->tx_argv[2]);
+	    if (DBWSnapToGrid != DBW_SNAP_USER)
+	    {
+		amountx = amounty = cmdParseCoord(w, cmd->tx_argv[2], TRUE, FALSE);
+	    }
+	    else
+	    {
+		switch (indx)
+		{
+		    case GEO_EAST: case GEO_WEST:
+			amountx = cmdParseCoord(w, cmd->tx_argv[2], TRUE, TRUE);
+			amounty = amountx;
+			break;
+		    case GEO_NORTH: case GEO_SOUTH:
+			amounty = cmdParseCoord(w, cmd->tx_argv[2], TRUE, FALSE);
+			amountx = amounty;
+			break;
+		    default:
+			amountx = cmdParseCoord(w, cmd->tx_argv[2], TRUE, TRUE);
+			amounty = cmdParseCoord(w, cmd->tx_argv[2], TRUE, FALSE);
+			break;
+		}
+	    }
 	}
-	else amount = 1;
+	else amountx = amounty = 1;
 
 	switch (indx)
 	{
 	    case GEO_NORTH:
 		xdelta = 0;
-		ydelta = amount;
+		ydelta = amounty;
 		break;
 	    case GEO_SOUTH:
 		xdelta = 0;
-		ydelta = -amount;
+		ydelta = -amounty;
 		break;
 	    case GEO_EAST:
-		xdelta = amount;
+		xdelta = amountx;
 		ydelta = 0;
 		break;
 	    case GEO_WEST:
-		xdelta = -amount;
+		xdelta = -amountx;
 		ydelta = 0;
 		break;
 	    case GEO_NORTHEAST:
-		xdelta = amount;
-		ydelta = amount;
+		xdelta = amountx;
+		ydelta = amounty;
 		break;
 	    case GEO_NORTHWEST:
-		xdelta = -amount;
-		ydelta = amount;
+		xdelta = -amountx;
+		ydelta = amounty;
 		break;
 	    case GEO_SOUTHEAST:
-		xdelta = amount;
-		ydelta = -amount;
+		xdelta = amountx;
+		ydelta = -amounty;
 		break;
 	    case GEO_SOUTHWEST:
-		xdelta = -amount;
-		ydelta = -amount;
+		xdelta = -amountx;
+		ydelta = -amounty;
 		break;
 	    default:
 		ASSERT(FALSE, "Bad direction in CmdMove");
@@ -1193,20 +1256,31 @@ CmdProperty(w, cmd)
     MagWindow *w;
     TxCommand *cmd;
 {
-    CellDef *editDef = EditCellUse->cu_def;
+    CellDef *def;
     int printPropertiesFunc();
     char *value;
     bool propfound;
 
+    /* Get the root definition of the window	*/
+    /* If window is NULL, pick up the edit def	*/
+
+    windCheckOnlyWindow(&w, DBWclientID);
+    if (w == (MagWindow *) NULL)
+    {
+	def = EditCellUse->cu_def;
+    }
+    else
+	def = ((CellUse *) w->w_surfaceID)->cu_def;
+
     if (cmd->tx_argc == 1)
     {
 	/* print all properties and their values */
-	DBPropEnum(editDef, printPropertiesFunc);
+	DBPropEnum(def, printPropertiesFunc);
     }
     else if (cmd->tx_argc == 2)
     {
 	/* print the value of the indicated property */
-	value = (char *)DBPropGet(editDef, cmd->tx_argv[1], &propfound);
+	value = (char *)DBPropGet(def, cmd->tx_argv[1], &propfound);
 	if (propfound)
 #ifdef MAGIC_WRAPPER
 	    Tcl_SetResult(magicinterp, value, NULL);
@@ -1219,13 +1293,13 @@ CmdProperty(w, cmd)
     else if (cmd->tx_argc == 3)
     {
 	if (strlen(cmd->tx_argv[2]) == 0)
-	    DBPropPut(editDef, cmd->tx_argv[1], NULL);
+	    DBPropPut(def, cmd->tx_argv[1], NULL);
 	else
 	{
 	    value = StrDup((char **)NULL, cmd->tx_argv[2]);
-	    DBPropPut(editDef, cmd->tx_argv[1], value);
+	    DBPropPut(def, cmd->tx_argv[1], value);
 	}
-	editDef->cd_flags |= (CDMODIFIED | CDGETNEWSTAMP);
+	def->cd_flags |= (CDMODIFIED | CDGETNEWSTAMP);
     }    
     else
     {
