@@ -27,7 +27,7 @@ static char rcsid[] = "$Header: /ufs/repository/magic/commands/CmdRS.c,v 1.8 200
 #include <string.h>
 #endif
 
-#include "magic/tclmagic.h"
+#include "tcltk/tclmagic.h"
 #include "misc/magic.h"
 #include "utils/stack.h"
 #include "utils/geometry.h"
@@ -48,6 +48,7 @@ static char rcsid[] = "$Header: /ufs/repository/magic/commands/CmdRS.c,v 1.8 200
 #include "netlist/netlist.h"
 #include "netmenu/netmenu.h"
 #include "select/select.h"
+#include "signals/signals.h"
 #include "sim/sim.h"
 
 extern void DisplayWindow();
@@ -227,7 +228,7 @@ CmdScaleGrid(w, cmd)
 
 	/* Scale cifinput and cifoutput */
 
-	CIFTechInputScale(scalen, scaled);
+	CIFTechInputScale(scalen, scaled, TRUE);
 	CIFTechOutputScale(scalen, scaled);
 
 	/* Scale drc rules */
@@ -298,6 +299,136 @@ scalegridusage:
     return;
 }
 
+#ifdef MAGIC_WRAPPER
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * CmdSearch --
+ *
+ *	This is a low-level search command that allows a hierarchical tile
+ *	search to execute a TCL procedure on each tile found.
+ *
+ *	Use: search [layers] <tcl_procedure>
+ *
+ *	<tcl_procedure> takes a list containing 4 integer values representing
+ *	the bounding box (llx, lly, urx, ury) of the tile, and one value
+ *	which is either a string containing the type name of the tile, or a
+ *	list of two layers in the case of a split tile, representing LEFT
+ *	and RIGHT types, respectively.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+void
+CmdSearch(w, cmd)
+    MagWindow *w;
+    TxCommand *cmd;
+{
+    SearchContext scx;
+    TileTypeBitMask visitMask;
+    int xMask, windowMask;
+    char *tclproc, *pptr;
+    int cmdTclProcSrFunc();     /* Fwd declaration */
+    CellDef *rootBoxDef;
+    MagWindow *window;
+
+    bzero(&scx, sizeof(SearchContext));
+    if (!ToolGetBox (&rootBoxDef, NULL))
+    {
+	TxError ("Box tool must be present\n");
+	return;
+    }
+    window = ToolGetBoxWindow (&scx.scx_area, &windowMask);
+    if (!window)
+    {
+	TxError ("Box tool does not exist in any window\n");
+	return;
+    }
+    xMask = ((DBWclientRec *)window->w_clientData)->dbw_bitmask;
+    if ((windowMask & ~xMask) != 0)
+    {
+	window = CmdGetRootPoint((Point *) NULL, (Rect *) NULL);
+	xMask = ((DBWclientRec *) window->w_clientData)->dbw_bitmask;
+	if ((windowMask & xMask) == 0)
+	{
+	    TxPrintf("The box is in more than one window;  use the cursor\n");
+	    TxPrintf("to select the one you want to select from.\n");
+	    return;
+	}
+    }
+    scx.scx_use = (CellUse *) window->w_surfaceID;
+    scx.scx_trans = GeoIdentityTransform;
+
+    if ((cmd->tx_argc == 1) || (cmd->tx_argc > 3))
+    {
+	TxError("Usage: search [layers] <tcl_procedure>\n");
+	return;
+    }
+
+    if (cmd->tx_argc == 3)
+    {
+	if (!CmdParseLayers(cmd->tx_argv[1], &visitMask))
+	    return;
+    }
+    else visitMask = DBAllButSpaceAndDRCBits;
+    pptr = cmd->tx_argv[cmd->tx_argc - 1];
+    if (*pptr == '{')           /* in-line procedure */
+    {
+	tclproc = StrDup(NULL, pptr + 1);
+	pptr = tclproc + strlen(tclproc) - 1;
+	if (*pptr == '}') *pptr = '\0';
+    }
+    else
+	tclproc = StrDup(NULL, cmd->tx_argv[cmd->tx_argc - 1]);
+
+    (void) DBTreeSrTiles(&scx, &visitMask, xMask,
+		cmdTclProcSrFunc, (ClientData) tclproc);
+   
+    freeMagic(tclproc);
+}   
+
+/*
+ * cmdTclProcSrFunc --
+ *  
+ * Filter procedure applied to tiles by cmdSearch() above.
+ */ 
+  
+int
+cmdTclProcSrFunc(tile, cxp)
+    Tile *tile;
+    TreeContext *cxp;
+{
+    Rect tileRect, rootRect;
+    SearchContext *scx = cxp->tc_scx;
+    char *tclproc = (char *)cxp->tc_filter->tf_arg;
+    char evalstr[512], *eptr;
+    int rval;
+
+    TiToRect(tile, &tileRect);
+    GeoTransRect(&scx->scx_trans, &tileRect, &rootRect);
+    if (GEO_OVERLAP((&tileRect), &cxp->tc_scx->scx_area))
+    {
+	sprintf(evalstr, "%s %d %d %d %d ", tclproc, rootRect.r_xbot,
+		rootRect.r_ybot, rootRect.r_xtop, rootRect.r_ytop);
+	eptr = evalstr + strlen(evalstr);
+#ifdef NONMANHATTAN
+	if (IsSplit(tile))
+	    sprintf(eptr, "{%s %s}\n", DBTypeLongNameTbl[SplitLeftType(tile)],
+			DBTypeLongNameTbl[SplitRightType(tile)]);
+	else
+#endif
+	    sprintf(eptr, "%s\n", DBTypeLongNameTbl[TiGetType(tile)]);
+
+	rval = Tcl_EvalEx(magicinterp, evalstr, -1, 0);
+	if (rval != TCL_OK) return 1;
+    }
+    /* Allow interrupts in case something goes wrong with the Tcl procedure */
+    if (SigInterruptPending) return 1;
+    return 0;
+}   
+
+#endif
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -329,7 +460,8 @@ CmdSee(w, cmd)
     int flags;
     bool off;
     char *arg;
-    TileTypeBitMask mask;
+    TileType i, j;
+    TileTypeBitMask mask, *rmask;
     DBWclientRec *crec;
 
     windCheckOnlyWindow(&w, DBWclientID);
@@ -385,24 +517,49 @@ CmdSee(w, cmd)
 
     if (off)
     {
-	int i;
 	for (i = 0; i < DBNumUserLayers; i++)
 	{
 	    if (TTMaskHasType(&mask, i))
 		TTMaskClearMask(&crec->dbw_visibleLayers,
 			&DBLayerTypeMaskTbl[i]);
 	}
+	for (; i < DBNumTypes; i++)
+	{
+	    /* This part handles stacked contact types.  The display	*/
+	    /* routine calls DBTreeSrUniqueTiles(), which displays	*/
+	    /* stacked contact types only on their home plane.  Thus	*/
+	    /* if we select a contact type, we should also select all	*/
+	    /* stacking types for that contact that are on the same	*/
+	    /* plane.							*/
+
+	    rmask = DBResidueMask(i);
+	    for (j = 0; j < DBNumUserLayers; j++)
+		if (TTMaskHasType(rmask, j))
+		    if (TTMaskHasType(&mask, j))
+			if (DBPlane(i) == DBPlane(j))
+			    TTMaskClearMask(&crec->dbw_visibleLayers,
+					&DBLayerTypeMaskTbl[i]);
+	}
 	crec->dbw_flags &= ~flags;
     }
     else
     {
-	int i;
 	for (i = 0; i < DBNumUserLayers; i++)
 	{
 	    if (TTMaskHasType(&mask, i))
 		TTMaskSetMask(&crec->dbw_visibleLayers,
 			&DBLayerTypeMaskTbl[i]);
 	}
+	for (; i < DBNumTypes; i++)
+	{
+	    rmask = DBResidueMask(i);
+	    for (j = 0; j < DBNumUserLayers; j++)
+		if (TTMaskHasType(rmask, j))
+		    if (TTMaskHasType(&mask, j))
+			if (DBPlane(i) == DBPlane(j))
+			    TTMaskSetMask(&crec->dbw_visibleLayers,
+					&DBLayerTypeMaskTbl[i]);
+	}
 	crec->dbw_flags |= flags;
     }
     WindAreaChanged(w, &w->w_screenArea);
@@ -1452,9 +1609,10 @@ CmdSnap(w, cmd)
     TxCommand *cmd;
 {
     static char *names[] = { "off", "internal", "lambda", "grid", "user", "on", 0 };
-    int n;
+    int n, mingrid;
+    DBWclientRec *crec;
 
-    if (cmd->tx_argc != 2) goto printit;
+    if (cmd->tx_argc < 2) goto printit;
 
     n = Lookup(cmd->tx_argv[1], names);
     if (n < 0)
@@ -1700,48 +1858,71 @@ CmdStretch(w, cmd)
 
     if (cmd->tx_argc > 1)
     {
-	int indx, amount;
+	int indx, amountx, amounty;
 
 	indx = GeoNameToPos(cmd->tx_argv[1], FALSE, TRUE);
 	if (indx < 0)
 	    return;
 	if (cmd->tx_argc == 3)
-	    amount = cmdParseCoord(cmd->tx_argv[2]);
-	else amount = 1;
+	{
+	    if (DBWSnapToGrid != DBW_SNAP_USER)
+	    {
+		amountx = amounty = cmdParseCoord(w, cmd->tx_argv[2], TRUE, FALSE);
+	    }
+	    else
+	    {
+		switch (indx)
+		{
+		    case GEO_EAST: case GEO_WEST:
+			amountx = cmdParseCoord(w, cmd->tx_argv[2], TRUE, TRUE);
+			amounty = amountx;
+			break;
+		    case GEO_NORTH: case GEO_SOUTH:
+			amounty = cmdParseCoord(w, cmd->tx_argv[2], TRUE, FALSE);
+			amountx = amounty;
+			break;
+		    default:
+			amountx = cmdParseCoord(w, cmd->tx_argv[2], TRUE, TRUE);
+			amounty = cmdParseCoord(w, cmd->tx_argv[2], TRUE, FALSE);
+			break;
+		}
+	    }
+	}
+	else amountx = amounty = 1;
 
 	switch (indx)
 	{
 	    case GEO_NORTH:
 		xdelta = 0;
-		ydelta = amount;
+		ydelta = amounty;
 		break;
 	    case GEO_SOUTH:
 		xdelta = 0;
-		ydelta = -amount;
+		ydelta = -amounty;
 		break;
 	    case GEO_EAST:
-		xdelta = amount;
+		xdelta = amountx;
 		ydelta = 0;
 		break;
 	    case GEO_WEST:
-		xdelta = -amount;
+		xdelta = -amountx;
 		ydelta = 0;
 		break;
 	    case GEO_NORTHEAST:
-		xdelta = amount;
-		ydelta = amount;
+		xdelta = amountx;
+		ydelta = amounty;
 		break;
 	    case GEO_NORTHWEST:
-		xdelta = -amount;
-		ydelta = amount;
+		xdelta = -amountx;
+		ydelta = amounty;
 		break;
 	    case GEO_SOUTHEAST:
-		xdelta = amount;
-		ydelta = -amount;
+		xdelta = amountx;
+		ydelta = -amounty;
 		break;
 	    case GEO_SOUTHWEST:
-		xdelta = -amount;
-		ydelta = -amount;
+		xdelta = -amountx;
+		ydelta = -amounty;
 		break;
 	    default:
 		ASSERT(FALSE, "Bad direction in CmdStretch");
