@@ -62,15 +62,24 @@ TileTypeBitMask CmdYMAllButSpace;
  * cmdParseCoord --
  *
  * Replaces the use of atoi() in command parsing of coordinates.  Allows
- * coordinates to be declared in internal units, lambda, or microns, and
- * translates to internal units accordingly.  It also allows coordinates
- * to be specified in floating-point.
+ * coordinates to be declared in internal units, lambda, user units, or
+ * microns, and translates to internal units accordingly.  It also allows
+ * coordinates to be specified in floating-point.
  *    A suffix of "i" indicates internal units, a suffix of "l" indicates
- * lambda, and a suffix in metric notation ("nm", "um", "mm", "cm")
- * indicates natural units.  Other valid units are "cu" or "centimicrons"
- * for centimicrons, or "microns" for um.
+ * lambda, a suffix of "g" indicates the user grid, and a suffix in metric
+ * notation ("nm", "um", "mm", "cm") indicates natural units.  Other valid
+ * units are "cu" or "centimicrons" for centimicrons, or "microns" for um.
  *    Units without any suffix are assumed to be in lambda if "snap"
- * (DBWSnapToGrid) is set to lambda, and internal units otherwise.
+ * (DBWSnapToGrid) is set to lambda, grid units if "snap" is set to the
+ * user grid, and internal units otherwise.
+ *
+ *    MagWindow argument w is used only with grid-based snapping, to find
+ * the value of the grid for the given window.  In this case, because the
+ * grid specifies an offset, "is_relative" specifies whether the given
+ * coordinate is a relative measurement (ignore offset) or an absolute
+ * position.  Because the grid can be different in X and Y, "is_x"
+ * specifies whether the given distance is in the X (TRUE) or Y (FALSE)
+ * direction.
  *
  * Results:
  *      Integer representing the given coordinate in internal units
@@ -82,12 +91,15 @@ TileTypeBitMask CmdYMAllButSpace;
  */
 
 int
-cmdParseCoord(arg)
+cmdParseCoord(w, arg, is_relative, is_x)
+    MagWindow *w;
     char *arg;
+    bool is_relative, is_x;
 {
     char *endptr;
     double dval = 0;
     int mscale;
+    DBWclientRec *crec;
 
     dval = strtod(arg, &endptr);
 
@@ -107,11 +119,38 @@ cmdParseCoord(arg)
 	return round(dval);
     }
     else if ((*endptr == 'i')
-		|| ((*endptr == '\0') && (DBWSnapToGrid != DBW_SNAP_LAMBDA)))
+		|| ((*endptr == '\0') && (DBWSnapToGrid == DBW_SNAP_INTERNAL)))
     {
 	/* internal units */
 	return round(dval);
     }
+    else if ((*endptr == 'g')
+		|| ((*endptr == '\0') && (DBWSnapToGrid == DBW_SNAP_USER)))
+    {
+	/* grid units */
+	if (w == (MagWindow *)NULL)
+	{
+	    windCheckOnlyWindow(&w, DBWclientID);
+	    if (w == (MagWindow *)NULL)
+		return round(dval);		/* Default, if window is unknown */
+	}
+	crec = (DBWclientRec *) w->w_clientData;
+	if (is_x)
+	{
+	    dval *= (double)(crec->dbw_gridRect.r_xtop
+			- crec->dbw_gridRect.r_xbot);
+	    if (!is_relative)
+		dval += (double)crec->dbw_gridRect.r_xbot;
+	}
+	else
+	{
+	    dval *= (double)(crec->dbw_gridRect.r_ytop
+			- crec->dbw_gridRect.r_ybot);
+	    if (!is_relative)
+		dval += (double)crec->dbw_gridRect.r_ybot;
+	}
+	return round(dval);
+    }
     else
     {
         /* natural units referred to the current cifoutput style */
