@@ -26,7 +26,7 @@ static char rcsid[] = "$Header: /ufs/repository/magic/commands/CmdTZ.c,v 1.7 200
 #include <ctype.h>
 #include <math.h>
 
-#include "magic/tclmagic.h"
+#include "tcltk/tclmagic.h"
 #include "misc/magic.h"
 #include "utils/geometry.h"
 #include "utils/utils.h"
@@ -85,6 +85,7 @@ extern void DisplayWindow();
 #define TECH_LAMBDA	5
 #define TECH_PLANES	6
 #define TECH_LAYERS	7
+#define TECH_DRC	8
 
 void
 CmdTech(w, cmd)
@@ -114,6 +115,7 @@ CmdTech(w, cmd)
 	"lambda			Show internal units per lambda", 
 	"planes			Show defined planes",
 	"layers			Show defined layers",
+	"drc <rule> <layer...>	Query DRC ruleset",
 	NULL
     };
 
@@ -210,6 +212,50 @@ CmdTech(w, cmd)
 		goto wrongNumArgs;
 	    break;
 
+	case TECH_DRC:
+	    /* Query the DRC database */
+	    if (cmd->tx_argc >= 4)
+	    {
+		TileType t1, t2;
+		int tresult;
+
+		t1 = DBTechNoisyNameType(cmd->tx_argv[3]);
+		if (t1 < 0) {
+		    TxError("No such layer %s\n", cmd->tx_argv[3]);
+		    break;
+		}
+		if (!strncmp(cmd->tx_argv[2], "width", 5))
+		{
+		    tresult = DRCGetDefaultLayerWidth(t1);
+#ifdef MAGIC_WRAPPER		
+		    Tcl_SetObjResult(magicinterp, Tcl_NewIntObj(tresult));
+#else
+		    TxPrintf("Minimum width is %d\n", tresult);
+#endif
+		}
+		else if (!strncmp(cmd->tx_argv[2], "spac", 4))
+		{
+		    if (cmd->tx_argc >= 5)
+		    {
+			t2 = DBTechNoisyNameType(cmd->tx_argv[4]);
+			if (t2 < 0) {
+			    TxError("No such layer %s\n", cmd->tx_argv[4]);
+		 	}
+		    }
+		    else
+			t2 = t1;
+		    tresult = DRCGetDefaultLayerSpacing(t1, t2);
+#ifdef MAGIC_WRAPPER		
+		    Tcl_SetObjResult(magicinterp, Tcl_NewIntObj(tresult));
+#else
+		    TxPrintf("Minimum spacing is %d\n", tresult);
+#endif
+		}
+	    }
+	    else
+		goto wrongNumArgs;
+	    break;
+
 	case TECH_LOAD:
 	    locargc = cmd->tx_argc;
 	    while ((*cmd->tx_argv[locargc - 1] == '-') && (locargc > 3))
@@ -285,7 +331,7 @@ CmdTech(w, cmd)
 		int d = DBLambda[0];
 		int n = DBLambda[1];
 
-		CIFTechInputScale(d, n);      
+		CIFTechInputScale(d, n, TRUE);
 		CIFTechOutputScale(d, n);
 		DRCTechScale(d, n);
 		ExtTechScale(d, n);
@@ -826,14 +872,15 @@ cmdWhatCellFunc(selUse, realUse, transform, foundAny)
 #define SWITCH		3
 #define TYPE		4
 #define VERTICAL	5
+#define SEGMENT		6
 
 void
 CmdWire(w, cmd)
     MagWindow *w;
     TxCommand *cmd;
 {
-    int option;
-    char **msg;
+    int option, locargc;
+    char **msg, *lastargv;
     TileType type;
     int width;
 
@@ -845,13 +892,16 @@ CmdWire(w, cmd)
 	"switch [layer width]   place contact and switch layers",
 	"type [layer width]     select the type and size of wires",
 	"vertical               add a new vertical wire leg",
+	"segment layer width x1 y1 x2 y2... [-noendcap]\n"
+	"			paint one or more wire segments",
 	NULL
     };
 
-    if (cmd->tx_argc < 2)
+    locargc = cmd->tx_argc;
+    if (locargc < 2)
     {
 	option = HELP;
-	cmd->tx_argc = 2;
+	locargc = 2;
     }
     else
     {
@@ -860,7 +910,7 @@ CmdWire(w, cmd)
 	{
 	    TxError("\"%s\" isn't a valid wire option.\n", cmd->tx_argv[1]);
 	    option = HELP;
-	    cmd->tx_argc = 2;
+	    locargc = 2;
 	}
     }
 
@@ -884,9 +934,9 @@ CmdWire(w, cmd)
 	    return;
 
 	case SWITCH:
-	    if (cmd->tx_argc == 2)
+	    if (locargc == 2)
 		WireAddContact(-1, 0);
-	    else if (cmd->tx_argc != 4)
+	    else if (locargc != 4)
 		goto badargs;
 	    else
 	    {
@@ -903,16 +953,16 @@ CmdWire(w, cmd)
 			    cmd->tx_argv[2]);
 		    return;
 		}
-		width = cmdParseCoord(cmd->tx_argv[3]);
+		width = cmdParseCoord(w, cmd->tx_argv[3], TRUE, TRUE);
 		WireAddContact(type, width);
 		return;
 	    }
 	    break;
 
 	case TYPE:
-	    if (cmd->tx_argc == 2)
+	    if (locargc == 2)
 		WirePickType(-1, 0);
-	    else if (cmd->tx_argc != 4)
+	    else if (locargc != 4)
 	    {
 		badargs:
 		TxError("Wrong arguments.  The correct syntax is\n");
@@ -934,7 +984,7 @@ CmdWire(w, cmd)
 			    cmd->tx_argv[2]);
 		    return;
 		}
-		width = cmdParseCoord(cmd->tx_argv[3]);
+		width = cmdParseCoord(w, cmd->tx_argv[3], TRUE, TRUE);
 		WirePickType(type, width);
 		return;
 	    }
@@ -943,6 +993,97 @@ CmdWire(w, cmd)
 	case VERTICAL:
 	    WireAddLeg((Rect *) NULL, (Point *) NULL, WIRE_VERTICAL);
 	    return;
+
+	case SEGMENT:
+	    {
+		Point *plist;
+		Rect r;
+		int pNum, i, j, points, wexpand;
+	 	CellDef *def = (CellDef *)NULL;
+		PaintUndoInfo ui;
+		bool endcap = TRUE;
+
+		if (EditCellUse != NULL)
+		    def = EditCellUse->cu_def;
+
+		if (def == NULL)
+		{
+		    TxError("No cell being edited\n");
+		    return;
+		}
+
+		lastargv = cmd->tx_argv[locargc - 1];
+		if (lastargv[0] == '-')
+		{
+		    if (!strcmp(lastargv, "-noendcap"))
+		    {
+			endcap = FALSE;
+			locargc--;
+		    }
+		}
+		if ((locargc < 8) || (locargc & 0x1))
+		{
+		    TxError("Usage: wire segment layer width x1 x2 y1 y2\n");
+		    return;
+		}
+		type = DBTechNoisyNameType(cmd->tx_argv[2]);
+		if (!StrIsInt(cmd->tx_argv[3]))
+		{
+		    TxError("Route segment width must be an integer value\n");
+		    return;
+		}
+		else
+		    width = atoi(cmd->tx_argv[3]);
+
+		/* By doubling DBLambda[1], we can get back the	*/
+		/* coordinates in 2x internal units, which is	*/
+		/* what is needed by routine PaintWireList.	*/
+
+		DBLambda[1] *= 2; 
+		points = (locargc - 4) / 2;
+		plist = (Point *)mallocMagic(points * sizeof(Point));
+		for (i = 0, j = 4; i < points; i++)
+		{
+		    plist[i].p_x = cmdParseCoord(w, cmd->tx_argv[j++], FALSE, TRUE);
+		    plist[i].p_y = cmdParseCoord(w, cmd->tx_argv[j++], FALSE, FALSE);
+		}
+		DBLambda[1] /= 2; 
+
+		def->cd_flags |= CDMODIFIED | CDGETNEWSTAMP;
+		ui.pu_def = def;
+		for (pNum = PL_PAINTBASE; pNum < DBNumPlanes; pNum++)
+		{
+		    if (DBPaintOnPlane(type, pNum))
+		    {
+			ui.pu_pNum = pNum;
+			PaintWireList(plist, points, width, endcap,
+				def->cd_planes[pNum],
+				DBStdPaintTbl(type, pNum), &ui);
+		    }
+		}
+
+		/* Determine the bounding box of the segment just drawn */
+		r.r_xbot = r.r_xtop = plist[0].p_x;
+		r.r_ybot = r.r_ytop = plist[0].p_y;
+		for (i = 1; i < points; i++)
+		    GeoIncludePoint(plist + i, &r);
+
+		wexpand = (int)(0.5 + (float)width * 1.414214);
+		r.r_xbot -= wexpand;
+		r.r_ybot -= wexpand;
+		r.r_xtop += wexpand;
+		r.r_ytop += wexpand;
+		r.r_xbot /= 2;
+		r.r_ybot /= 2;
+		r.r_xtop /= 2;
+		r.r_ytop /= 2;
+
+		DBWAreaChanged(def, &r, DBW_ALLWINDOWS, &DBAllButSpaceBits);
+		DBReComputeBbox(def);
+		DRCCheckThis (def, TT_CHECKPAINT, &r);
+		freeMagic(plist);
+	    }
+	    break;
     }
 }
 
@@ -1089,6 +1230,9 @@ cmdWriteallFunc(def, autoWrite)
  * Side effects:
  *	Sets EditCellUse.
  *
+ * Notes:
+ *	This does not yet implement the "-force" option.
+ *
  * ----------------------------------------------------------------------------
  */
 
@@ -1114,7 +1258,7 @@ CmdXload(w, cmd)
     {
 	if (CmdIllegalChars(cmd->tx_argv[1], "[],", "Cell name"))
 	    return;
-	DBWxloadWindow(w, cmd->tx_argv[1]);
+	DBWloadWindow(w, cmd->tx_argv[1], FALSE, TRUE);
     }
-    else DBWxloadWindow(w, (char *) NULL);
+    else DBWloadWindow(w, (char *) NULL, FALSE, TRUE);
 }
