@@ -50,6 +50,8 @@ static char rcsid[] = "$Header: /ufs/repository/magic/commands/CmdWizard.c,v 1.2
 
 extern void cmdPsearchStats();
 
+void cmdStatsHier(CellDef *, int, CellDef *);
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -187,6 +189,7 @@ CmdExtractTest(w, cmd)
 /*
  * ----------------------------------------------------------------------------
  *
+ * showTech --
  * CmdShowtech --
  *
  * Usage:
@@ -204,6 +207,149 @@ CmdExtractTest(w, cmd)
  * ----------------------------------------------------------------------------
  */
 
+void
+showTech(outf, verbose)
+    FILE *outf;		/* File to which information is to be output */
+    bool verbose;	/* If TRUE, output detailed erase table */
+{
+    int i, j;
+    int pNum;
+    bool first, any;
+
+    fprintf(outf, "Technology %s\n", DBTechName);
+    fprintf(outf, "%d tile planes, %d tile types\n\n",
+		DBNumPlanes, DBNumTypes);
+    fprintf(outf, "Planes:\n");
+    for (i = 0; i < DBNumPlanes; i++)
+	fprintf(outf, "%s\t%s\n", DBPlaneShortName(i),
+		DBPlaneLongName(i));
+
+    fprintf(outf, "\n");
+    fprintf(outf, "Types:\n");
+    for (i = 0; i < DBNumTypes; i++) {
+	int pl ; char *spl ;
+
+	pl = DBPlane(i);
+	spl = ( pl <= 0 || pl > DBNumPlanes ) ? "??" : DBPlaneLongName(pl);
+	fprintf(outf, "%s\t%s\t%s\n", spl,
+		DBTypeShortName(i), DBTypeLongName(i));
+	}
+
+    fprintf(outf, "\n");
+    fprintf(outf, "\014Connectivity:\n");
+    for (j = 0; j < DBNumTypes; j++)
+	for (i = 0; i < j; i++)
+	    if (DBConnectsTo(i, j))
+		fprintf(outf, "%s :: %s\n",
+			DBTypeLongName(j), DBTypeLongName(i));
+    fprintf(outf, "\n");
+
+    fprintf(outf, "\n\014Component Layers:\n");
+    for (i = 0; i < DBNumUserLayers; i++)
+	for (j = 0; j < DBNumUserLayers; j++)
+	    if ((j != i) && TTMaskHasType(&DBComponentTbl[j], i))
+		fprintf(outf, "%s is a component of %s\n",
+		    DBTypeLongName(i), DBTypeLongName(j));
+    fprintf(outf, "\n");
+
+    fprintf(outf, "\014Planes affected by painting:\n");
+    fprintf(outf, "Type                  Planes\n");
+    fprintf(outf, "----                  ------\n");
+    for (i = 0; i < DBNumTypes; i++)
+    {
+	fprintf(outf, "%-22.22s", DBTypeLongName(i));
+	first = TRUE;
+	for (pNum = 0; pNum < DBNumPlanes; pNum++)
+	{
+	    if (DBPaintOnPlane(i, pNum))
+	    {
+		if (first)
+		    first = FALSE;
+		else
+		    fprintf(outf, ", ");
+		fprintf(outf, "%s", DBPlaneLongName(pNum));
+	    }
+	}
+	fprintf(outf, "\n");
+    }
+
+    fprintf(outf, "\014Planes affected by erasing:\n");
+    fprintf(outf, "Type                  Planes\n");
+    fprintf(outf, "----                  ------\n");
+    for (i = 0; i < DBNumTypes; i++)
+    {
+	fprintf(outf, "%-22.22s", DBTypeLongName(i));
+	first = TRUE;
+	for (pNum = 0; pNum < DBNumPlanes; pNum++)
+	{
+	    if (DBEraseOnPlane(i, pNum))
+	    {
+		if (!first)
+		    fprintf(outf, ", ");
+		first = FALSE;
+		fprintf(outf, "%s", DBPlaneLongName(pNum));
+	    }
+	}
+	fprintf(outf, "\n");
+    }
+
+    for (pNum = PL_PAINTBASE; pNum < DBNumPlanes; pNum++)
+    {
+	fprintf(outf, "\014Paint: %s\n", DBPlaneLongName(pNum));
+	fprintf(outf, "=======================================\n");
+	for (i = 0; i < DBNumTypes; i++)
+	{
+	    if (i == TT_SPACE || DBPlane(i) == pNum)
+	    {
+		any = FALSE;
+		for (j = 0; j < DBNumTypes; j++)
+		{
+		    if (!verbose && (i == TT_SPACE || j == TT_SPACE))
+			continue;
+		    if (DBStdPaintEntry(i, j, pNum) != i)
+		    {
+			fprintf(outf, "%s + %s --> %s\n",
+				DBTypeLongName(i), DBTypeLongName(j),
+				DBTypeLongName(DBStdPaintEntry(i, j, pNum)));
+			any = TRUE;
+		    }
+		}
+		if (any)
+		    fprintf(outf,
+				"--------------------------------------\n");
+	    }
+	}
+    }
+
+    for (pNum = PL_PAINTBASE; pNum < DBNumPlanes; pNum++)
+    {
+	fprintf(outf, "\014Erase: %s\n", DBPlaneLongName(pNum));
+	fprintf(outf, "=======================================\n");
+	for (i = 0; i < DBNumTypes; i++)
+	{
+	    if (i == TT_SPACE || DBPlane(i) == pNum)
+	    {
+		any = FALSE;
+		for (j = 0; j < DBNumTypes; j++)
+		{
+		    if (!verbose && i == j)
+			continue;
+		    if (DBStdEraseEntry(i, j, pNum) != i)
+		    {
+			fprintf(outf, "%s - %s --> %s\n",
+				DBTypeLongName(i), DBTypeLongName(j),
+				DBTypeLongName(DBStdEraseEntry(i, j, pNum)));
+			any = TRUE;
+		    }
+		}
+		if (any)
+		    fprintf(outf,
+				"--------------------------------------\n");
+	    }
+	}
+    }
+}
+
 void
 CmdShowtech(w, cmd)
     MagWindow *w;
@@ -247,147 +393,6 @@ CmdShowtech(w, cmd)
 	(void) fclose(outf);
 }
 
-showTech(outf, verbose)
-    FILE *outf;		/* File to which information is to be output */
-    bool verbose;	/* If TRUE, output detailed erase table */
-{
-    int i, j;
-    int pNum;
-    bool first, any;
-
-    (void) fprintf(outf, "Technology %s\n", DBTechName);
-    (void) fprintf(outf, "%d tile planes, %d tile types\n\n",
-		DBNumPlanes, DBNumTypes);
-    (void) fprintf(outf, "Planes:\n");
-    for (i = 0; i < DBNumPlanes; i++)
-	(void) fprintf(outf, "%s\t%s\n", DBPlaneShortName(i),
-		DBPlaneLongName(i));
-
-    (void) fprintf(outf, "\n");
-    (void) fprintf(outf, "Types:\n");
-    for (i = 0; i < DBNumTypes; i++) {
-	int pl ; char *spl ;
-
-	pl = DBPlane(i);
-	spl = ( pl <= 0 || pl > DBNumPlanes ) ? "??" : DBPlaneLongName(pl);
-	(void) fprintf(outf, "%s\t%s\t%s\n", spl,
-		DBTypeShortName(i), DBTypeLongName(i));
-	}
-
-    (void) fprintf(outf, "\n");
-    (void) fprintf(outf, "\014Connectivity:\n");
-    for (j = 0; j < DBNumTypes; j++)
-	for (i = 0; i < j; i++)
-	    if (DBConnectsTo(i, j))
-		fprintf(outf, "%s :: %s\n",
-			DBTypeLongName(j), DBTypeLongName(i));
-    (void) fprintf(outf, "\n");
-
-    (void) fprintf(outf, "\n\014Component Layers:\n");
-    for (i = 0; i < DBNumUserLayers; i++)
-	for (j = 0; j < DBNumUserLayers; j++)
-	    if ((j != i) && TTMaskHasType(&DBComponentTbl[j], i))
-		fprintf(outf, "%s is a component of %s\n",
-		    DBTypeLongName(i), DBTypeLongName(j));
-    (void) fprintf(outf, "\n");
-
-    (void) fprintf(outf, "\014Planes affected by painting:\n");
-    (void) fprintf(outf, "Type                  Planes\n");
-    (void) fprintf(outf, "----                  ------\n");
-    for (i = 0; i < DBNumTypes; i++)
-    {
-	(void) fprintf(outf, "%-22.22s", DBTypeLongName(i));
-	first = TRUE;
-	for (pNum = 0; pNum < DBNumPlanes; pNum++)
-	{
-	    if (DBPaintOnPlane(i, pNum))
-	    {
-		if (first)
-		    first = FALSE;
-		else
-		    (void) fprintf(outf, ", ");
-		(void) fprintf(outf, "%s", DBPlaneLongName(pNum));
-	    }
-	}
-	(void) fprintf(outf, "\n");
-    }
-
-    (void) fprintf(outf, "\014Planes affected by erasing:\n");
-    (void) fprintf(outf, "Type                  Planes\n");
-    (void) fprintf(outf, "----                  ------\n");
-    for (i = 0; i < DBNumTypes; i++)
-    {
-	(void) fprintf(outf, "%-22.22s", DBTypeLongName(i));
-	first = TRUE;
-	for (pNum = 0; pNum < DBNumPlanes; pNum++)
-	{
-	    if (DBEraseOnPlane(i, pNum))
-	    {
-		if (!first)
-		    (void) fprintf(outf, ", ");
-		first = FALSE;
-		(void) fprintf(outf, "%s", DBPlaneLongName(pNum));
-	    }
-	}
-	(void) fprintf(outf, "\n");
-    }
-
-    for (pNum = PL_PAINTBASE; pNum < DBNumPlanes; pNum++)
-    {
-	(void) fprintf(outf, "\014Paint: %s\n", DBPlaneLongName(pNum));
-	(void) fprintf(outf, "=======================================\n");
-	for (i = 0; i < DBNumTypes; i++)
-	{
-	    if (i == TT_SPACE || DBPlane(i) == pNum)
-	    {
-		any = FALSE;
-		for (j = 0; j < DBNumTypes; j++)
-		{
-		    if (!verbose && (i == TT_SPACE || j == TT_SPACE))
-			continue;
-		    if (DBStdPaintEntry(i, j, pNum) != i)
-		    {
-			(void) fprintf(outf, "%s + %s --> %s\n",
-				DBTypeLongName(i), DBTypeLongName(j),
-				DBTypeLongName(DBStdPaintEntry(i, j, pNum)));
-			any = TRUE;
-		    }
-		}
-		if (any)
-		    (void) fprintf(outf,
-				"--------------------------------------\n");
-	    }
-	}
-    }
-
-    for (pNum = PL_PAINTBASE; pNum < DBNumPlanes; pNum++)
-    {
-	(void) fprintf(outf, "\014Erase: %s\n", DBPlaneLongName(pNum));
-	(void) fprintf(outf, "=======================================\n");
-	for (i = 0; i < DBNumTypes; i++)
-	{
-	    if (i == TT_SPACE || DBPlane(i) == pNum)
-	    {
-		any = FALSE;
-		for (j = 0; j < DBNumTypes; j++)
-		{
-		    if (!verbose && i == j)
-			continue;
-		    if (DBStdEraseEntry(i, j, pNum) != i)
-		    {
-			(void) fprintf(outf, "%s - %s --> %s\n",
-				DBTypeLongName(i), DBTypeLongName(j),
-				DBTypeLongName(DBStdEraseEntry(i, j, pNum)));
-			any = TRUE;
-		    }
-		}
-		if (any)
-		    (void) fprintf(outf,
-				"--------------------------------------\n");
-	    }
-	}
-    }
-}
 
 /*
  * ----------------------------------------------------------------------------
@@ -503,7 +508,6 @@ cmdStatsFunc(def, outf)
     FILE *outf;
 {
     int cmdStatsCount(), cmdStatsOutput();
-    int cmdStatsHier();
     struct countClient cc;
     int total;
     TileType t;
@@ -521,13 +525,13 @@ cmdStatsFunc(def, outf)
     {
 	if (totalTiles[t])
 	{
-	    (void) fprintf(outf, "%s\tTOTAL\t%s\t%d\n",
+	    fprintf(outf, "%s\tTOTAL\t%s\t%d\n",
 		    def->cd_name, DBTypeLongName(t), totalTiles[t]);
 	    total += totalTiles[t];
 	}
     }
 
-    (void) fprintf(outf, "%s\tTOTAL\tALL\t%d\n", def->cd_name, total);
+    fprintf(outf, "%s\tTOTAL\tALL\t%d\n", def->cd_name, total);
     return (0);
 }
 
@@ -693,7 +697,7 @@ cmdStatsOutput(def, cc)
 	if (ci->ci_hierCount[t] | ci->ci_count[t])
 	{
 	    /* Root-def this-def type-name num-flat num-paint */
-	    (void) fprintf(cc->cc_outFile, "%s\t%s\t%s\t%d\t%d\n",
+	    fprintf(cc->cc_outFile, "%s\t%s\t%s\t%d\t%d\n",
 		    cc->cc_rootDef->cd_name, def->cd_name,
 		    DBTypeLongName(t),
 		    ci->ci_hierCount[t] + ci->ci_count[t], ci->ci_count[t]);
@@ -705,7 +709,7 @@ cmdStatsOutput(def, cc)
     /* Root-def this-def ALL num-flat num-paint */
     if (hiercount | count)
     {
-	(void) fprintf(cc->cc_outFile, "%s\t%s\tALL\t%d\t%d\n",
+	fprintf(cc->cc_outFile, "%s\t%s\tALL\t%d\t%d\n",
 		cc->cc_rootDef->cd_name, def->cd_name,
 		hiercount + count, count);
     }
