@@ -379,7 +379,7 @@ DBCellCopyLabels(scx, mask, xMask, targetUse, pArea)
 	    (void) DBEraseLabelsByContent(def, &labTargetRect, targetPos,
 		    -1, lab->lab_text);
 	    (void) DBPutLabel(def, &labTargetRect, targetPos,
-		    lab->lab_text, lab->lab_type, lab->lab_flags);
+		    lab->lab_text, lab->lab_type, lab->lab_flags, TRUE);
 	    if (pArea != NULL)
 		(void) GeoIncludeAll(&labTargetRect, pArea);
 	}
@@ -455,7 +455,7 @@ dbCopyAllPaint(tile, cxp)
     Rect sourceRect, targetRect;
     PaintUndoInfo ui;
     CellDef *def;
-    TileType type;
+    TileType type = TiGetTypeExact(tile);
     int pNum;
 
     /*
@@ -466,38 +466,18 @@ dbCopyAllPaint(tile, cxp)
      */
 
 #ifdef NONMANHATTAN
-    Rect cliprect;
     bool splittile = FALSE;
-    TileType exact = TiGetTypeExact(tile);
-    int o1, o2, o3, dir, side;
-
+    TileType dinfo = 0;
+    
     if (IsSplit(tile))
     {
 	splittile = TRUE;
-	if (SplitSide(tile))
-	    type = SplitRightType(tile);
-	else
-	    type = SplitLeftType(tile);
-
-	/* Change the tile type according to the transformation */
-
-        o1 = ((scx->scx_trans.t_e > 0) || (scx->scx_trans.t_d > 0)) ? 1 : 0;
-        o2 = ((scx->scx_trans.t_a > 0) || (scx->scx_trans.t_b > 0)) ? 1 : 0;
-        o3 = (scx->scx_trans.t_a != 0) ? 1 : 0;
-        dir = SplitDirection(tile) ^ o1 ^ o2;
-        side = SplitSide(tile) ^ o2 ^ (SplitDirection(tile) | o3);
-
-	if (side)
-	    exact = TT_DIAGONAL | TT_SIDE | SplitLeftType(tile) |
-			(SplitRightType(tile) << 14);
-	else
-	    exact = TT_DIAGONAL | SplitRightType(tile) | (SplitLeftType(tile) << 14);
-
-	if (dir) exact |= TT_DIRECTION;
+	dinfo = DBTransformDiagonal(type, &scx->scx_trans);
+	type = (SplitSide(tile)) ? SplitRightType(tile) :
+			SplitLeftType(tile);
     }
-    else
 #endif
-    type = TiGetType(tile);
+
     if (type == TT_SPACE)
 	return 0;
 
@@ -510,6 +490,7 @@ dbCopyAllPaint(tile, cxp)
     GEOTRANSRECT(&scx->scx_trans, &sourceRect, &targetRect);
 
     ui.pu_def = def = arg->caa_targetUse->cu_def;
+
     def->cd_flags |= CDMODIFIED|CDGETNEWSTAMP;
 
 #ifdef NONMANHATTAN
@@ -522,7 +503,7 @@ dbCopyAllPaint(tile, cxp)
 	Rect rrect, orect;
 	int np, i, j;
 
-	GrClipTriangle(&targetRect, &arg->caa_rect, TRUE, dir, side, points, &np);
+	GrClipTriangle(&targetRect, &arg->caa_rect, TRUE, dinfo, points, &np);
 	
 	if (np == 0)
 	   return(0);
@@ -551,8 +532,7 @@ dbCopyAllPaint(tile, cxp)
 		rrect.r_ybot = points[0].p_y;
 		rrect.r_ytop = points[2].p_y;
 		GeoCanonicalRect(&rrect, &targetRect);
-		splittile = FALSE;
-		exact = type;
+		dinfo = 0;
 	    }
 	    else if (np >= 4) /* Process extra rectangles in the area */
 	    {
@@ -628,7 +608,7 @@ splitdone:
 	{
 	    ui.pu_pNum = pNum;
 #ifdef NONMANHATTAN
-	    (*dbCurPaintPlane)(def->cd_planes[pNum], exact,
+	    (*dbCurPaintPlane)(def->cd_planes[pNum], dinfo,
 			&targetRect, dbCurPaintTbl[pNum][type], &ui);
 #else
 	    (*dbCurPaintPlane)(def->cd_planes[pNum], &targetRect,
@@ -801,7 +781,7 @@ dbCellCopyCellsFunc(scx, arg)
     newUse = DBCellNewUse(def, (char *) use->cu_id);
     if (!DBLinkCell(newUse, arg->caa_targetUse->cu_def))
     {
-	FREE((char *) newUse->cu_id);
+	freeMagic((char *) newUse->cu_id);
 	newUse->cu_id = NULL;
 	(void) DBLinkCell(newUse, arg->caa_targetUse->cu_def);
     }
