@@ -25,7 +25,7 @@ static char rcsid[] = "$Header: /ufs/repository/magic/database/DBcellname.c,v 1.
 #include <string.h>
 #endif
 
-#include "magic/tclmagic.h"
+#include "tcltk/tclmagic.h"
 #include "misc/magic.h"
 #include "utils/hash.h"
 #include "utils/utils.h"
@@ -41,6 +41,7 @@ static char rcsid[] = "$Header: /ufs/repository/magic/database/DBcellname.c,v 1.
 #include "textio/textio.h"
 #include "dbwind/dbwind.h"
 #include "main/main.h"
+#include "drc/drc.h"
 
 /*
  * Hash tables for CellDefs and CellUses
@@ -216,9 +217,9 @@ DBCellDelete(cellname)
 	if (celluse->cu_parent == (CellDef *) NULL)
 	{
 	    WindUnload(celluse);
-	    FREE(celluse->cu_id);
+	    freeMagic(celluse->cu_id);
 	}
-	FREE((char *)celluse);
+	freeMagic((char *)celluse);
     }
     celldef->cd_parents = (CellUse *)NULL;
 
@@ -1118,7 +1119,7 @@ DBCellDefAlloc()
     CellDef *cellDef;
     int pNum;
 
-    MALLOC(CellDef *, cellDef, sizeof (CellDef));
+    cellDef = (CellDef *) mallocMagic((unsigned) (sizeof (CellDef)));
     cellDef->cd_flags = 0;
     cellDef->cd_bbox.r_xbot = 0;
     cellDef->cd_bbox.r_ybot = 0;
@@ -1172,7 +1173,7 @@ DBCellNewUse(cellDef, useName)
 {
     CellUse *cellUse;
 
-    MALLOC(CellUse *, cellUse, sizeof (CellUse));
+    cellUse = (CellUse *) mallocMagic((unsigned) (sizeof (CellUse)));
     cellUse->cu_id = StrDup((char **) NULL, useName);
     cellUse->cu_expandMask = 0;
     cellUse->cu_transform = GeoIdentityTransform;
@@ -1277,6 +1278,11 @@ DBCellDeleteDef(cellDef)
     HashSetValue(entry, (ClientData) NULL);
     if (cellDef->cd_props)
 	DBPropClearAll(cellDef);
+
+    /* Need to check the DRC pending queue, and remove	*/
+    /* this cell if it's there.				*/
+    DRCRemovePending(cellDef);
+
     DBCellDefFree(cellDef);
     return TRUE;
 }
@@ -1310,9 +1316,9 @@ DBCellDefFree(cellDef)
     Label *lab;
 
     if (cellDef->cd_file != (char *) NULL)
-	FREE(cellDef->cd_file);
+	freeMagic(cellDef->cd_file);
     if (cellDef->cd_name != (char *) NULL)
-	FREE(cellDef->cd_name);
+	freeMagic(cellDef->cd_name);
 
     /*
      * We want the following searching to be non-interruptible
@@ -1331,11 +1337,11 @@ DBCellDefFree(cellDef)
     }
 
     for (lab = cellDef->cd_labels; lab; lab = lab->lab_next)
-	FREE((char *) lab);
+	freeMagic((char *) lab);
     SigEnableInterrupts();
     HashKill(&cellDef->cd_idHash);
 
-    FREE((char *) cellDef);
+    freeMagic((char *) cellDef);
 }
 
 /*
@@ -1373,7 +1379,7 @@ DBCellDeleteUse(cellUse)
 
     cellDef = cellUse->cu_def;
     if (cellUse->cu_id != (char *) NULL)
-	FREE(cellUse->cu_id);
+	freeMagic(cellUse->cu_id);
     cellUse->cu_id = (char *) NULL;
     cellUse->cu_def = (CellDef *) NULL;
 
@@ -1391,7 +1397,7 @@ DBCellDeleteUse(cellUse)
 	}
     }
 
-    FREE((char *) cellUse);
+    freeMagic((char *) cellUse);
     return (TRUE);
 }
 
@@ -1724,7 +1730,7 @@ dbFindNamesFunc(use, parentDef)
 	    TxError("Duplicate instance-id for cell %s (%s) will be renamed\n",
 			use->cu_def->cd_name, use->cu_id);
 	    DBUnLinkCell(use, parentDef);
-	    FREE(use->cu_id);
+	    freeMagic(use->cu_id);
 	    use->cu_id = (char *) NULL;
 	}
 	HashSetValue(he, use);
