@@ -34,6 +34,12 @@ static char rcsid[] = "$Header: /ufs/repository/magic/database/DBcellsrch.c,v 1.
 #include "windows/windows.h"
 #include "main/main.h"
 
+/* Quick hack for dbScalePlanes() access to the CIF/GDS paint table */
+
+#ifdef CIF_MODULE
+extern PaintResultType CIFPaintTable[];
+#endif
+
 /*
  * The following structure is used to accumulate information about
  * the types of tiles visible underneath a given point in the database.
@@ -113,11 +119,60 @@ DBTreeSrTiles(scx, mask, xMask, func, cdarg)
     filter.tf_arg = cdarg;
     filter.tf_mask = mask;
     filter.tf_xmask = xMask;
+#ifdef NONMANHATTAN
+    filter.tf_dinfo = 0;
+#endif
     filter.tf_planes = DBTechTypesToPlanes(mask);
 
     return dbCellTileSrFunc(scx, &filter);
 }
 
+#ifdef NONMANHATTAN
+
+/*
+ * DBTreeSrNMTiles --
+ *	This is a variant of the above in which the search is over
+ *	a non-Manhattan area.
+ */
+
+int
+DBTreeSrNMTiles(scx, dinfo, mask, xMask, func, cdarg)
+    SearchContext *scx;		/* Pointer to search context specifying
+				 * a cell use to search, an area in the
+				 * coordinates of the cell's def, and a
+				 * transform back to "root" coordinates.
+				 */
+    TileType dinfo;		/* Type containing information about the
+				 * diagonal area to search.
+				 */
+    TileTypeBitMask *mask;	/* Only tiles with a type for which
+				 * a bit in this mask is on are processed.
+				 */
+    int xMask;			/* All subcells are visited recursively
+				 * until we encounter uses whose flags,
+				 * when anded with xMask, are not
+				 * equal to xMask.
+				 */
+    int (*func)();		/* Function to apply at each qualifying tile */
+    ClientData cdarg;		/* Client data for above function */
+{
+    int dbCellTileSrFunc();
+    TreeFilter filter;
+
+    /* Set up the filter and call the recursive filter function */
+
+    filter.tf_func = func;
+    filter.tf_arg = cdarg;
+    filter.tf_mask = mask;
+    filter.tf_xmask = xMask;
+    filter.tf_dinfo = dinfo;
+    filter.tf_planes = DBTechTypesToPlanes(mask);
+
+    return dbCellTileSrFunc(scx, &filter);
+}
+
+#endif
+
 /*
  * dbCellTileSrFunc --
  *
@@ -150,6 +205,17 @@ dbCellTileSrFunc(scx, fp)
     for (pNum = PL_PAINTBASE; pNum < DBNumPlanes; pNum++)
 	if (PlaneMaskHasPlane(fp->tf_planes, pNum))
 	{
+#ifdef NONMANHATTAN
+	    if (fp->tf_dinfo & TT_DIAGONAL)
+	    {
+		TileType dinfo = DBTransformDiagonal(fp->tf_dinfo, &scx->scx_trans);
+		if (DBSrPaintNMArea((Tile *) NULL, def->cd_planes[pNum],
+			dinfo, &scx->scx_area, fp->tf_mask,
+			fp->tf_func, (ClientData) &context))
+		    return 1;
+	    }
+	    else
+#endif
 	    if (DBSrPaintArea((Tile *) NULL, def->cd_planes[pNum],
 		    &scx->scx_area, fp->tf_mask,
 		    fp->tf_func, (ClientData) &context))
@@ -224,7 +290,7 @@ DBTreeSrUniqueTiles(scx, mask, xMask, func, cdarg)
 }
 
 /*
- * dbCellTileSrFunc --
+ * dbCellUniqueTileSrFunc --
  *
  * Recursive filter procedure applied to the cell by DBTreeSrUniqueTiles().
  * This is similar to dbCellTileSrFunc, except that for each plane searched,
@@ -242,7 +308,7 @@ dbCellUniqueTileSrFunc(scx, fp)
     CellDef *def = scx->scx_use->cu_def;
     int pNum;
 
-    ASSERT(def != (CellDef *) NULL, "dbCellTileSrFunc");
+    ASSERT(def != (CellDef *) NULL, "dbCellUniqueTileSrFunc");
     if (!DBIsExpand(scx->scx_use, fp->tf_xmask))
 	return 0;
     if ((def->cd_flags & CDAVAILABLE) == 0)
@@ -343,7 +409,6 @@ DBNoTreeSrTiles(scx, mask, xMask, func, cdarg)
     int (*func)();		/* Function to apply at each qualifying tile */
     ClientData cdarg;		/* Client data for above function */
 {
-    int dbCellTileSrFunc();
     TreeContext context;
     TreeFilter filter;
     CellUse *cellUse = scx->scx_use;
@@ -1433,6 +1498,7 @@ struct scaleArg {
    int scaled;
    int pnum;
    Plane *ptarget;
+   bool doCIF;
 };
 
 /*
@@ -1447,18 +1513,20 @@ struct scaleArg {
  */
 
 void
-dbScalePlane(oldplane, newplane, pnum, scalen, scaled)
+dbScalePlane(oldplane, newplane, pnum, scalen, scaled, doCIF)
     Plane *oldplane, *newplane;
     int pnum;
     int scalen, scaled;
+    bool doCIF;
 {
-    int dbTileScaleFunc();
+    int dbTileScaleFunc();		/* forward declaration */
     struct scaleArg arg;
 
     arg.scalen = scalen;
     arg.scaled = scaled;
     arg.ptarget = newplane;
     arg.pnum = pnum;
+    arg.doCIF = doCIF;
     (void) DBSrPaintArea((Tile *) NULL, oldplane, &TiPlaneRect,
 		&DBAllButSpaceBits, dbTileScaleFunc, (ClientData) &arg);
 }
@@ -1506,10 +1574,14 @@ dbTileScaleFunc(tile, scvals)
     if (IsSplit(tile))
 	type = (SplitSide(tile)) ? SplitRightType(tile) : SplitLeftType(tile);
     DBNMPaintPlane(scvals->ptarget, exact, &targetRect,
-		DBStdPaintTbl(type, scvals->pnum), (PaintUndoInfo *)NULL);
+		((scvals->doCIF) ? CIFPaintTable :
+		DBStdPaintTbl(type, scvals->pnum)),
+		(PaintUndoInfo *)NULL);
 #else
     DBPaintPlane(scvals->ptarget, &targetRect,
-		DBStdPaintTbl(type, scvals->pnum), (PaintUndoInfo *)NULL);
+		((scvals->doCIF) ? CIFPaintTable :
+		DBStdPaintTbl(type, scvals->pnum)),
+		(PaintUndoInfo *)NULL);
 #endif
     return 0;
 }
@@ -1618,7 +1690,8 @@ dbScaleCell(cellDef, scalen, scaled)
 	if (cellDef->cd_planes[pNum] == NULL) continue;
 	newplane = DBNewPlane((ClientData) TT_SPACE);
 	DBClearPaintPlane(newplane);
-	dbScalePlane(cellDef->cd_planes[pNum], newplane, pNum, scalen, scaled);
+	dbScalePlane(cellDef->cd_planes[pNum], newplane, pNum,
+		scalen, scaled, FALSE);
         DBFreePaintPlane(cellDef->cd_planes[pNum]);
 	TiFreePlane(cellDef->cd_planes[pNum]);
         cellDef->cd_planes[pNum] = newplane;
