@@ -50,7 +50,7 @@ static char rcsid[] = "$Header: /ufs/repository/magic/database/DBconnect.c,v 1.7
 struct conSrArg
 {
     CellDef *csa_def;			/* Definition being searched. */
-    Plane *csa_plane;			/* Current plane being searched. */
+    int csa_plane;			/* Index of current plane being searched. */
     TileTypeBitMask *csa_connect;	/* Table indicating what connects
 					 * to what.
 					 */
@@ -71,6 +71,9 @@ typedef struct
 {
     Rect		area;		/* Area to process */
     TileTypeBitMask	*connectMask;	/* Connection mask for search */
+#ifdef NONMANHATTAN
+    TileType		dinfo;		/* Info about triangular search areas */
+#endif
 } conSrArea;
 
 struct conSrArg2
@@ -90,6 +93,50 @@ struct conSrArg2
 
 #define CSA2_LIST_START_SIZE 256
 
+#ifdef NONMANHATTAN
+
+/*
+ *-----------------------------------------------------------------
+ * DBTransformDiagonal --
+ *
+ *	Resolve geometric transformations on diagonally-split tiles
+ *	Assumes that we have already determined that this tile is
+ *	split.
+ *
+ * Results:
+ *	A tile type containing embedded diagonal and side information.
+ *	Note that this tile type does NOT contain any actual type
+ *	information.
+ *
+ * Side Effects:
+ *	None.
+ *-----------------------------------------------------------------
+ */
+
+TileType
+DBTransformDiagonal(oldtype, trans)
+    TileType oldtype;
+    Transform *trans;
+{
+    TileType dinfo;
+    int o1, o2, o3, dir, side;
+	
+    o1 = ((trans->t_e > 0) || (trans->t_d > 0)) ? 1 : 0;
+    o2 = ((trans->t_a > 0) || (trans->t_b > 0)) ? 1 : 0;
+    o3 = (trans->t_a != 0) ? 1 : 0;
+    dir = (oldtype & TT_DIRECTION) ? 1 : 0;
+    side = ((oldtype & TT_SIDE) ? 1 : 0) ^ o2 ^ (dir | o3);
+    dir ^= o1 ^ o2;
+
+    dinfo = TT_DIAGONAL;
+    if (side) dinfo |= TT_SIDE;
+    if (dir) dinfo |= TT_DIRECTION;
+
+    return dinfo;
+}
+
+#endif
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -188,7 +235,7 @@ DBSrConnect(def, startArea, mask, connect, bounds, func, clientData)
     csa.csa_clientData = clientData;
     csa.csa_clear = FALSE;
     csa.csa_connect = connect;
-    csa.csa_plane = def->cd_planes[startPlane];
+    csa.csa_plane = startPlane;
     if (dbSrConnectFunc(startTile, &csa) != 0) result = 1;
 
     /* Pass 2.  Don't call any client function, just clear the marks.
@@ -198,7 +245,7 @@ DBSrConnect(def, startArea, mask, connect, bounds, func, clientData)
     SigDisableInterrupts();
     csa.csa_clientFunc = NULL;
     csa.csa_clear = TRUE;
-    csa.csa_plane = def->cd_planes[startPlane];
+    csa.csa_plane = startPlane;
     (void) dbSrConnectFunc(startTile, &csa);
     SigEnableInterrupts();
 
@@ -281,7 +328,7 @@ dbSrConnectFunc(tile, csa)
 
     if (csa->csa_clientFunc != NULL)
     {
-	if ((*csa->csa_clientFunc)(tile, csa->csa_clientData) != 0)
+	if ((*csa->csa_clientFunc)(tile, csa->csa_plane, csa->csa_clientData) != 0)
 	    return 1;
     }
 
@@ -434,10 +481,20 @@ donesides:
 	GEO_EXPAND(&newArea, 1, &newArea);
 	for (i = PL_TECHDEPBASE; i < DBNumPlanes; i++)
 	{
-	    newcsa.csa_plane = newcsa.csa_def->cd_planes[i];
-	    if (DBSrPaintArea((Tile *) NULL, newcsa.csa_plane,
-		&newArea, connectMask, dbSrConnectFunc,
-		(ClientData) &newcsa) != 0) return 1;
+	    newcsa.csa_plane = i;
+#ifdef NONMANHATTAN
+	    if (IsSplit(tile))
+	    {
+		if (DBSrPaintNMArea((Tile *) NULL, csa->csa_def->cd_planes[i],
+			TiGetTypeExact(tile), &newArea, connectMask,
+			dbSrConnectFunc, (ClientData) &newcsa) != 0)
+		    return 1;
+	    }
+	    else
+#endif
+	    if (DBSrPaintArea((Tile *) NULL, csa->csa_def->cd_planes[i],
+			&newArea, connectMask, dbSrConnectFunc,
+			(ClientData) &newcsa) != 0) return 1;
 	}
     }
 
@@ -467,26 +524,11 @@ donesides:
  */
 
 int
-dbcUnconnectFunc(tile, connectMask)
-    Tile *tile;			    /* Current tile. */
-    TileTypeBitMask *connectMask;
+dbcUnconnectFunc(tile, clientData)
+    Tile *tile;				/* Current tile	*/
+    ClientData clientData;		/* Unused.	*/
     
 {
-#ifdef NONMANHATTAN
-    TileType loctype;
-
-    /* We want to ignore those split tiles whose opposite corner contains */
-    /* the correct type;  otherwise, we will drop into an infinite loop	  */
-    /* selecting and unselecting the same tile.				  */
-
-    if (IsSplit(tile))
-    {
-	loctype = (SplitSide(tile)) ? SplitLeftType(tile) : SplitRightType(tile);
-	if (TTMaskHasType(connectMask, loctype))
-	    return 0;
-    }
-#endif
-
     return 1;
 }
 
@@ -592,7 +634,10 @@ dbcConnectFunc(tile, cx)
     Rect *srArea;
     SearchContext *scx = cx->tc_scx;
     SearchContext scx2;
-    TileType newtype, loctype;
+    TileType loctype = TiGetTypeExact(tile);
+#ifdef NONMANHATTAN
+    TileType dinfo = 0;
+#endif
     int pNum;
     CellDef *def;
 
@@ -623,37 +668,12 @@ dbcConnectFunc(tile, cx)
     if (GEO_RECTNULL(&newarea)) return 0;
 
 #ifdef NONMANHATTAN
-    /* Resolve geometric transformations on diagonally-split tiles */
-
     if (IsSplit(tile))
     {
-	int o1, o2, o3, dir, side;
-	
-	o1 = ((scx->scx_trans.t_e > 0) || (scx->scx_trans.t_d > 0)) ? 1 : 0;
-	o2 = ((scx->scx_trans.t_a > 0) || (scx->scx_trans.t_b > 0)) ? 1 : 0;
-	o3 = (scx->scx_trans.t_a != 0) ? 1 : 0;
-	dir = SplitDirection(tile) ^ o1 ^ o2;
-	side = SplitSide(tile) ^ o2 ^ (SplitDirection(tile) | o3);
-
-	if (side == SplitSide(tile))
-	    newtype = TT_DIAGONAL | SplitLeftType(tile) |
-			(SplitRightType(tile) << 14);
-	else
-	    newtype = TT_DIAGONAL | SplitRightType(tile) |
-			(SplitLeftType(tile) << 14);
-
-	if (dir) newtype |= TT_DIRECTION;
-	if (side)
-	{
-	    newtype |= TT_SIDE;
-	    loctype = (newtype & TT_RIGHTMASK) >> 14;
-	}
-	else
-	    loctype = newtype & TT_LEFTMASK;
+	dinfo = DBTransformDiagonal(loctype, &scx->scx_trans);
+	loctype = (SplitSide(tile)) ? SplitRightType(tile) : SplitLeftType(tile);
     }
-    else
 #endif
-    loctype = newtype = TiGetTypeExact(tile);
 
     /* See if the destination cell contains stuff over the whole
      * current area (on its home plane) that is connected to it.
@@ -671,13 +691,20 @@ dbcConnectFunc(tile, cx)
     if (DBIsContact(loctype))
     {
 	TileType ctype;
+	TileTypeBitMask *rMask = DBResidueMask(loctype);
 
 	TTMaskSetOnlyType(&notConnectMask, loctype);
 
+	/* Differenct contact types may share residues (6/18/04) */
+	for (ctype = TT_TECHDEPBASE; ctype < DBNumUserLayers; ctype++)
+	    if (TTMaskEqual(rMask, DBResidueMask(ctype)))
+		TTMaskSetType(&notConnectMask, ctype);
+
 	/* The mask of contact types must include all stacked contacts */
 	for (ctype = DBNumUserLayers; ctype < DBNumTypes; ctype++)
-	    if TTMaskHasType(DBResidueMask(ctype), loctype)
+	    if (TTMaskHasType(DBResidueMask(ctype), loctype))
 		TTMaskSetType(&notConnectMask, ctype);
+
 	TTMaskCom(&notConnectMask);
     }
     else
@@ -689,10 +716,17 @@ dbcConnectFunc(tile, cx)
     /* which have not already been painted.			*/
 
     def = csa2->csa2_use->cu_def;
+#ifdef NONMANHATTAN
+    if (DBSrPaintNMArea((Tile *) NULL, def->cd_planes[pNum],
+		dinfo, &newarea, &notConnectMask, dbcUnconnectFunc,
+		(ClientData) NULL) == 0)
+	return 0;
+#else
     if (DBSrPaintArea((Tile *) NULL, def->cd_planes[pNum],
 		&newarea, &notConnectMask, dbcUnconnectFunc,
-		(ClientData) connectMask) == 0)
+		(ClientData) NULL) == 0)
 	return 0;
+#endif
 
     /* Paint this tile into the destination cell.  This
      * marks its area has having been processed.  Then recycle
@@ -700,7 +734,7 @@ dbcConnectFunc(tile, cx)
      */
 
 #ifdef NONMANHATTAN
-    DBNMPaintPlane(def->cd_planes[pNum], newtype,
+    DBNMPaintPlane(def->cd_planes[pNum], dinfo,
 		&newarea, DBStdPaintTbl(loctype, pNum),
 		(PaintUndoInfo *) NULL);
 #else
@@ -730,14 +764,14 @@ dbcConnectFunc(tile, cx)
 #ifdef NONMANHATTAN
     /* Only extend those sides bordering the diagonal tile */
 	  
-    if (newtype & TT_DIAGONAL)
+    if (dinfo & TT_DIAGONAL)
     {
-	if (newtype & TT_SIDE)			/* right */
+	if (dinfo & TT_SIDE)			/* right */
 	    newarea.r_xtop += 1;
 	else					/* left */
 	    newarea.r_xbot -= 1;
-	if (((newtype & TT_SIDE) >> 1)
-		== (newtype & TT_DIRECTION))	/* top */
+	if (((dinfo & TT_SIDE) >> 1)
+		== (dinfo & TT_DIRECTION))	/* top */
 	    newarea.r_ytop += 1;
 	else					/* bottom */
 	    newarea.r_ybot -= 1;
@@ -770,6 +804,9 @@ dbcConnectFunc(tile, cx)
 	{
 	    newlist[i].area = csa2->csa2_list[i].area;
 	    newlist[i].connectMask = csa2->csa2_list[i].connectMask;
+#ifdef NONMANHATTAN
+	    newlist[i].dinfo = csa2->csa2_list[i].dinfo;
+#endif
 	    
 	}
 	freeMagic((char *)csa2->csa2_list);
@@ -778,6 +815,9 @@ dbcConnectFunc(tile, cx)
 
     csa2->csa2_list[csa2->csa2_top].area = newarea;
     csa2->csa2_list[csa2->csa2_top].connectMask = connectMask;
+#ifdef NONMANHATTAN
+    csa2->csa2_list[csa2->csa2_top].dinfo = dinfo;
+#endif
 
     return 0;
 }
@@ -840,6 +880,9 @@ DBTreeCopyConnect(scx, mask, xMask, connect, area, destUse)
 {
     struct conSrArg2 csa2;
     TileTypeBitMask *newmask;
+#ifdef NONMANHATTAN
+    TileType newtype;
+#endif
 
     csa2.csa2_use = destUse;
     csa2.csa2_xMask = xMask;
@@ -861,7 +904,17 @@ DBTreeCopyConnect(scx, mask, xMask, connect, area, destUse)
     {
 	newmask = csa2.csa2_list[csa2.csa2_top].connectMask;
 	scx->scx_area = csa2.csa2_list[csa2.csa2_top].area;
+#ifdef NONMANHATTAN
+	newtype = csa2.csa2_list[csa2.csa2_top].dinfo;
+#endif
 	csa2.csa2_top--;
+
+#ifdef NONMANHATTAN
+	if (newtype & TT_DIAGONAL)
+	    DBTreeSrNMTiles(scx, newtype, newmask, xMask, dbcConnectFunc,
+			(ClientData) &csa2);
+	else
+#endif
 	DBTreeSrTiles(scx, newmask, xMask, dbcConnectFunc, (ClientData) &csa2);
     }
     freeMagic((char *)csa2.csa2_list);
