@@ -64,6 +64,10 @@ bool DBVerbose = TRUE;
 char *dbFgets();
 FILE *dbReadOpen();
 int DBFileOffset;
+bool dbReadLabels();
+bool dbReadElements();
+bool dbReadProperties();
+bool dbReadUse();
 
 /*
  * ----------------------------------------------------------------------------
@@ -228,15 +232,19 @@ file_is_not_writeable(name)
  */
 
 bool
-DBCellRead(cellDef, name, setFileName)
+DBCellRead(cellDef, name, ignoreTech)
     CellDef *cellDef;	/* Pointer to definition of cell to be read in */
     char *name;		/* Name of file from which to read definition.
 			 * If NULL, then use cellDef->cd_file; if that
 			 * is NULL try the name of the cell.
 			 */
-    bool setFileName;	/* If TRUE then cellDef->cd_file should be updated
-			 * to point to the name of the file from which the
-			 * cell was loaded.
+    bool ignoreTech;	/* If FALSE then the technology of the file MUST
+			 * match the current technology, or else the
+			 * subroutine will return an error condition
+			 * without reading anything.  If TRUE, a
+			 * warning will be printed if the technology
+			 * names do not match, but an attempt will be
+			 * made to read the file anyway.
 			 */
 {
     int cellStamp = 0, rectCount = 0, rectReport = 10000;
@@ -246,7 +254,8 @@ DBCellRead(cellDef, name, setFileName)
     FILE *f;
     Rect *rp;
     int c;
-    TileType type;
+    TileType type, rtype;
+    TileTypeBitMask *rmask;
     Plane *plane;
     Rect r;
     int n = 1, d = 1;
@@ -254,7 +263,7 @@ DBCellRead(cellDef, name, setFileName)
     if (cellDef->cd_flags & CDAVAILABLE)
 	return (TRUE);
 
-    if ((f = dbReadOpen(cellDef, name, setFileName)) == NULL)
+    if ((f = dbReadOpen(cellDef, name, TRUE)) == NULL)
 	return (FALSE);
 
     /*
@@ -291,16 +300,26 @@ DBCellRead(cellDef, name, setFileName)
 	}
 	if (strcmp(DBTechName, tech) != 0)
 	{
-	    TxError("Cell %s has wrong technology %s\n", cellDef->cd_name,
-			tech);
-	    TxError("Use command \"load tech\" if you want to switch technologies.\n");
-	    (void) fclose(f);
+	    TxError("Cell %s has technology \"%s\", but current "
+			"technology is \"%s\"\n", cellDef->cd_name,
+			tech, DBTechName);
+	    if (ignoreTech)
+		TxPrintf("Will attempt to read cell anyway.\n");
+	    else
+	    {
+		TxError("Use command \"tech load\" if you want to switch"
+			" technologies, or use\n");
+		TxError("\"cellname delete %s\" and \"load %s -force\" to"
+			" force the cell to load as technology %s\n",
+			cellDef->cd_name, cellDef->cd_name, DBTechName);
+		(void) fclose(f);
 #ifdef FILE_LOCKS
-	    flock_close(f, cellDef, name);
+		flock_close(f, cellDef, name);
 #endif
-	    f = NULL;
-	    SigEnableInterrupts();
-	    return (FALSE);
+		f = NULL;
+		SigEnableInterrupts();
+		return (FALSE);
+	    }
 	}
 	if (dbFgets(line, sizeof line, f) == NULL)
 	    goto badfile;
@@ -345,7 +364,7 @@ DBCellRead(cellDef, name, setFileName)
     ReduceFraction(&n, &d);
     if (d > 1)
     {
-	CIFTechInputScale(1, d);
+	CIFTechInputScale(1, d, TRUE);
 	CIFTechOutputScale(1, d);
 	DRCTechScale(1, d);
 	ExtTechScale(1, d);
@@ -415,19 +434,23 @@ DBCellRead(cellDef, name, setFileName)
 			cellDef->cd_name);
 	}
 
-	/* Encapsulates painting information for DBPaintPlane */
-	if ((type > 0) && (DBPlane(type) > 0))
-	{
-	    ptable = DBStdPaintTbl(type, DBPlane(type));
-	    plane = cellDef->cd_planes[DBPlane(type)];
-
-	    /*
-	     * Record presence of material in cell.
-	     */
+	/*
+	 * Record presence of material in cell.
+	 */
 	    
-	    TTMaskSetType(&cellDef->cd_types, type);
+	if (DBPlane(type) > 0)
+	{
+	    if (type < DBNumUserLayers)
+		TTMaskSetType(&cellDef->cd_types, type);
+	    else
+	    {
+		/* Separate stacked contact types into their components */
+	        rmask = DBResidueMask(type);
+		for (rtype = TT_SPACE + 1; rtype < DBNumUserLayers; rtype++)
+		    if (TTMaskHasType(rmask, rtype))
+			TTMaskSetType(&cellDef->cd_types, type);
+	    }
 	}
-	else plane = NULL;
 
 	/*
 	 * The following loop is executed once for each line
@@ -452,6 +475,9 @@ nextrect:
 #ifdef NONMANHATTAN  
 		if ((dir = GetRect(f, 3, rp, n)) == 0) goto badfile;
 		dir >>= 1;
+		dinfo = TT_DIAGONAL | ((dir & 0x2) ? TT_SIDE : 0) |
+			((((dir & 0x2) >> 1) ^ (dir & 0x1)) ?
+			TT_DIRECTION : 0);
 #else
 		if (!GetRect(f, 3, rp, n)) goto badfile;
 		TxPrintf("Error: cell %s contains non-manhattan geometry\n",
@@ -462,7 +488,13 @@ nextrect:
 #endif
 	    }
             else  
-	    if (!GetRect(f, 4, rp, n)) goto badfile;
+	    {
+#ifdef NONMANHATTAN  
+		dinfo = 0;
+#endif
+		if (!GetRect(f, 4, rp, n)) goto badfile;
+	    }
+
 	    if ((++rectCount % rectReport == 0) && DBVerbose)
 	    {
 		TxPrintf("%s: %d rects\n", cellDef->cd_name, rectCount);
@@ -473,48 +505,46 @@ nextrect:
 	     * Only add a new rectangle if it is non-null, and if the
 	     * layer is reasonable.
 	     */
-	    if (!GEO_RECTNULL(rp) && (plane != NULL))
+	    if (!GEO_RECTNULL(rp))
 	    {
-		/*
-		 * We use DBPaintPlane, so inter-plane effects of painting
-		 * tiles do not occur during read-in.  For files generated
-		 * by Magic itself, this is not a problem; other clients
-		 * should beware.
-		 */
-#ifdef NONMANHATTAN
-		if (c == 't')
+		/*------------------------------------------------------*/
+		/* The complicated use of DBPaintPlane() has been	*/
+		/* replaced with a simpler call to DBPaint().  HOWEVER	*/
+		/* there are instances where this can cause unexpected	*/
+		/* behavior.  Namely, magic-7.2 allows, e.g., m456c	*/
+		/* painted over m3c;  this apparently works although	*/
+		/* image on the metal4 plane cannot represent both	*/
+		/* contact types.  magic-7.3 disallows this, so for the	*/
+		/* same technology file (no stackable types), m3c gets	*/
+		/* eliminated by the paint rules!  This condition is	*/
+		/* left AS-IS.  Caveat end-user.			*/
+		/*				   	Tim 7/8/04	*/
+		/*------------------------------------------------------*/
+
+		TileType loctype;
+
+		if (type >= DBNumUserLayers)
 		{
-		    dinfo = TT_DIAGONAL | ((dir & 0x2) ? TT_SIDE : 0) |
-				((((dir & 0x2) >> 1) ^ (dir & 0x1)) ?
-				TT_DIRECTION : 0);
-		    DBNMPaintPlane(plane, dinfo, rp, ptable,
-				(PaintUndoInfo *) NULL);
+		    rmask = DBResidueMask(type);
+		    for (rtype = TT_SPACE + 1; rtype < DBNumUserLayers; rtype++)
+			if (TTMaskHasType(rmask, rtype))
+			{
+			    loctype = rtype;
+#ifdef NONMANHATTAN
+			    if (dinfo & TT_SIDE) loctype <<= 14;
+			    loctype |= dinfo;
+#endif
+			    DBPaint(cellDef, rp, loctype);
+			}
 		}
 		else
-#endif
-		DBPaintPlane(plane, rp, ptable, (PaintUndoInfo *) NULL);
-
-		/* If a contact, paint it on all connected planes */
-		if (DBConnPlanes[type])
 		{
-		    int pNum;
-
-		    for (pNum = PL_TECHDEPBASE; pNum < DBNumPlanes; pNum++)
-			if (PlaneMaskHasPlane(DBConnPlanes[type], pNum))
-			{
+		    loctype = type;
 #ifdef NONMANHATTAN
-			    if (c == 't')
-			    {
-				DBNMPaintPlane(cellDef->cd_planes[pNum],
-					dinfo, rp,  DBStdPaintTbl(type, pNum),
-					(PaintUndoInfo *) NULL);
-			    }
-			    else
+		    if (dinfo & TT_SIDE) loctype <<= 14;
+		    loctype |= dinfo;
 #endif
-			    DBPaintPlane(cellDef->cd_planes[pNum], rp,
-				    DBStdPaintTbl(type, pNum),
-				    (PaintUndoInfo *) NULL);
-			}
+		    DBPaint(cellDef, rp, loctype);
 		}
 	    }
 	}
@@ -671,7 +701,7 @@ dbReadOpen(cellDef, name, setFileName)
 	else {
 	    TxError("Cell %s couldn't be found\n", cellDef->cd_name);
 #ifdef FILE_LOCKS
-	    MALLOC(char *, realname, strlen(cellDef->cd_name) + strlen(DBSuffix) + 1);
+	    realname = (char *) mallocMagic((unsigned) (strlen(cellDef->cd_name) + strlen(DBSuffix) + 1));
 	    (void) sprintf(realname, "%s%s", cellDef->cd_name, DBSuffix);
 	    cellDef->cd_file = StrDup(&cellDef->cd_file, realname);
 #endif
@@ -1108,8 +1138,8 @@ dbReadElements(cellDef, line, len, f, scale)
 	    *textend = '"';
 	    ntok += sscanf(textend + 1, "%99[^\n]", flags);
 	    ntok -= 4;
-	    DBWElementParseFlags(NULL, elementname, "persistent");
 	}
+	DBWElementParseFlags(NULL, elementname, "persistent");
 
 	/* Set the style(s) for this element */
 	tstr = styles;
@@ -1326,7 +1356,19 @@ dbReadLabels(cellDef, line, len, f, scale)
 Warning: label \"%s\" attached to unknown type \"%s\"\n", text, layername);
 	    type = TT_SPACE;
 	}
+	else if (type >= DBNumUserLayers)
+	{
+	    TileTypeBitMask *rmask;
+	    TileType rtype;
 
+	    /* Don't stick labels on stacked types; choose the	*/
+	    /* topmost (last, usually) residue contact type.	*/
+
+	    rmask = DBResidueMask(type);
+	    for (rtype = TT_SPACE + 1; rtype < DBNumUserLayers; rtype++)
+		if (TTMaskHasType(rmask, rtype))
+		    type = rtype;
+	}
 	(void) DBPutLabel(cellDef, &r, orient, text, type, 0);
 
 nextlabel:
@@ -1722,7 +1764,7 @@ DBCellWriteFile(cellDef, f)
     {
 	FPRINTF(f, "<< elements >>\n");
 	FPRINTF(f, estring);
-	FREE(estring);
+	freeMagic(estring);
     }
 
     /* And any properties */
@@ -1849,7 +1891,7 @@ DBCellWrite(cellDef, fileName)
      */
     if (fileName)
     {
-	MALLOC(char *, realname, strlen(fileName)+strlen(DBSuffix)+1);
+	realname = (char *) mallocMagic((unsigned) (strlen(fileName)+strlen(DBSuffix)+1));
 	(void) sprintf(realname, "%s%s", fileName, DBSuffix);
 	/* Bug fix: 7/17/99, Michael D. Godfrey:  Forces		*/
 	/* cd_name and cd_file to ALWAYS be the same, otherwise ugly	*/
@@ -1863,7 +1905,7 @@ DBCellWrite(cellDef, fileName)
     }
     else if (cellDef->cd_name)
     {
-	MALLOC(char *, realname, strlen(cellDef->cd_name)+strlen(DBSuffix)+1);
+	realname = (char *) mallocMagic((unsigned) (strlen(cellDef->cd_name)+strlen(DBSuffix)+1));
 	(void) sprintf(realname, "%s%s", cellDef->cd_name, DBSuffix);
     }
     else return (FALSE);
@@ -1898,7 +1940,7 @@ DBCellWrite(cellDef, fileName)
     /*
      * Figure name of the temp file to write.
      */
-    MALLOC(char *, tmpname, strlen(expandname) + strlen(template) + 1);
+    tmpname = (char *) mallocMagic((unsigned) (strlen(expandname) + strlen(template) + 1));
     (void) sprintf(tmpname, "%s%s", expandname, template);
     tmpres = mkstemp(tmpname);
 
@@ -2036,8 +2078,8 @@ to a safe place before executing any more Magic commands.\n",
 
 cleanup:
     SigEnableInterrupts();
-    FREE(realname);
-    FREE(tmpname);
+    freeMagic(realname);
+    freeMagic(tmpname);
     return result;
 }
 
