@@ -48,27 +48,23 @@ struct nldata
 };
 
 /*
- * The following statically allocated variables are used to
- * pass the arguments of DBSearchLabel down to the filter
- * function.  We use statics rather than a client procedure
- * becuase re_exec was initially used for pattern matching, and
- * it already kept information statically.
+ * The following structure is used to pass the arguments of
+ * DBSearchLabel down to the filter function.
  */
 
 #define	MAXLABPATHSIZE	256
 
-int (*labSrFunc)();		/* Function to apply to each label found */
-ClientData labSrArg;		/* Client data of caller */
-char labSrStr[MAXLABPATHSIZE];	/* String buffer in which the full pathname
-				 * of each label is assembled for handing
-				 * to the filter function.
-				 */
-char *labSrPattern;		/* Pattern for matching. */
+typedef struct {
+    char *labSrPattern;		/* Pattern for matching. */
+    int (*labSrFunc)();		/* Function to apply to each label found */
+    ClientData labSrArg;	/* Client data of caller */
+} labSrStruct;
 
 /* Forward declarations */
 
 extern void DBTreeFindUse();
 
+bool dbParseArray();
 
 /*
  * ----------------------------------------------------------------------------
@@ -128,14 +124,20 @@ DBSearchLabel(scx, mask, xMask, pattern, func, cdarg)
 {
     TerminalPath tpath;
     int dbSrLabelFunc();
+    labSrStruct labsr;
+    char labSrStr[MAXLABPATHSIZE];	/* String buffer in which the full pathname
+				 	* of each label is assembled for handing
+				 	* to the filter function.
+				 	*/
 
-    labSrPattern = pattern;
-    labSrFunc = func;
-    labSrArg = cdarg;
+    labsr.labSrPattern = pattern;
+    labsr.labSrFunc = func;
+    labsr.labSrArg = cdarg;
     tpath.tp_first = tpath.tp_next = labSrStr;
-    tpath.tp_last = &labSrStr[sizeof labSrStr - 2];
+    tpath.tp_last = &(labSrStr[sizeof(labSrStr) - 2]);
 
-    if (DBTreeSrLabels(scx, mask, xMask, &tpath, dbSrLabelFunc, (ClientData) 0))
+    if (DBTreeSrLabels(scx, mask, xMask, &tpath, dbSrLabelFunc,
+		(ClientData) &labsr))
 	return 1;
     else return 0;
 }
@@ -159,16 +161,17 @@ DBSearchLabel(scx, mask, xMask, pattern, func, cdarg)
  */
 
 int
-dbSrLabelFunc(scx, label, tpath)
+dbSrLabelFunc(scx, label, tpath, labsr)
     SearchContext *scx;		/* Contains pointer to use in which label
 				 * occurred, and transform back to root
 				 * coordinates.
 				 */
     Label *label;		/* Label itself */
     TerminalPath *tpath;	/* Full pathname of the terminal */
+    labSrStruct *labsr;		/* Information passed to this routine */
 {
-    if (Match(labSrPattern, label->lab_text))
-	if ((*labSrFunc)(scx, label, tpath, labSrArg))
+    if (Match(labsr->labSrPattern, label->lab_text))
+	if ((*labsr->labSrFunc)(scx, label, tpath, labsr->labSrArg))
 	    return 1;
     return 0;
 }
@@ -506,7 +509,7 @@ DBNearestLabel(cellUse, area, point, xMask, labelArea, labelName, length)
     if (labelName == NULL) tp = NULL, name = NULL;
     else
     {
-	MALLOC(char *, name, length);
+	name = (char *) mallocMagic((unsigned) (length));
 	tPath.tp_first = name;
 	tPath.tp_next = name;
 	tPath.tp_last = name + length - 1;
@@ -524,7 +527,7 @@ DBNearestLabel(cellUse, area, point, xMask, labelArea, labelName, length)
     (void) DBTreeSrLabels(&scx, &DBAllTypeBits, xMask, tp, dbNearestLabelFunc,
 	(ClientData) &funcData);
 
-    if (name) FREE(name);
+    if (name) freeMagic(name);
     
     if (!funcData.nld_gotLabel) return FALSE;
     else return TRUE;
