@@ -94,7 +94,7 @@ int dbPaintDebug = 0;
 	Tile *xtile = otile, *xxnew, *xp; \
 	int x = xcoord; \
  \
-	MALLOC(Tile *, xxnew, sizeof (Tile)); \
+	xxnew = (Tile *) mallocMagic((unsigned) (sizeof (Tile))); \
 	xxnew->ti_client = (ClientData) MINFINITY; \
  \
 	LEFT(xxnew) = x, BOTTOM(xxnew) = BOTTOM(xtile); \
@@ -148,8 +148,7 @@ int dbPaintDebug = 0;
  * found while enumerating 'area', that gives the result type for this
  * operation.  The semantics of painting, erasing, and "writing" (storing
  * a new type in the area without regard to the previous contents) are
- * all encapsulated in this table.  (Note:  There is now a procedure called
- * DBPaintPlaneByProc for those cases where a table doesn't suffice.)
+ * all encapsulated in this table.
  *
  * If undo is desired, 'undo' should point to a PaintUndoInfo struct
  * that contains everything needed to build an undo record.  Otherwise,
@@ -263,8 +262,18 @@ enumerate:
 	else
 #endif
 	newType = resultTbl[oldType];
+
+//	/* Sounded good, but violates maximum horiz. strips!	*/
+//	/* Beware violating maximum horiz. strips property	*/
+//	/* unless you want to change the code in a lot of	*/
+//	/* places. . .						*/
+//
+//	if (oldType == newType)
+//	     goto paintdone; /* nothing to do */
+
 	if (oldType != newType)
 	{
+
 	    /*
 	     * Clip the tile against the clipping rectangle.
 	     * Merging is only necessary if we clip to the left or to
@@ -505,428 +514,6 @@ enumerate:
 #endif	/* PAINTDEBUG */
 	}
 
-
-	/***
-	 ***		END OF PAINT CODE
-	 *** ---------- BACK TO AREA SEARCH ----------
-	 ***/
-paintdone:
-	/* Move right if possible */
-	tpnew = TR(tile);
-	if (LEFT(tpnew) < area->r_xtop)
-	{
-	    /* Move back down into clipping area if necessary */
-	    while (BOTTOM(tpnew) >= clipTop) tpnew = LB(tpnew);
-	    if (BOTTOM(tpnew) >= BOTTOM(tile) || BOTTOM(tile) <= area->r_ybot)
-	    {
-		tile = tpnew;
-		goto enumerate;
-	    }
-	}
-
-	/* Each iteration returns one tile further to the left */
-	while (LEFT(tile) > area->r_xbot)
-	{
-	    /* Move left if necessary */
-	    if (BOTTOM(tile) <= area->r_ybot)
-		goto done;
-
-	    /* Move down if possible; left otherwise */
-	    tpnew = LB(tile); tile = BL(tile);
-	    if (BOTTOM(tpnew) >= BOTTOM(tile) || BOTTOM(tile) <= area->r_ybot)
-	    {
-		tile = tpnew;
-		goto enumerate;
-	    }
-	}
-	/* At left edge -- walk down to next tile along the left edge */
-	for (tile = LB(tile); RIGHT(tile) <= area->r_xbot; tile = TR(tile))
-	    /* Nothing */;
-    }
-
-done:
-    plane->pl_hint = tile;
-}
-
-/*
- * ----------------------------------------------------------------------------
- *
- * DBPaintPlaneByProc --
- *
- * Just like "DBPaintPlane", but it calls a client-supplied procedure to
- * compute the new tile type instead of looking it up in a table.
- *
- * See "DBPaintPlane" for more details on how this works.
- *
- * Results:
- *	None.
- *
- * Side effects:
- *	Modifies the database plane that contains the given tile.
- *
- * IMPLEMENTATION NOTES:
- *	1) This procedure is identical to "DBPaintPlane" except for the arguments
- *	   and the line marked "CHANGED".  Be sure to keep the two procedures in 
- *	   sync when updating code!
- *	2) Use of this function for non-manhattan geometry (NONMANHATTAN defined)
- *	   is superceded by DBNMPaintPlane(), as that function determines if
- *	   a diagonal is being painted, and may split the area to be painted
- *	   before calling DBPaintPlaneByProc() or DBPaintPlane() on each sub-
- *	   area, as required.
- *
- * REMINDER:
- *	Callers should remember to set the CDMODIFIED and CDGETNEWSTAMP
- *	bits in the cell definition containing the plane being painted.
- *
- * ----------------------------------------------------------------------------
- */
-
-void
-DBPaintPlaneByProc(plane, area, proc, parg, undo)
-    Plane *plane;		/* Plane whose paint is to be modified */
-    Rect *area;	/* Area to be changed */
-    int (*proc)();		/* Proc to determine new type of material.
-				 * This proc should be of the form:
-				 *
-				 *    int 
-				 *    foo(oldtype, parg)
-				 *        int oldtype;
-				 *	  Clientdata parg;
-				 *    {
-				 *        return int;
-				 *    }
-				 *
-				 * Oldtype is the type of material that
-				 * is already there, and the proc should
-				 * return the type that is to replace it.
-				 * "parg" is used to pass other information
-				 * to the procedure.
-				 */
-    ClientData parg;
-    PaintUndoInfo *undo;	/* Record containing everything needed to
-				 * save undo entries for this operation.
-				 * If NULL, the undo package is not used.
-				 */
-
-{
-    Point start;
-    int clipTop, mergeFlags;
-    TileType oldType, newType;
-    Tile *tile, *newtile;
-    Tile *tpnew;	/* Used for area search */
-    Tile *tp;		/* Used for paint */
-#ifdef NONMANHATTAN
-    bool haschanged;
-#endif
-
-    if (area->r_xtop <= area->r_xbot || area->r_ytop <= area->r_ybot)
-	return;
-
-    /*
-     * The following is a modified version of the area enumeration
-     * algorithm.  It expects the in-line paint code below to leave
-     * 'tile' pointing to the tile from which we should continue the
-     * search.
-     */
-
-    start.p_x = area->r_xbot;
-    start.p_y = area->r_ytop - 1;
-    tile = plane->pl_hint;
-    GOTOPOINT(tile, &start);
-
-    /* Each iteration visits another tile on the LHS of the search area */
-    while (TOP(tile) > area->r_ybot)
-    {
-	/***
-	 *** AREA SEARCH.
-	 *** Each iteration enumerates another tile.
-	 ***/
-enumerate:
-	if (SigInterruptPending)
-	    break;
-
-	clipTop = TOP(tile);
-	if (clipTop > area->r_ytop) clipTop = area->r_ytop;
-	oldType = TiGetTypeExact(tile);
-
-#ifdef	PAINTDEBUG
-	if (dbPaintDebug)
-	    dbPaintShowTile(tile, undo, "area enum");
-#endif	/* PAINTDEBUG */
-
-	/***
-	 *** ---------- THE FOLLOWING IS IN-LINE PAINT CODE ----------
-	 ***/
-
-	/*
-	 * Set up the directions in which we will have to
-	 * merge initially.  Clipping can cause some of these
-	 * to be turned off.
-	 */
-	mergeFlags = MRG_TOP | MRG_LEFT;
-	if (RIGHT(tile) >= area->r_xtop) mergeFlags |= MRG_RIGHT;
-	if (BOTTOM(tile) <= area->r_ybot) mergeFlags |= MRG_BOTTOM;
-
-	/*
-	 * The following is a kludge for plowing that should go away
-	 * once the plowing code gets stable.  Make sure that the intermediate
-	 * coordinate of this tile is reset to "uninitialized".
-	 */
-	tile->ti_client = (ClientData) MINFINITY;
-
-	/*
-	 * Determine new type of this tile.
-	 * Change the type if necessary.
-	 */
-
-#ifdef NONMANHATTAN
-	haschanged = FALSE;
-#endif
-	newType = (*proc)(oldType, parg);	/* CHANGED */
-
-	if (oldType != newType)
-	{
-	    /*
-	     * Clip the tile against the clipping rectangle.
-	     * Merging is only necessary if we clip to the left or to
-	     * the right, and then only to the top or the bottom.
-	     * We do the merge in-line for efficiency.
-	     */
-
-	    /* Clip up */
-	    if (TOP(tile) > area->r_ytop)
-	    {
-#ifdef NONMANHATTAN
-		if (IsSplit(tile))
-		{
-		    haschanged = TiNMSplitY(&tile, &newtile, area->r_ytop, 1, undo);
-		    if (!IsSplit(tile))
-		    {
-			oldType = TiGetTypeExact(tile);
-			newType = (*proc)(oldType, parg);
-		    }
-		}
-		else
-		{
-#endif
-		newtile = TiSplitY(tile, area->r_ytop);
-		TiSetBody(newtile, TiGetBody(tile));
-#ifdef NONMANHATTAN
-		}
-#endif
-		mergeFlags &= ~MRG_TOP;
-	    }
-
-#ifdef NONMANHATTAN
-	    /* Clipping diagonals can cause the new tile to no longer be */
-	    /* in the search path!					 */
-	    if (RIGHT(tile) <= area->r_xbot)
-		goto paintdone;
-#endif
-
-	    /* Clip down */
-	    if (BOTTOM(tile) < area->r_ybot)
-	    {
-#ifdef NONMANHATTAN
-		if (IsSplit(tile))
-		{
-		    haschanged = TiNMSplitY(&tile, &newtile, area->r_ybot, 0, undo);
-		    if (!IsSplit(tile))
-		    {
-			oldType = TiGetTypeExact(tile);
-			newType = (*proc)(oldType, parg);
-		    }
-		}
-		else
-		{
-#endif
-		newtile = tile, tile = TiSplitY(tile, area->r_ybot);
-		TiSetBody(tile, TiGetBody(newtile));
-#ifdef NONMANHATTAN
-		}
-#endif
-		mergeFlags &= ~MRG_BOTTOM;
-	    }
-
-#ifdef NONMANHATTAN
-	    /* Clipping diagonals can cause the new tile to no longer be */
-	    /* in the search path!					 */
-	    if (RIGHT(tile) <= area->r_xbot)
-		goto paintdone;
-#endif
-
-	    /* Clip right */
-	    if (RIGHT(tile) > area->r_xtop)
-	    {
-#ifdef NONMANHATTAN
-		if (IsSplit(tile))
-		{
-		    haschanged = TiNMSplitX(&tile, &newtile, area->r_xtop, 1, undo);
-		    if (!IsSplit(tile))
-		    {
-			oldType = TiGetTypeExact(tile);
-			newType = (*proc)(oldType, parg);
-		    }
-		}
-		else
-		{
-#endif
-		TISPLITX(newtile, tile, area->r_xtop);
-		TiSetBody(newtile, TiGetBody(tile));
-#ifdef NONMANHATTAN
-		}
-#endif
-		mergeFlags &= ~MRG_RIGHT;
-
-		/* Merge the outside tile to its top */
-		tp = RT(newtile);
-		if (CANMERGE_Y(newtile, tp)) TiJoinY(newtile, tp, plane);
-
-		/* Merge the outside tile to its bottom */
-		tp = LB(newtile);
-		if (CANMERGE_Y(newtile, tp)) TiJoinY(newtile, tp, plane);
-	    }
-
-#ifdef NONMANHATTAN
-	    /* Clipping diagonals can cause the new tile	*/
-	    /* to no longer be in the search path!		*/
-	    if (BOTTOM(tile) >= area->r_ytop)
-		goto paintdone;
-#endif
-
-	    /* Clip left */
-	    if (LEFT(tile) < area->r_xbot)
-	    {
-#ifdef NONMANHATTAN
-		if (IsSplit(tile))
-		{
-		    haschanged = TiNMSplitX(&tile, &newtile, area->r_xbot, 0, undo);
-		    if (!IsSplit(tile))
-		    {
-			oldType = TiGetTypeExact(tile);
-			newType = (*proc)(oldType, parg);
-		    }
-		}
-		else
-		{
-#endif
-		newtile = tile;
-		TISPLITX(tile, tile, area->r_xbot);
-		TiSetBody(tile, TiGetBody(newtile));
-#ifdef NONMANHATTAN
-		}
-#endif
-		mergeFlags &= ~MRG_LEFT;
-
-		/* Merge the outside tile to its top */
-		tp = RT(newtile);
-		if (CANMERGE_Y(newtile, tp)) TiJoinY(newtile, tp, plane);
-
-		/* Merge the outside tile to its bottom */
-		tp = LB(newtile);
-		if (CANMERGE_Y(newtile, tp)) TiJoinY(newtile, tp, plane);
-	    }
-
-#ifdef NONMANHATTAN
-	    /* Clipping diagonals can cause the new tile	*/
-	    /* to no longer be in the search path!		*/
-	    if (BOTTOM(tile) >= area->r_ytop)
-		goto paintdone;
-#endif
-
-#ifdef	PAINTDEBUG
-	    if (dbPaintDebug)
-		dbPaintShowTile(tile, undo, "after clip");
-#endif	/* PAINTDEBUG */
-	}
-
-#ifdef  NONMANHATTAN
-	if (newType & TT_DIAGONAL)
-	{
-	    /* If left and right types of a diagonal tile are	*/
-	    /* the same, revert back to a rectangular tile.	*/
-
-	    if ((newType & TT_LEFTMASK) == ((newType & TT_RIGHTMASK) >> 14))
-	    {
-	        newType &= TT_LEFTMASK;
-		if (undo && UndoIsEnabled())
-		    DBPAINTUNDO(tile, newType, undo);
-		TiSetBody(tile, newType);
-	    }
-	    else
-		mergeFlags = 0;
-	}
-#endif  /* NONMANHATTAN */
-
-	/*
-	 * Merge the tile back into the parts of the plane that have
-	 * already been visited.  Note that if we clipped in a particular
-	 * direction we avoid merging in that direction.
-	 *
-	 * We avoid calling dbPaintMerge if at all possible.
-	 */
-	if (mergeFlags & MRG_LEFT)
-	{
-	    for (tp = BL(tile); BOTTOM(tp) < TOP(tile); tp = RT(tp))
-		if (TiGetTypeExact(tp) == newType)
-		{
-		    tile = dbPaintMerge(tile, newType, plane, mergeFlags, undo);
-		    goto paintdone;
-		}
-	    mergeFlags &= ~MRG_LEFT;
-	}
-	if (mergeFlags & MRG_RIGHT)
-	{
-	    for (tp = TR(tile); TOP(tp) > BOTTOM(tile); tp = LB(tp))
-		if (TiGetTypeExact(tp) == newType)
-		{
-		    tile = dbPaintMerge(tile, newType, plane, mergeFlags, undo);
-		    goto paintdone;
-		}
-	    mergeFlags &= ~MRG_RIGHT;
-	}
-
-	/*
-	 * Cheap and dirty merge -- we don't have to merge to the
-	 * left or right, so the top/bottom merge is very fast.
-	 *
-	 * Now it's safe to change the type of this tile, and
-	 * record the event on the undo list.
-	 */
-#ifdef NONMANHATTAN
-	if (undo && UndoIsEnabled())
-	    if (haschanged || (oldType != newType))
-#else
-	if (undo && oldType != newType && UndoIsEnabled())
-#endif
-	    DBPAINTUNDO(tile, newType, undo);
-	TiSetBody(tile, newType);
-
-#ifdef	PAINTDEBUG
-	if (dbPaintDebug)
-	    dbPaintShowTile(tile, undo, "changed type");
-#endif	/* PAINTDEBUG */
-
-	if (mergeFlags & MRG_TOP)
-	{
-	    tp = RT(tile);
-	    if (CANMERGE_Y(tile, tp)) TiJoinY(tile, tp, plane);
-#ifdef	PAINTDEBUG
-	    if (dbPaintDebug)
-		dbPaintShowTile(tile, undo, "merged up (CHEAP)");
-#endif	/* PAINTDEBUG */
-	}
-	if (mergeFlags & MRG_BOTTOM)
-	{
-	    tp = LB(tile);
-	    if (CANMERGE_Y(tile, tp)) TiJoinY(tile, tp, plane);
-#ifdef	PAINTDEBUG
-	    if (dbPaintDebug)
-		dbPaintShowTile(tile, undo, "merged down (CHEAP)");
-#endif	/* PAINTDEBUG */
-	}
-
-
 	/***
 	 ***		END OF PAINT CODE
 	 *** ---------- BACK TO AREA SEARCH ----------
@@ -984,6 +571,9 @@ done:
  * painting, except where sub-splitting diagonal tiles requires a change
  * in tile type.
  *
+ * Use the resultTbl such that we ONLY fracture tiles that interact
+ * with the type to be painted.  This prevents "frivolous fracturing".
+ *
  * Results:
  *	None.
  *
@@ -994,9 +584,12 @@ done:
  */
 
 void
-DBFracturePlane(plane, area, undo)
+DBFracturePlane(plane, area, resultTbl, undo)
     Plane *plane;		/* Plane whose paint is to be modified */
     Rect *area;	/* Area to be changed */
+    PaintResultType *resultTbl;	/* Paint table, to pinpoint those tiles
+				 * that interact with the paint type.
+				 */
     PaintUndoInfo *undo;	/* Record containing everything needed to
 				 * save undo entries for this operation.
 				 * If NULL, the undo package is not used.
@@ -1037,6 +630,23 @@ enumerate:
 
 	clipTop = TOP(tile);
 	if (clipTop > area->r_ytop) clipTop = area->r_ytop;
+
+	/* Important!  Determine if the tile actually interacts	*/
+	/* with the type to be painted.  If not, then ignore it	*/
+
+	oldType = TiGetLeftType(tile);
+	if (resultTbl[oldType] == oldType)
+	{
+	    if (IsSplit(tile))
+	    {
+		oldType = TiGetRightType(tile);
+		if (resultTbl[oldType] == oldType)
+		    goto paintdone;
+	    }
+	    else
+		goto paintdone;
+	}
+
 	oldType = TiGetTypeExact(tile);
 
 #ifdef	PAINTDEBUG
@@ -1048,10 +658,13 @@ enumerate:
 	 *** ---------- THE FOLLOWING IS IN-LINE PAINT CODE ----------
 	 ***/
 
-	if (RIGHT(tile) < area->r_xtop) mergeFlags |= MRG_RIGHT;
-	if (BOTTOM(tile) > area->r_ybot) mergeFlags |= MRG_BOTTOM;
+	/* These merges are opposite to what is done in the	*/
+	/* DBPaintPlane routine.				*/
+
 	if (LEFT(tile) > area->r_xbot) mergeFlags |= MRG_LEFT;
+	if (RIGHT(tile) < area->r_xtop) mergeFlags |= MRG_RIGHT;
 	if (TOP(tile) < area->r_ytop) mergeFlags |= MRG_TOP;
+	if (BOTTOM(tile) > area->r_ybot) mergeFlags |= MRG_BOTTOM;
 
 	/*
 	 * Clip the tile against the clipping rectangle.
@@ -1078,7 +691,7 @@ enumerate:
 	/* Clipping diagonals can cause the new tile to no longer be */
 	/* in the search path!					 */
 	if (RIGHT(tile) <= area->r_xbot)
-	    goto paintdone;
+	    goto paintmerge;
 
 	/* Clip down */
 	if (BOTTOM(tile) < area->r_ybot)
@@ -1098,7 +711,7 @@ enumerate:
 	/* Clipping diagonals can cause the new tile to no longer be */
 	/* in the search path!					 */
 	if (RIGHT(tile) <= area->r_xbot)
-	    goto paintdone;
+	    goto paintmerge;
 
 	/* Clip right */
 	if (RIGHT(tile) > area->r_xtop)
@@ -1126,7 +739,7 @@ enumerate:
 	/* Clipping diagonals can cause the new tile	*/
 	/* to no longer be in the search path!		*/
 	if (BOTTOM(tile) >= area->r_ytop)
-	    goto paintdone;
+	    goto paintmerge;
 
 	/* Clip left */
 	if (LEFT(tile) < area->r_xbot)
@@ -1155,13 +768,15 @@ enumerate:
 	/* Clipping diagonals can cause the new tile	*/
 	/* to no longer be in the search path!		*/
 	if (BOTTOM(tile) >= area->r_ytop)
-	    goto paintdone;
+	    goto paintmerge;
 
 #ifdef	PAINTDEBUG
 	if (dbPaintDebug)
 	    dbPaintShowTile(tile, undo, "after clip");
 #endif	/* PAINTDEBUG */
+	goto paintdone;		/* skip the merge */
 
+paintmerge:
 	/*
 	 * Merge the tile back into the parts of the plane that have
 	 * already been visited.  Note that if we clipped in a particular
@@ -1179,6 +794,7 @@ enumerate:
 		}
 	    mergeFlags &= ~MRG_LEFT;
 	}
+
 	if (mergeFlags & MRG_RIGHT)
 	{
 	    for (tp = TR(tile); TOP(tp) > BOTTOM(tile); tp = LB(tp))
@@ -1190,6 +806,7 @@ enumerate:
 	    mergeFlags &= ~MRG_RIGHT;
 	}
 
+
 	/*
 	 * Cheap and dirty merge -- we don't have to merge to the
 	 * left or right, so the top/bottom merge is very fast.
@@ -1205,6 +822,7 @@ enumerate:
 		dbPaintShowTile(tile, undo, "merged up (CHEAP)");
 #endif	/* PAINTDEBUG */
 	}
+
 	if (mergeFlags & MRG_BOTTOM)
 	{
 	    tp = LB(tile);
@@ -1215,7 +833,6 @@ enumerate:
 #endif	/* PAINTDEBUG */
 	}
 
-
 	/***
 	 ***		END OF PAINT CODE
 	 *** ---------- BACK TO AREA SEARCH ----------
@@ -1258,6 +875,352 @@ done:
     plane->pl_hint = tile;
 }
 
+/*
+ * ----------------------------------------------------------------------------
+ * dbMergeNMTilesFunc --
+ *
+ *	Procedure to detect when an area of non-Manhattan tiles can be 
+ *	merged together into a single, larger non-Manhattan tile.
+ *
+ * Results:
+ *	Return 1 if any splitting was done.
+ *
+ * Side Effects:
+ *	Messes with the corner-stitched database.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+typedef struct {
+   Plane *plane;
+   PaintResultType *resultTbl;
+} mergeNMStruct;
+
+int
+dbMergeNMTilesFunc(tile, cdata)
+    Tile *tile;
+    ClientData cdata;
+{
+    mergeNMStruct *nmstruct = (mergeNMStruct *)cdata;
+    Plane *plane = nmstruct->plane;
+    PaintResultType *resultTbl = nmstruct->resultTbl;
+    Tile *tp;
+    int aspecta, aspectb;
+    bool tileOK = FALSE;
+    Rect rect;
+    TileType ttype, htype;
+    TileTypeBitMask tmask;
+
+    int dbcUnconnectFunc();	/* forward reference */
+
+    if (!IsSplit(tile))
+	return 0;
+
+    ttype = TiGetTypeExact(tile);
+    tileOK = FALSE;
+
+    if (SplitDirection(tile) != 0)
+    {
+	/* find tile at NW corner */
+	tp = RT(tile);
+	while (LEFT(tp) >= LEFT(tile))
+	    tp = BL(tp);
+	if ((RIGHT(tp) == LEFT(tile)) && (BOTTOM(tp) == TOP(tile)))
+	    tileOK = TRUE;
+    }
+    else
+    {
+	/* find tile at NE corner */
+	tp = RT(tile);
+	tp = TR(tp);
+	while (BOTTOM(tp) > TOP(tile))
+	    tp = LB(tp);
+	if ((RIGHT(tile) == LEFT(tp)) && (BOTTOM(tp) == TOP(tile)))
+	    tileOK = TRUE;
+    }
+    if (!IsSplit(tp) || !tileOK || (TiGetTypeExact(tp) != ttype)
+			|| (SplitDirection(tp) != SplitDirection(tile)))
+	return 0;
+
+    aspecta = (RIGHT(tp) - LEFT(tp)) * (TOP(tile) - BOTTOM(tile));
+    aspectb = (RIGHT(tile) - LEFT(tile)) * (TOP(tp) - BOTTOM(tp));
+    if (aspecta != aspectb)
+	return 0;
+
+    TTMaskSetOnlyType(&tmask, TiGetLeftType(tile));
+    TTMaskCom(&tmask);
+    if (SplitDirection(tile) == 0)
+    {
+	rect.r_xbot = LEFT(tile);
+	rect.r_xtop = RIGHT(tile);
+	rect.r_ybot = TOP(tile);
+	rect.r_ytop = TOP(tp);
+    }
+    else
+    {
+	rect.r_xbot = LEFT(tp);
+	rect.r_ybot = BOTTOM(tile);
+	rect.r_xtop = LEFT(tile);
+	rect.r_ytop = BOTTOM(tp);
+    }
+
+    /* Check for correct material in opposite corners	*/
+    /* Use dbcUnconnectFunc, which just returns 1 for	*/
+    /* tiles of the wrong type.				*/
+
+    if (DBSrPaintArea(tile, plane, &rect, &tmask, dbcUnconnectFunc,
+		(ClientData)NULL))
+	return 0;
+
+    TTMaskSetOnlyType(&tmask, TiGetRightType(tile));
+    TTMaskCom(&tmask);
+
+    if (SplitDirection(tile) == 0)
+    {
+	rect.r_xbot = LEFT(tp);
+	rect.r_ybot = BOTTOM(tile);
+	rect.r_xtop = LEFT(tile);
+	rect.r_ytop = BOTTOM(tp);
+    }
+    else
+    {
+	rect.r_xbot = LEFT(tile);
+	rect.r_ybot = TOP(tile);
+	rect.r_xtop = RIGHT(tile);
+	rect.r_ytop = TOP(tp);
+    }
+
+    if (DBSrPaintArea(tile, plane, &rect, &tmask, dbcUnconnectFunc,
+		(ClientData)NULL))
+	return 0;
+
+    /* Good to go! */
+
+    if (SplitDirection(tile) == 0)
+    {
+	rect.r_xbot = LEFT(tile);
+	rect.r_ybot = BOTTOM(tile);
+	rect.r_xtop = RIGHT(tp);
+	rect.r_ytop = TOP(tp);
+    }
+    else
+    {
+	rect.r_xbot = LEFT(tp);
+	rect.r_ybot = BOTTOM(tile);
+	rect.r_xtop = RIGHT(tile);
+	rect.r_ytop = TOP(tp);
+    }
+
+    TxPrintf("non-Manhattan merge possible at %d %d to %d %d\n",
+		rect.r_xbot, rect.r_ybot, rect.r_xtop, rect.r_ytop);
+
+    /* Erase the area, then paint back with ttype		*/
+    /* This is a non-operation as far as "undo" is concerned	*/
+
+    /* No, this is no good because using DBPaintPlane means we	*/
+    /* are not guaranteed that the result is a single tile in	*/
+    /* the area we want to merge.  We need to merge the tiles	*/
+    /* inside the area ourselves.				*/
+
+    htype = TiGetLeftType(tile);
+    if (htype == TT_SPACE)
+	htype = TiGetRightType(tile);
+
+    DBPaintPlane(plane, &rect, resultTbl, (PaintUndoInfo *)NULL);
+
+    tile = plane->pl_hint;
+    GOTOPOINT(tile, &(rect.r_ll));
+    TiSetBody(tile, ttype);
+
+    return 1;
+}
+
+/*
+ * ----------------------------------------------------------------------------
+ * DBUnFracturePlane --
+ *
+ *	DBFracturePlane() specifically prohibits merging across the
+ *	fracture boundary.  Therefore, for tiles that are otherwise
+ *	untouched by the DBNMPaintPlane() routine, we need to complete
+ *	the merging to ensure the integrity of the maximum-horizontal-
+ *	stripes database.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	Modifies the database plane that contains the given tile.
+ *
+ * Notes:
+ *	This routine should check for simple cases where two
+ *	non-Manhattan tiles can be merged into one larger tile.
+ *	Procedure:  Find the two tiles at the endpoints of the
+ *	tile's diagonal.  If either one is a non-Manhattan tile
+ *	with the same (exact) type and tile aspect ratio (same
+ *	angle of the diagonal), then check the other two areas
+ *	for all-left-type and all-right-type.  If this check
+ *	passes, erase the area and paint a single non-Manhattan
+ *	tile.  Or is it easy enough to do the merges & splits?
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+
+void
+DBUnFracturePlane(plane, area, undo)
+    Plane *plane;		/* Plane whose paint is to be modified */
+    Rect *area;			/* Area to be changed */
+    PaintUndoInfo *undo;	/* Record containing everything needed to
+				 * save undo entries for this operation.
+				 * If NULL, the undo package is not used.
+				 */
+{
+    Point start;
+    int clipTop, mergeFlags;
+    TileType ttype;
+    Tile *tile, *newtile;
+    Tile *tpnew;	/* Used for area search */
+    Tile *tp;		/* Used for paint */
+    Rect areaex;
+
+    if (area->r_xtop <= area->r_xbot || area->r_ytop <= area->r_ybot)
+	return;
+
+    /*
+     * Increase the search area by 1 in all directions to catch tiles
+     * on the outside of the fracture boundary.
+     */
+    areaex.r_xbot = area->r_xbot - 1;
+    areaex.r_ybot = area->r_ybot - 1;
+    areaex.r_xtop = area->r_xtop + 1;
+    areaex.r_ytop = area->r_ytop + 1;
+
+    /*
+     * The following is a modified version of the area enumeration
+     * algorithm.  It expects the in-line paint code below to leave
+     * 'tile' pointing to the tile from which we should continue the
+     * search.
+     */
+
+    start.p_x = areaex.r_xbot;
+    start.p_y = areaex.r_ytop - 1;
+    tile = plane->pl_hint;
+    GOTOPOINT(tile, &start);
+
+    /* Each iteration visits another tile on the LHS of the search area */
+    /* Note that this iteration visits tiles all tiles that are inside	*/
+    /* OR WHICH TOUCH the search area.  This catches tiles generated by	*/
+    /* TiNMSplit(X,Y)() that were outside of the search area but need	*/
+    /* to be merged into the surrounding area.				*/
+
+    while (TOP(tile) > areaex.r_ybot)
+    {
+	/***
+	 *** AREA SEARCH.
+	 *** Each iteration enumerates another tile.
+	 ***/
+enumerate:
+	if (SigInterruptPending)
+	    break;
+
+	clipTop = TOP(tile);
+	if (clipTop > areaex.r_ytop) clipTop = areaex.r_ytop;
+
+	/* These merges are opposite to what is done in the	*/
+	/* DBPaintPlane routine.				*/
+
+	mergeFlags = MRG_LEFT | MRG_RIGHT | MRG_TOP | MRG_BOTTOM;
+	ttype = TiGetTypeExact(tile);
+
+	/*
+	 * Merge the tile back into the parts of the plane that have
+	 * already been visited.  Note that if we clipped in a particular
+	 * direction we avoid merging in that direction.
+	 *
+	 * We avoid calling dbPaintMerge if at all possible.
+	 */
+	if (mergeFlags & MRG_LEFT)
+	{
+	    for (tp = BL(tile); BOTTOM(tp) < TOP(tile); tp = RT(tp))
+		if (TiGetTypeExact(tp) == ttype)
+		{
+		    tile = dbPaintMerge(tile, ttype, plane, mergeFlags, undo);
+		    goto paintdone;
+		}
+	    mergeFlags &= ~MRG_LEFT;
+	}
+
+	if (mergeFlags & MRG_RIGHT)
+	{
+	    for (tp = TR(tile); TOP(tp) > BOTTOM(tile); tp = LB(tp))
+		if (TiGetTypeExact(tp) == ttype)
+		{
+		    tile = dbPaintMerge(tile, ttype, plane, mergeFlags, undo);
+		    goto paintdone;
+		}
+	    mergeFlags &= ~MRG_RIGHT;
+	}
+
+	/*
+	 * Cheap and dirty merge -- we don't have to merge to the
+	 * left or right, so the top/bottom merge is very fast.
+	 *
+	 */
+
+	if (mergeFlags & MRG_TOP)
+	{
+	    tp = RT(tile);
+	    if (CANMERGE_Y(tile, tp)) TiJoinY(tile, tp, plane);
+	}
+
+	if (mergeFlags & MRG_BOTTOM)
+	{
+	    tp = LB(tile);
+	    if (CANMERGE_Y(tile, tp)) TiJoinY(tile, tp, plane);
+	}
+
+	/***
+	 ***		END OF PAINT CODE
+	 *** ---------- BACK TO AREA SEARCH ----------
+	 ***/
+paintdone:
+	/* Move right if possible */
+	tpnew = TR(tile);
+	if (LEFT(tpnew) < areaex.r_xtop)
+	{
+	    /* Move back down into clipping area if necessary */
+	    while (BOTTOM(tpnew) >= clipTop) tpnew = LB(tpnew);
+	    if (BOTTOM(tpnew) >= BOTTOM(tile) || BOTTOM(tile) <= areaex.r_ybot)
+	    {
+		tile = tpnew;
+		goto enumerate;
+	    }
+	}
+
+	/* Each iteration returns one tile further to the left */
+	while (LEFT(tile) > areaex.r_xbot)
+	{
+	    /* Move left if necessary */
+	    if (BOTTOM(tile) <= areaex.r_ybot)
+		goto done;
+
+	    /* Move down if possible; left otherwise */
+	    tpnew = LB(tile); tile = BL(tile);
+	    if (BOTTOM(tpnew) >= BOTTOM(tile) || BOTTOM(tile) <= areaex.r_ybot)
+	    {
+		tile = tpnew;
+		goto enumerate;
+	    }
+	}
+	/* At left edge -- walk down to next tile along the left edge */
+	for (tile = LB(tile); RIGHT(tile) <= areaex.r_xbot; tile = TR(tile))
+	    /* Nothing */;
+    }
+
+done:
+    plane->pl_hint = tile;
+}
+
 /*
  * ----------------------------------------------------------------------------
  * DBDiagonalProc(type) --
@@ -1266,14 +1229,14 @@ done:
  *	Argument "cdata", gives direction and side of diagonal, and the
  *	paint result table for the given diagonal side.
  *
- *	This routine will be called separately for each side of the
- *	diagonal, if necessary.
+ *	If the result cannot be described with a single tile, then
+ *	return -1.
  *
- *	This routine is called only from DBPaintPlaneByProc(), which itself
- *	is called only from DBNMPaintPlane().
+ *	This routine is called only from DBNMPaintPlane().
  * 
  * Results:
- *	Returns the new type to be painted.
+ *	Returns the new type to be painted, or -1 if the result can
+ *	only be generated by quartering the original tile.
  *
  * Side Effects:
  *	None.
@@ -1282,47 +1245,97 @@ done:
 
 int
 DBDiagonalProc(oldtype, cdata)
-    int oldtype;
+    TileType oldtype;
     ClientData cdata;
 {
-    int newtype;
+    TileType old_n, old_s, old_e, old_w;
+    TileType new_n, new_s, new_e, new_w;
+    TileType newtype;
     DiagInfo *dinfo = (DiagInfo *)cdata;
     PaintResultType *resultTbl = dinfo->resultTbl;
 
+    /* Disassemble old and new types into four quadrants, find the	*/
+    /* paint result for each quadrant, then reassemble the result.	*/
+
     if (oldtype & TT_DIAGONAL)
     {
-	if (dinfo->side == 0)
-	    newtype = (resultTbl[oldtype & TT_LEFTMASK])
-			| (oldtype & TT_RIGHTMASK);
+	old_e = (oldtype & TT_RIGHTMASK) >> 14;
+	old_w = oldtype & TT_LEFTMASK;
+
+	if (oldtype & TT_DIRECTION)
+	{
+	    old_n = old_e;
+	    old_s = old_w;
+	}
 	else
-	    newtype = (resultTbl[(oldtype & TT_RIGHTMASK) >> 14] << 14)
-			| (oldtype & TT_LEFTMASK);
+	{
+	    old_n = old_w;
+	    old_s = old_e;
+	}
+    }
+    else
+	old_n = old_s = old_e = old_w = oldtype;
+
+    /* Determine result for each quadrant */
+
+    if (dinfo->side == 0)
+    {
+	new_e = old_e;
+	new_w = resultTbl[old_w];
     }
     else
     {
-	newtype = (dinfo->side == 0) ?
-		resultTbl[oldtype] : (resultTbl[oldtype] << 14);
-	/* Preserve oldtype on the side not painted */
-	newtype |= (dinfo->side == 0) ?
-		oldtype << 14 : oldtype;
+	new_e = resultTbl[old_e];
+	new_w = old_w;
     }
 
-    /* For purposes of the undo command, record which side we just painted */
+    if (dinfo->dir == dinfo->side)
+    {
+	new_n = resultTbl[old_n];
+	new_s = old_s;
+    }
+    else
+    {
+	new_n = old_n;
+	new_s = resultTbl[old_s];
+    }
+
+    /* Now reassemble */
+
+    if ((new_n == new_e) && (new_s == new_w))
+    {
+	if (new_n == new_w)
+	{
+	    newtype = new_n;	/* Turned back into a rectangle */
+	    return newtype;
+	}
+	else 
+	{
+	    newtype = new_w | (new_e << 14);
+	    newtype |= (TT_DIAGONAL | TT_DIRECTION);
+	}
+    }
+    else if ((new_n == new_w) && (new_s == new_e))
+    {
+	newtype = new_w | (new_e << 14);
+	newtype |= TT_DIAGONAL;
+    }
+    else
+	return -1;
+
+    /* For purposes of "undo" recording, record which side we just painted */
     if (dinfo->side)
 	newtype |= TT_SIDE;
 
-    if (dinfo->dir) newtype |= TT_DIRECTION;
-
-    return newtype | TT_DIAGONAL;
-}
-
+    return newtype;
+}	
+    
 /*
  * ----------------------------------------------------------------------------
  *
  * DBNMPaintPlane --
  *
- * Non-Manhattan PaintPlane function (wrapper for DBPaintPlane and
- * DBPaintPlaneByProc)
+ * Non-Manhattan PaintPlane function (wrapper for DBPaintPlane)
  *
  * Due to the intricacies of painting diagonals, it is necessary to
  * perform a search on the area to paint and return a list of sub-areas
@@ -1365,6 +1378,7 @@ DBNMPaintPlane(plane, exacttype, area, resultTbl, undo)
     LinkedRect *lhead, *lr, *newlr;
     int xc, yc, width, height, xref, yref;
     int resstate;
+    mergeNMStruct nmstruct;
 
     if (exacttype & TT_DIAGONAL)
     {
@@ -1378,14 +1392,14 @@ DBNMPaintPlane(plane, exacttype, area, resultTbl, undo)
 	width = area->r_xtop - area->r_xbot;
 
 	/* Break the plane up around the specified boundary */
-	DBFracturePlane(plane, area, undo);
+	DBFracturePlane(plane, area, resultTbl, undo);
 
 	/* Find all tiles under the area to paint, and make a	*/
 	/* linked list out of them.				*/
 
 	lhead = NULL;
-	(void) DBSrPaintArea((Tile *) NULL, plane,
-            area, &DBAllTypeBits, dbNMEnumFunc, (ClientData) &lhead);
+	(void) DBSrPaintArea((Tile *) NULL, plane, area,
+		&DBAllTypeBits, dbNMEnumFunc, (ClientData) &lhead);
 
 	/*--------------------------------------------------------*/
 	/* If there's only one tile underneath, we can just paint */
@@ -1398,19 +1412,100 @@ DBNMPaintPlane(plane, exacttype, area, resultTbl, undo)
 	if (lr != NULL)
 	    if (lhead->r_next == NULL)
 	    {
+		Point start;
+		Tile *tile;
+		TileType oldType, newType;
+
 	        GeoClip(&lhead->r_r, area);
-		DBPaintPlaneByProc(plane, &lhead->r_r, (int *)DBDiagonalProc,
-			(ClientData) &dinfo, undo);
-		freeMagic((char *) lr);
-	        return;
+		start.p_x = area->r_xbot;
+		start.p_y = area->r_ytop - 1;
+		tile = plane->pl_hint;
+		GOTOPOINT(tile, &start);
+		oldType = TiGetTypeExact(tile);
+		newType = DBDiagonalProc(oldType, &dinfo);
+
+		/* Ignore tiles that don't change type.	*/
+
+		if (newType == oldType)
+		{
+		    freeMagic((char *) lr);
+	            return;
+		}
+
+		/* Watch for the worst-case scenario of attempting to	*/
+		/* draw a triangle on top of another triangle with the	*/
+		/* diagonal in the opposite direction.  This forces	*/
+		/* subsplitting into quarters.  But if we're down to	*/
+		/* width=1 or height=1, we cannot subdivide any further	*/
+		/* so we stop the madness.				*/
+
+		else if (newType == -1)
+		{
+		    if ((width == 1) || (height == 1))
+		    {
+			freeMagic((char *) lr);
+			return;
+		    }
+
+		    /* lr->r_r is drawn & quartered */
+
+		    lr->r_r.r_xtop -= (width >> 1);
+		    lr->r_r.r_ytop -= (height >> 1);
+
+		    newlr = (LinkedRect *) mallocMagic(sizeof(LinkedRect));
+		    newlr->r_r.r_xbot = area->r_xbot;
+		    newlr->r_r.r_xtop = lr->r_r.r_xtop;
+		    newlr->r_r.r_ybot = lr->r_r.r_ytop;
+		    newlr->r_r.r_ytop = area->r_ytop;
+		    newlr->r_next = (LinkedRect *)NULL;
+		    lr->r_next = newlr;
+
+		    newlr = (LinkedRect *) mallocMagic(sizeof(LinkedRect));
+		    newlr->r_r.r_xbot = lr->r_r.r_xtop;
+		    newlr->r_r.r_xtop = area->r_xtop;
+		    newlr->r_r.r_ybot = area->r_ybot;
+		    newlr->r_r.r_ytop = lr->r_r.r_ytop;
+		    newlr->r_next = lr->r_next;
+		    lr->r_next = newlr;
+
+		    newlr = (LinkedRect *) mallocMagic(sizeof(LinkedRect));
+		    newlr->r_r.r_xbot = lr->r_r.r_xtop;
+		    newlr->r_r.r_xtop = area->r_xtop;
+		    newlr->r_r.r_ybot = lr->r_r.r_ytop;
+		    newlr->r_r.r_ytop = area->r_ytop;
+		    newlr->r_next = lr->r_next;
+		    lr->r_next = newlr;
+		}
+		else
+		{
+		    /* Easier case:  Triangles have the diagonal in the	*/
+		    /* same direction, so we just find the new paint	*/
+		    /* types.  Tile may become a non-split tile, in	*/
+		    /* which case we call DBPaintPlane() to make sure	*/
+		    /* that the tile is properly merged back into the	*/
+		    /* surrounding area, if applicable.			*/
+
+		    if (newType & TT_DIAGONAL)
+		    {
+			if (undo && UndoIsEnabled())
+			    DBPAINTUNDO(tile, newType, undo);
+			TiSetBody(tile, newType);
+		    }
+		    else
+			DBPaintPlane(plane, &(lr->r_r), resultTbl, undo);
+
+		    freeMagic((char *) lr);
+		    /* goto nmmerge; */
+	            return;
+		}
 	    }
 
 	/*------------------------------------------------------*/
 	/* Further subdivide the rects so that every rect is 	*/
 	/* either a (non-clipped) triangle or a rectangle.	*/
-	/* Call DBPaintPlaneByProc() on triangular areas, and	*/
-	/* DBPaintPlane() on rectangular areas.  Reject areas	*/
-	/* which contain no paint.				*/
+	/* Recursively call this function on triangular areas,	*/
+	/* and DBPaintPlane() on rectangular areas.  Reject	*/
+	/* areas which contain no paint.			*/
 	/*------------------------------------------------------*/
 
 	while (lr != NULL)
@@ -1418,8 +1513,7 @@ DBNMPaintPlane(plane, exacttype, area, resultTbl, undo)
 	    resstate = RES_DIAG;
 
 	    /* Clip to area:  DBFracturePlane() is supposed to do this, */
-	    /* but I need to fix it.  Currently it can still merge	*/
-	    /* across the fracture boundary.				*/
+	    /* so this step should not be necessary.			*/
 	    GeoClip(&lr->r_r, area);
 
 	    /* Split off left */
@@ -1533,7 +1627,7 @@ paintrect:
 		     (resstate == RES_RIGHT && dinfo.side)) {
 		DBPaintPlane(plane, &(lr->r_r), resultTbl, undo);
 	    }
-	    /* else: rectangle does not contain type and should be ignored. */
+	    /* else: Rectangle does not contain type and should be ignored. */
 nextrect:
 	    lr = lr->r_next;
 	}
@@ -1544,9 +1638,27 @@ nextrect:
 	    freeMagic((char *) lr);
 	    lr = lr->r_next;
 	}
+
+	/* Restore the maximum horizontal strips policy on the	*/
+	/* plane around the fractured boundary.			*/
+
+	DBUnFracturePlane(plane, area, undo);
     }
     else
 	DBPaintPlane(plane, area, resultTbl, undo);
+
+// nmmerge:
+//
+//  /* Walk the area, looking for non-Manhattan merges.	*/
+//  /* This is experimental code, disabled for now.	*/
+//  /* Also see "goto nmmerge" line, above.		*/
+//
+//  nmstruct.plane = plane;
+//  nmstruct.resultTbl = resultTbl;
+//
+//  while (DBSrPaintArea((Tile *) NULL, plane, area, &DBAllTypeBits,
+//	dbMergeNMTilesFunc, (ClientData) &nmstruct));
+
 }
 
 /*
@@ -1619,7 +1731,7 @@ dbNMEnumFunc(tile, arg)
 void
 DBPaintPlaneMergeOnce(plane, exacttype, area, resultTbl, undo)
     Plane *plane;		/* Plane whose paint is to be modified */
-    TileType exacttype;		/* Source tile type, including diagonal info */
+    TileType exacttype;		/* Source tile type diagonal info (only) */
     Rect *area;	/* Area to be changed */
     PaintResultType *resultTbl;	/* Table, indexed by the type of tile already
 				 * present in the plane, giving the type to
