@@ -117,7 +117,7 @@ DBTechInitContact()
  *
  * Magic-7.3 additional syntax:
  *
- *	stackable type1 [type2 [type3 ...]]
+ *	stackable type1 [type2 [alias12] [type3 [alias13] ...]]
  *
  * where type1 and type2 are contact types, allows type1 and type2
  * to be drawn on top of one other by generating an extra contact
@@ -125,7 +125,14 @@ DBTechInitContact()
  * plane.  If more than two types are specified, then type1 will be
  * set to be stackable with each of the indicated types.  If only
  * type1 is specified, then type1 will be made stackable with all
- * other (existing) contact types.
+ * other (existing) contact types.  Wherever a layer name exists
+ * after a known type that is not a known layer name, it is
+ * considered to be an alias name for the stacked type.  e.g.,
+ * "stackable pc via pm12contact" is valid if "pc" and "via" are
+ * defined types.  Thie provides compatibility with layout files
+ * created with technology files that explicitly define stacked
+ * contact types, especially those from magic-7.2 and earlier
+ * versions.
  *
  *	stackable
  *
@@ -180,7 +187,7 @@ DBTechAddContact(sectionName, argc, argv)
 	}
 	else if (!strcmp(*argv, "stackable"))
 	{
-	    TileType stackType, newType;
+	    TileType stackType, newType = -1;
 	    LayerInfo *lim, *lin;
 
 	    if (argc == 1)
@@ -207,9 +214,12 @@ DBTechAddContact(sectionName, argc, argv)
 		}
 		else
 		{
+		    TileType lastType = TT_SPACE;
+		    char *primary;
+
 		    while (--argc > 1)
 		    {
-			stackType = DBTechNoisyNameType(*++argv);
+			stackType = DBTechNameType(*++argv);
 			if (stackType >= 0)
 			{
 			    newType = dbTechAddOneStackedContact(contactType,
@@ -218,8 +228,23 @@ DBTechAddContact(sectionName, argc, argv)
 				TechError("Contact types %s and %s do not stack\n",
 					DBTypeLongNameTbl[contactType],
 					DBTypeLongNameTbl[stackType]);
+			    lastType = stackType;
+			}
+			else if (lastType >= TT_SPACE)
+			{
+			    /* (*argv) becomes an alias name for layer newType */
+			    if (newType < 0)
+				TechError("Contact type %s unknown or contact "
+					"missing in stackable statement\n", *argv);
+			    else
+				DBTechAddNameToType(*argv, newType);
+			    lastType = TT_SPACE;
+			}
+			else
+			{
+			    DBTechNoisyNameType(*argv);
+			    lastType = TT_SPACE;
 			}
-			else DBTechNoisyNameType(*argv);
 		    }
 		}
 	    }
@@ -340,6 +365,12 @@ dbTechAddOneStackedContact(type1, type2)
 	if (!TTMaskEqual(&mmask, &DBZeroTypeBits))
 	    return -2; /* Contact type exists, so don't create one. */
 
+	/* Also check if there is a stacking type made of these contact */
+	/* images.  If we already have one, don't re-make it.		*/
+
+	else if (DBTechFindStacking(type1, type2) != -1)
+	    return -2; /* Stacking type exists, so don't create one. */
+
 	/* All clear to set the residue bitmask for this contact type */
 
 	/* Diagnostic */
@@ -516,6 +547,14 @@ dbTechContactResidues(argc, argv, contactType)
      * if so, disallow contactType.
      *
      * Can this restriction be lifted?  ---Tim 07/24/03
+     * This restriction is partially lifted, as one can create a non-stacked
+     * type such as "pad" having the same residues as one stacked type,
+     * such as "m123c".  Due to the way magic handles stacked types, these
+     * will not be confused.	---Tim 05/11/04
+     *
+     * Restriction entirely lifted 6/18/04.  Error message left as a warning
+     * until it is clear that there are no unwanted side effects of having
+     * two contact types with identical residues.
      */
 
     /* Find if there exists an image with the same residue mask */
@@ -528,13 +567,14 @@ dbTechContactResidues(argc, argv, contactType)
     
     if (!TTMaskEqual(&mmask, &DBZeroTypeBits))
     {   
-	TechError("Contact residues for %s identical to those for ",
+	TxPrintf("Contact residues for %s identical to those for ",
 		DBTypeLongName(contactType));
+
 	for (imageType = TT_TECHDEPBASE; imageType < DBNumTypes; imageType++)
 	    if (TTMaskHasType(&mmask, imageType))
-		TxError("%s ", DBTypeLongName(imageType));
-	TxError("\n");
-	return -1;
+		TxPrintf("%s ", DBTypeLongName(imageType));
+
+	TxPrintf("\n");
     }  
 
     /* All clear to set the residue bitmask for this contact type */
