@@ -171,15 +171,37 @@ DBTechSetVersion(sectionName, argc, argv)
     int argc;
     char *argv[];
 {
-    if (argc != 2) goto usage;
-    if (strcmp(argv[0], "version") == 0) {
+    char *contline;
+    int n, slen;
+
+    if (argc < 2) goto usage;
+    if (strcmp(argv[0], "version") == 0)
+    {
 	(void) StrDup(&DBTechVersion, argv[1]);
+	for (n = 2; n < argc; n++)
+	{
+	    slen = strlen(DBTechVersion);
+	    contline = mallocMagic(strlen(argv[n]) + slen + 1);
+	    sprintf(contline, "%s\n%s", DBTechVersion, argv[n]);
+	    freeMagic(DBTechVersion);
+	    DBTechVersion = contline;
+	}
 	return TRUE;
     }
-    if (strcmp(argv[0], "description") == 0) {
+    if (strcmp(argv[0], "description") == 0)
+    {
 	(void) StrDup(&DBTechDescription, argv[1]);
+	for (n = 2; n < argc; n++)
+	{
+	    slen = strlen(DBTechDescription);
+	    contline = mallocMagic(strlen(argv[n]) + slen + 1);
+	    sprintf(contline, "%s\n%s", DBTechDescription, argv[n]);
+	    freeMagic(DBTechDescription);
+	    DBTechDescription = contline;
+	}
 	return TRUE;
     }
+
 usage:
     TechError("Badly formed version line\nUsage: {version text}|{description text}\n");
     return FALSE;
