@@ -29,7 +29,7 @@ static char rcsid[] = "$Header: /ufs/repository/magic/database/DBtechname.c,v 1.
 #include <string.h>
 #endif
 
-#include "magic/tclmagic.h"
+#include "tcltk/tclmagic.h"
 #include "misc/magic.h"
 #include "utils/geometry.h"
 #include "utils/utils.h"
@@ -119,13 +119,13 @@ DBTechNameTypes(typename,bitmask)
     slash = index(typename, '/');
     if (slash != NULL) *slash = 0;
     type = (TileType) dbTechNameLookup(typename, &dbTypeNameLists);
+    if (type < 0) return type;
     if (slash == NULL)
     {
     	 TTMaskSetType(bitmask,type);
 	 return type;
     } 
     *slash = '/';
-    if (type < 0) return type;
 
     /* There's a plane qualification.  Locate the image. */
 
