@@ -254,6 +254,35 @@ DBTechAddPlane(sectionName, argc, argv)
     DBPlaneLongNameTbl[DBNumPlanes++] = cp;
     return TRUE;
 }
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * DBTechAddNameToType --
+ *
+ * Add a new name (alias) to an existing tile type.
+ *
+ * Results:
+ *	None.
+ *
+ * Side Effects:
+ *	Adds "newname" to dbTypeNameLists.  May change the primary name
+ *	of the type stored in the pointer array DBTypeLongNameTbl[].
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+void
+DBTechAddNameToType(newname, ttype)
+    char *newname;
+    TileType ttype;
+{
+    char *cp;
+
+    cp = dbTechNameAdd(newname, (ClientData) ttype, &dbTypeNameLists);
+    DBTypeLongNameTbl[ttype] = cp;
+}
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -599,7 +628,7 @@ dbTechNameAddOne(name, cdata, isPrimary, ptable)
 	    break;
 
     /* Create a new name */
-    MALLOC(NameList *, new, sizeof (NameList));
+    new = (NameList *) mallocMagic((unsigned) (sizeof (NameList)));
     new->sn_name = StrDup((char **) NULL, name);
     new->sn_value = cdata;
     new->sn_primary = isPrimary;
