@@ -41,6 +41,278 @@ struct dbCheck
 };
 
 int dbCheckMaxHFunc(), dbCheckMaxVFunc();
+
+#ifdef NONMANHATTAN
+
+/*
+ * --------------------------------------------------------------------
+ *
+ * DBSrPaintNMArea --
+ *
+ * Find all tiles overlapping a given triangular area whose types are
+ * contained in the mask supplied.  Apply the given procedure to each
+ * such tile.  The procedure should be of the following form:
+ *
+ *	int
+ *	func(tile, cdata)
+ *	    Tile *tile;
+ *	    ClientData cdata;
+ *	{
+ *	}
+ *
+ * This is equivalent to DBSrPaintArea, but is used when only one
+ * diagonal half (nonmanhattan extension) of the area should be searched.
+ * The bitmask for the diagonal are passed in "ttype" (diagonal, side and
+ * direction only are used, as in DBNMPaintPlane).
+ *
+ * Results:
+ *	0 is returned if the search completed normally.  1 is returned
+ *	if it aborted.
+ *
+ * Side effects:
+ *	Whatever side effects result from application of the
+ *	supplied procedure.
+ *
+ * Notes:
+ *	Do not call this routine on an infinite search area (e.g., area
+ *	== &TiPlaneRect), even if "ttype" is set appropriately.
+ * --------------------------------------------------------------------
+ */
+
+#define IGNORE_LEFT 1
+#define IGNORE_RIGHT 2
+
+int
+DBSrPaintNMArea(hintTile, plane, ttype, rect, mask, func, arg)
+    Tile *hintTile;		/* Tile at which to begin search, if not NULL.
+				 * If this is NULL, use the hint tile supplied
+				 * with plane.
+				 */
+    Plane *plane;		/* Plane in which tiles lie.  This is used to
+				 * provide a hint tile in case hintTile == NULL.
+				 * The hint tile in the plane is updated to be
+				 * the last tile visited in the area
+				 * enumeration.
+				 */
+    TileType ttype;		/* Information about the non-manhattan area to
+			 	 * search; zero if area is manhattan.
+			 	 */
+    Rect *rect;			/* Area to search.  This area should not be
+				 * degenerate.  Tiles must OVERLAP the area.
+				 */
+    TileTypeBitMask *mask;	/* Mask of those paint tiles to be passed to
+				 * func.
+				 */
+    int (*func)();		/* Function to apply at each tile */
+    ClientData arg;		/* Additional argument to pass to (*func)() */
+{
+    Point start;
+    Tile *tp, *tpnew;
+    TileType tpt;
+    int rheight, rwidth, rmax, f1, f2, f3, f4;
+    bool ignore_sides;
+
+    /* If the search area is not diagonal, return the result of the	*/
+    /* standard (manhattan) search function.				*/
+
+    if (!(ttype & TT_DIAGONAL))
+	return DBSrPaintArea(hintTile, plane, rect, mask, func, arg);
+
+    start.p_x = rect->r_xbot;
+    start.p_y = rect->r_ytop - 1;
+    tp = hintTile ? hintTile : plane->pl_hint;
+    GOTOPOINT(tp, &start);
+
+    /* Each iteration visits another tile on the LHS of the search area */
+    while (TOP(tp) > rect->r_ybot)
+    {
+	/* Each iteration enumerates another tile */
+nm_enum:
+	plane->pl_hint = tp;
+	if (SigInterruptPending)
+	    return (1);
+
+	/* Check if the tile is in the (nonmanhattan) area, and continue */ 
+	/* the tile enumeration if it is not.				 */
+	/* Watch for calculations involving (M)INFINITY in tile (tp)!	 */
+
+	rheight = rect->r_ytop - rect->r_ybot;
+	rwidth = rect->r_xtop - rect->r_xbot;
+	rmax = MAX(rheight, rwidth);
+	f1 = (BOTTOM(tp) > MINFINITY + 2) ?
+		((rect->r_ytop - BOTTOM(tp)) * rwidth) : INFINITY - 2;
+	f2 = (TOP(tp) < INFINITY - 2) ?
+		((TOP(tp) - rect->r_ybot) * rwidth) : INFINITY - 2;
+
+	if (ttype & TT_SIDE)
+	{
+	    /* Outside-of-triangle check---ignore sub-integer slivers */
+	    if (RIGHT(tp) < INFINITY - 2)
+	    {
+		f3 = (rect->r_xtop - RIGHT(tp)) * rheight;
+		f3 += rmax;
+	    }
+	    else
+		f3 = MINFINITY + 2;
+	    if ((ttype & TT_DIRECTION) ? (f2 < f3) : (f1 < f3))
+		goto enum_next;
+	}
+	else
+	{
+	    /* Outside-of-triangle check---ignore sub-integer slivers */
+	    if (LEFT(tp) > MINFINITY + 2)
+	    {
+		f4 = (LEFT(tp) - rect->r_xbot) * rheight;
+		f4 += rmax;
+	    }
+	    else
+		f4 = MINFINITY + 2;
+	    if ((ttype & TT_DIRECTION) ? (f1 < f4) : (f2 < f4))
+		goto enum_next;
+	}
+
+	/* Secondary checks---if tile is also non-Manhattan, is		*/
+	/* either side of it outside the area?  If so, restrict it.	*/
+	/* This check is only necessary if the split directions are	*/
+	/* the same, so we have to see if either of the neighboring	*/
+	/* points is also inside the search triangle.			*/
+
+	ignore_sides = 0;
+
+	if (IsSplit(tp))
+	{
+	    if (!TTMaskHasType(mask, SplitLeftType(tp)))
+		ignore_sides |= IGNORE_LEFT;
+	    if (!TTMaskHasType(mask, SplitRightType(tp)))
+		ignore_sides |= IGNORE_RIGHT;
+
+	    tpt = TiGetTypeExact(tp);
+	    if ((tpt & TT_DIRECTION) == (ttype & TT_DIRECTION))
+	    {
+		f3 = (LEFT(tp) > MINFINITY + 2) ?
+			((rect->r_xtop - LEFT(tp)) * rheight) : INFINITY - 2;
+		f4 = (RIGHT(tp) < INFINITY - 2) ?
+			((RIGHT(tp) - rect->r_xbot) * rheight) : INFINITY - 2;
+
+		if (ttype & TT_SIDE)
+		{
+		    /* Ignore sub-integer slivers */
+		    if (f4 != INFINITY - 2) f4 -= rmax;
+		    if (f3 != INFINITY - 2) f3 += rmax;
+		    
+		    if (ttype & TT_DIRECTION)
+		    {
+			if ((f2 < f3) && (f1 > f4))
+			    /* Tile bottom left is outside search area */
+			    ignore_sides |= IGNORE_LEFT;
+		    }
+		    else
+		    {
+			if ((f1 < f3) && (f2 > f4))
+			    /* Tile top left is outside search area */
+			    ignore_sides |= IGNORE_LEFT;
+		    }
+		}
+		else
+		{
+		    /* Ignore sub-integer slivers */
+		    if (f4 != INFINITY - 2) f4 += rmax;
+		    if (f3 != INFINITY - 2) f3 -= rmax;
+		    
+		    if (ttype & TT_DIRECTION)
+		    {
+			if ((f2 > f3) && (f1 < f4))
+			    /* Tile top right is outside search area */
+			    ignore_sides |= IGNORE_RIGHT;
+		    }
+		    else
+		    {
+			if ((f1 > f3) && (f2 < f4))
+			    /* Tile bottom right is outside search area */
+			    ignore_sides |= IGNORE_RIGHT;
+		    }
+		}
+	    }
+
+	    /* If the tile is larger than the search area or overlaps	*/
+	    /* the search area, we need to check if one of the sides	*/
+	    /* of the tile is disjoint from the search area.		*/
+
+	    rheight = TOP(tp) - BOTTOM(tp);
+	    rwidth = RIGHT(tp) - LEFT(tp);
+	    rmax = MAX(rheight, rwidth);
+	    f1 = (TOP(tp) < INFINITY - 2) ?
+			((TOP(tp) - rect->r_ybot) * rwidth) : INFINITY - 2;
+	    f2 = (BOTTOM(tp) > MINFINITY + 2) ?
+			((rect->r_ytop - BOTTOM(tp)) * rwidth) : INFINITY - 2;
+	    f3 = (RIGHT(tp) < INFINITY - 2) ?
+			((RIGHT(tp) - rect->r_xtop) * rheight) : INFINITY - 2;
+	    f4 = (LEFT(tp) > MINFINITY + 2) ?
+			((rect->r_xbot - LEFT(tp)) * rheight) : INFINITY - 2;
+
+	    /* ignore sub-integer slivers */
+	    if (f4 < INFINITY - 2) f4 += rmax;
+	    if (f3 < INFINITY - 2) f3 += rmax;
+
+	    if (SplitDirection(tp) ? (f1 < f4) : (f2 < f4))
+		ignore_sides |= IGNORE_LEFT;
+
+	    if (SplitDirection(tp) ? (f2 < f3) : (f1 < f3))
+		ignore_sides |= IGNORE_RIGHT;
+
+	    /* May call function twice to paint both sides of   */
+	    /* the split tile, if necessary.                    */
+
+	    if (!(ignore_sides & IGNORE_LEFT))
+	    {
+		TiSetBody(tp, (ClientData)(tpt & ~TT_SIDE)); /* bit clear */
+		if ((*func)(tp, arg)) return (1);
+	    }
+	    if (!(ignore_sides & IGNORE_RIGHT))
+	    {
+		TiSetBody(tp, (ClientData)(tpt | TT_SIDE)); /* bit set */
+		if ((*func)(tp, arg)) return (1);
+	    }
+        }
+	else
+	    if (TTMaskHasType(mask, TiGetType(tp)) && (*func)(tp, arg))
+		return (1);
+
+enum_next:
+	tpnew = TR(tp);
+	if (LEFT(tpnew) < rect->r_xtop)
+	{
+	    while (BOTTOM(tpnew) >= rect->r_ytop) tpnew = LB(tpnew);
+	    if (BOTTOM(tpnew) >= BOTTOM(tp) || BOTTOM(tp) <= rect->r_ybot)
+	    {
+		tp = tpnew;
+		goto nm_enum;
+	    }
+	} 
+
+	/* Each iteration returns one tile further to the left */
+	while (LEFT(tp) > rect->r_xbot)
+	{
+	    if (BOTTOM(tp) <= rect->r_ybot) 
+		return (0);
+	    tpnew = LB(tp);
+	    tp = BL(tp);
+	    if (BOTTOM(tpnew) >= BOTTOM(tp) || BOTTOM(tp) <= rect->r_ybot)
+	    {
+		tp = tpnew;
+		goto nm_enum;
+	    }
+	}
+
+	/* At left edge -- walk down to next tile along the left edge */
+	for (tp = LB(tp); RIGHT(tp) <= rect->r_xbot; tp = TR(tp))
+	    /* Nothing */;
+    }
+    return (0);
+}
+
+#endif	/* NONMANHATTAN */
+
 
 /*
  * --------------------------------------------------------------------
@@ -203,113 +475,7 @@ enumerate:
     }
     return (0);
 }
-
-/*
- * --------------------------------------------------------------------
- *
- * DBSrPaintAreaReverse --
- *
- * Identical to DBSrPaintArea, except that tiles are visited
- * from right to left, bottom to top.  Used when plowing either
- * up or to the left.
- *
- * Results:
- *	0 is returned if the search completed normally.  1 is returned
- *	if it aborted.
- *
- * Side effects:
- *	Whatever side effects result from application of the
- *	supplied procedure.
- *
- * --------------------------------------------------------------------
- */
 
-int
-DBSrPaintAreaReverse(hintTile, plane, rect, mask, func, arg)
-    Tile *hintTile;		/* Tile at which to begin search, if not NULL.
-				 * If this is NULL, use the hint tile supplied
-				 * with plane.
-				 */
-    Plane *plane;	/* Plane in which tiles lie.  This is used to
-				 * provide a hint tile in case hintTile == NULL.
-				 * The hint tile in the plane is updated to be
-				 * the last tile visited in the area
-				 * enumeration.
-				 */
-    Rect *rect;	/* Area to search */
-    TileTypeBitMask *mask;	/* Mask of those paint tiles to be passed to
-				 * func.
-				 */
-    int (*func)();		/* Function to apply at each tile */
-    ClientData arg;		/* Additional argument to pass to (*func)() */
-{
-    Point start;
-    Tile *tp, *tpnew;
-    Rect biggerRect;
-
-    /* If the rectangle has a zero x- or y-size, make a copy of it
-     * with a size of one.  For finding purposes, a degenerate area
-     * is equivalent to one with a dimension of 1, except that this
-     * code doesn't work right with a degenerate area if it falls
-     * on a tile boundary.
-     */
-    
-    if ((rect->r_xbot == rect->r_xtop) || (rect->r_ybot == rect->r_ytop))
-    {
-	biggerRect = *rect;
-	rect = &biggerRect;
-	if (rect->r_xbot == rect->r_xtop) rect->r_xtop++;
-	if (rect->r_ybot == rect->r_ytop) rect->r_ytop++;
-    }
-
-    start.p_x = rect->r_xtop - 1;
-    start.p_y = rect->r_ybot;
-    tp = hintTile ? hintTile : plane->pl_hint;
-    GOTOPOINT(tp, &start);
-
-    /* Each iteration visits another tile on the RHS of the search area */
-    while (BOTTOM(tp) < rect->r_ytop)
-    {
-	/* Each iteration enumerates another tile */
-enumerate:
-	plane->pl_hint = tp;
-	if (SigInterruptPending)
-	    return (1);
-
-	if (TTMaskHasType(mask, TiGetType(tp)) && (*func)(tp, arg))
-	    return (1);
-
-	tpnew = BL(tp);
-	if (RIGHT(tpnew) > rect->r_xbot)
-	{
-	    while (TOP(tpnew) <= rect->r_ybot) tpnew = RT(tpnew);
-	    if (TOP(tpnew) <= TOP(tp) || TOP(tp) >= rect->r_ytop)
-	    {
-		tp = tpnew;
-		goto enumerate;
-	    }
-	}
-
-	/* Each iteration returns one tile further to the right */
-	while (RIGHT(tp) < rect->r_xtop)
-	{
-	    if (TOP(tp) >= rect->r_ytop)
-		return (0);
-	    tpnew = RT(tp);
-	    tp = TR(tp);
-	    if (TOP(tpnew) <= TOP(tp) || TOP(tp) >= rect->r_ytop)
-	    {
-		tp = tpnew;
-		goto enumerate;
-	    }
-	}
-
-	/* At right edge -- walk down to next tile along the right edge */
-	for (tp = RT(tp); LEFT(tp) >= rect->r_xtop; tp = BL(tp))
-	    /* Nothing */;
-    }
-    return (0);
-}
 
 /*
  * --------------------------------------------------------------------
@@ -417,7 +583,8 @@ enumerate:
 		{
 		    TiSetBody(tp, (ClientData)((TileType)TiGetBody(tp)
 				& ~TT_SIDE));  /* bit clear */
-		    if ((*func)(tp, arg)) return (1);
+		    if ((tp->ti_client == client) && (*func)(tp, arg))
+			return (1);
 		}
 	    }
 
@@ -430,7 +597,8 @@ enumerate:
 		{
 		    TiSetBody(tp, (ClientData)((TileType)TiGetBody(tp)
 				| TT_SIDE));      /* bit set */
-		    if ((*func)(tp, arg)) return (1);
+		    if ((tp->ti_client == client) && (*func)(tp, arg))
+			return (1);
 		}
 	    }
 	}
@@ -600,7 +768,7 @@ enumerate:
 	/* Each iteration returns one tile further to the right */
 	while (RIGHT(tp) < rect->r_xtop)
 	{
-	    FREE((char *) tp); 
+	    freeMagic((char *) tp); 
 	    tpnew = RT(tp);
 	    tp = TR(tp);
 	    if (CLIP_TOP(tpnew) <= CLIP_TOP(tp) && BOTTOM(tpnew) < rect->r_ytop)
@@ -610,7 +778,7 @@ enumerate:
 	    }
 	}
 
-	FREE((char *) tp); 
+	freeMagic((char *) tp); 
 	/* At right edge -- walk up to next tile along the right edge */
 	tp = RT(tp);
 	if (BOTTOM(tp) < rect->r_ytop) {
@@ -684,10 +852,10 @@ dbFreeCellFunc(tile)
 	    use->cu_parent = (CellDef *) NULL;
 	    (void) DBCellDeleteUse(use);
 	}
-	FREE((char *) body);
+	freeMagic((char *) body);
     }
 
-    FREE((char *) tile);
+    freeMagic((char *) tile);
     return 0;
 }
 
