@@ -55,6 +55,15 @@ extern void CmdTech(), CmdTool(), CmdUnexpand();
 extern void CmdUpsidedown(), CmdWhat(), CmdWire(), CmdWriteall();
 extern void CmdGoto(), CmdFlatten(), CmdXload();
 
+/*
+ * TCL-only extensions
+ */
+
+#ifdef MAGIC_WRAPPER
+extern void CmdSearch();
+extern void CmdCellsearch();
+#endif
+
 /*
  * Nonmanhattan geometry extensions
  */
@@ -467,8 +476,26 @@ DBWInitCommands()
 	CmdAutoExtToSim, FALSE);
     WindAddCommand(DBWclientID,
 	"exttospice [args]	convert extracted file(s) to a SPICE format file;"
-	" type\n\t\t\t\"exttopice help\" for information on options",
+	" type\n\t\t\t\"exttospice help\" for information on options",
 	CmdAutoExtToSpice, FALSE);
+
+    /* Add the aliases ext2spice and ext2sim */
+    WindAddCommand(DBWclientID,
+	"ext2sim [args]	convert extracted file(s) to a sim format file;"
+	" type\n\t\t\t\"ext2sim help\" for information on options",
+	CmdAutoExtToSim, FALSE);
+    WindAddCommand(DBWclientID,
+	"ext2spice [args]	convert extracted file(s) to a SPICE format file;"
+	" type\n\t\t\t\"ext2spice help\" for information on options",
+	CmdAutoExtToSpice, FALSE);
+
+    WindAddCommand(DBWclientID,
+	"search [layers] proc	execute a TCL procedure on each tile in the box area.",
+	CmdSearch, FALSE);
+    WindAddCommand(DBWclientID,
+	"cellsearch [instance] proc\n\
+			execute a TCL procedure on each tile in the box area.",
+	CmdCellsearch, FALSE);
 #endif
 
 
