@@ -174,6 +174,11 @@ DBWredisplay(w, rootArea, clipArea)
 	    rootArea->r_xtop, rootArea->r_ytop);
     */
 
+#ifdef MAGIC_WRAPPER
+    /* Honor the display suspend state */
+    if (GrDisplayStatus == DISPLAY_SUSPEND) return;
+#endif
+
     GrLock(w, TRUE);
 
     /* Round up the redisplay area by 1 pixel on all sides.  This
