@@ -14,10 +14,9 @@
 
 #include <stdio.h>
 #include <string.h>
+#include <math.h>
 
-#ifdef MAGIC_WRAPPER
-#include "magic/tclmagic.h"
-#endif
+#include "tcltk/tclmagic.h"
 #include "misc/magic.h"
 #include "utils/geometry.h"
 #include "tiles/tile.h"
@@ -80,6 +79,24 @@ static CellDef *dbwelemRootDef;		/* To pass root cell definition from
 					 * DBWElementAdd.
 					 */
 
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * AppendString ---
+ *
+ *	Support function for DBWPrintElements().  Allocates memory for and
+ *	adds string "newstr" to the string pointed to by "oldstr".  If
+ *	"postfix" is non-NULL, this is also appended to "oldstr".
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	Memory reallocation of *oldstr.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
 void
 AppendString(oldstr, newstr, postfix)
     char **oldstr;
@@ -100,12 +117,38 @@ AppendString(oldstr, newstr, postfix)
     {
 	strcpy(tmpstr, *oldstr);
 	strcat(tmpstr, newstr);
-	FREE(*oldstr);
+	freeMagic(*oldstr);
     }
     if (postfix != NULL) strcat(tmpstr, postfix);
     *oldstr = tmpstr;
 }
 
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * AppendFlag --
+ *
+ *	Support function for DBWPrintElements to assist in generating
+ *	the string of comma-separated flags for each element printed
+ *	to the output.
+ *
+ * Results:
+ *	None.
+ *
+ * Side Effects:
+ *	Space is allocated and text added to "rstr".  The pointer to
+ *	boolean "flagset" is set to TRUE, indicating that at least one
+ *	flag has been written to the output.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+void AppendFlag(char **rstr, bool *flagset, char *fname)
+{
+    AppendString(rstr, *flagset ? "," : " ", fname);
+    *flagset = TRUE;
+}
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -135,7 +178,7 @@ DBWPrintElements(cellDef, flagmask)
     char *rstr = NULL;	/* allocated return string */
     char istr[10];
     styleptr sptr;
-    int flagset;
+    bool flagset;
 
     /* These must match the order of text sizes in graphics/graphics.h */
     static char *textSizes[] = {"small", "medium", "large", "xlarge",
@@ -158,12 +201,9 @@ DBWPrintElements(cellDef, flagmask)
 
 		/* print dstyle(s) */
 		for (sptr = elem->stylelist; sptr != NULL; sptr = sptr->next)
-		{
-		    AppendString(&rstr, GrStyleTable[sptr->style].longname, NULL);
-		    if (sptr->next != NULL)
-			AppendString(&rstr, ",", NULL);
-		}
-		AppendString(&rstr, " ");
+		    AppendString(&rstr, GrStyleTable[sptr->style].longname,
+			((sptr->next == NULL) ? " " : ","));
+
 		/* print start point */
 		sprintf(istr, "%d", elem->area.r_xbot);
 		AppendString(&rstr, istr, " ");
@@ -186,17 +226,18 @@ DBWPrintElements(cellDef, flagmask)
 			sprintf(istr, "%d", elem->area.r_xtop);
 			AppendString(&rstr, istr, " ");
 			sprintf(istr, "%d", elem->area.r_ytop);
+			AppendString(&rstr, istr, NULL);
 			/* any non-default flags? */
+			flagset = FALSE;
 			if (elem->flags & DBW_ELEMENT_LINE_HALFX)
-			    AppendString(&rstr, " halfx");
+			    AppendFlag(&rstr, &flagset, "halfx");
 			if (elem->flags & DBW_ELEMENT_LINE_HALFY)
-			{
-			    AppendString(&rstr,
-					(elem->flags & DBW_ELEMENT_LINE_HALFX) ?
-					"," : " ");
-			    AppendString(&rstr, "halfy");
-			}
-			AppendString(&rstr, istr, "\n");
+			    AppendFlag(&rstr, &flagset, "halfy");
+			if (elem->flags & DBW_ELEMENT_LINE_ARROWL)
+			    AppendFlag(&rstr, &flagset, "arrowleft");
+			if (elem->flags & DBW_ELEMENT_LINE_ARROWR)
+			    AppendFlag(&rstr, &flagset, "arrowright");
+			AppendString(&rstr, "\n", NULL);
 		        break;
 		    case ELEMENT_TEXT:
 		        /* label text */
@@ -204,23 +245,15 @@ DBWPrintElements(cellDef, flagmask)
 		        AppendString(&rstr, elem->text, NULL);
 		        AppendString(&rstr, "\"", NULL);
 			/* any non-default flags? */
-			flagset = 0x0;
+			flagset = FALSE;
 			if (((elem->flags & DBW_ELEMENT_TEXT_POS) >> 4) !=
 					GEO_CENTER)
-			    flagset |= 0x1;
+			    AppendFlag(&rstr, &flagset, GeoPosToName((elem->flags &
+					DBW_ELEMENT_TEXT_POS) >> 4));
 			if (((elem->flags & DBW_ELEMENT_TEXT_SIZE) >> 1) !=
 					GR_TEXT_MEDIUM)
-			    flagset |= 0x2;
-			if (flagset != 0)
-			    AppendString(&rstr, " ", NULL);
-			if (flagset & 0x1)
-			    AppendString(&rstr, GeoPosToName((elem->flags &
-					DBW_ELEMENT_TEXT_POS) >> 4),
-					(flagset & 0x2) ? "," : NULL);
-			if (flagset & 0x2)
-			    AppendString(&rstr, textSizes[(elem->flags &
-					DBW_ELEMENT_TEXT_SIZE) >> 1], 
-					NULL);
+			    AppendFlag(&rstr, &flagset, textSizes[(elem->flags &
+					DBW_ELEMENT_TEXT_SIZE) >> 1]);
 		        AppendString(&rstr, "\n", NULL);
 		        break;
 		}
@@ -296,7 +329,7 @@ DBWElementRedraw(window, plane)
 	    Rect transArea;
 	    int offx, offy;
 
-	    WindSurfaceToScreen(window, &unitArea, &transArea);
+	    WindSurfaceToScreenNoClip(window, &unitArea, &transArea);
 	    offx = (transArea.r_xtop - transArea.r_xbot) >> 1;
 	    offy = (transArea.r_ytop - transArea.r_ybot) >> 1;
 
@@ -329,10 +362,6 @@ DBWElementRedraw(window, plane)
 		    case ELEMENT_RECT:
 			GrFastBox(&screenArea, NULL);
 			break;
-		    case ELEMENT_LINE:
-			GrClipLine(screenArea.r_xbot, screenArea.r_ybot,
-				screenArea.r_xtop, screenArea.r_ytop);
-			break;
 		    case ELEMENT_TEXT:
 			p.p_x = screenArea.r_xbot;
 			p.p_y = screenArea.r_ybot;
@@ -341,6 +370,72 @@ DBWElementRedraw(window, plane)
 				((elem->flags & DBW_ELEMENT_TEXT_SIZE) >> 1),
 				FALSE, &(window->w_screenArea), (Rect *)NULL);
 			break;
+		    case ELEMENT_LINE:
+			GrClipLine(screenArea.r_xbot, screenArea.r_ybot,
+				screenArea.r_xtop, screenArea.r_ytop);
+#ifdef NONMANHATTAN
+			/* Draw arrowheads on endpoints, if flags are set */
+
+			if (elem->flags & (DBW_ELEMENT_LINE_ARROWL |
+					DBW_ELEMENT_LINE_ARROWR)) {
+			    static Rect unitArea = {{0, 0}, {1, 1}};
+			    Rect transArea;
+			    Point polyp[4];
+			    double theta, r;
+			    int i, offx, offy;
+
+			    WindSurfaceToScreenNoClip(window, &unitArea, &transArea);
+			    offx = (transArea.r_xtop - transArea.r_xbot) >> 1;
+			    offy = (transArea.r_ytop - transArea.r_ybot) >> 1;
+			    WindSurfaceToScreenNoClip(window, &elem->area, &screenArea);
+			    if (elem->flags & DBW_ELEMENT_LINE_HALFX)
+			    {
+                                screenArea.r_xbot += offx;
+                                screenArea.r_xtop += offx;
+			    }
+			    if (elem->flags & DBW_ELEMENT_LINE_HALFY)
+			    {
+                                screenArea.r_ybot += offy;
+                                screenArea.r_ytop += offy;
+			    }
+			    theta = atan2((double)(screenArea.r_ytop
+					- screenArea.r_ybot), (double)
+                                         (screenArea.r_xtop - screenArea.r_xbot));
+			    r = (double)(transArea.r_xtop - transArea.r_xbot);
+			    if (elem->flags & DBW_ELEMENT_LINE_ARROWL)
+			    {
+				for (i = 0; i < 4; i++)
+				{
+				    polyp[i].p_x = screenArea.r_xbot;
+				    polyp[i].p_y = screenArea.r_ybot;
+				}
+				polyp[1].p_x += (int)(r * cos(theta + 0.2));
+				polyp[1].p_y += (int)(r * sin(theta + 0.2));
+				polyp[2].p_x += (int)(r * 0.9 * cos(theta));
+				polyp[2].p_y += (int)(r * 0.9 * sin(theta));
+				polyp[3].p_x += (int)(r * cos(theta - 0.2));
+				polyp[3].p_y += (int)(r * sin(theta - 0.2));
+				GrFillPolygon(polyp, 4);
+			    }
+
+			    if (elem->flags & DBW_ELEMENT_LINE_ARROWR)
+			    {
+				for (i = 0; i < 4; i++)
+				{
+				    polyp[i].p_x = screenArea.r_xtop;
+				    polyp[i].p_y = screenArea.r_ytop;
+				}
+				polyp[1].p_x -= (int)(r * cos(theta + 0.2));
+				polyp[1].p_y -= (int)(r * sin(theta + 0.2));
+				polyp[2].p_x -= (int)(r * 0.9 * cos(theta));
+				polyp[2].p_y -= (int)(r * 0.9 * sin(theta));
+				polyp[3].p_x -= (int)(r * cos(theta - 0.2));
+				polyp[3].p_y -= (int)(r * sin(theta - 0.2));
+				GrFillPolygon(polyp, 4);
+			    }
+			}
+#endif
+			break;
 		}
 	    }
 	}
@@ -812,7 +907,9 @@ DBWElementParseFlags(MagWindow *w, char *ename, char *flagstr)
     HashEntry *entry;
     int flidx, newflags;
 
-    static char *lineOffset[] = {"halfx", "halfy", "exactx", "exacty", NULL};
+    static char *lineOffset[] = {"halfx", "halfy", "exactx", "exacty",
+		"arrowleft", "arrowbottom", "arrowright", "arrowtop",
+		"plainleft", "plainbottom", "plainright", "plaintop", NULL};
     static char *textSizes[] = {"small", "medium", "large", "xlarge",
 	"default", NULL}; 
     static char *genFlags[] = {"persistent", "temporary", NULL}; 
@@ -888,6 +985,22 @@ DBWElementParseFlags(MagWindow *w, char *ename, char *flagstr)
 			    case 3:
 				newflags &= ~(DBW_ELEMENT_LINE_HALFY);
 				break;
+			    case 4:
+			    case 5:
+				newflags |= DBW_ELEMENT_LINE_ARROWL;
+				break;
+			    case 6:
+			    case 7:
+				newflags |= DBW_ELEMENT_LINE_ARROWR;
+				break;
+			    case 8:
+			    case 9:
+				newflags &= ~(DBW_ELEMENT_LINE_ARROWL);
+				break;
+			    case 10:
+			    case 11:
+				newflags &= ~(DBW_ELEMENT_LINE_ARROWR);
+				break;
 			    default:
 				TxError("No such line element flag \"%s\"\n",
 					flagstr);
