@@ -125,10 +125,10 @@ DBWcreate(window, argc, argv)
 
     window->w_clientData = (ClientData) crec;
     if (argc > 0)
-	DBWloadWindow(window, argv[0]);
+	DBWloadWindow(window, argv[0], TRUE, FALSE);
     else if (ToolGetBox(&boxDef, &box))
     {
-	DBWloadWindow(window, boxDef->cd_name);
+	DBWloadWindow(window, boxDef->cd_name, TRUE, FALSE);
 
 	/* Zoom in on the box, leaving a 10% border or at least 2 units
 	 * on each side.
@@ -146,7 +146,7 @@ DBWcreate(window, argc, argv)
     }
     else
     {
-	DBWloadWindow(window, (char *) NULL);
+	DBWloadWindow(window, (char *) NULL, TRUE, FALSE);
     }
     return TRUE;
 }
@@ -245,7 +245,7 @@ dbwReloadFunc(w, name)
     MagWindow *w;
     char *name;
 {
-    DBWloadWindow(w, name);
+    DBWloadWindow(w, name, TRUE, FALSE);
     return (0);
 }
 
@@ -269,13 +269,19 @@ dbwReloadFunc(w, name)
  *	to the topmost cell in the new window.  Otherwise, the edit
  *	cell doesn't change.
  *
+ *	If "expand" is true, unexpands all subcells of the root cell.
+ *
  * ----------------------------------------------------------------------------
  */
 
 void
-DBWloadWindow(window, name)
+DBWloadWindow(window, name, ignoreTech, expand)
     MagWindow *window;	/* Identifies window to which cell is to be bound */
     char *name;		/* Name of new cell to be bound to this window */
+    bool ignoreTech;	/* If FALSE, indicates that the technology of
+			 * the layout must match the current technology.
+			 */
+    bool expand;	/* Indicates whether or not to expand the cell */
 {
     CellDef *newEditDef;
     CellUse *newEditUse;
@@ -286,6 +292,7 @@ DBWloadWindow(window, name)
 #ifdef FILE_LOCKS
     FILE *dummy = NULL;
 #endif
+   int UnexpandFunc();		/* forward declaration */
 
     loadBox.r_xbot = loadBox.r_ybot = 0;
     loadBox.r_xtop = loadBox.r_ytop = 1;
@@ -326,7 +333,7 @@ DBWloadWindow(window, name)
 	if (newEditDef == (CellDef *) NULL)
 	    newEditDef = DBCellNewDef(name, (char *) NULL);
 
-	if (!DBCellRead(newEditDef, (char *) NULL, TRUE))
+	if (!DBCellRead(newEditDef, (char *) NULL, ignoreTech))
 	{
 	    TxPrintf("Creating new cell\n");
 	    DBCellSetAvail(newEditDef);
@@ -354,156 +361,12 @@ DBWloadWindow(window, name)
     (void) StrDup(&(newEditUse->cu_id), "Topmost cell in the window");
     DBExpand(newEditUse,
 	((DBWclientRec *)window->w_clientData)->dbw_bitmask, TRUE);
-    if (newEdit)
-    {
-	if (EditCellUse && EditRootDef)
-	{
-	    DBWUndoOldEdit(EditCellUse, EditRootDef,
-		    &EditToRootTransform, &RootToEditTransform);
-	    DBWUndoNewEdit(newEditUse, newEditDef,
-		    &GeoIdentityTransform, &GeoIdentityTransform);
-	}
-	if (newEditUse->cu_def->cd_flags & CDNOEDIT)
-	{
-	    newEdit = FALSE;
-	}
-	else
-	{
-	    EditCellUse = newEditUse;
-	    EditRootDef = newEditDef;
-	}
 
-	EditToRootTransform = GeoIdentityTransform;
-	RootToEditTransform = GeoIdentityTransform;
-    }
-
-    /* enforce a minimum size of 60 and a border of 10% around the sides */
-    xadd = MAX(0, (60 - (loadBox.r_xtop - loadBox.r_xbot)) / 2) + 
-	(loadBox.r_xtop - loadBox.r_xbot + 1) / 10;
-    yadd = MAX(0, (60 - (loadBox.r_ytop - loadBox.r_ybot)) / 2) +
-	(loadBox.r_ytop - loadBox.r_ybot + 1) / 10;
-    loadBox.r_xbot -= xadd;  loadBox.r_xtop += xadd;
-    loadBox.r_ybot -= yadd;  loadBox.r_ytop += yadd;
-
-    window->w_bbox = &(newEditUse->cu_def->cd_bbox);
-    res = WindLoad(window, DBWclientID, (ClientData) newEditUse, &loadBox);
-    ASSERT(res, "DBWcreate");
-
-    /* Update the captions in all windows to reflect the new
-     * edit cell.  Also, if we've got a new edit cell, we need
-     * to explicitly ask for redisplay, because there could be
-     * another window on this cell somewhere else, and it needs
-     * to be redisplayed too (if it's just the new window, that
-     * is taken care of during WindLoad).
-     */
-    CmdSetWindCaption(EditCellUse, EditRootDef);
-    if (newEdit)
-	DBWAreaChanged(newEditDef, &newEditDef->cd_bbox, DBW_ALLWINDOWS,
-	    &DBAllButSpaceBits);
-}
-
-
-/*
- * ----------------------------------------------------------------------------
- *
- * DBWxloadWindow
- *
- *	Replace the root cell of a window by the specified cell.
- *
- *	A cell name of NULL causes the cell with name "(UNNAMED)" to be
- *	created if it does not already exist, or used if it does.
- *
- * Results:
- *	None.
- *
- * Side effects:
- *	If there is currently no edit use, or if this is the only
- *	window containing the edit cell, then set the edit cell
- *	to the topmost cell in the new window.  Otherwise, the edit
- *	cell doesn't change.
- *	Unexpands all subcells of the root cell.
- *
- * ----------------------------------------------------------------------------
- */
-
-void
-DBWxloadWindow(window, name)
-    MagWindow *window;	/* Identifies window to which cell is to be bound */
-    char *name;		/* Name of new cell to be bound to this window */
-{
-    CellDef *newEditDef;
-    CellUse *newEditUse;
-    void DisplayWindow();
-    int res, newEdit;
-    int xadd, yadd;
-    Rect loadBox;
-    int UnexpandFunc();		/* Forward reference. */
-#ifdef FILE_LOCKS
-    FILE *dummy = NULL;
-#endif
-
-    loadBox.r_xbot = loadBox.r_ybot = 0;
-    loadBox.r_xtop = loadBox.r_ytop = 1;
-
-    /* See if we're to change the edit cell */
-    newEdit = !WindSearch((WindClient) DBWclientID, (ClientData) NULL,
-			    (Rect *) NULL, dbwLoadFunc, (ClientData) window);
-
-    if (name == (char *) NULL)
-    {
-	/*
-	 * If there is an existing unnamed cell, we use it.
-	 * Otherwise, we create one afresh.
-	 */
-	newEditDef = DBCellLookDef(UNNAMED);
-	if (newEditDef == (CellDef *) NULL)
-	{
-	    newEditDef = DBCellNewDef(UNNAMED, (char *) NULL);
-	    DBCellSetAvail(newEditDef);
-	}
-    }
-    else
-    {
-	/*
-	 * Name specified.
-	 * First try to find it in main memory, then try to
-	 * read it from disk.
-	 */
-	newEditDef = DBCellLookDef(name);
-	if (newEditDef == (CellDef *) NULL)
-	    newEditDef = DBCellNewDef(name, (char *) NULL);
-
-	if (!DBCellRead(newEditDef, (char *) NULL, TRUE))
-	{
-	    TxPrintf("Creating new cell\n");
-	    DBCellSetAvail(newEditDef);
-#ifdef FILE_LOCKS
-	    flock_test_set(newEditDef->cd_file, dummy);
-#endif
-	}
-	else
-	{
-	    /* DBCellRead doesn't dare to change bounding boxes, so
-	     * we have to call DBReComputeBbox here (we know that it's
-	     * safe).
-	     */
-	    DBReComputeBbox(newEditDef);
-	    loadBox = newEditDef->cd_bbox;
-	}
-    }
-
-    /*
-     * Attach the new cell to the selected window.
-     */
-
-    newEditUse = DBCellNewUse(newEditDef, (char *) NULL);
-    (void) StrDup(&(newEditUse->cu_id), "Topmost cell in the window");
-    DBExpand(newEditUse,
-	((DBWclientRec *)window->w_clientData)->dbw_bitmask, TRUE);
-    DBExpandAll(newEditUse, &(newEditUse->cu_bbox),
-	((DBWclientRec *)window->w_clientData)->dbw_bitmask, FALSE,
-	UnexpandFunc,
-	(ClientData) (((DBWclientRec *)window->w_clientData)->dbw_bitmask));
+    if (expand)
+	DBExpandAll(newEditUse, &(newEditUse->cu_bbox),
+		((DBWclientRec *)window->w_clientData)->dbw_bitmask,
+		FALSE, UnexpandFunc,
+		(ClientData) (((DBWclientRec *)window->w_clientData)->dbw_bitmask));
 
     if (newEdit)
     {
@@ -514,7 +377,6 @@ DBWxloadWindow(window, name)
 	    DBWUndoNewEdit(newEditUse, newEditDef,
 		    &GeoIdentityTransform, &GeoIdentityTransform);
 	}
-
 	if (newEditUse->cu_def->cd_flags & CDNOEDIT)
 	{
 	    newEdit = FALSE;
