@@ -62,7 +62,7 @@ static Rect boxRootArea;		/* Root def coords */
  * grid always, instead of snapping to the usual 1x1 grid.  If the value
  * is DBW_SNAP_INTERNAL, the box gets snapped to the internal grid.
  */
-bool DBWSnapToGrid = DBW_SNAP_LAMBDA;
+int DBWSnapToGrid = DBW_SNAP_LAMBDA;
 
 /* Forward reference: */
 
