@@ -8,7 +8,8 @@
  * rcsid $Header: /ufs/repository/magic/dbwind/dbwtech.h,v 1.1.1.1 2000/03/29 18:35:24 rajit Exp $
  */
 
-#define	_DISTECH
+#ifndef _DBWTECH_H
+#define	_DBWTECH_H
 
 extern TileTypeBitMask	*DBWStyleToTypesTbl;
 
@@ -17,3 +18,4 @@ extern TileTypeBitMask	*DBWStyleToTypesTbl;
 /* forward declarations */
 int  DBWTechParseStyle();
 
+#endif /* _DBWTECH_H */
