@@ -102,7 +102,6 @@ areaCheck(tile, arg)
     if ((rect.r_xbot >= rect.r_xtop) || (rect.r_ybot >= rect.r_ytop))
 	return 0;
 
-#ifdef DRC_EUCLIDEAN
     /* Apply extra clipping for radial distance check on corner
      * extensions.
      */
@@ -133,7 +132,6 @@ areaCheck(tile, arg)
 	    }
 	}
     }
-#endif
 
     (*(arg->dCD_function)) (arg->dCD_celldef, &rect,
 	arg->dCD_cptr, arg->dCD_clientData);
@@ -257,9 +255,7 @@ drcTile (tile, arg)
     unsigned char stackpending = (char)0;
 
     arg->dCD_constraint = &errRect;
-#ifdef DRC_EUCLIDEAN
     arg->dCD_radial = 0; 
-#endif          
 
     /*
      * If we were interrupted, we want to
@@ -350,12 +346,23 @@ drcTile (tile, arg)
 	    /*
 	     * Stacked contact types have no rules.  Any stacked contact
 	     * type must have DRC rules applied to each of its constituent
-	     * types.  Start with the first type encountered for each.
+	     * types.  But, if the residue mask for the stacking type
+	     * contains the other type, then this is a "false" edge, and
+	     * we will not process it.  Otherwise, start with the first
+	     * type encountered for each.
 	     */
 	    stackpending = (unsigned char)0x0;
 	    if (tt >= DBNumUserLayers)
 	    {
 		rMask = DBResidueMask(tt);
+		if (to >= DBNumUserLayers)
+		{
+		    TileTypeBitMask combined;
+		    TTMaskAndMask3(&combined, rMask, DBResidueMask(to));
+		    if (!TTMaskIsZero(&combined))
+			continue;
+		}
+		else if (TTMaskHasType(rMask, to)) continue;
 		stackpending |= (unsigned char)0x01;
 		for (tt = TT_TECHDEPBASE; tt < DBNumUserLayers; tt++)
 		    if (TTMaskHasType(rMask, tt))
@@ -364,6 +371,7 @@ drcTile (tile, arg)
 	    if (to >= DBNumUserLayers)
 	    {
 		rMask = DBResidueMask(to);
+		if (TTMaskHasType(rMask, tt)) continue;
 		stackpending |= (unsigned char)0x02;
 		for (to = TT_TECHDEPBASE; to < DBNumUserLayers; to++)
 		    if (TTMaskHasType(rMask, to))
@@ -389,7 +397,13 @@ drcTile (tile, arg)
 
 		    /* Contact images exist in multple planes; only check	*/
 		    /* rules that are for the plane being checked.		*/
-		    if (arg->dCD_plane != cptr->drcc_plane) continue;
+		    /* NOTE:  Ideally, the plane of the edge should be recorded	*/
+		    /* in the rule; i.e., we should have cptr->drcc_edgeplane	*/
+		    /* and compare that to arg->dCD_plane.  However, the	*/
+		    /* following eliminates most of the redundant checks.	*/
+		
+		    if ((!(cptr->drcc_flags & DRC_XPLANE)) &&
+				(arg->dCD_plane != cptr->drcc_plane)) continue;
 
 		    DRCstatRules++;
 		    errRect.r_ytop = edgeTop;
@@ -439,9 +453,8 @@ drcTile (tile, arg)
 			if (TTMaskHasType(&cptr->drcc_corner, TiGetTopType(tp)))
 		 	{
 			    errRect.r_ybot -= cptr->drcc_cdist;
-#ifdef DRC_EUCLIDEAN
-			    arg->dCD_radial |= 0x1000;
-#endif
+			    if (DRCEuclidean)
+				arg->dCD_radial |= 0x1000;
 			}
 
 			if (cptr->drcc_flags & DRC_BOTHCORNERS)
@@ -456,9 +469,8 @@ drcTile (tile, arg)
 			    if (TTMaskHasType(&cptr->drcc_corner, TiGetBottomType(tp)))
 			    {
 				errRect.r_ytop += cptr->drcc_cdist;
-#ifdef DRC_EUCLIDEAN
-				arg->dCD_radial |= 0x2000;
-#endif
+				if (DRCEuclidean)
+				    arg->dCD_radial |= 0x2000;
 			    }
 			}
 
@@ -491,9 +503,8 @@ drcTile (tile, arg)
 			if (TTMaskHasType(&cptr->drcc_corner, TiGetBottomType(tp)))
 			{
 			    errRect.r_ytop += cptr->drcc_cdist;
-#ifdef DRC_EUCLIDEAN
-			    arg->dCD_radial |= 0x8000;
-#endif
+			    if (DRCEuclidean)
+				arg->dCD_radial |= 0x8000;
 			}
 
 			if (cptr->drcc_flags & DRC_BOTHCORNERS)
@@ -508,9 +519,8 @@ drcTile (tile, arg)
 			    if (TTMaskHasType(&cptr->drcc_corner, TiGetTopType(tp)))
 			    {
 				errRect.r_ybot -= cptr->drcc_cdist;
-#ifdef DRC_EUCLIDEAN
-				arg->dCD_radial |= 0x4000;
-#endif
+				if (DRCEuclidean)
+				    arg->dCD_radial |= 0x4000;
 			    }
 			}
 
@@ -532,13 +542,11 @@ drcTile (tile, arg)
 			arg->dCD_which = DRC_RIGHT;
 #endif
 		    }
-#ifdef DRC_EUCLIDEAN
 		    if (arg->dCD_radial)
 		    {
 			arg->dCD_radial &= 0xf000;
 			arg->dCD_radial |= (0xfff & cptr->drcc_cdist);
 		    }
-#endif
 
 		    /* Trigger rule:  If rule check passed without	*/
 		    /* errors, do the next rule.  Otherwise, skip	*/
@@ -679,6 +687,14 @@ checkbottom:
 	    if (tt >= DBNumUserLayers)
 	    {
 		rMask = DBResidueMask(tt);
+		if (to >= DBNumUserLayers)
+		{
+		    TileTypeBitMask combined;
+		    TTMaskAndMask3(&combined, rMask, DBResidueMask(to));
+		    if (!TTMaskIsZero(&combined))
+			continue;
+		}
+		else if (TTMaskHasType(rMask, to)) continue;
 		stackpending |= (unsigned char)0x01;
 		for (tt = TT_TECHDEPBASE; tt < DBNumUserLayers; tt++)
 		    if (TTMaskHasType(rMask, tt))
@@ -687,6 +703,7 @@ checkbottom:
 	    if (to >= DBNumUserLayers)
 	    {
 		rMask = DBResidueMask(to);
+		if (TTMaskHasType(rMask, tt)) continue;
 		stackpending |= (unsigned char)0x02;
 		for (to = TT_TECHDEPBASE; to < DBNumUserLayers; to++)
 		    if (TTMaskHasType(rMask, to))
@@ -712,7 +729,8 @@ checkbottom:
 		{
 		    /* Contact images exist in multple planes; only check	*/
 		    /* rules that are for the plane being checked.		*/
-		    if (arg->dCD_plane != cptr->drcc_plane) continue;
+		    if ((!(cptr->drcc_flags & DRC_XPLANE)) &&
+				(arg->dCD_plane != cptr->drcc_plane)) continue;
 
 		    DRCstatRules++;
 		    errRect.r_xbot = edgeLeft;
@@ -751,9 +769,8 @@ checkbottom:
 		 	if (TTMaskHasType(&cptr->drcc_corner, TiGetLeftType(tp)))
 			{
 			    errRect.r_xtop += cptr->drcc_cdist; 	
-#ifdef DRC_EUCLIDEAN
-			    arg->dCD_radial |= 0x4000;
-#endif
+			    if (DRCEuclidean)
+				arg->dCD_radial |= 0x4000;
 			}
 
 			if (cptr->drcc_flags & DRC_BOTHCORNERS)
@@ -767,9 +784,8 @@ checkbottom:
 			    if (TTMaskHasType(&cptr->drcc_corner, TiGetRightType(tp)))
 			    {
 				errRect.r_xbot -= cptr->drcc_cdist; 	
-#ifdef DRC_EUCLIDEAN
-				arg->dCD_radial |= 0x1000;
-#endif
+				if (DRCEuclidean)
+				    arg->dCD_radial |= 0x1000;
 			    }
 			}
 
@@ -804,9 +820,8 @@ checkbottom:
 			if (TTMaskHasType(&cptr->drcc_corner, TiGetTopType(tp)))
 			{
 			    errRect.r_xbot -= cptr->drcc_cdist;
-#ifdef DRC_EUCLIDEAN
-			    arg->dCD_radial |= 0x2000;
-#endif
+			    if (DRCEuclidean)
+				arg->dCD_radial |= 0x2000;
 			}
 
 			if (cptr->drcc_flags & DRC_BOTHCORNERS)
@@ -820,9 +835,8 @@ checkbottom:
 			    if (TTMaskHasType(&cptr->drcc_corner, TiGetTopType(tp)))
 			    {
 				errRect.r_xtop += cptr->drcc_cdist;
-#ifdef DRC_EUCLIDEAN
-				arg->dCD_radial |= 0x8000;
-#endif
+				if (DRCEuclidean)
+				    arg->dCD_radial |= 0x8000;
 			    }
 			}
 
@@ -844,13 +858,11 @@ checkbottom:
 			arg->dCD_which = DRC_TOP;
 #endif
 		    }
-#ifdef DRC_EUCLIDEAN
 		    if (arg->dCD_radial)
 		    {
 			arg->dCD_radial &= 0xf000;
 			arg->dCD_radial |= (0xfff & cptr->drcc_cdist);
 		    }
-#endif
 		    /* Trigger rule:  If trigger check passed, do	*/
 		    /* the next rule.  Otherwise, skip it.		*/
 
