@@ -63,6 +63,18 @@ global unsigned char DRCStatus = DRC_NOT_RUNNING;
 
 global bool DRCBackGround = TRUE;
 
+/* Global variable, settable by outside world, that enables
+ * and disables Euclidean DRC checking.  If enabled, magic
+ * computes corner-to-corner distances using a Euclidean metric.
+ * If disabled, magic computes corner-to-corner distances using
+ * a Manhattan metric.
+ *
+ * Note:  should this feature be settable from the technology
+ * file, rather than from the command line?
+ */
+
+global bool DRCEuclidean = FALSE;
+
 /* Global variable that defines the size of chunks into which
  * to decompose large DRC areas.  Also used to guarantee a
  * canonical determination of interaction areas.  May be
@@ -257,6 +269,49 @@ DRCCheckThis (celldef, operation, area)
     }
 }
 
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * DRCRemovePending --
+ *
+ *	Remove a cell def from the DRC pending list.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	DRCPendingRoot linked list modified
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+void
+DRCRemovePending(def)
+   CellDef *def;
+{
+    DRCPendingCookie *p, *plast;
+
+    p = DRCPendingRoot;
+    plast = NULL;
+
+    while (p != NULL)
+    {
+	if (p->dpc_def == def)
+	{
+	    if (plast == NULL)
+		DRCPendingRoot = p->dpc_next;
+	    else
+		plast->dpc_next = p->dpc_next;
+
+	    freeMagic(p);
+	    return;
+	}
+	plast = p;
+	p = p->dpc_next;
+    }
+}
+
 /*
  * ----------------------------------------------------------------------------
  *
