@@ -132,7 +132,7 @@ drcWhyClear()
 	char * old = (char *) drcWhyList;
 	(void) StrDup(&(drcWhyList->dwl_string), (char *) NULL);
 	drcWhyList = drcWhyList->dwl_next;
-	FREE(old);
+	freeMagic(old);
     }
 }
 
@@ -157,7 +157,7 @@ drcWhyDup(why)
 {
     struct drcwhylist * new;
 
-    MALLOC(struct drcwhylist *, new, sizeof *new);
+    new = (struct drcwhylist *) mallocMagic((unsigned) (sizeof *new));
     new->dwl_string = StrDup((char **) NULL, why);
     new->dwl_next = drcWhyList;
     drcWhyList = new;
@@ -253,9 +253,9 @@ DRCTechInit()
 	    {
 		char * old = (char *) dp;
 		dp = dp->drcc_next;
-		FREE(old);
+		freeMagic(old);
 	    }
-	    MALLOC(DRCCookie *, dp, sizeof (DRCCookie));
+	    dp = (DRCCookie *) mallocMagic((unsigned) (sizeof (DRCCookie)));
 	    dp->drcc_dist = -1;
 	    dp->drcc_next = (DRCCookie *) NULL;
 	    DRCRulesTbl[i][j] = dp;
@@ -521,8 +521,8 @@ drcWidth(argc, argv)
 		    && TTMaskHasType(&setC, i) && TTMaskHasType(&set, j))
 	    {
 		/* find bucket preceding the new one we wish to insert */
-		dp = drcFindBucket(i, j);
-		MALLOC(DRCCookie *, dpnew, sizeof (DRCCookie));
+		dp = drcFindBucket(i, j, distance);
+		dpnew = (DRCCookie *) mallocMagic ((unsigned)sizeof (DRCCookie));
 		ASSIGN(dpnew, distance, dp->drcc_next, set, set, why,
 			    distance, DRC_FORWARD, plane);
 
@@ -600,7 +600,7 @@ drcArea(argc, argv)
 	    {
 		/* find bucket preceding the new one we wish to insert */
 		dp = drcFindBucket(i, j, horizon);
-		MALLOC(DRCCookie *, dpnew, sizeof (DRCCookie));
+		dpnew = (DRCCookie *) mallocMagic((unsigned) (sizeof (DRCCookie)));
 		ASSIGN(dpnew, horizon, dp->drcc_next, set, set, why,
 			    distance, DRC_AREA|DRC_FORWARD, plane);
 
@@ -700,7 +700,7 @@ drcMaxwidth(argc, argv)
 	    {
 		/* find bucket preceding the new one we wish to insert */
 		dp = drcFindBucket(i, j, distance);
-		MALLOC(DRCCookie *, dpnew, sizeof (DRCCookie));
+		dpnew = (DRCCookie *) mallocMagic((unsigned) (sizeof (DRCCookie)));
 		ASSIGN(dpnew, distance, dp->drcc_next, set, set, why,
 			    distance, DRC_MAXWIDTH|bend, plane);
 
@@ -907,7 +907,7 @@ drcSpacing(argc, argv)
 		for (curPlane = PL_TECHDEPBASE; curPlane < DBNumPlanes; curPlane++)
 		{
 		    if (!PlaneMaskHasPlane(planes2, curPlane)) continue;
-		    MALLOC(DRCCookie *, dpnew, sizeof (DRCCookie));
+		    dpnew = (DRCCookie *) mallocMagic((unsigned) (sizeof (DRCCookie)));
 		    TTMaskClearMask3(&tmp1, &DBPlaneTypes[curPlane], &set2);
 		    TTMaskAndMask3(&tmp2, &DBPlaneTypes[curPlane], &setR);
 
@@ -926,7 +926,7 @@ drcSpacing(argc, argv)
 		        ASSIGN(dpnew, distance, dp->drcc_next, tmp1, tmp2,
 				why, distance, DRC_FORWARD, curPlane);
 
-			MALLOC(DRCCookie *, dptrig, sizeof (DRCCookie));
+			dptrig = (DRCCookie *) mallocMagic((unsigned) (sizeof (DRCCookie)));
 		        ASSIGN(dptrig, wwidth, dpnew, set1, set1, why,
 				wwidth, DRC_REVERSE | DRC_TRIGGER, curPlane);
 			dp->drcc_next = dptrig;
@@ -964,7 +964,7 @@ drcSpacing(argc, argv)
 			 
 			/* find bucket preceding new one we wish to insert */
 			dp = drcFindBucket(j, i, distance);
-			MALLOC(DRCCookie *, dpnew, sizeof (DRCCookie));
+			dpnew = (DRCCookie *) mallocMagic((unsigned) (sizeof (DRCCookie)));
 			if (widerule)
 			{
 			    DRCCookie *dptrig;
@@ -974,7 +974,7 @@ drcSpacing(argc, argv)
 			    ASSIGN(dpnew, distance, dp->drcc_next,
 				tmp1, tmp2, why, distance,
 				DRC_REVERSE | DRC_BOTHCORNERS, curPlane);
-			    MALLOC(DRCCookie *, dptrig, sizeof (DRCCookie));
+			    dptrig = (DRCCookie *) mallocMagic((unsigned) (sizeof (DRCCookie)));
 			    ASSIGN(dptrig, wwidth, dpnew, set1, set1, why,
 				wwidth, DRC_FORWARD | DRC_TRIGGER, curPlane);
 			    dp->drcc_next = dptrig;
@@ -1009,7 +1009,7 @@ drcSpacing(argc, argv)
 		{
 		    if (!PlaneMaskHasPlane(planes1, curPlane)) continue;
 
-		    MALLOC(DRCCookie *, dpnew, sizeof (DRCCookie));
+		    dpnew = (DRCCookie *) mallocMagic ((unsigned) (sizeof (DRCCookie)));
 		    TTMaskClearMask3(&tmp1, &DBPlaneTypes[curPlane], &set1);
 		    TTMaskAndMask3(&tmp2, &DBPlaneTypes[curPlane], &setRreverse);
 
@@ -1039,11 +1039,11 @@ drcSpacing(argc, argv)
 			 * rules only used with touching_ok which must be 
 			 * single plane)
 			 * so am not setting DRC_XPLANE as above.
-			 * /
+			 */
 			 
 			/* find bucket preceding new one we wish to insert */
 			dp = drcFindBucket(j, i, distance);
-			MALLOC(DRCCookie *, dpnew, sizeof (DRCCookie));
+			dpnew = (DRCCookie *) mallocMagic((unsigned) (sizeof (DRCCookie)));
 			ASSIGN(dpnew, distance, dp->drcc_next,
 				tmp1, tmp2, why, distance, 
 				DRC_REVERSE | DRC_BOTHCORNERS, curPlane);
@@ -1103,7 +1103,7 @@ drcSpacing(argc, argv)
 		    }
 
 		    /* create new check and add it to list */
-		    MALLOC(DRCCookie *, dpnew, sizeof (DRCCookie));
+		    dpnew = (DRCCookie *) mallocMagic((unsigned) (sizeof (DRCCookie)));
 		    TTMaskClearMask3(&tmp1, &DBPlaneTypes[curPlane], &set2);
 		    TTMaskZero(&tmp2);
 
@@ -1247,8 +1247,6 @@ drcEdge(argc, argv)
 	}
     }
 
-    setM = tmp3;
-
     /* "plane" argument deprecated; kept for backward compatibility only */
 
     if (argc == 9 && ((tmpPlane < 0) || (tmpPlane != checkPlane)))
@@ -1262,7 +1260,7 @@ drcEdge(argc, argv)
 	    {
 		/* Find bucket preceding the new one we wish to insert */
 		dp = drcFindBucket(i, j, distance);
-		MALLOC(DRCCookie *, dpnew, sizeof (DRCCookie));
+		dpnew = (DRCCookie *) mallocMagic((unsigned) (sizeof (DRCCookie)));
 		ASSIGN(dpnew, distance, dp->drcc_next,
 		    setM, setC, why, cdist, DRC_FORWARD, checkPlane);
 		if (fourway) dpnew->drcc_flags |= DRC_BOTHCORNERS;
@@ -1274,7 +1272,7 @@ drcEdge(argc, argv)
 		{
 		    /* find bucket preceding new one we wish to insert */
 		    dp = drcFindBucket(j, i, distance);
-		    MALLOC(DRCCookie *, dpnew, sizeof (DRCCookie));
+		    dpnew = (DRCCookie *) mallocMagic ((unsigned) (sizeof (DRCCookie)));
 		    ASSIGN(dpnew,distance,dp->drcc_next,
 			    setM, setC, why, cdist, DRC_REVERSE, checkPlane);
 		    dpnew->drcc_flags |= DRC_BOTHCORNERS;
@@ -1364,7 +1362,7 @@ drcOverhang(argc, argv)
 	    {
 		/* Find bucket preceding the new one we wish to insert */
 		dp = drcFindBucket(i, j, distance);
-		MALLOC(DRCCookie *, dpnew, sizeof (DRCCookie));
+		dpnew = (DRCCookie *) mallocMagic ((unsigned) (sizeof (DRCCookie)));
 		ASSIGN(dpnew, distance, dp->drcc_next,
 			setM, setC, why, 0, DRC_FORWARD, plane);
 		dpnew->drcc_flags |= DRC_BOTHCORNERS;
@@ -1372,7 +1370,7 @@ drcOverhang(argc, argv)
 
 		/* find bucket preceding new one we wish to insert */
 		dp = drcFindBucket(j, i, distance);
-		MALLOC(DRCCookie *, dpnew, sizeof (DRCCookie));
+		dpnew = (DRCCookie *) mallocMagic((unsigned) (sizeof (DRCCookie)));
 		ASSIGN(dpnew, distance, dp->drcc_next,
 			setM, setC, why, 0, DRC_REVERSE, plane);
 		dpnew->drcc_flags |= DRC_BOTHCORNERS;
@@ -1452,7 +1450,7 @@ drcRectOnly(argc, argv)
 	    {
 		/* Find bucket preceding the new one we wish to insert */
 		dp = drcFindBucket(i, j, 1);
-		MALLOC(DRCCookie *, dpnew, sizeof (DRCCookie));
+		dpnew = (DRCCookie *) mallocMagic((unsigned) (sizeof (DRCCookie)));
 		ASSIGN(dpnew, 1, dp->drcc_next,
 		    set2, setC, why, 1, DRC_FORWARD, plane);
 		dpnew->drcc_flags |= DRC_BOTHCORNERS;
@@ -1460,7 +1458,7 @@ drcRectOnly(argc, argv)
 
 		/* find bucket preceding new one we wish to insert */
 		dp = drcFindBucket(j, i, 1);
-		MALLOC(DRCCookie *, dpnew, sizeof (DRCCookie));
+		dpnew = (DRCCookie *) mallocMagic((unsigned) (sizeof (DRCCookie)));
 		ASSIGN(dpnew,1,dp->drcc_next,
 			    set2, setC, why, 1, DRC_REVERSE, plane);
 		dpnew->drcc_flags |= DRC_BOTHCORNERS;
@@ -1563,7 +1561,7 @@ drcSurround(argc, argv)
 	    {
 		/* Find bucket preceding the new one we wish to insert */
 		dp = drcFindBucket(i, j, distance);
-		MALLOC(DRCCookie *, dpnew, sizeof (DRCCookie));
+		dpnew = (DRCCookie *) mallocMagic((unsigned) (sizeof (DRCCookie)));
 		ASSIGN(dpnew, distance, dp->drcc_next,
 			invM, set2, why, distance, DRC_FORWARD, plane1);
 		if (plane2 != plane1)
@@ -1573,7 +1571,7 @@ drcSurround(argc, argv)
 
 		/* find bucket preceding new one we wish to insert */
 		dp = drcFindBucket(j, i, distance);
-		MALLOC(DRCCookie *, dpnew, sizeof (DRCCookie));
+		dpnew = (DRCCookie *) mallocMagic((unsigned) (sizeof (DRCCookie)));
 		ASSIGN(dpnew, distance, dp->drcc_next,
 			invM, set2, why, distance, DRC_REVERSE, plane1);
 		if (plane2 != plane1)
@@ -1586,7 +1584,7 @@ drcSurround(argc, argv)
 
     if (strcmp(presence, "absence_illegal")) return distance;
 
-    /* To do---add extra rule when presence of the surrounding	*/
+    /* Add an extra rule when presence of the surrounding	*/
     /* layer is required.  Rule is different if planes match.	*/
 
     if (plane1 == plane2)
@@ -1605,7 +1603,7 @@ drcSurround(argc, argv)
 		{
 		    /* Find bucket preceding the new one we wish to insert */
 		    dp = drcFindBucket(i, j, distance);
-		    MALLOC(DRCCookie *, dpnew, sizeof (DRCCookie));
+		    dpnew = (DRCCookie *) mallocMagic((unsigned) (sizeof (DRCCookie)));
 		    ASSIGN(dpnew, distance, dp->drcc_next, set1,
 				invM, why, distance, DRC_FORWARD, plane2);
 		    dpnew->drcc_flags |= DRC_BOTHCORNERS;
@@ -1613,7 +1611,7 @@ drcSurround(argc, argv)
 
 		    /* find bucket preceding new one we wish to insert */
 		    dp = drcFindBucket(j, i, distance);
-		    MALLOC(DRCCookie *, dpnew, sizeof (DRCCookie));
+		    dpnew = (DRCCookie *) mallocMagic((unsigned) (sizeof (DRCCookie)));
 		    ASSIGN(dpnew,distance,dp->drcc_next, set1,
 				invM, why, distance, DRC_REVERSE, plane2);
 		    dpnew->drcc_flags |= DRC_BOTHCORNERS;
@@ -1628,7 +1626,7 @@ drcSurround(argc, argv)
 		{
 		    /* Find bucket preceding the new one we wish to insert */
 		    dp = drcFindBucket(i, j, distance);
-		    MALLOC(DRCCookie *, dpnew, sizeof (DRCCookie));
+		    dpnew = (DRCCookie *) mallocMagic((unsigned) (sizeof (DRCCookie)));
 		    ASSIGN(dpnew, distance, dp->drcc_next,
 				set2, invM, why, distance, DRC_FORWARD, plane2);
 		    dpnew->drcc_flags |= DRC_BOTHCORNERS | DRC_XPLANE;
@@ -1636,7 +1634,7 @@ drcSurround(argc, argv)
 
 		    /* find bucket preceding new one we wish to insert */
 		    dp = drcFindBucket(j, i, distance);
-		    MALLOC(DRCCookie *, dpnew, sizeof (DRCCookie));
+		    dpnew = (DRCCookie *) mallocMagic((unsigned) (sizeof (DRCCookie)));
 		    ASSIGN(dpnew,distance,dp->drcc_next,
 				set2, invM, why, distance, DRC_REVERSE, plane2);
 		    dpnew->drcc_flags |= DRC_BOTHCORNERS | DRC_XPLANE;
@@ -1836,7 +1834,7 @@ drcRectangle(argc, argv)
 
 		/* Find bucket preceding the new one we wish to insert */
 		dp = drcFindBucket(i, j, distance);
-		MALLOC(DRCCookie *, dpnew, sizeof (DRCCookie));
+		dpnew = (DRCCookie *) mallocMagic((unsigned) (sizeof (DRCCookie)));
 		ASSIGN(dpnew, distance, dp->drcc_next, 
 		    nottypes, DBAllTypeBits, why, distance, 
 		    DRC_FORWARD, plane);
@@ -1844,7 +1842,7 @@ drcRectangle(argc, argv)
 
 		/* Find bucket preceding the new one we wish to insert */
 		dp = drcFindBucket(j, i, distance); /* note: j, i not i, j */
-		MALLOC(DRCCookie *, dpnew, sizeof (DRCCookie));
+		dpnew = (DRCCookie *) mallocMagic((unsigned) (sizeof (DRCCookie)));
 		ASSIGN(dpnew, distance, dp->drcc_next, 
 		    nottypes, DBAllTypeBits, why, distance, 
 		    DRC_REVERSE, plane);
@@ -1863,7 +1861,7 @@ drcRectangle(argc, argv)
 			    dp = dp->drcc_next)
 			; /* null body */
 
-		    MALLOC(DRCCookie *, dpnew, sizeof (DRCCookie));
+		    dpnew = (DRCCookie *) mallocMagic ((unsigned) (sizeof (DRCCookie)));
 		    ASSIGN(dpnew, distance, dp->drcc_next, 
 			types, DBZeroTypeBits, why, even, 
 			DRC_RECTSIZE, plane);
@@ -2002,7 +2000,7 @@ DRCTechFinal()
 	    dpp = &( DRCRulesTbl [i][j]);
 	    dummy = *dpp;
 	    *dpp = dummy->drcc_next;
-	    FREE ((char *) dummy); 
+	    freeMagic((char *) dummy); 
 	}
     }
 
@@ -2032,6 +2030,14 @@ DRCTechFinal()
 		    if (TTMaskHasType(&dp->drcc_mask, i)) continue;
 		}
 		else if (TTMaskHasType(&dp->drcc_mask, j)) continue;
+
+		/* In magic 7.3, this edge may be legal on planes other	*/
+		/* than drcc_plane if i or j is a contact type or	*/
+		/* TT_SPACE.						*/
+
+		if (DBIsContact(i) || DBIsContact(j) || i == TT_SPACE ||
+			j == TT_SPACE) continue;
+
 		keep = dp;
 		goto illegalEdge;
 	    }
@@ -2046,7 +2052,7 @@ DRCTechFinal()
 	    {
 		if (dp == keep) continue;
 		/* Don't free the shared drcc_why string here! */
-		FREE((char *) dp);
+		freeMagic((char *) dp);
 		drcRulesOptimized += 1;
 	    }
 	    DRCRulesTbl[i][j] = keep;
@@ -2144,14 +2150,14 @@ DRCTechFinal()
 		/* Trigger rules */
 		if (dptrig != NULL)
 		{
-		    FREE((char *)dp->drcc_next);
+		    freeMagic((char *)dp->drcc_next);
 		    *dp2back = dp->drcc_next->drcc_next;
 		}
 		else
 		    *dp2back = dp->drcc_next;
 
 		/* Don't free the shared drcc_why string here! */
-		FREE ((char *) dp);
+		freeMagic((char *) dp);
 		drcRulesOptimized += 1;
 	    }
 	}
@@ -2284,3 +2290,104 @@ DRCTechScale(scalen, scaled)
     TechHalo /= scalen;
 }
 
+/* The following routines are used by the "tech" command (and in other places,
+ * such as the LEF file reader) to query the DRC database.
+ */
+
+/*
+ *-----------------------------------------------------------------------------
+ *  DRCGetDefaultLayerWidth ---
+ *
+ *	Determine a default layer width from the DRC width rules
+ *	of a layer.  Break on the first rule found, since rules
+ *	are ordered from shortest to longest distance.
+ *
+ * Results:
+ *	The minimum width of the magic layer, in magic internal units
+ *
+ * Side effects:
+ *	None.
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+int
+DRCGetDefaultLayerWidth(ttype)
+    TileType ttype;
+{
+    int routeWidth = 0;
+    DRCCookie *cptr;
+    TileTypeBitMask *set;
+
+    for (cptr = DRCRulesTbl[TT_SPACE][ttype]; cptr != (DRCCookie *) NULL;
+	cptr = cptr->drcc_next)
+    {
+	if (cptr->drcc_flags == DRC_FORWARD)
+	{
+	    set = &cptr->drcc_mask;
+	    if (TTMaskEqual(set, &cptr->drcc_corner))
+		if ((cptr->drcc_plane == DBPlane(ttype)) &&
+			(cptr->drcc_dist == cptr->drcc_cdist))
+		{
+		    routeWidth = cptr->drcc_dist;
+		    /* Diagnostic */
+		    /*
+		    TxPrintf("DRC: Layer %s has default width %d\n",
+		    DBTypeLongNameTbl[ttype], routeWidth);
+		    */
+		    break;
+		}
+	}
+    }
+    return routeWidth;
+}
+
+/*
+ *-----------------------------------------------------------------------------
+ *  DRCGetDefaultLayerSpacing ---
+ *
+ *	Determine a default layer-to-layer spacing from the DRC width
+ *	rules of a layer.  Break on the first rule found, since rules
+ *	are ordered from shortest to longest distance.
+ *
+ * Results:
+ *	The minimum spacing between the specified magic layer types,
+ *	in magic internal units
+ *
+ * Side effects:
+ *	None.
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+int
+DRCGetDefaultLayerSpacing(ttype1, ttype2)
+    TileType ttype1, ttype2;
+{
+    int routeSpacing = 0;
+    DRCCookie *cptr;
+    TileTypeBitMask *set;
+
+    for (cptr = DRCRulesTbl[ttype1][TT_SPACE]; cptr != (DRCCookie *) NULL;
+	cptr = cptr->drcc_next)
+    {
+	if (cptr->drcc_flags == DRC_FORWARD)
+	{
+	    set = &cptr->drcc_mask;
+	    if (!TTMaskHasType(set, ttype2))
+		if ((cptr->drcc_plane == DBPlane(ttype2)) &&
+			(cptr->drcc_dist == cptr->drcc_cdist))
+		{
+		    routeSpacing = cptr->drcc_dist;
+		    /* Diagnostic */
+		    /*
+		    TxPrintf("DRC: Layer %s has default spacing %d to layer %s\n",
+			DBTypeLongNameTbl[ttype1], routeSpacing,
+			DBTypeLongNameTbl[ttype2]);
+		    */
+		    break;
+		}
+	}
+    }
+    return routeSpacing;
+}
