@@ -22,10 +22,7 @@ static char rcsid[] = "$Header: /ufs/repository/magic/ext2sim/ext2sim.c,v 1.5 20
 #include <ctype.h>
 #include <math.h>	/* for sqrt() in bipolar L,W calculation */
 
-#ifdef MAGIC_WRAPPER
-#include "magic/tclmagic.h"
-#endif
-
+#include "tcltk/tclmagic.h"
 #include "misc/magic.h"
 #include "utils/geometry.h"
 #include "utils/hash.h"
@@ -107,14 +104,14 @@ typedef struct {
 
 #define initNodeClient(node) \
 { \
-	MALLOC(ClientData,(node)->efnode_client,sizeof(nodeClient)); \
+	(node)->efnode_client = (ClientData) mallocMagic((unsigned) (sizeof(nodeClient))); \
 	(( nodeClient *)(node)->efnode_client)->visitMask = (long) 0; \
 }
 
 
 #define initNodeClientHier(node) \
 { \
-	MALLOC(ClientData,(node)->efnode_client,sizeof(nodeClientHier)); \
+	(node)->efnode_client = (ClientData) mallocMagic((unsigned) (sizeof(nodeClientHier))); \
 	((nodeClientHier *) (node)->efnode_client)->visitMask = (long) 0; \
 }
 
@@ -133,13 +130,13 @@ int  esDevsMerged;
 #define addDevMult(f) \
 {  \
         if ( esFMult == NULL ) { \
-          MALLOC(float *, esFMult, esFMSize*sizeof(float));  \
+          esFMult = (float *) mallocMagic((unsigned) (esFMSize*sizeof(float)));  \
         } else if ( esFMIndex >= esFMSize ) {  \
           int i;  \
           float *op = esFMult ;  \
-          MALLOC(float *, esFMult, (esFMSize = esFMSize*2)*sizeof(float)); \
+          esFMult = (float *) mallocMagic((unsigned) ((esFMSize = esFMSize*2)*sizeof(float))); \
           for ( i = 0 ; i < esFMSize/2 ; i++ ) esFMult[i] = op[i]; \
-          if (op) FREE(op); \
+          if (op) freeMagic(op); \
         }  \
         esFMult[esFMIndex++] = (float)(f); \
 }
@@ -198,6 +195,10 @@ Exttosim_Init(interp)
     if (WindReplaceCommand(DBWclientID, "exttosim", CmdExtToSim) < 0)
 	return TCL_ERROR;
 
+    /* ext2sim is an alias for exttosim */
+    if (WindReplaceCommand(DBWclientID, "ext2sim", CmdExtToSim) < 0)
+	return TCL_ERROR;
+
     Tcl_PkgProvide(interp, "Exttosim", "7.3");
     return TCL_OK;
 }
@@ -425,12 +426,12 @@ CmdExtToSim(w, cmd)
 	    EFScale = 0;
 	    if (EFArgTech)
 	    {
-		FREE(EFArgTech);
+		freeMagic(EFArgTech);
 		EFArgTech = NULL;
 	    }
 	    if (EFSearchPath)
 	    {
-		FREE(EFSearchPath);
+		freeMagic(EFSearchPath);
 		EFArgTech = NULL;
 	    }
 	    break;
@@ -531,7 +532,7 @@ runexttosim:
     EFResistThreshold = LocResistThreshold;
 
     /* Read the hierarchical description of the input circuit */
-    if (EFReadFile(inName, FALSE, esDoExtResis) == FALSE)
+    if (EFReadFile(inName, FALSE, esDoExtResis, FALSE) == FALSE)
     {
 	EFDone();
 	return /* TCL_ERROR */;
@@ -577,8 +578,8 @@ runexttosim:
 	/* Tcl variable substitution for substrate node names */
 	if (subname && (subname[0] == '$'))
 	{
-	    char *resstr;
-	    resstr = Tcl_GetVar(magicinterp, &subname[1], TCL_GLOBAL_ONLY);
+	    resstr = (char *)Tcl_GetVar(magicinterp, &subname[1],
+			TCL_GLOBAL_ONLY);
 	    if (resstr != NULL) fetInfo[i].defSubs = resstr;
 	}
     }
@@ -603,7 +604,7 @@ runexttosim:
 	TxPrintf("Devices merged: %d\n", esDevsMerged);
 	esFMIndex = 0;
 	for (p = devMergeList; p != NULL; p = p->next)
-	    FREE(p);
+	    freeMagic(p);
 	devMergeList = NULL;
     }
 
@@ -709,7 +710,7 @@ main(argc, argv)
     }
 
     /* Read the hierarchical description of the input circuit */
-    if (EFReadFile(inName, FALSE, esDoExtResis) == FALSE)
+    if (EFReadFile(inName, FALSE, esDoExtResis, FALSE) == FALSE)
     {
 	exit(1);
     }
@@ -730,7 +731,7 @@ main(argc, argv)
 	EFVisitDevs(simmergeVisit, (ClientData) NULL);
 	TxPrintf("Devices merged: %d\n", esDevsMerged);
 	esFMIndex = 0;
-	for (p = devMergeList; p != NULL; p = p->next) FREE(p);
+	for (p = devMergeList; p != NULL; p = p->next) freeMagic(p);
     }
 
     EFVisitDevs(simdevVisit, (ClientData) NULL);
@@ -874,7 +875,7 @@ simmainArgs(pargc, pargv)
 	    ndx = efBuildAddStr(EFDevTypes, &EFDevNumTypes, MAXDEVTYPES, cp);
 	    fetInfo[ndx].resClassSD = rClass;
 	    fetInfo[ndx].resClassSub = rClassSub;
-	    MALLOC(char *, fetInfo[ndx].defSubs, (strlen(subsNode)+1));
+	    fetInfo[ndx].defSubs = (char *) mallocMagic((unsigned) (strlen(subsNode)+1));
 	    strcpy(fetInfo[ndx].defSubs,subsNode);
 	    TxError("Info: fet %s(%d) sdRclass=%d subRclass=%d dSub=%s\n", 
 	    	cp, ndx, fetInfo[ndx].resClassSD, fetInfo[ndx].resClassSub, 
@@ -1181,7 +1182,7 @@ FILE *outf;
     	he = EFHNConcatLook(prefix, suffix, "substrate");
     	if (he == NULL)
     	{
-		(void) fprintf(outf, "errGnd!");
+		fprintf(outf, "errGnd!");
 		return;
     	}
     	/* Canonical name */
@@ -1310,7 +1311,7 @@ simdevOutNode(prefix, suffix, name, outf)
     he = EFHNConcatLook(prefix, suffix, name);
     if (he == NULL)
     {
-	(void) fprintf(outf, " GND");
+	fprintf(outf, " GND");
 	return;
     }
 
@@ -1356,11 +1357,11 @@ int simcapVisit(hierName1, hierName2, cap)
     if (cap <= EFCapThreshold)
 	return 0;
 
-    (void) fprintf(esSimF, "C ");
+    fprintf(esSimF, "C ");
     EFHNOut(hierName1, esSimF);
-    (void) fprintf(esSimF, " ");
+    fprintf(esSimF, " ");
     EFHNOut(hierName2, esSimF);
-    (void) fprintf(esSimF, esCapFormat, cap);
+    fprintf(esSimF, esCapFormat, cap);
     return 0;
 }
 
@@ -1398,11 +1399,11 @@ simresistVisit(hierName1, hierName2, res)
 {
     res = (res+ 500)/1000;
 
-    (void) fprintf(esSimF, "r ");
+    fprintf(esSimF, "r ");
     EFHNOut(hierName1, esSimF);
-    (void) fprintf(esSimF, " ");
+    fprintf(esSimF, " ");
     EFHNOut(hierName2, esSimF);
-    (void) fprintf(esSimF, " %d\n", res);
+    fprintf(esSimF, " %d\n", res);
     return 0;
 }
 
@@ -1443,23 +1444,23 @@ int simnodeVisit(node, res, cap)
     res = (res + 500) / 1000;
     if (cap > EFCapThreshold)
     {
-	(void) fprintf(esSimF, "C ");
+	fprintf(esSimF, "C ");
 	EFHNOut(hierName, esSimF);
-	(void) fprintf(esSimF, esCapFormat, cap);
+	fprintf(esSimF, esCapFormat, cap);
     }
     if (res > EFResistThreshold)
     {
-	(void) fprintf(esSimF, "R ");
+	fprintf(esSimF, "R ");
 	EFHNOut(hierName, esSimF);
-	(void) fprintf(esSimF, " %d\n", res);
+	fprintf(esSimF, " %d\n", res);
     }
     if (node->efnode_attrs && !esNoAttrs)
     {
-	(void) fprintf(esSimF, "A ");
+	fprintf(esSimF, "A ");
 	EFHNOut(hierName, esSimF);
 	for (fmt = " %s", ap = node->efnode_attrs; ap; ap = ap->efa_next)
 	{
-	    (void) fprintf(esSimF, fmt, ap->efa_text);
+	    fprintf(esSimF, fmt, ap->efa_text);
 	    fmt = ",%s";
 	}
 	putc('\n', esSimF);
@@ -1472,19 +1473,19 @@ int simnodeVisit(node, res, cap)
 	{
 	    if (isGlob && EFHNIsGlob(nn->efnn_hier))
 		continue;
-	    (void) fprintf(esAliasF, "= ");
+	    fprintf(esAliasF, "= ");
 	    EFHNOut(hierName, esAliasF);
-	    (void) fprintf(esAliasF, " ");
+	    fprintf(esAliasF, " ");
 	    EFHNOut(nn->efnn_hier, esAliasF);
-	    (void) fprintf(esAliasF, "\n");
+	    fprintf(esAliasF, "\n");
 	}
     }
 
     if (esLabF)
     {
-	(void) fprintf(esLabF, "94 ");
+	fprintf(esLabF, "94 ");
 	EFHNOut(hierName, esLabF);
-	(void) fprintf(esLabF, " %d %d %s;\n",
+	fprintf(esLabF, " %d %d %s;\n",
 		    node->efnode_loc.r_xbot, node->efnode_loc.r_ybot,
 		    EFLayerNames[node->efnode_type]);
     }
@@ -1514,7 +1515,7 @@ Dev    *dev;
 {
 	   devMerge *fp;
 
-	MALLOC (devMerge *, fp, sizeof(devMerge));
+	fp = (devMerge *) mallocMagic((unsigned) (sizeof(devMerge)));
 	fp->l = l; fp->w = w;
 	fp->g = g; fp->s = s;
 	fp->d = d; fp->b = b;
@@ -1577,9 +1578,9 @@ Dev    *dev;
 		char *t; \
 		int l1 = strlen(a1); \
 		int l2 = strlen(a2); \
-		MALLOC(char *, t, (l1+l2)+1); \
+		t = (char *) mallocMagic((unsigned) (l1+l2)+1); \
 		t = (char *) strcat(a1,a2); \
-		FREE(a1); \
+		freeMagic(a1); \
 		a1 = t ; \
 	} \
 }
@@ -1659,7 +1660,7 @@ int l, w;
 		setDevMult(fp->esFMIndex, DEV_KILLED); 
 		setDevMult(cfp->esFMIndex, m);
 		esDevsMerged++;
-		FREE(fp);
+		freeMagic(fp);
 		return 0;
 	  }
 	}
