@@ -23,10 +23,7 @@ static char rcsid[] = "$Header: /ufs/repository/magic/ext2spice/ext2spice.c,v 1.
 #include <string.h>
 #endif
 
-#ifdef MAGIC_WRAPPER
-#include "magic/tclmagic.h"
-#endif
-
+#include "tcltk/tclmagic.h"
 #include "misc/magic.h"
 #include "utils/malloc.h"
 #include "utils/geometry.h"
@@ -54,12 +51,16 @@ void CmdExtToSpice();
 int spcmainArgs();
 int spccapVisit(), spcdevVisit(), spcnodeVisit(), subcktVisit(), spcresistVisit(), 
     devMergeVisit(), devDistJunctVisit();
+int subcktUndef();
 EFNode *spcdevSubstrate();
 char *nodeSpiceName();
 
+#define AUTO	2	/* TRUE | FALSE | AUTO for esDoSubckt */
+
 /* Options specific to ext2spice */
 bool esDoExtResis = FALSE;
 bool esDoPorts = TRUE;
+int  esDoSubckt = AUTO;
 bool esDevNodesOnly = FALSE;
 bool esNoAttrs = FALSE;
 bool esHierAP = FALSE;
@@ -134,14 +135,14 @@ typedef struct {
 
 /*
  * Used to mark the nodes which are connected to devs. initMask  is set to
- * DEV_CONNET_MASK only when we are in visitDevs 
+ * DEV_CONNECT_MASK only when we are in visitDevs 
  */
 #define	DEV_CONNECT_MASK	((unsigned long)1<<(sizeof(long)*BITSPERCHAR-1))
 unsigned long	initMask = 0;
 
 #define initNodeClient(node) \
 { \
-	MALLOC(ClientData,(node)->efnode_client,sizeof(nodeClient)); \
+	(node)->efnode_client = (ClientData) mallocMagic((unsigned) (sizeof(nodeClient))); \
 	 (( nodeClient *)(node)->efnode_client)->spiceNodeName = NULL; \
 	(( nodeClient *)(node)->efnode_client)->m_w.visitMask = (unsigned long)initMask; \
 }
@@ -149,7 +150,7 @@ unsigned long	initMask = 0;
 
 #define initNodeClientHier(node) \
 { \
-	MALLOC(ClientData,(node)->efnode_client,sizeof(nodeClientHier)); \
+	(node)->efnode_client = (ClientData) mallocMagic((unsigned)(sizeof(nodeClientHier))); \
 	((nodeClientHier *) (node)->efnode_client)->m_w.visitMask = (unsigned long) 0; \
 }
 
@@ -192,13 +193,13 @@ int nodeVisitDebug();
 #define addDevMult(f) \
 {  \
 	if ( esFMult == NULL ) { \
-	  MALLOC(float *, esFMult, esFMSize*sizeof(float));  \
+	  esFMult = (float *) mallocMagic((unsigned) (esFMSize*sizeof(float)));  \
 	} else if ( esFMIndex >= esFMSize ) {  \
 	  int i;  \
 	  float *op = esFMult ;  \
-	  MALLOC(float *, esFMult, (esFMSize = esFMSize*2)*sizeof(float)); \
+	  esFMult = (float *) mallocMagic((unsigned) ((esFMSize = esFMSize*2)*sizeof(float))); \
 	  for ( i = 0 ; i < esFMSize/2 ; i++ ) esFMult[i] = op[i]; \
-	  if (op) FREE(op); \
+	  if (op) freeMagic(op); \
 	}  \
 	esFMult[esFMIndex++] = (float)(f); \
 }
@@ -227,6 +228,7 @@ devMerge *devMergeList = NULL ;
 
 /* Forward declaration */
 int _ext2spice_start();
+void topVisit();
 
 /*
  * ----------------------------------------------------------------------------
@@ -254,6 +256,10 @@ Exttospice_Init(interp)
     if (WindReplaceCommand(DBWclientID, "exttospice", CmdExtToSpice) < 0)
 	return TCL_ERROR;
 
+    /* Add "ext2spice" as an alias for "exttospice" */
+    if (WindReplaceCommand(DBWclientID, "ext2spice", CmdExtToSpice) < 0)
+	return TCL_ERROR;
+
     Tcl_PkgProvide(interp, "Exttospice", "7.3");
     return TCL_OK;
 }
@@ -290,14 +296,14 @@ CmdExtToSpice(w, cmd)
     int argc = cmd->tx_argc;
     char **argv = cmd->tx_argv;
     char **msg;
-    bool err_result;
+    bool err_result, locDoSubckt;
     char *resstr;
 
     short sd_rclass;
     short sub_rclass;
     char *devname;
     char *subname;
-    int idx;
+    int idx, idx2;
 
     static EFCapValue LocCapThreshold = 2;
     static int LocResistThreshold = INFINITE_THRESHOLD; 
@@ -337,9 +343,17 @@ CmdExtToSpice(w, cmd)
 	"no",
 	"false",
 	"off",
+	"automatic", 
+	"top",
+	"descend",
 	NULL
     };
 
+    typedef enum {
+	IDX_YES, IDX_TRUE, IDX_ON, IDX_NO, IDX_FALSE, IDX_OFF,
+	IDX_AUTO, IDX_TOP, IDX_DESCEND
+    } yesnoType;  
+
     if (cmd->tx_argc > 1)
     {
 	option = Lookup(cmd->tx_argv[1], cmdExtToSpcOption);
@@ -362,19 +376,68 @@ CmdExtToSpice(w, cmd)
 	    else if (idx < 3) esDoExtResis = TRUE;
 	    else esDoExtResis = FALSE;
 	    break;
+
 	case EXTTOSPC_SUBCIRCUITS:
 	    if (cmd->tx_argc == 2)
 	    {
 		Tcl_SetResult(magicinterp, (esDoPorts) ? "on" : "off", NULL);
 		return;
 	    }
-	    else if (cmd->tx_argc != 3)
-		goto usage;
 	    idx = Lookup(cmd->tx_argv[2], yesno);
-	    if (idx < 0) goto usage;
-	    else if (idx < 3) esDoPorts = TRUE;
-	    else esDoPorts = FALSE;
+	    switch (idx) {
+		case IDX_YES: case IDX_TRUE: case IDX_ON:
+		    esDoPorts = TRUE;
+		    return;
+		    break;
+		case IDX_NO: case IDX_FALSE: case IDX_OFF:
+		    esDoPorts = FALSE;
+		    return;
+		    break;
+		case IDX_DESCEND:
+		    if (cmd->tx_argc == 3)
+		    {
+			Tcl_SetResult(magicinterp, (esDoPorts) ? "on" : "off", NULL);
+			return;
+		    }
+		    break;
+		case IDX_TOP:
+		    if (cmd->tx_argc == 3)
+		    {
+			Tcl_SetResult(magicinterp, (esDoSubckt) ? "on" : "off", NULL);
+			return;
+		    }
+		    break;
+		case IDX_AUTO:
+		default:
+		    goto usage;
+		    break;
+	    }
+
+	    if (cmd->tx_argc != 4) goto usage;
+	    idx2 = Lookup(cmd->tx_argv[3], yesno);
+	    switch (idx2) {
+		case IDX_YES: case IDX_TRUE: case IDX_ON:
+		    if (idx == IDX_DESCEND)
+			esDoPorts = TRUE;
+		    else
+			esDoSubckt = TRUE;
+		    break;
+		case IDX_NO: case IDX_FALSE: case IDX_OFF:
+		    if (idx == IDX_DESCEND)
+			esDoPorts = FALSE;
+		    else
+			esDoSubckt = FALSE;
+		    break;
+		case IDX_AUTO:
+		    esDoSubckt = AUTO;
+		    break;
+		case IDX_DESCEND:
+		case IDX_TOP:
+		default:
+		    goto usage;
+	    }
 	    break;
+
 	case EXTTOSPC_FORMAT:
 	    if (cmd->tx_argc == 2)
 	    {
@@ -478,12 +541,12 @@ CmdExtToSpice(w, cmd)
 	    EFScale = 0;
 	    if (EFArgTech)
 	    {
-		FREE(EFArgTech);
+		freeMagic(EFArgTech);
 		EFArgTech = NULL;
 	    }
 	    if (EFSearchPath)
 	    {
-		FREE(EFSearchPath);
+		freeMagic(EFSearchPath);
 		EFArgTech = NULL;
 	    }
 	    break;
@@ -569,7 +632,7 @@ runexttospice:
     }
 
     /* Read the hierarchical description of the input circuit */
-    if (EFReadFile(inName, esDoPorts, esDoExtResis) == FALSE)
+    if (EFReadFile(inName, TRUE, esDoExtResis, FALSE) == FALSE)
     {
 	EFDone();
         return;
@@ -613,12 +676,17 @@ runexttospice:
 	/* Tcl variable substitution for substrate node names */
 	if (subname && (subname[0] == '$'))
 	{
-	    char *resstr;
-	    resstr = Tcl_GetVar(magicinterp, &subname[1], TCL_GLOBAL_ONLY);
+	    resstr = (char *)Tcl_GetVar(magicinterp, &subname[1],
+			TCL_GLOBAL_ONLY);
 	    if (resstr != NULL) fetInfo[i].defSubs = resstr;
 	}
     }
 
+    /* Keep a pointer to the "GND" variable, if it exists. */
+
+    resstr = (char *)Tcl_GetVar(magicinterp, "GND", TCL_GLOBAL_ONLY);
+    if (resstr == NULL) resstr = "GND";		/* default value */
+
     /* Write the output file */
 
     fprintf(esSpiceF, "* %s file created from %s.ext - technology: %s\n\n",
@@ -642,6 +710,21 @@ runexttospice:
     }
     EFFlatBuild(inName, flatFlags);
 
+    /* Determine if this is a subcircuit */
+    locDoSubckt = FALSE;
+    if (esDoSubckt == AUTO) {
+	if (efFlatRootDef->def_flags & DEF_SUBCIRCUIT)
+	    locDoSubckt = TRUE;
+    }
+    if ((esDoSubckt == TRUE) || (locDoSubckt == TRUE))
+	topVisit(efFlatRootDef);
+
+    /* If we don't want to write subcircuit calls, remove the	*/
+    /* subcircuit flag from all cells at this time.		*/
+
+    if (!esDoPorts)
+	EFVisitSubcircuits(subcktUndef, (ClientData) NULL);
+
     initMask = ( esDistrJunct  ) ? (unsigned long)0 : DEV_CONNECT_MASK;
 
     if (esMergeDevsA || esMergeDevsC)
@@ -653,7 +736,7 @@ runexttospice:
 	    devMerge *p;
 
 	    for (p = devMergeList; p != NULL; p = p->next)
-		FREE(p);
+		freeMagic(p);
 	    devMergeList = NULL;
 	}
     }
@@ -668,9 +751,12 @@ runexttospice:
     }
     EFVisitResists(spcresistVisit, (ClientData) NULL);
     EFVisitSubcircuits(subcktVisit, (ClientData) NULL);
-    (void) sprintf( esCapFormat, "C%%d %%s GND %%.%dlffF%%s", esCapAccuracy);
+    (void) sprintf( esCapFormat, "C%%d %%s %s %%.%dlffF%%s", resstr, esCapAccuracy);
     EFVisitNodes(spcnodeVisit, (ClientData) NULL);
 
+    if ((esDoSubckt == TRUE) || (locDoSubckt == TRUE))
+	fprintf(esSpiceF, ".ends\n");
+
     if (esFormat == HSPICE) 
 	printSubcktDict();
 
@@ -754,7 +840,7 @@ main(argc, argv)
     }
 
     /* Read the hierarchical description of the input circuit */
-    if (EFReadFile(inName, esDoPorts, esDoExtResis) == FALSE)
+    if (EFReadFile(inName, TRUE, esDoExtResis, FALSE) == FALSE)
     {
 	exit (1);
     }
@@ -780,6 +866,21 @@ main(argc, argv)
     }
     EFFlatBuild(inName, flatFlags);
 
+    /* Determine if this is a subcircuit */
+    locDoSubckt = FALSE;
+    if (esDoSubckt == AUTO) {
+	if (efFlatRootDef->def_flags & DEF_SUBCIRCUIT)
+	    locDoSubckt = TRUE;
+    }
+    if ((esDoSubckt == TRUE) || (locDoSubckt == TRUE))
+	topVisit(efFlatRootDef);
+
+    /* If we don't want to write subcircuit calls, remove the	*/
+    /* subcircuit flag from all cells at this time.		*/
+
+    if (!esDoPorts)
+	EFVisitSubcircuits(subcktUndef, (ClientData) NULL);
+
     initMask = ( esDistrJunct  ) ? (unsigned long)0 : DEV_CONNECT_MASK ;
 
     if ( esMergeDevsA || esMergeDevsC ) {
@@ -789,7 +890,7 @@ main(argc, argv)
 	{
 	  devMerge *p;
 
-	  for ( p = devMergeList ; p != NULL ; p=p->next ) FREE(p);
+	  for ( p = devMergeList ; p != NULL ; p=p->next ) freeMagic(p);
 	}
     } else if ( esDistrJunct )
      	EFVisitDevs(devDistJunctVisit, (ClientData) NULL);
@@ -804,6 +905,9 @@ main(argc, argv)
     (void) sprintf( esCapFormat, "C%%d %%s GND %%.%dlffF%%s", esCapAccuracy);
     EFVisitNodes(spcnodeVisit, (ClientData) NULL);
 
+    if ((esDoSubckt == TRUE) || (locDoSubckt == TRUE))
+	fprintf(esSpiceF, ".ends\n");
+
     if ( esFormat == HSPICE ) 
 	printSubcktDict();
 
@@ -934,7 +1038,7 @@ spcmainArgs(pargc, pargv)
 	        TxError("DEV_CONNECT_MASK and or nodeClient\n");
 		exit (1);
 	    }
-	    MALLOC(char *, fetInfo[ndx].defSubs, (strlen(subsNode)+1));
+	    fetInfo[ndx].defSubs = (char *) mallocMagic((unsigned) (strlen(subsNode)+1));
 	    strcpy(fetInfo[ndx].defSubs,subsNode);
 	    TxError("info: dev %s(%d) sdRclass=%d subRclass=%d dSub=%s\n", 
 	    cp, ndx, fetInfo[ndx].resClassSD, fetInfo[ndx].resClassSub, 
@@ -1055,16 +1159,18 @@ DevTerm *term;
  * ----------------------------------------------------------------------------
  */
 int
-subcktVisit(use, hierName, cdata)
+subcktVisit(use, hierName, is_top)
     Use *use;
     HierName *hierName;
-    ClientData cdata;	/* unused */
+    bool is_top;		/* TRUE if this is the top-level cell */
 {
     EFNode *snode;
     Def *def = use->use_def;
-    HierName *nodeName;
+    EFNodeName *nodeName;
     int portorder, portmax;
 
+    if (is_top == TRUE) return 0;	/* Ignore the top-level cell */
+
     fprintf(esSpiceF, "X%d", esSbckNum++);
 
     /* This is not a DEV, but "spcdevOutNode" is a general-purpose routine that */
@@ -1079,10 +1185,13 @@ subcktVisit(use, hierName, cdata)
 		snode != &def->def_firstn;
 		snode = (EFNode *) snode->efnode_next)
     {
-	portorder = snode->efnode_name->efnn_node->efnode_flags;
-	if (!(portorder & EF_PORT)) continue;
-	else portorder >>= PORT_SHIFT;
-	if (portorder > portmax) portmax = portorder;
+	if (!(snode->efnode_flags & EF_PORT)) continue;
+	for (nodeName = snode->efnode_name; nodeName != NULL; nodeName =
+			nodeName->efnn_next)
+	{
+	    portorder = nodeName->efnn_port;
+	    if (portorder > portmax) portmax = portorder;
+	}
     }
 
     if (portmax < 0)
@@ -1094,9 +1203,12 @@ subcktVisit(use, hierName, cdata)
         for (snode = (EFNode *) def->def_firstn.efnode_next;
 		snode != &def->def_firstn;
 		snode = (EFNode *) snode->efnode_next)
-	    if (snode->efnode_name->efnn_node->efnode_flags & EF_PORT)
-		spcdevOutNode(hierName, snode->efnode_name->efnn_hier,
-				"subcircuit", esSpiceF); 
+	    if (snode->efnode_flags & EF_PORT)
+		for (nodeName = snode->efnode_name; nodeName != NULL;
+			nodeName = nodeName->efnn_next)
+		    if (nodeName->efnn_port >= 0)
+			spcdevOutNode(hierName, nodeName->efnn_hier,
+					"subcircuit", esSpiceF); 
     }
     else
     {
@@ -1110,15 +1222,20 @@ subcktVisit(use, hierName, cdata)
 			snode != &def->def_firstn;
 			snode = (EFNode *) snode->efnode_next)
 	    {
-		int portidx = snode->efnode_name->efnn_node->efnode_flags;
-		if (!(portidx & EF_PORT)) continue;
-		else portidx >>= PORT_SHIFT;
-		if (portidx == portorder)
+		if (!(snode->efnode_flags & EF_PORT)) continue;
+		for (nodeName = snode->efnode_name; nodeName != NULL;
+			nodeName = nodeName->efnn_next)
 		{
-		    spcdevOutNode(hierName, snode->efnode_name->efnn_hier,
-				"subcircuit", esSpiceF); 
-		    break;
+		    int portidx = nodeName->efnn_port;
+		    if (portidx == portorder)
+		    {
+			spcdevOutNode(hierName, nodeName->efnn_hier,
+					"subcircuit", esSpiceF); 
+
+			break;
+		    }
 		}
+		if (nodeName != NULL) break;
 	    }
 	    portorder++;
 	}
@@ -1129,6 +1246,138 @@ subcktVisit(use, hierName, cdata)
     return 0;
 }
 
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * subcktUndef --
+ *
+ * Procedure to remove the DEF_SUBCIRCUIT flag from all subcells.
+ * Called by EFVisitSubcircuits().
+ *
+ * Results:
+ *	Returns 0 always.
+ *
+ * Side effects:
+ *	Undefines the DEF_SUBCIRCUIT flag in each encountered Def.
+ *
+ * ----------------------------------------------------------------------------
+ */
+int
+subcktUndef(use, hierName, is_top)
+    Use *use;
+    HierName *hierName;
+    bool is_top;	/* TRUE if this is the top-level cell */
+{
+    Def *def = use->use_def;
+
+    def->def_flags &= ~(DEF_SUBCIRCUIT);
+    return 0;
+}
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * topVisit --
+ *
+ * Procedure to output a subcircuit definition to the .spice file.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	Writes to the file esSpiceF.
+ *
+ * Format of a .spice subcircuit definition:
+ *
+ *	.subckt name node1 node2 ... noden
+ *
+ * where
+ *	node1 node2 ... noden are the nodes connecting to the ports of
+ *	the subcircuit.  "name" is the name of the cell def.
+ *
+ * ----------------------------------------------------------------------------
+ */
+ 
+void
+topVisit(def)
+    Def *def;
+{
+    EFNode *snode;
+    EFNodeName *sname, *nodeName;
+    HashSearch hs;
+    HashEntry *he;
+    int portorder, portmax;
+
+    fprintf(esSpiceF, ".subckt %s", def->def_name);
+
+    /* Note that the ports of the subcircuit will not necessarily be	*/
+    /* ALL the entries in the hash table, so we have to check.		*/
+
+    HashStartSearch(&hs);
+    portmax = -1;
+
+    while (he = HashNext(&def->def_nodes, &hs))
+    {
+	sname = (EFNodeName *) HashGetValue(he);
+	snode = sname->efnn_node;
+	if (!(snode->efnode_flags & EF_PORT)) continue;
+	for (nodeName = sname; nodeName != NULL; nodeName = nodeName->efnn_next)
+	{
+	    portorder = nodeName->efnn_port;
+	    if (portorder > portmax) portmax = portorder;
+	}
+    }
+    
+    if (portmax < 0)
+    {
+	/* No port order declared; print them as we see them.	*/
+	/* This shouldn't happen for proper .ext files written	*/
+	/* by the magic extractor.				*/
+
+	HashStartSearch(&hs);
+	while (he = HashNext(&def->def_nodes, &hs))
+	{
+	    sname = (EFNodeName *) HashGetValue(he);
+	    snode = sname->efnn_node;
+
+	    if (snode->efnode_name->efnn_node->efnode_flags & EF_PORT)
+		fprintf(esSpiceF, " %s", he->h_key.h_name);
+	}
+    }
+    else
+    {
+	/* Port numbers need not start at zero or be contiguous. */
+	/* They will be printed in numerical order.              */
+
+	portorder = 0;
+	while (portorder <= portmax)
+	{
+	    HashStartSearch(&hs);
+	    while (he = HashNext(&def->def_nodes, &hs))
+	    {
+		int portidx;
+
+		sname = (EFNodeName *) HashGetValue(he);
+		snode = sname->efnn_node;
+
+		if (!(snode->efnode_flags & EF_PORT)) continue;
+		for (nodeName = sname; nodeName != NULL; nodeName = nodeName->efnn_next)
+		{
+		    portidx = nodeName->efnn_port;
+		    if (portidx == portorder)
+		    {
+			fprintf(esSpiceF, " %s", he->h_key.h_name);
+			break;
+		    }
+		}
+		if (nodeName != NULL) break;
+	    }
+	    portorder++;
+	}
+    }
+    fprintf(esSpiceF, "\n");
+}
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1224,34 +1473,36 @@ spcdevVisit(dev, hierName, trans)
 	    devchar = 'C';
 	    break;
 	case DEV_SUBCKT:
+	case DEV_RSUBCKT:
 	    devchar = 'X';
 	    break;
     }
-    (void) fprintf(esSpiceF, "%c", devchar);
+    fprintf(esSpiceF, "%c", devchar);
 
     /* Device index is taken from gate attributes if attached;	*/
     /* otherwise, the device is numbered in sequence.		*/
 
     if (gate->dterm_attrs)
-	(void) fprintf(esSpiceF, "%s", gate->dterm_attrs);
+	fprintf(esSpiceF, "%s", gate->dterm_attrs);
     else
     {
 	switch (dev->dev_class)
 	{
 	    case DEV_RES:
-		(void) fprintf(esSpiceF, "%d", esResNum++);
+		fprintf(esSpiceF, "%d", esResNum++);
 		break;
 	    case DEV_DIODE:
-		(void) fprintf(esSpiceF, "%d", esDiodeNum++);
+		fprintf(esSpiceF, "%d", esDiodeNum++);
 		break;
 	    case DEV_CAP:
-		(void) fprintf(esSpiceF, "%d", esCapNum++);
+		fprintf(esSpiceF, "%d", esCapNum++);
 		break;
 	    case DEV_SUBCKT:
-		(void) fprintf(esSpiceF, "%d", esSbckNum++);
+	    case DEV_RSUBCKT:
+		fprintf(esSpiceF, "%d", esSbckNum++);
 		break;
 	    default:
-		(void) fprintf(esSpiceF, "%d", esDevNum++);
+		fprintf(esSpiceF, "%d", esDevNum++);
 		break;
 	}
     }
@@ -1289,6 +1540,14 @@ spcdevVisit(dev, hierName, trans)
 
 	    spcdevOutNode(hierName, gate->dterm_node->efnode_name->efnn_hier,
 			name, esSpiceF);
+
+	    /* Drop through to below (no break statement) */
+
+	case DEV_RSUBCKT:
+	    /* RC-like subcircuits are exactly like other subcircuits	*/
+	    /* except that the "gate" node is treated as an identifier	*/
+	    /* only and is not output.					*/
+
 	    if (dev->dev_nterm > 1)
 		spcdevOutNode(hierName, source->dterm_node->efnode_name->efnn_hier,
 			name, esSpiceF);
@@ -1304,29 +1563,26 @@ spcdevVisit(dev, hierName, trans)
 	    }
 	    fprintf(esSpiceF, " %s", EFDevTypes[dev->dev_type]);
 
-	    /* If this thing looks like a FET (3 or 4 terminals), then	*/
-	    /* write its length and width.				*/
+	    /* Write its length and width and pass them as	*/
+	    /* parameters to the subcircuit call.		*/
 
-	    if (dev->dev_nterm > 2)
+	    GeoTransRect(trans, &dev->dev_rect, &r);
+	    scale = GeoScale(trans);
+
+	    sdM = getCurDevMult();
+
+	    if (esFormat == HSPICE)
 	    {
-		GeoTransRect(trans, &dev->dev_rect, &r);
-		scale = GeoScale(trans);
-
-		sdM = getCurDevMult();
-
-		if (esFormat == HSPICE)
-		{
-		    fprintf(esSpiceF, " w=%d l=%d", w*scale, l*scale);
-		    if (sdM != 1.0)
-			fprintf(esSpiceF, " M=%g", sdM);
-		}
-		else
-		{
-		    fprintf(esSpiceF, " w=%gu l=%gu",
+		fprintf(esSpiceF, " w=%d l=%d", w*scale, l*scale);
+		if (sdM != 1.0)
+		    fprintf(esSpiceF, " M=%g", sdM);
+	    }
+	    else
+	    {
+		fprintf(esSpiceF, " w=%gu l=%gu",
 				(float)w * scale * esScale * sdM,
 				(float)l * scale * esScale);
-		    sdM = 1.0;
-		}
+		sdM = 1.0;
 	    }
 	    break;
 
@@ -1523,13 +1779,13 @@ spcdevVisit(dev, hierName, trans)
 	    if (!esNoAttrs)
 	    {
 		if (gate->dterm_attrs || source->dterm_attrs || drain->dterm_attrs)
-		    (void) fprintf(esSpiceF,"\n**devattr");
+		    fprintf(esSpiceF,"\n**devattr");
 		if (gate->dterm_attrs)
-		    (void) fprintf(esSpiceF, " g=%s", gate->dterm_attrs);
+		    fprintf(esSpiceF, " g=%s", gate->dterm_attrs);
 		if (source->dterm_attrs) 
-		    (void) fprintf(esSpiceF, " s=%s", source->dterm_attrs);
+		    fprintf(esSpiceF, " s=%s", source->dterm_attrs);
 		if (drain->dterm_attrs) 
-		    (void) fprintf(esSpiceF, " d=%s", drain->dterm_attrs);
+		    fprintf(esSpiceF, " d=%s", drain->dterm_attrs);
 	    }
 	    break;
     }
@@ -1730,7 +1986,7 @@ spcdevOutNode(prefix, suffix, name, outf)
     he = EFHNConcatLook(prefix, suffix, name);
     if (he == NULL)
     {
-	(void) fprintf(outf, " errGnd!");
+	fprintf(outf, " errGnd!");
 	return;
     }
     nn = (EFNodeName *) HashGetValue(he);
@@ -1846,12 +2102,14 @@ spcnodeVisit(node, res, cap)
     char *fmt, *nsn;
     EFAttr *ap;
 
-    if ( node->efnode_client )
-    isConnected =  ( esDistrJunct ) ?
-	(((nodeClient *)node->efnode_client)->m_w.widths != NULL) :
-        (((nodeClient *)node->efnode_client)->m_w.visitMask&DEV_CONNECT_MASK);
-
-     if ( !isConnected && esDevNodesOnly) 
+    if (node->efnode_client)
+    {
+	isConnected = (esDistrJunct) ?
+		(((nodeClient *)node->efnode_client)->m_w.widths != NULL) :
+        	((((nodeClient *)node->efnode_client)->m_w.visitMask
+		& DEV_CONNECT_MASK) != 0);
+    }
+    if (!isConnected && esDevNodesOnly) 
 	return 0;
 
     hierName = (HierName *) node->efnode_name->efnn_hier;
@@ -1871,10 +2129,10 @@ spcnodeVisit(node, res, cap)
     }
     if (node->efnode_attrs && !esNoAttrs)
     {
-	(void) fprintf(esSpiceF, "**nodeattr %s :",nsn );
+	fprintf(esSpiceF, "**nodeattr %s :",nsn );
 	for (fmt = " %s", ap = node->efnode_attrs; ap; ap = ap->efa_next)
 	{
-	    (void) fprintf(esSpiceF, fmt, ap->efa_text);
+	    fprintf(esSpiceF, fmt, ap->efa_text);
 	    fmt = ",%s";
 	}
 	putc('\n', esSpiceF);
@@ -1892,7 +2150,6 @@ nodeVisitDebug(node, res, cap)
 {
     EFNodeName *nn;
     HierName *hierName;
-    bool isConnected;
     char *fmt, *nsn;
     EFAttr *ap;
     
@@ -2026,7 +2283,6 @@ char *efHNSprintfPrefix(hierName, str)
     return ++str;
 }
 
-
 /*
  * ----------------------------------------------------------------------------
  *
@@ -2155,7 +2411,7 @@ Dev    *dev;
 {
 	   devMerge *fp;
 
-	MALLOC (devMerge *, fp, sizeof(devMerge));
+	fp = (devMerge *) mallocMagic((unsigned) (sizeof(devMerge)));
 	fp->l = l; fp->w = w;
 	fp->g = g; fp->s = s;
 	fp->d = d; fp->b = b;
@@ -2220,9 +2476,9 @@ Dev    *dev;
 		char *t; \
 		int l1 = strlen(a1); \
 		int l2 = strlen(a2); \
-		MALLOC(char *, t, (l1+l2)+1); \
+		t = (char *) mallocMagic((unsigned) ((l1+l2)+1)); \
 		t = (char *) strcat(a1,a2); \
-		FREE(a1); \
+		freeMagic(a1); \
 		a1 = t ; \
 	} \
 }
@@ -2338,7 +2594,7 @@ mergeThem:
 		setDevMult(cfp->esFMIndex, m);
 		esDevsMerged++;
 		/* Need to do attribute stuff here */
-		FREE(fp);
+		freeMagic(fp);
 		return 0;
 	  }
 	}
@@ -2375,7 +2631,7 @@ EFNode *n;
 		initNodeClient(n);
 	nc = (nodeClient *) n->efnode_client;
 	if ( nc->m_w.widths == NULL ) {
-		MALLOC(float *, (nc->m_w.widths), sizeof(float)*efNumResistClasses);
+		(nc->m_w.widths) = (float *) mallocMagic((unsigned) sizeof(float)*efNumResistClasses);
 		for ( i=0; i<EFDevNumTypes ; i++ )
 			nc->m_w.widths[i] = 0.0;
 	}
