@@ -303,7 +303,7 @@ EFArgs(argc, argv, err_result, argsProc, cdata)
     /* Eliminate trailing .ext from input name */
     if ((cp = rindex(inname, '.')) && strcmp(cp, ".ext") == 0)
     {
-	MALLOC(char *, realIn, cp - inname + 1);
+	realIn = (char *) mallocMagic((unsigned)(cp - inname + 1));
 	(void) strncpy(realIn, inname, cp - inname);
 	realIn[cp - inname] = '\0';
 	inname = realIn;
