@@ -61,6 +61,9 @@ EFNode *efBuildDevNode();
 void efNodeAddName();
 void efNodeMerge();
 
+bool efConnBuildName();
+bool efConnInitSubs();
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -162,14 +165,15 @@ efBuildNode(def, nodeName, nodeCap, x, y, layerName, av, ac)
     }
 
     /* Allocate a new node with 'nodeName' as its single name */
-    MALLOC(EFNodeName *, newname, sizeof (EFNodeName));
+    newname = (EFNodeName *) mallocMagic((unsigned)(sizeof (EFNodeName)));
     newname->efnn_hier = EFStrToHN((HierName *) NULL, nodeName);
+    newname->efnn_port = -1;	/* No port assignment */
     newname->efnn_next = NULL;
     HashSetValue(he, (char *) newname);
 
     /* New node itself */
     size = sizeof (EFNode) + (efNumResistClasses - 1) * sizeof (PerimArea);
-    MALLOC(EFNode *, newnode, size);
+    newnode = (EFNode *) mallocMagic((unsigned)(size));
     newnode->efnode_flags = 0;
     newnode->efnode_cap = nodeCap;
     newnode->efnode_attrs = (EFAttr *) NULL;
@@ -241,7 +245,7 @@ efBuildAttr(def, nodeName, r, layerName, text)
     nn = (EFNodeName *) HashGetValue(he);
 
     size = ATTRSIZE(strlen(text));
-    MALLOC(EFAttr *, ap, size);
+    ap = (EFAttr *) mallocMagic((unsigned)(size));
     (void) strcpy(ap->efa_text, text);
     ap->efa_type =
 	efBuildAddStr(EFLayerNames, &EFLayerNumNames, MAXTYPES, layerName);
@@ -357,7 +361,7 @@ efBuildKill(def, name)
 {
     Kill *kill;
 
-    MALLOC(Kill *, kill, sizeof (Kill));
+    kill = (Kill *) mallocMagic((unsigned)(sizeof (Kill)));
     kill->kill_name = EFStrToHN((HierName *) NULL, name);
     kill->kill_next = def->def_kills;
     def->def_kills = kill;
@@ -496,10 +500,13 @@ efBuildDevice(def, class, type, r, argc, argv)
 	    if (hasModel)
 		argstart = 2;
 	    break;
+	case DEV_RSUBCKT:
+	    argstart = 2;
+	    break;
     }
     nterminals = (argc - argstart) / 3;
 
-    MALLOC(Dev *, newdev, (unsigned) DevSize(nterminals));
+    newdev = (Dev *) mallocMagic((unsigned) DevSize(nterminals));
     newdev->dev_nterm = nterminals;
     newdev->dev_rect = *r;
     newdev->dev_type = efBuildAddStr(EFDevTypes, &EFDevNumTypes, MAXDEVTYPES, type);
@@ -522,6 +529,10 @@ efBuildDevice(def, class, type, r, argc, argv)
 	    if (strncmp(argv[2], "None", 4) != 0)
 		newdev->dev_subsnode = efBuildDevNode(def, argv[2], TRUE);
 	    break;
+	case DEV_RSUBCKT:
+	    newdev->dev_values.dev_lw.dev_length = atoi(argv[0]);
+	    newdev->dev_values.dev_lw.dev_width = atoi(argv[1]);
+	    break;
 	case DEV_DIODE:		/* do nothing */
 	    break;
 	case DEV_RES:
@@ -529,24 +540,25 @@ efBuildDevice(def, class, type, r, argc, argv)
 	    {
 		newdev->dev_values.dev_lw.dev_length = atoi(argv[0]);
 		newdev->dev_values.dev_lw.dev_width = atoi(argv[1]);
-		newdev->dev_values.dev_res = 0.0;
 	    }
 	    else if (StrIsNumeric(argv[0]))
 	    {
 		newdev->dev_values.dev_res = (float)atof(argv[0]);
-		newdev->dev_values.dev_lw.dev_length = 0;
-		newdev->dev_values.dev_lw.dev_width = 0;
 	    }
 	    else
 	    {
 		if (hasModel)
+		{
 		    efReadError("Error: expected L and W, got %s %s\n", argv[0],
 				argv[1]);
+		    newdev->dev_values.dev_lw.dev_length = 0;
+		    newdev->dev_values.dev_lw.dev_width = 0;
+		}
 		else
+		{
 		    efReadError("Error: expected resistance value, got %s\n", argv[0]);
-		newdev->dev_values.dev_res = 0.0;
-		newdev->dev_values.dev_lw.dev_length = 0;
-		newdev->dev_values.dev_lw.dev_width = 0;
+		    newdev->dev_values.dev_res = 0.0;
+		}
 	    }
 	    break;
 	case DEV_CAP:
@@ -554,24 +566,25 @@ efBuildDevice(def, class, type, r, argc, argv)
 	    {
 		newdev->dev_values.dev_lw.dev_length = atoi(argv[0]);
 		newdev->dev_values.dev_lw.dev_width = atoi(argv[1]);
-		newdev->dev_values.dev_cap = 0.0;
 	    }
 	    else if (StrIsNumeric(argv[0]))
 	    {
 		newdev->dev_values.dev_cap = (float)atof(argv[0]);
-		newdev->dev_values.dev_lw.dev_length = 0;
-		newdev->dev_values.dev_lw.dev_width = 0;
 	    }
 	    else
 	    {
 		if (hasModel)
+		{
 		    efReadError("Error: expected L and W, got %s %s\n", argv[0],
 				argv[1]);
+		    newdev->dev_values.dev_lw.dev_length = 0;
+		    newdev->dev_values.dev_lw.dev_width = 0;
+		}
 		else
+		{
 		    efReadError("Error: expected capacitance value, got %s\n", argv[0]);
-		newdev->dev_values.dev_cap = 0.0;
-		newdev->dev_values.dev_lw.dev_length = 0;
-		newdev->dev_values.dev_lw.dev_width = 0;
+		    newdev->dev_values.dev_cap = 0.0;
+		}
 	    }
 	    break;
     }
@@ -611,7 +624,7 @@ efBuildDevice(def, class, type, r, argc, argv)
  *	the global node name table.  If it doesn't already exist, create a
  *	EFNode for it, and set capacitance and area/perimeter to zero.
  *	Set the efnode_flags value to EF_PORT, with the port number encoded
- *	in the field above the flag bits (PORT_SHIFT bits up).
+ *	in the efNodeName structure.
  *
  * ----------------------------------------------------------------------------
  */
@@ -637,7 +650,10 @@ efBuildPortNode(def, name, idx, x, y, layername)
 	nn = (EFNodeName *) HashGetValue(he);
     }
     if (nn != (EFNodeName *) NULL)
-	nn->efnn_node->efnode_flags |= (EF_PORT | (idx << PORT_SHIFT));
+    {
+	nn->efnn_node->efnode_flags |= EF_PORT;
+	nn->efnn_port = idx;
+    }
 }
 
 
@@ -778,7 +794,7 @@ efBuildUse(def, subDefName, subUseId, ta, tb, tc, td, te, tf)
     if (newdef == NULL)
 	newdef = efDefNew(subDefName);
 
-    MALLOC(Use *, newuse, sizeof (Use));
+    newuse = (Use *) mallocMagic((unsigned)(sizeof (Use)));
     newuse->use_def = newdef;
     newuse->use_trans.t_a = ta;
     newuse->use_trans.t_b = tb;
@@ -840,7 +856,7 @@ efBuildConnect(def, nodeName1, nodeName2, deltaC, av, ac)
     unsigned size = sizeof (Connection)
 		    + (efNumResistClasses - 1) * sizeof (PerimArea);
 
-    MALLOC(Connection *, conn, size);
+    conn = (Connection *) mallocMagic((unsigned)(size));
     if (efConnInitSubs(conn, nodeName1, nodeName2))
     {
 	conn->conn_cap = (EFCapValue) deltaC;
@@ -884,7 +900,7 @@ efBuildResistor(def, nodeName1, nodeName2, resistance)
 {
     Connection *conn;
 
-    MALLOC(Connection *, conn, sizeof (Connection));
+    conn = (Connection *) mallocMagic((unsigned)(sizeof (Connection)));
     if (efConnInitSubs(conn, nodeName1, nodeName2))
     {
 	conn->conn_res = resistance;
@@ -921,7 +937,7 @@ efBuildCap(def, nodeName1, nodeName2, cap)
 {
     Connection *conn;
 
-    MALLOC(Connection *, conn, sizeof (Connection));
+    conn = (Connection *) mallocMagic((unsigned)(sizeof (Connection)));
     if (efConnInitSubs(conn, nodeName1, nodeName2))
     {
 	conn->conn_cap = (EFCapValue) cap;
@@ -979,9 +995,9 @@ efConnInitSubs(conn, nodeName1, nodeName2)
     return TRUE;
 
 bad:
-    if (c1->cn_name) FREE((char *) c1->cn_name);
-    if (c2->cn_name) FREE((char *) c2->cn_name);
-    FREE((char *) conn);
+    if (c1->cn_name) freeMagic((char *) c1->cn_name);
+    if (c2->cn_name) freeMagic((char *) c2->cn_name);
+    freeMagic((char *) conn);
     return FALSE;
 }
 
@@ -1122,7 +1138,7 @@ efNodeAddName(node, he, hn)
     EFNodeName *newnn;
     EFNodeName *oldnn;
 
-    MALLOC(EFNodeName *, newnn, sizeof (EFNodeName));
+    newnn = (EFNodeName *) mallocMagic((unsigned)(sizeof (EFNodeName)));
     newnn->efnn_node = node;
     newnn->efnn_hier = hn;
     HashSetValue(he, (char *) newnn);
@@ -1222,6 +1238,37 @@ efNodeMerge(node1, node2)
 	     */
 	    nnlast->efnn_next = node1->efnode_name;
 	    node1->efnode_name = node2->efnode_name;
+
+	    /*
+	     * Choose the new location only if node2's location is a valid one,
+	     * i.e, node2 wasn't created before it was mentioned.  This is mainly
+	     * to deal with new fets, resistors, and capacitors created by resistance
+	     * extraction, which appear with their full hierarchical names in the
+	     * .ext file for the root cell.
+	     *
+	     * This code has been moved up from below so that the original
+	     * location and type will be prefered when the original name
+	     * is preferred.
+	     *
+	     * I am purposefully subverting the original specification that
+	     * the node refer to the bottom corner of the network.  Does
+	     * this have any effect on exttosim or exttospice?
+	     *
+	     *					Tim, 6/14/04
+	     */
+	    if (node2->efnode_type > 0)
+	    {
+		node1->efnode_loc = node2->efnode_loc;
+		node1->efnode_type = node2->efnode_type;
+
+		if (node2->efnode_loc.r_ybot < node1->efnode_loc.r_ybot
+		    || (node2->efnode_loc.r_ybot == node1->efnode_loc.r_ybot
+		    && node2->efnode_loc.r_xbot < node1->efnode_loc.r_xbot))
+		{
+//		    node1->efnode_loc = node2->efnode_loc;
+//		    node1->efnode_type = node2->efnode_type;
+		}
+	    }
 	}
 	else
 	{
@@ -1245,24 +1292,6 @@ efNodeMerge(node1, node2)
 	node2->efnode_attrs = (EFAttr *) NULL;	/* Sanity */
     }
 
-    /*
-     * Choose the new location only if node2's location is a valid one,
-     * i.e, node2 wasn't created before it was mentioned.  This is mainly
-     * to deal with new fets, resistors, and capacitors created by resistance
-     * extraction, which appear with their full hierarchical names in the
-     * .ext file for the root cell.
-     */
-    if (node2->efnode_type > 0)
-    {
-	if (node2->efnode_loc.r_ybot < node1->efnode_loc.r_ybot
-	    || (node2->efnode_loc.r_ybot == node1->efnode_loc.r_ybot
-		    && node2->efnode_loc.r_xbot < node1->efnode_loc.r_xbot))
-	{
-	    node1->efnode_loc = node2->efnode_loc;
-	    node1->efnode_type = node2->efnode_type;
-	}
-    }
-
     /* Unlink node2 from list for def */
     node2->efnode_prev->efnhdr_next = node2->efnode_next;
     node2->efnode_next->efnhdr_prev = node2->efnode_prev;
@@ -1274,8 +1303,16 @@ efNodeMerge(node1, node2)
     if ((node2->efnode_flags & EF_DEVTERM) == 0)
 	node1->efnode_flags &= ~EF_DEVTERM;
 
+    /*
+     * If node2 has the EF_PORT flag set but node1 does not,
+     * then copy the port record in the flags to node1.
+     */
+    if ((node2->efnode_flags & EF_PORT) &&
+		!(node1->efnode_flags & EF_PORT))
+	node1->efnode_flags |= EF_PORT;
+
     /* Get rid of node2 */
-    FREE((char *) node2);
+    freeMagic((char *) node2);
 }
 
 /*
@@ -1316,7 +1353,7 @@ efFreeNodeTable(table)
 	{
 	    for (hn = nn->efnn_hier; hn; hn = hn->hn_parent)
 		(void) HashFind(&efFreeHashTable, (char *) hn);
-	    FREE((char *) nn);
+	    freeMagic((char *) nn);
 	}
 }
 
@@ -1349,8 +1386,8 @@ efFreeNodeList(head)
 	    node = (EFNode *) node->efnode_next)
     {
 	for (ap = node->efnode_attrs; ap; ap = ap->efa_next)
-	    FREE((char *) ap);
-	FREE((char *) node);
+	    freeMagic((char *) ap);
+	freeMagic((char *) node);
     }
 }
 
@@ -1374,7 +1411,7 @@ void
 efFreeConn(conn)
     Connection *conn;
 {
-    if (conn->conn_name1) FREE(conn->conn_name1);
-    if (conn->conn_name2) FREE(conn->conn_name2);
-    FREE((char *) conn);
+    if (conn->conn_name1) freeMagic(conn->conn_name1);
+    if (conn->conn_name2) freeMagic(conn->conn_name2);
+    freeMagic((char *) conn);
 }
