@@ -117,18 +117,18 @@ EFFlatBuild(name, flags)
 
     if (flags & EF_FLATNODES)
     {
-	(void) efFlatNodes(&efFlatContext);
-	(void) efFlatKills(&efFlatContext);
+	efFlatNodes(&efFlatContext);
+	efFlatKills(&efFlatContext);
 	efFlatGlob();
     }
 
     /* Must happen after kill processing */
     if (flags & EF_FLATCAPS)
-	(void) efFlatCaps(&efFlatContext);
+	efFlatCaps(&efFlatContext);
 
     /* Distances are independent of kill processing */
     if (flags & EF_FLATDISTS)
-	(void) efFlatDists(&efFlatContext);
+	efFlatDists(&efFlatContext);
 
     if (efHNStats) efHNPrintSizes("after building flattened table");
 }
@@ -167,6 +167,7 @@ EFFlatDone()
     HashKill(&efHNUseHashTable);
 }
 
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -201,22 +202,68 @@ EFFlatDone()
 efFlatNodes(hc)
     HierContext *hc;
 {
-//  if (!(hc->hc_use->use_def->def_flags & DEF_SUBCIRCUIT))
-//  {
-//	/* Recursively flatten each use, except in defined subcircuits */
-	(void) efHierSrUses(hc, efFlatNodes, (ClientData) NULL);
-//  }
+    (void) efHierSrUses(hc, efFlatNodes, (ClientData) NULL);
 
     /* Add all our own nodes to the table */
-    efAddNodes(hc);
+    efAddNodes(hc, FALSE);
 
     /* Process our own connections and adjustments */
-//  if (!(hc->hc_use->use_def->def_flags & DEF_SUBCIRCUIT))
+    (void) efAddConns(hc);
+
+    return (0);
+}
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * efFlatNodesStdCell --
+ *
+ * Recursive procedure to flatten the nodes in hc->hc_use->use_def,
+ * using a depth-first post-order traversal of the hierarchy.  We stop
+ * whenever we reach a subcircuit definition, only enumerating its ports.
+ *
+ * Algorithm:
+ *	We first recursivly call efFlatNodes for all of our children uses.
+ *	This adds their node names to the global node table.  Next we add
+ *	our own nodes to the table.  Some nodes will have to be merged
+ *	by connections made in this def, or at least will require adjustments
+ *	to their resistance or capacitance.  We walk down the connection
+ *	list hc->hc_use->use_def->def_conns to do this merging.  Whenever
+ *	two nodes merge, the EFNodeName list for the resulting node is
+ *	rearranged to begin with the highest precedence name from the lists
+ *	for the two nodes being combined.  See efNodeMerge for a discussion
+ *	of precedence.
+ *
+ * Results:
+ *	Returns 0 to keep efHierSrUses going.
+ *
+ * Side effects:
+ *	Adds node names to the table of flattened node names efNodeHashTable.
+ *	May merge nodes from the list efNodeList as per the connection
+ *	list hc->hc_use->use_def->def_conns.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+efFlatNodesStdCell(hc)
+    HierContext *hc;
+{
+    if (!(hc->hc_use->use_def->def_flags & DEF_SUBCIRCUIT))
+    {
+  	/* Recursively flatten each use, except in defined subcircuits */
+	(void) efHierSrUses(hc, efFlatNodesStdCell, (ClientData) NULL);
+    }
+
+    /* Add all our own nodes to the table */
+    efAddNodes(hc, TRUE);
+
+    /* Process our own connections and adjustments */
+    if (!(hc->hc_use->use_def->def_flags & DEF_SUBCIRCUIT))
 	(void) efAddConns(hc);
 
     return (0);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -224,7 +271,8 @@ efFlatNodes(hc)
  *
  * Add all the nodes defined by the def 'hc->hc_use->use_def' to the
  * global symbol table.  Each global name is prefixed by the hierarchical
- * name component hc->hc_hierName.
+ * name component hc->hc_hierName.  If "stdcell" is TRUE, we ONLY add nodes
+ * that are defined ports.  Otherwise, we add all nodes.
  *
  * Results:
  *	None.
@@ -235,8 +283,9 @@ efFlatNodes(hc)
  * ----------------------------------------------------------------------------
  */
 
-efAddNodes(hc)
+efAddNodes(hc, stdcell)
     HierContext *hc;
+    bool stdcell;
 {
     Def *def = hc->hc_use->use_def;
     EFNodeName *nn, *newname, *oldname;
@@ -246,7 +295,7 @@ efAddNodes(hc)
     int scale, size, asize;
     HashEntry *he;
     Transform t;
-//  bool is_subcircuit = (def->def_flags & DEF_SUBCIRCUIT) ? TRUE : FALSE;
+    bool is_subcircuit = (def->def_flags & DEF_SUBCIRCUIT) ? TRUE : FALSE;
 
     scale = def->def_scale;
     t = hc->hc_trans;
@@ -259,14 +308,15 @@ efAddNodes(hc)
 	    node = (EFNode *) node->efnode_next)
     {
 	/* In subcircuits, only enumerate the ports */
-//	if (is_subcircuit && !(node->efnode_flags & EF_PORT)) continue;
+ 	if (stdcell && is_subcircuit && !(node->efnode_flags & EF_PORT))
+	    continue;
 
-	MALLOC(EFNode *, newnode, size);
+	newnode = (EFNode *) mallocMagic((unsigned)(size));
 	newnode->efnode_attrs = (EFAttr *) NULL;
 	for (ap = node->efnode_attrs; ap; ap = ap->efa_next)
 	{
 	    asize = ATTRSIZE(strlen(ap->efa_text));
-	    MALLOC(EFAttr *, newap, asize);
+	    newap = (EFAttr *) mallocMagic((unsigned)(asize));
 	    (void) strcpy(newap->efa_text, ap->efa_text);
 	    GeoTransRect(&t, &ap->efa_loc, &newap->efa_loc);
 	    newap->efa_type = ap->efa_type;
@@ -323,7 +373,7 @@ efAddNodes(hc)
 	     * first (since the first name is the "canonical" name for the
 	     * node).  The order of the remaining names will be reversed.
 	     */
-	    MALLOC(EFNodeName *, newname, sizeof (EFNodeName));
+	    newname = (EFNodeName *) mallocMagic((unsigned)(sizeof (EFNodeName)));
 	    HashSetValue(he, (char *) newname);
 	    newname->efnn_node = newnode;
 	    newname->efnn_hier = hierName;
@@ -340,7 +390,7 @@ efAddNodes(hc)
 	}
     }
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -516,7 +566,7 @@ efFlatGlob()
 	     * Create a new EFNodeName that points to nodeFlat, but
 	     * don't link it in to nodeFlat->efnode_name yet.
 	     */
-	    MALLOC(EFNodeName *, nameGlob, sizeof (EFNodeName));
+	    nameGlob = (EFNodeName *) mallocMagic((unsigned)(sizeof (EFNodeName)));
 	    HashSetValue(heGlob, (ClientData) nameGlob);
 	    nameGlob->efnn_node = nodeFlat;
 	    nameGlob->efnn_hier = (HierName *) heGlob->h_key.h_ptr;
@@ -570,7 +620,7 @@ efFlatGlob()
 	}
 	else
 	{
-	    FREE((char *) nameGlob);
+	    freeMagic((char *) nameGlob);
 	    EFHNFree(hnGlob, (HierName *) NULL, HN_GLOBAL);
 	}
     }
@@ -628,7 +678,7 @@ efFlatGlobCopy(hierName)
     int size;
 
     size = HIERNAMESIZE(strlen(hierName->hn_name));
-    MALLOC(HierName *, hNew, size);
+    hNew = (HierName *) mallocMagic((unsigned)(size));
     (void) strcpy(hNew->hn_name, hierName->hn_name);
     hNew->hn_parent = (HierName *) NULL;
     hNew->hn_hash = hierName->hn_hash;
@@ -903,7 +953,7 @@ double c;
 {
 	EFCapValue *capp = (EFCapValue *)HashGetValue(he);
 	if(capp == NULL) {
-		MALLOC(EFCapValue *, capp, sizeof(EFCapValue));
+		capp = (EFCapValue *) mallocMagic((unsigned)(sizeof(EFCapValue)));
 		HashSetValue(he, capp);
 	}
 	*capp = (EFCapValue) c;
