@@ -78,7 +78,7 @@ efHierSrUses(hc, func, cdata)
     Use *u;
 
     /* We should not descend into subcircuit "black boxes" */
-    if (hc->hc_use->use_def->def_flags & DEF_SUBCIRCUIT) return 0;
+//  if (hc->hc_use->use_def->def_flags & DEF_SUBCIRCUIT) return 0;
 
     for (u = hc->hc_use->use_def->def_uses; u; u = u->use_next)
     {
