@@ -24,7 +24,7 @@ static char rcsid[] = "$Header: /ufs/repository/magic/extflat/EFname.c,v 1.4 200
 #include <string.h>
 #include <math.h>
 
-#include "magic/tclmagic.h"
+#include "tcltk/tclmagic.h"
 #include "misc/magic.h"
 #include "utils/geometry.h"
 #include "utils/geofast.h"
@@ -86,7 +86,7 @@ EFHNIsGlob(hierName)
 {
 #ifdef MAGIC_WRAPPER
     char *retstr;
-    retstr = Tcl_GetVar(magicinterp, hierName->hn_name, TCL_GLOBAL_ONLY);
+    retstr = (char *)Tcl_GetVar(magicinterp, hierName->hn_name, TCL_GLOBAL_ONLY);
     if (retstr != NULL) return TRUE;
 #endif
     return hierName->hn_name[strlen(hierName->hn_name) - 1] == '!';
@@ -122,7 +122,7 @@ EFHNIsGND(hierName)
     if (hierName->hn_parent != (HierName *) NULL) return FALSE;
 
 #ifdef MAGIC_WRAPPER
-    retstr = Tcl_GetVar(magicinterp, "GND", TCL_GLOBAL_ONLY);
+    retstr = (char *)Tcl_GetVar(magicinterp, "GND", TCL_GLOBAL_ONLY);
     if (retstr != NULL)
 	if (!strcmp(hierName->hn_name, retstr)) return TRUE;
 #endif
@@ -161,7 +161,7 @@ EFHNConcat(prefix, suffix)
 	    prev = new, suffix = suffix->hn_parent)
     {
 	size = HIERNAMESIZE(strlen(suffix->hn_name));
-	MALLOC(HierName *, new, size);
+	new = (HierName *) mallocMagic((unsigned)(size));
 	if (efHNStats) efHNRecord(size, HN_CONCAT);
 	new->hn_hash = suffix->hn_hash;
 	(void) strcpy(new->hn_name, suffix->hn_name);
@@ -223,7 +223,7 @@ EFStrToHN(prefix, suffixStr)
 	if (*cp == '/' || *cp == '\0')
 	{
 	    size = HIERNAMESIZE(cp - slashPtr);
-	    MALLOC(HierName *, hierName, size);
+	    hierName = (HierName *) mallocMagic((unsigned)(size));
 	    if (efHNStats) efHNRecord(size, HN_ALLOC);
 	    efHNInit(hierName, slashPtr, cp);
 	    hierName->hn_parent = prefix;
@@ -451,7 +451,7 @@ EFHNFree(hierName, prefix, type)
 	if (hn == prefix)
 	    break;
 
-	FREE((char *) hn);
+	freeMagic((char *) hn);
 	if (efHNStats)
 	{
 	    int len = strlen(hn->hn_name);
@@ -636,7 +636,7 @@ efHNFromUse(hc, prefix)
     }
 
     size = HIERNAMESIZE(strlen(namePtr));
-    MALLOC(HierName *, hierName, size);
+    hierName = (HierName *) mallocMagic ((unsigned)(size));
     if (efHNStats) efHNRecord(size, HN_FROMUSE);
     efHNInit(hierName, namePtr, (char *) NULL);
     hierName->hn_parent = prefix;
@@ -645,7 +645,7 @@ efHNFromUse(hc, prefix)
     he = HashFind(&efHNUseHashTable, (char *) hierName);
     if (HashGetValue(he))
     {
-	FREE((char *) hierName);
+	freeMagic((char *) hierName);
 	return (HierName *) HashGetValue(he);
     }
     HashSetValue(he, (ClientData) hierName);
@@ -847,7 +847,7 @@ efHNDistCopy(dist)
 {
     Distance *distNew;
 
-    MALLOC(Distance *, distNew, sizeof (Distance));
+    distNew = (Distance *) mallocMagic ((unsigned)(sizeof (Distance)));
     *distNew = *dist;
     return (char *) distNew;
 }
@@ -871,7 +871,7 @@ efHNDistKill(dist)
     for (hn = dist->dist_2; hn; hn = hn->hn_parent)
 	(void) HashFind(&efFreeHashTable, (char *) hn);
 
-    FREE((char *) dist);
+    freeMagic((char *) dist);
 }
 
 /*
@@ -950,7 +950,7 @@ efHNDump()
 
     HashStartSearch(&hs);
     while (he = HashNext(&efNodeHashTable, &hs))
-	(void) fprintf(f, "%s\n", EFHNToStr((HierName *) he->h_key.h_ptr));
+	fprintf(f, "%s\n", EFHNToStr((HierName *) he->h_key.h_ptr));
 
     (void) fclose(f);
 }
