@@ -26,7 +26,7 @@ static char rcsid[] = "$Header: /ufs/repository/magic/extflat/EFread.c,v 1.2 200
 #include <string.h>
 #include <ctype.h>
 
-#include "magic/tclmagic.h"
+#include "tcltk/tclmagic.h"
 #include "misc/magic.h"
 #include "utils/geometry.h"
 #include "utils/hash.h"
@@ -41,7 +41,7 @@ static char rcsid[] = "$Header: /ufs/repository/magic/extflat/EFread.c,v 1.2 200
 #ifndef MAGIC_WRAPPER
 /* This must match the definition for extDevTable in extract/ExtBasic.c */
 char *extDevTable[] = {"fet", "mosfet", "bjt", "devres", "devcap",
-		"diode", "subckt", NULL};
+		"diode", "subckt", "rsubckt", NULL};
 #endif
 
 /*
@@ -91,7 +91,7 @@ char *efReadFileName;	/* Name of file currently being read */
 int efReadLineNum;	/* Current line number in above file */
 
 /* Data local to this file */
-static int efReadDef();
+static bool efReadDef();
 
 /* atoCap - convert a string to a EFCapValue */
 #define	atoCap(s)	((EFCapValue)atof(s))
@@ -121,18 +121,18 @@ static int efReadDef();
  */
 
 bool
-EFReadFile(name, dosubckt, resist)
+EFReadFile(name, dosubckt, resist, noscale)
     char *name; /* Name of def to be read in */
     bool dosubckt, resist;
 {
     Def *def;
-    int  rc;
+    bool  rc;
 
     def = efDefLook(name);
     if (def == NULL)
 	def = efDefNew(name);
 
-    rc = efReadDef(def, dosubckt, resist);
+    rc = efReadDef(def, dosubckt, resist, noscale);
     if (EFArgTech) EFTech = StrDup((char **) NULL, EFArgTech);
     if (EFScale == 0) EFScale = 1;
 
@@ -161,9 +161,9 @@ EFReadFile(name, dosubckt, resist)
  */
 
 bool
-efReadDef(def, dosubckt, resist)
+efReadDef(def, dosubckt, resist, noscale)
    Def *def;
-   bool dosubckt, resist;
+   bool dosubckt, resist, noscale;
 {
     int argc, ac, n;
     EFCapValue cap;
@@ -176,6 +176,7 @@ efReadDef(def, dosubckt, resist)
     Rect r;
     bool rc = TRUE;
     bool DoResist = resist;
+    bool DoSubCircuit = dosubckt;
 
     /* Mark def as available */
     def->def_flags |= DEF_AVAILABLE;
@@ -231,11 +232,14 @@ readfile:
 		    efReadError("Bad linear scaling = 0; reset to 1.\n");
 		    lscale = 1;
 		}
-		def->def_scale = lscale;
-		if (EFScale != lscale)
+		if (noscale == FALSE)
 		{
-		    if (EFScale != 0) efScaleChanged = TRUE, EFScale = 1;
-		    else EFScale = lscale;
+		    def->def_scale = lscale;
+		    if (EFScale != lscale)
+		    {
+			if (EFScale != 0) efScaleChanged = TRUE, EFScale = 1;
+			else EFScale = lscale;
+		    }
 		}
 		break;
 
@@ -260,9 +264,10 @@ readfile:
 		break;
 
 	    /* replaces "fet" (below) */
-	    /* device mosfet|bjt|subckt type xlo ylo xhi yhi area perim */
-	    /*		[substrate] GATE T1 T2 ...			*/
-	    /* device res|cap type xlo ylo xhi yhi value GATE T1 T2 ... */
+	    /* device mosfet|bjt|subckt	type xlo ylo xhi yhi		*/
+	    /*		area perim [substrate] GATE T1 T2 ...		*/
+	    /* device res|cap|rsubckt type xlo ylo xhi yhi value	*/
+	    /*		GATE T1 T2 ...					*/
 	    case DEVICE:
 		r.r_xbot = atoi(argv[3]);
 		r.r_ybot = atoi(argv[4]);
@@ -296,6 +301,9 @@ readfile:
 			else
 			   ac = 9;	/* Has device L and W */
 			break;
+		    case DEV_RSUBCKT:
+			ac = 9;	/* Similar to RES, with a model */
+			break;
 		}
 		if (((argc - ac) % 3) != 0)
 		{
@@ -358,7 +366,7 @@ readfile:
 
 	    /* port name num xl yl xh yh type */
 	    case PORT:
-		if (dosubckt)
+		if (DoSubCircuit)
 		{
 		    DoResist = FALSE;
 		    def->def_flags |= DEF_SUBCIRCUIT;
@@ -512,10 +520,16 @@ resistChanged:
 	    goto readfile;
     }
 
+    /* If we are considering standard cells, only the first level of	*/
+    /* subcircuits is meaningful.					*/
+
+    if (def->def_flags & DEF_SUBCIRCUIT)
+	DoSubCircuit = FALSE;
+
     /* Read in each def that has not yet been read in */
     for (use = def->def_uses; use; use = use->use_next)
 	if ((use->use_def->def_flags & DEF_AVAILABLE) == 0)
-	    if(efReadDef(use->use_def, dosubckt, resist) != TRUE)
+	    if(efReadDef(use->use_def, DoSubCircuit, resist, noscale) != TRUE)
 		rc = FALSE;
 
     return rc;
