@@ -43,6 +43,9 @@ static char rcsid[] = "$Header: /ufs/repository/magic/extflat/EFsym.c,v 1.2 2001
 #include "extflat/extflat.h"
 #include "extflat/EFint.h"
 
+/* Forward declarations */
+bool efSymAdd();
+
 HashTable efSymHash;
 
 /*
