@@ -44,6 +44,7 @@ extern HierContext efFlatContext;
 extern void efDevFixLW();
 extern void efHNOutPrefix();
 
+bool efDevKilled();
 
 /*
  * ----------------------------------------------------------------------------
@@ -62,15 +63,16 @@ extern void efHNOutPrefix();
  * call the user-supplied procedure (*subProc)(), which should be of
  * the following form:
  *
- * 	(*subProc)(use, hierName, cdata)
+ * 	(*subProc)(use, hierName, is_top)
  *	    Use *use;
  *	    HierName *hierName;
- *	    ClientData cdata;
+ *	    Boolean is_top;
  *	{
  *	}
  *
- * The procedure should return 0 normally, or 1 to abort the
- * search.
+ * is_top will be TRUE for the top-level cell, and FALSE for all
+ * other cells.  The procedure should return 0 normally, or 1 to abort
+ * the search.
  *
  * Results:
  *	Returns 0 if terminated normally, or 1 if the search
@@ -88,10 +90,26 @@ EFVisitSubcircuits(subProc, cdata)
     ClientData cdata;
 {
     CallArg ca;
+    HierContext *hc;
+    int efVisitSubcircuits();   /* Forward declaration */
+
+    /* If the top-level def is defined as a subcircuit, call topProc */
+
+    hc = &efFlatContext;
+    if (hc->hc_use->use_def->def_flags & DEF_SUBCIRCUIT)
+	if ((*subProc)(hc->hc_use, hc->hc_hierName, TRUE))
+	    return 1;
+
+    /* For each subcell of the top-level def that is defined as */
+    /* a subcircuit, call subProc.				*/
 
     ca.ca_proc = subProc;
     ca.ca_cdata = cdata;
-    return efVisitSubcircuits(&efFlatContext, (ClientData) &ca);
+
+    if (efHierSrUses(hc, efVisitSubcircuits, (ClientData) &ca))
+	return 1;
+
+    return 0;
 }
 
 /*
@@ -159,6 +177,8 @@ EFGetLengthAndWidth(dev, lptr, wptr)
 	case DEV_MOSFET:
 	case DEV_BJT:
 	case DEV_SUBCKT:
+	case DEV_RES:
+	case DEV_CAP:
 	    l = dev->dev_values.dev_lw.dev_length;
 	    w = dev->dev_values.dev_lw.dev_width;
 	    break;
@@ -191,7 +211,7 @@ EFGetLengthAndWidth(dev, lptr, wptr)
 	    break;
 
 	default:
-	    l = w = 0;	/* RES and CAP devices shouldn't use l & w. */
+	    l = w = 0;	/* DIODE device shouldn't use l & w. */
 	    break;
     }
 
