@@ -638,9 +638,9 @@ extArrayAdjust(ha, et1, et2)
 
 	ck = (CoupleKey *) he->h_key.h_words;
 	name = extArrayNodeName(ck->ck_1, ha, et1, et2);
-	(void) fprintf(ha->ha_outf, "cap \"%s\" ", name);
+	fprintf(ha->ha_outf, "cap \"%s\" ", name);
 	name = extArrayNodeName(ck->ck_2, ha, et1, et2);
-	(void) fprintf(ha->ha_outf, "\"%s\" %lg\n", name, cap);
+	fprintf(ha->ha_outf, "\"%s\" %lg\n", name, cap);
     }
 }
 
@@ -908,7 +908,7 @@ extArrayHardNode(tp, def, ha)
 	LabelList *ll;
 
 	lreg = (LabRegion *) extGetRegion(tp);
-	MALLOC(LabelList *, ll, sizeof (LabelList));
+	ll = (LabelList *) mallocMagic((unsigned) (sizeof (LabelList)));
 	lreg->lreg_labels = ll;
 	ll->ll_next = (LabelList *) NULL;
 	ll->ll_label = arg.hw_label;
