@@ -26,9 +26,7 @@ static char sccsid[] = "@(#)ExtBasic.c	4.13 MAGIC (Berkeley) 12/5/85";
 #include <string.h>
 #include <math.h>
 
-#ifdef MAGIC_WRAPPER
-#include "magic/tclmagic.h"
-#endif
+#include "tcltk/tclmagic.h"
 #include "misc/magic.h"
 #include "utils/geometry.h"
 #include "utils/geofast.h"
@@ -54,7 +52,7 @@ static char sccsid[] = "@(#)ExtBasic.c	4.13 MAGIC (Berkeley) 12/5/85";
 /* ext2spice/ext2sim are compiled as separate programs (i.e., non-Tcl)	   */
 
 char *extDevTable[] = {"fet", "mosfet", "bjt", "devres", "devcap",
-	"diode", "subckt", NULL};
+	"diode", "subckt", "rsubckt", NULL};
 
 /* --------------------- Data local to this file ---------------------- */
 
@@ -126,6 +124,7 @@ void extOutputTrans();
 void extTransOutTerminal();
 void extTransBad();
 
+bool extLabType();
 
 
 /*
@@ -352,7 +351,7 @@ extOutputNodes(nodeList, outFile)
 	for (ll = reg->nreg_labels; ll; ll = ll->ll_next)
 	    if (ll->ll_attr == LL_PORTATTR)
 	    {
-		(void) fprintf(outFile, "port \"%s\" %d %d %d %d %d %s\n",
+		fprintf(outFile, "port \"%s\" %d %d %d %d %d %s\n",
 			ll->ll_label->lab_text,
 			ll->ll_label->lab_flags & PORT_NUM_MASK,
 			ll->ll_label->lab_rect.r_xbot,
@@ -381,16 +380,16 @@ extOutputNodes(nodeList, outFile)
 	text = extNodeName((LabRegion *) reg);
 	intR = (reg->nreg_resist+rround)/ExtCurStyle->exts_resistScale;
 	finC = reg->nreg_cap/ExtCurStyle->exts_capScale;
-	(void) fprintf(outFile, "node \"%s\" %d %lg", text, intR, finC);
+	fprintf(outFile, "node \"%s\" %d %lg", text, intR, finC);
 
 	/* Output its location (lower-leftmost point and type name) */
-	(void) fprintf(outFile, " %d %d %s",
+	fprintf(outFile, " %d %d %s",
 		    reg->nreg_ll.p_x, reg->nreg_ll.p_y,
 		    DBTypeShortName(reg->nreg_type));
 
 	/* Output its area and perimeter for each resistivity class */
 	for (n = 0; n < ExtCurStyle->exts_numResistClasses; n++)
-	    (void) fprintf(outFile, " %d %d", reg->nreg_pa[n].pa_area,
+	    fprintf(outFile, " %d %d", reg->nreg_pa[n].pa_area,
 				reg->nreg_pa[n].pa_perim);
 	(void) putc('\n', outFile);
 
@@ -400,7 +399,7 @@ extOutputNodes(nodeList, outFile)
 	    {
 		/* Don't output the trailing character for node attributes */
 		lab = ll->ll_label;
-		(void) fprintf(outFile, "attr %s %d %d %d %d %s \"",
+		fprintf(outFile, "attr %s %d %d %d %d %s \"",
 			    text, lab->lab_rect.r_xbot, lab->lab_rect.r_ybot,
 			          lab->lab_rect.r_xtop, lab->lab_rect.r_ytop,
 			          DBTypeShortName(lab->lab_type));
@@ -408,7 +407,7 @@ extOutputNodes(nodeList, outFile)
 		n = strlen(cp) - 1;
 		while (n-- > 0)
 		    putc(*cp++, outFile);
-		(void) fprintf(outFile, "\"\n");
+		fprintf(outFile, "\"\n");
 	    }
 
 	/* Output the alternate names for the node */
@@ -417,7 +416,7 @@ extOutputNodes(nodeList, outFile)
 	    {
 		for (ll = ll->ll_next; ll; ll = ll->ll_next)
 		     if (extLabType(ll->ll_label->lab_text, LABTYPE_NAME))
-			(void) fprintf(outFile, "equiv \"%s\" \"%s\"\n",
+			fprintf(outFile, "equiv \"%s\" \"%s\"\n",
 					    text, ll->ll_label->lab_text);
 		break;
 	    }
@@ -959,10 +958,11 @@ extOutputTrans(def, transList, outFile)
 	}
 #ifdef MAGIC_WRAPPER
 	/* If subsName is a Tcl variable (begins with "$"), make the	*/
-	/* variable substitution, if one exists.			*/
-	else if (subsName && subsName[0] == '$')
+	/* variable substitution, if one exists.  Ignore double-$.	*/
+	else if (subsName && subsName[0] == '$' && subsName[1] != '$')
 	{
-	    char *varsub = Tcl_GetVar(magicinterp, &subsName[1], TCL_GLOBAL_ONLY);
+	    char *varsub = (char *)Tcl_GetVar(magicinterp, &subsName[1],
+				TCL_GLOBAL_ONLY);
 	    if (varsub != NULL) subsName = varsub;
 	}
 #endif
@@ -990,7 +990,7 @@ extOutputTrans(def, transList, outFile)
 	switch (ExtCurStyle->exts_deviceClass[t])
 	{
 	    case DEV_FET:	/* old style, perimeter & area */
-		(void) fprintf(outFile, " %d %d \"%s\"",
+		fprintf(outFile, " %d %d \"%s\"",
 		    reg->treg_area, extTransRec.tr_perim, 
 				(subsName == NULL) ? "None" : subsName);
 		break;
@@ -998,6 +998,7 @@ extOutputTrans(def, transList, outFile)
 	    case DEV_MOSFET:	/* new style, length & width */
 	    case DEV_BJT:
 	    case DEV_SUBCKT:
+	    case DEV_RSUBCKT:
 		length = extTransRec.tr_gatelen / 2;	/* (default) */
 		width = 0;
 		isAnnular = FALSE;
@@ -1051,8 +1052,8 @@ extOutputTrans(def, transList, outFile)
 
 		    for (i = 0; i < n; i++)
 			for (lb = extSpecialBounds[i]; lb != NULL; lb = lb->b_next)
-			    FREE((char *)lb);
-		    FREE((char *)extSpecialBounds);
+			    freeMagic((char *)lb);
+		    freeMagic((char *)extSpecialBounds);
 
 		    /* Put the region list back the way we found it: */
 		    /* Re-mark with extTransRec.tr_gatenode */
@@ -1063,7 +1064,7 @@ extOutputTrans(def, transList, outFile)
 		    (void) ExtFindNeighbors(reg->treg_tile, arg.fra_pNum, &arg);
 		}
 
-		(void) fprintf(outFile, " %d %d \"%s\"", length, width,
+		fprintf(outFile, " %d %d \"%s\"", length, width,
 			(subsName == NULL) ? "None" : subsName);
 		break;
 
@@ -1077,7 +1078,7 @@ extOutputTrans(def, transList, outFile)
 		for (n = 0; extTransRec.tr_termlen[n] != 0; n++)
 		{
 		    width += extTransRec.tr_termlen[n];
-		    length -= width;
+		    length -= extTransRec.tr_termlen[n];
 		}
 		if (width)
 		{
@@ -1103,9 +1104,9 @@ extOutputTrans(def, transList, outFile)
 		}
 
 		if (hasModel)	/* semiconductor resistor */
-		    (void) fprintf(outFile, " %d %d", length, width);
+		    fprintf(outFile, " %d %d", (length / 2), (width / 2));
 		else		/* regular resistor */
-		    (void) fprintf(outFile, " %g", dres / 1000.0); /* mOhms -> Ohms */
+		    fprintf(outFile, " %g", dres / 1000.0); /* mOhms -> Ohms */
 
 		break;
 
@@ -1122,7 +1123,7 @@ extOutputTrans(def, transList, outFile)
 					"Capacitor has zero width");
 		    else width <<= 2;	/* total perimeter / 4 */
 
-		    (void) fprintf(outFile, " %d %d", reg->treg_area / width,
+		    fprintf(outFile, " %d %d", reg->treg_area / width,
 				width);
 		}
 		else
@@ -1130,7 +1131,7 @@ extOutputTrans(def, transList, outFile)
 		    dcap = (ExtCurStyle->exts_transGateCap[t] * reg->treg_area) +
 			(ExtCurStyle->exts_transSDCap[t] * extTransRec.tr_perim);
 
-		    (void) fprintf(outFile, " %g", dcap / 1000.0);  /* aF -> fF */
+		    fprintf(outFile, " %g", dcap / 1000.0);  /* aF -> fF */
 		}
 		break;
 	}
@@ -1289,6 +1290,8 @@ extTransPerimFunc(bp)
 	    extTransRec.tr_termvector[thisterm].p_y = 0;
 	    extTransRec.tr_termpos[thisterm].pnum = DBPlane(TiGetType(bp->b_outside));
 	    extTransRec.tr_termpos[thisterm].pt = bp->b_outside->ti_ll;
+
+	    /* Find the total area of this terminal */
 	}
 	else			/* update the region tile position */
 	{
@@ -1516,7 +1519,7 @@ extAnnularPerimFunc(bp)
 			}   
 		    }
 		}
-		FREE((char *)lastlb);
+		freeMagic((char *)lastlb);
 	
 		/* New segment cannot extend more than two existing segments */
 		break;
@@ -1575,22 +1578,22 @@ extTransOutTerminal(lreg, ll, whichTerm, len, outFile)
     char fmt;
 
 
-    (void) fprintf(outFile, " \"%s\" %d", extNodeName(lreg), len);
+    fprintf(outFile, " \"%s\" %d", extNodeName(lreg), len);
     for (fmt = ' '; ll; ll = ll->ll_next) 
 	if (ll->ll_attr == whichTerm)
 	{
-	    (void) fprintf(outFile, "%c\"", fmt);
+	    fprintf(outFile, "%c\"", fmt);
 	    cp = ll->ll_label->lab_text;
 	    n = strlen(cp) - 1;
 	    while (n-- > 0)
 		putc(*cp++, outFile);
 	    ll->ll_attr = LL_NOATTR;
-	    (void) fprintf(outFile, "\"");
+	    fprintf(outFile, "\"");
 	    fmt = ',';
 	}
 
     if (fmt == ' ')
-	(void) fprintf(outFile, " 0");
+	fprintf(outFile, " 0");
 }
 
 /*
@@ -1697,7 +1700,7 @@ extTransFirst(tile, arg)
 {
     TransRegion *reg;
 
-    MALLOC(TransRegion *, reg, sizeof (TransRegion));
+    reg = (TransRegion *) mallocMagic((unsigned) (sizeof (TransRegion)));
     reg->treg_next = (TransRegion *) NULL;
     reg->treg_labels = (LabelList *) NULL;
     reg->treg_pnum = DBNumPlanes;
@@ -1803,6 +1806,14 @@ extNodeAreaFunc(tile, arg)
     Rect r;
     PlaneAndArea pla;
 
+#ifdef NONMANHATTAN
+    if (IsSplit(tile))
+    {
+	type = (SplitSide(tile)) ? SplitRightType(tile) : SplitLeftType(tile);
+	if (type == TT_SPACE) return 0;		/* Should not happen */
+    }
+#endif
+
     /* Compute the resistance for the previous region */
     if (old = (NodeRegion *) arg->fra_region)
 	if (ExtOptions & EXT_DORESISTANCE)
@@ -1811,7 +1822,7 @@ extNodeAreaFunc(tile, arg)
     /* Allocate a new node */
     nclasses = ExtCurStyle->exts_numResistClasses;
     n = sizeof (NodeRegion) + (sizeof (PerimArea) * (nclasses - 1));
-    MALLOC(NodeRegion *, reg, (unsigned) n);
+    reg = (NodeRegion *) mallocMagic((unsigned) n);
     reg->nreg_labels = (LabelList *) NULL;
     reg->nreg_cap = (CapValue) 0;
     reg->nreg_resist = 0;
@@ -1855,14 +1866,14 @@ extNodeAreaFunc(tile, arg)
         type = TiGetTypeExact(tile);
 
 	/* Contacts are replaced by their residues when calculating */
-	/* perimeter capacitance.				    */
+	/* area/perimeter capacitance and resistance.		    */
 
 	residue = (DBIsContact(type)) ?
 		DBPlaneToResidue(type, tilePlaneNum) : type;
 
 	mask = &ExtCurStyle->exts_nodeConn[type];
-	resMask = &ExtCurStyle->exts_typesResistChanged[type];
-	resistClass = ExtCurStyle->exts_typeToResistClass[type];
+	resMask = &ExtCurStyle->exts_typesResistChanged[residue];
+	resistClass = ExtCurStyle->exts_typeToResistClass[residue];
 
 	/*
 	 * Make sure the lower-leftmost point in the node is
@@ -1884,9 +1895,14 @@ extNodeAreaFunc(tile, arg)
 	    area = (r.r_xtop - r.r_xbot) * (r.r_ytop - r.r_ybot);
 	}
 	else area = TILEAREA(tile);
+
+#ifdef NONMANHATTAN
+	if (IsSplit(tile)) area /= 2;	/* Split tiles are 1/2 area! */
+#endif
+
 	if (resistClass != -1)
 	    extResistArea[resistClass] += area;
-	reg->nreg_cap += area * ExtCurStyle->exts_areaCap[type];
+	reg->nreg_cap += area * ExtCurStyle->exts_areaCap[residue];
 
 	/*
 	 * Walk along all four sides of tile.
@@ -1915,18 +1931,23 @@ topside:
 	    else len = MIN(RIGHT(tile), RIGHT(tp)) - MAX(LEFT(tile), LEFT(tp));
 #ifdef NONMANHATTAN
             if (IsSplit(tp))
-                t = SplitBottomType(tp);
+	    {
+        	t = SplitBottomType(tp);
+		tp->ti_body = (ClientData)((TileType)tp->ti_body & ~TT_SIDE);
+		if (!SplitDirection(tp))
+		    tp->ti_body = (ClientData)((TileType)tp->ti_body | TT_SIDE);
+	    }
             else
 #endif
 	    t = TiGetTypeExact(tp);
 	    tres = (DBIsContact(t)) ? DBPlaneToResidue(t, arg->fra_pNum) : t;
-	    if (TTMaskHasType(mask, t) && tp->ti_client == extUnInit)
+	    if (TTMaskHasType(mask, tres) && tp->ti_client == extUnInit)
 	    {
 		PUSHTILE(tp, tilePlaneNum);
 	    }
 	    if ((capval = ExtCurStyle->exts_perimCap[residue][tres]) != (CapValue) 0)
 		reg->nreg_cap += capval * len;
-	    if (TTMaskHasType(resMask, t) && resistClass != -1)
+	    if (TTMaskHasType(resMask, tres) && resistClass != -1)
 		extResistPerim[resistClass] += len;
 	}
 
@@ -1950,18 +1971,21 @@ leftside:
 	    else len = MIN(TOP(tile), TOP(tp)) - MAX(BOTTOM(tile), BOTTOM(tp));
 #ifdef NONMANHATTAN
             if (IsSplit(tp))
+	    {
                 t = SplitRightType(tp);
+		tp->ti_body = (ClientData)((TileType)tp->ti_body | TT_SIDE);
+	    }
             else
 #endif
 	    t = TiGetTypeExact(tp);
 	    tres = (DBIsContact(t)) ? DBPlaneToResidue(t, arg->fra_pNum) : t;
-	    if (TTMaskHasType(mask, t) && tp->ti_client == extUnInit)
+	    if (TTMaskHasType(mask, tres) && tp->ti_client == extUnInit)
 	    {
 		PUSHTILE(tp, tilePlaneNum);
 	    }
 	    if ((capval = ExtCurStyle->exts_perimCap[residue][tres]) != (CapValue) 0)
 		reg->nreg_cap += capval * len;
-	    if (TTMaskHasType(resMask, t) && resistClass != -1)
+	    if (TTMaskHasType(resMask, tres) && resistClass != -1)
 		extResistPerim[resistClass] += len;
 	}
 
@@ -1986,18 +2010,23 @@ bottomside:
 	    else len = MIN(RIGHT(tile), RIGHT(tp)) - MAX(LEFT(tile), LEFT(tp));
 #ifdef NONMANHATTAN
             if (IsSplit(tp))
+	    {
                 t = SplitTopType(tp);
+		tp->ti_body = (ClientData)((TileType)tp->ti_body & ~TT_SIDE);
+		if (SplitDirection(tp))
+		    tp->ti_body = (ClientData)((TileType)tp->ti_body | TT_SIDE);
+	    }
             else
 #endif
 	    t = TiGetTypeExact(tp);
 	    tres = (DBIsContact(t)) ? DBPlaneToResidue(t, arg->fra_pNum) : t;
-	    if (TTMaskHasType(mask, t) && tp->ti_client == extUnInit)
+	    if (TTMaskHasType(mask, tres) && tp->ti_client == extUnInit)
 	    {
 		PUSHTILE(tp, tilePlaneNum);
 	    }
 	    if ((capval = ExtCurStyle->exts_perimCap[residue][tres]) != (CapValue) 0)
 		reg->nreg_cap += capval * len;
-	    if (TTMaskHasType(resMask, t) && resistClass != -1)
+	    if (TTMaskHasType(resMask, tres) && resistClass != -1)
 		extResistPerim[resistClass] += len;
 	}
 
@@ -2021,18 +2050,21 @@ rightside:
 	    else len = MIN(TOP(tile), TOP(tp)) - MAX(BOTTOM(tile), BOTTOM(tp));
 #ifdef NONMANHATTAN
             if (IsSplit(tp))
+	    {
                 t = SplitLeftType(tp);
+		tp->ti_body = (ClientData)((TileType)tp->ti_body & ~TT_SIDE);
+	    }
             else
 #endif
 	    t = TiGetTypeExact(tp);
 	    tres = (DBIsContact(t)) ? DBPlaneToResidue(t, arg->fra_pNum) : t;
-	    if (TTMaskHasType(mask, t) && tp->ti_client == extUnInit)
+	    if (TTMaskHasType(mask, tres) && tp->ti_client == extUnInit)
 	    {
 		PUSHTILE(tp, tilePlaneNum);
 	    }
 	    if ((capval = ExtCurStyle->exts_perimCap[residue][tres]) != (CapValue) 0)
 		reg->nreg_cap += capval * len;
-	    if (TTMaskHasType(resMask, t) && resistClass != -1)
+	    if (TTMaskHasType(resMask, tres) && resistClass != -1)
 		extResistPerim[resistClass] += len;
 	}
 
@@ -2064,8 +2096,9 @@ donesides:
 		    else        
 #endif              
 		    t = TiGetTypeExact(tp);
+		    tres = (DBIsContact(t)) ? DBPlaneToResidue(t, pNum) : t;
 
-		    if (tp->ti_client == extUnInit && TTMaskHasType(mask, t))
+		    if (tp->ti_client == extUnInit && TTMaskHasType(mask, tres))
 			PUSHTILE(tp, pNum);
 		}
 	}
