@@ -232,9 +232,9 @@ extOutputCoupling(table, outFile)
 
 	ck = (CoupleKey *) he->h_key.h_words;
 	text = extNodeName((LabRegion *) ck->ck_1);
-	(void) fprintf(outFile, "cap \"%s\" ", text);
+	fprintf(outFile, "cap \"%s\" ", text);
 	text = extNodeName((LabRegion *) ck->ck_2);
-	(void) fprintf(outFile, "\"%s\" %lg\n", text, cap);
+	fprintf(outFile, "\"%s\" %lg\n", text, cap);
     }
 }
 
@@ -348,11 +348,14 @@ extAddOverlap(tbelow, ecpls)
     CapValue c;
     Tile *tabove = ecpls->tile;
 
-    /* Do nothing if both tiles are connected */
+    /* Check if both tiles are connected.  If they are, we don't need   */
+    /* to check for shielding material, and we don't want to add any    */
+    /* coupling capacitance between them.  However, we *do* want to     */
+    /* subtract off any substrate (area) capacitance previously added   */
+    /* (Correction made 4/29/04 by Tim from a tip by Jeff Sondeen).     */
+
     rabove = (NodeRegion *) extGetRegion(tabove);
     rbelow = (NodeRegion *) extGetRegion(tbelow);
-    if (rabove == rbelow)
-	return (0);
 
     /* Compute the area of overlap */
     ov.o_clip.r_xbot = MAX(LEFT(tbelow), LEFT(tabove));
@@ -407,10 +410,8 @@ extAddOverlap(tbelow, ecpls)
     /* If any capacitance remains, add this record to the table */
     if (ov.o_area > 0)
     {
-	int oa = 
-	    ExtCurStyle->exts_planeOrder[DBPlane(TiGetType(tabove))];
-	int ob = 
-	    ExtCurStyle->exts_planeOrder[DBPlane(TiGetType(tbelow))];
+	int oa = ExtCurStyle->exts_planeOrder[DBPlane(ta)];
+	int ob = ExtCurStyle->exts_planeOrder[DBPlane(tb)];
 	if (oa > ob)
 	{
 	    /*
@@ -430,6 +431,9 @@ extAddOverlap(tbelow, ecpls)
 	    extNregAdjustCap(rabove, 0.0, 
 		"obsolete_overlap (skipped, wrong direction)");
 	    
+        /* If the regions are the same, skip this part */
+        if (rabove == rbelow) return (0);
+
 	/* Find the coupling hash record */
 	if (rabove < rbelow) ck.ck_1 = rabove, ck.ck_2 = rbelow;
 	else ck.ck_1 = rbelow, ck.ck_2 = rabove;
@@ -443,7 +447,6 @@ extAddOverlap(tbelow, ecpls)
 		ov.o_area, "overlap");
 	extSetCapValue(he, c);
     }
-
     return (0);
 }
 
@@ -701,9 +704,6 @@ extSideOverlap(tp, esws)
     CapValue cap;
     CoupleKey ck;
 
-    if (rtp == rbp)
-	return (0);
-
     if (bp->b_segment.r_xtop == bp->b_segment.r_xbot)
     {
 	length = MIN(bp->b_segment.r_ytop, TOP(tp))
@@ -724,6 +724,10 @@ extSideOverlap(tp, esws)
     ta = TiGetType(bp->b_inside);
     tb = TiGetType(tp);
 
+    /* Avoid doing the code below if we're just going to return without */
+    /* changing anything.                                               */
+    if ((tb == TT_SPACE) && (rtp == rbp)) return (0);
+
     /* Revert any contacts to their residues */
     if (DBIsContact(ta))
 	ta = DBPlaneToResidue(ta, esws->plane_of_boundary);
@@ -773,13 +777,14 @@ extSideOverlap(tp, esws)
 		    break;
 		}
 	    }
-	    cap += e->ec_cap * ov.o_area;
+	    if (rtp != rbp)
+		cap += e->ec_cap * ov.o_area;
 	    areaAccountedFor += ov.o_area;
 	}
     }
     
     /* Add in the new capacitance. */
-    if (TiGetType(tp) == TT_SPACE)
+    if (tb == TT_SPACE)
     {
 	/* Is tp a space tile?  If so, extGetRegion points to garbage;  	
 	 * make terminal 2 point to ground.
@@ -789,16 +794,8 @@ extSideOverlap(tp, esws)
     }
     else
     {
-	int oa;
-	int ob;
-	if (rtp < rbp) ck.ck_1 = rtp, ck.ck_2 = rbp;
-	    else ck.ck_1 = rbp, ck.ck_2 = rtp;
-	he = HashFind(extCoupleHashPtr, (char *) &ck);
-	if (CAP_DEBUG) extAdjustCouple(he, cap, "sideoverlap");
-	extSetCapValue(he, cap + extGetCapValue(he));
-
-	oa = ExtCurStyle->exts_planeOrder[DBPlane(TiGetType(bp->b_inside))];
-	ob = ExtCurStyle->exts_planeOrder[DBPlane(TiGetType(tp))];
+	int oa = ExtCurStyle->exts_planeOrder[DBPlane(ta)];
+	int ob = ExtCurStyle->exts_planeOrder[DBPlane(tb)];
 	if (oa > ob)
 	{
 	    /* If the overlapped tile is between the substrate and the boundary
@@ -807,16 +804,15 @@ extSideOverlap(tp, esws)
 	     * we now know it is shielded from the substrate.
 	     */
 	    CapValue subcap;
-	    TileType intype = TiGetType(bp->b_inside);
 	    TileType outtype = TiGetType(bp->b_outside);
 
 	    /* Decompose contacts into their residues */
-	    if (DBIsContact(intype))
-		intype = DBPlaneToResidue(intype, esws->plane_of_boundary);
+	    if (DBIsContact(ta))
+		ta = DBPlaneToResidue(ta, esws->plane_of_boundary);
 	    if (DBIsContact(outtype))
 		outtype = DBPlaneToResidue(outtype, esws->plane_of_boundary);
 
-	    subcap = (ExtCurStyle->exts_perimCap[intype][outtype] *
+	    subcap = (ExtCurStyle->exts_perimCap[ta][outtype] *
 			MIN(areaAccountedFor, length));
 	    rbp->nreg_cap -= subcap;
 	    if (CAP_DEBUG)
@@ -824,8 +820,17 @@ extSideOverlap(tp, esws)
 	} else if (CAP_DEBUG)
 	    extNregAdjustCap(rbp, 0.0, 
 		"obsolete_perimcap (skipped, wrong direction)");
-    }
 
+	/* If the nodes are electrically connected, then we don't add	*/
+	/* any side overlap capacitance to the node.			*/
+	if (rtp == rbp) return (0);
+
+	if (rtp < rbp) ck.ck_1 = rtp, ck.ck_2 = rbp;
+	    else ck.ck_1 = rbp, ck.ck_2 = rtp;
+	he = HashFind(extCoupleHashPtr, (char *) &ck);
+	if (CAP_DEBUG) extAdjustCouple(he, cap, "sideoverlap");
+	extSetCapValue(he, cap + extGetCapValue(he));
+    }
     return (0);
 }
 
