@@ -213,7 +213,7 @@ extHierConnectFunc2(cum, ha)
 	    nn->nn_node = node1;
 	    nn->nn_next = node1->node_names;
 	    node1->node_names = node2->node_names;
-	    FREE((char *) node2);
+	    freeMagic((char *) node2);
 	}
     }
     else if (r.r_xtop > r.r_xbot && r.r_ytop > r.r_ybot)
@@ -387,27 +387,27 @@ extOutputConns(table, outf)
 	    if (nnext = nn->nn_next)
 	    {
 		/* First merge */
-		(void) fprintf(outf, "merge \"%s\" \"%s\" %lg",
+		fprintf(outf, "merge \"%s\" \"%s\" %lg",
 				nn->nn_name, nnext->nn_name, c);
 		for (n = 0; n < ExtCurStyle->exts_numResistClasses; n++)
-		    (void) fprintf(outf, " %d %d",
+		    fprintf(outf, " %d %d",
 				node->node_pa[n].pa_area,
 				node->node_pa[n].pa_perim);
-		(void) fprintf(outf, "\n");
+		fprintf(outf, "\n");
 		nn->nn_node = (Node *) NULL;		/* Processed */
 
 		/* Subsequent merges */
 		for (nn = nnext; nnext = nn->nn_next; nn = nnext)
 		{
-		    (void) fprintf(outf, "merge \"%s\" \"%s\" %lg\n",
+		    fprintf(outf, "merge \"%s\" \"%s\" %lg\n",
 				    nn->nn_name, nnext->nn_name, c);
 		    nn->nn_node = (Node *) NULL;	/* Processed */
 		}
 	    }
 	    nn->nn_node = (Node *) NULL;
-	    FREE((char *) node);
+	    freeMagic((char *) node);
 	}
-	FREE((char *) nfirst);
+	freeMagic((char *) nfirst);
     }
 }
 
@@ -441,8 +441,8 @@ extHierNewNode(he)
 
     nclasses = ExtCurStyle->exts_numResistClasses;
     n = (nclasses - 1) * sizeof (PerimArea) + sizeof (Node);
-    MALLOC(NodeName *, nn, sizeof (NodeName));
-    MALLOC(Node *, node, (unsigned) n);
+    nn = (NodeName *) mallocMagic((unsigned) (sizeof (NodeName)));
+    node = (Node *) mallocMagic((unsigned) n);
 
     nn->nn_node = node;
     nn->nn_next = (NodeName *) NULL;
@@ -486,7 +486,7 @@ extHierLabFirst(tile, arg)
 {
     LabRegion *new;
 
-    MALLOC(LabRegion *, new, sizeof (LabRegion));
+    new = (LabRegion *) mallocMagic((unsigned) (sizeof (LabRegion)));
     new->lreg_next = (LabRegion *) NULL;
     new->lreg_labels = (LabelList *) NULL;
     new->lreg_pnum = DBNumPlanes;
@@ -551,7 +551,7 @@ extHierNewOne()
     }
     else
     {
-	MALLOC(ExtTree *, et, sizeof (ExtTree));
+	et = (ExtTree *) mallocMagic((unsigned)(sizeof (ExtTree)));
 	(void) sprintf(defname, "__EXTTREE%d__", extHierOneNameSuffix++);
 	DBNewYank(defname, &et->et_use, &dummy);
     }
