@@ -296,17 +296,17 @@ extLength(rootUse, f)
 	    for (rLab = rList; rLab; rLab = rLab->lab_next)
 	    {
 		extPathPairDistance(dLab, rLab, &min, &max);
-		(void) fprintf(f, "distance %s %s %d %d\n",
+		fprintf(f, "distance %s %s %d %d\n",
 			dLab->lab_text, rLab->lab_text, min, max);
 	    }
 
 	    /* Free the driver label */
-	    FREE((char *) dLab);
+	    freeMagic((char *) dLab);
 	}
 
 	/* Free all the receiver labels built up during this iteration */
 	for (rLab = rList; rLab; rLab = rLab->lab_next)
-	    FREE((char *) rLab);
+	    freeMagic((char *) rLab);
 
 	/* For sanity since we've freed the driver label list */
 	HashSetValue(he, (ClientData) NULL);
@@ -502,7 +502,7 @@ extLengthLabelsFunc(scx, label, tpath)
     /* Allocate and fill in a new hierarchical label */
     len = strlen(tpath->tp_first) + sizeof (Label)
 	- sizeof newLab->lab_text + 1;
-    MALLOC(Label *, newLab, (unsigned) len);
+    newLab = (Label *) mallocMagic((unsigned) len);
     newLab->lab_type = label->lab_type;
     GeoTransPos(&scx->scx_trans, label->lab_pos, newLab->lab_pos);
     GeoTransRect(&scx->scx_trans, &label->lab_rect, &newLab->lab_rect);
@@ -579,7 +579,7 @@ extPathLabelFunc(rect, text, childLab, pLabList)
     int len;
 
     len = strlen(text) + sizeof (Label) - sizeof lab->lab_text + 1;
-    MALLOC(Label *, lab, (unsigned) len);
+    lab = (Label *) mallocMagic((unsigned) len);
     lab->lab_type = childLab->lab_type;
     lab->lab_rect = *rect;
     lab->lab_pos = GEO_CENTER;	/* Irrelevant */
