@@ -533,11 +533,11 @@ extSubtreeOutputCoupling(ha)
 
 	NODETOTILE(ck->ck_1, &ha->ha_cumFlat, tp);
 	name = extSubtreeTileToNode(tp, &ha->ha_cumFlat, ha, TRUE);
-	(void) fprintf(ha->ha_outf, "cap \"%s\" ", name);
+	fprintf(ha->ha_outf, "cap \"%s\" ", name);
 
 	NODETOTILE(ck->ck_2, &ha->ha_cumFlat, tp);
 	name = extSubtreeTileToNode(tp, &ha->ha_cumFlat, ha, TRUE);
-	(void) fprintf(ha->ha_outf, "\"%s\" %lg\n", name, cap);
+	fprintf(ha->ha_outf, "\"%s\" %lg\n", name, cap);
     }
 }
 
@@ -719,6 +719,7 @@ extSubtreeTileToNode(tp, et, ha, doHard)
     LabRegion *reg;
     int pNum;
     Rect r;
+    TileType ttype;
 
     /* If there is a label list, use it */
     if (extHasRegion(tp, extUnInit))
@@ -739,15 +740,48 @@ extSubtreeTileToNode(tp, et, ha, doHard)
 	 * Make sure we've got a valid tile -- interactions with interrupts
 	 * can cause problems.
 	 */
-	pNum = DBPlane(TiGetType(tp));
-	if (pNum >= PL_PAINTBASE &&
-		DBSrPaintArea((Tile *) NULL,
-		    et->et_lookNames->cd_planes[DBPlane(TiGetType(tp))],
-		    &r, &DBAllButSpaceBits, extConnFindFunc, (ClientData) &reg))
+#ifdef NONMANHATTAN
+	if (IsSplit(tp))
 	{
-	    if (SigInterruptPending)
-		return ("(none)");
-	    return (extNodeName(reg));
+	    if (SplitSide(tp))
+		ttype = SplitRightType(tp);
+	    else
+		ttype = SplitLeftType(tp);
+	}
+	else
+#endif
+	ttype = TiGetTypeExact(tp);
+
+	pNum = DBPlane(ttype);
+	if (pNum >= PL_PAINTBASE)
+	{
+#ifdef NONMANHATTAN
+	    if (IsSplit(tp))
+	    {
+		if (DBSrPaintNMArea((Tile *) NULL,
+			et->et_lookNames->cd_planes[pNum],
+			TiGetTypeExact(tp), &r, &DBAllButSpaceBits,
+			extConnFindFunc, (ClientData) &reg))
+		{
+		    if (SigInterruptPending)
+			return ("(none)");
+		    return (extNodeName(reg));
+		}
+	    }
+	    else
+	    {
+#endif
+		if (DBSrPaintArea((Tile *) NULL,
+			et->et_lookNames->cd_planes[pNum], &r,
+			&DBAllButSpaceBits, extConnFindFunc, (ClientData) &reg))
+		{
+		    if (SigInterruptPending)
+			return ("(none)");
+		    return (extNodeName(reg));
+		}
+#ifdef NONMANHATTAN
+	    }
+#endif
 	}
     }
 
@@ -839,17 +873,29 @@ extSubtreeHardNode(tp, et, ha)
 {
     LabRegion *lreg = (LabRegion *) extGetRegion(tp);
     CellDef *def = et->et_use->cu_def;
-    TileType type = TiGetType(tp);
+    TileType ttype;
     char labelBuf[4096];
     LabelList *ll;
     HardWay arg;
 
     ASSERT(lreg->lreg_labels == NULL, "extSubtreeHardNode");
 
+#ifdef NONMANHATTAN
+    if (IsSplit(tp))
+    {
+	if (SplitSide(tp))
+	    ttype = SplitRightType(tp);
+	else
+	    ttype = SplitLeftType(tp);
+    }
+    else
+#endif
+	ttype = TiGetTypeExact(tp);
+
     arg.hw_ha = ha;
     arg.hw_label = (Label *) NULL;
-    arg.hw_mask = DBPlaneTypes[DBPlane(type)];
-    TTMaskAndMask(&arg.hw_mask, &DBConnectTbl[type]);
+    arg.hw_mask = DBPlaneTypes[DBPlane(ttype)];
+    TTMaskAndMask(&arg.hw_mask, &DBConnectTbl[ttype]);
     arg.hw_tpath.tp_last = &labelBuf[sizeof labelBuf - 3];
     arg.hw_tpath.tp_first = arg.hw_tpath.tp_next = labelBuf;
     arg.hw_prefix = TRUE;
@@ -878,7 +924,7 @@ extSubtreeHardNode(tp, et, ha)
      */
     if (arg.hw_label)
     {
-	MALLOC(LabelList *, ll, sizeof (LabelList));
+	ll = (LabelList *) mallocMagic((unsigned) (sizeof (LabelList)));
 	lreg->lreg_labels = ll;
 	ll->ll_next = (LabelList *) NULL;
 	ll->ll_label = arg.hw_label;
