@@ -29,7 +29,7 @@ static char sccsid[] = "@(#)ExtTech.c	4.8 MAGIC (Berkeley) 10/26/85";
 #endif
 #include <stdlib.h>		/* for strtod() */
 
-#include "magic/tclmagic.h"
+#include "tcltk/tclmagic.h"
 #include "misc/magic.h"
 #include "utils/utils.h"
 #include "utils/geometry.h"
@@ -66,7 +66,7 @@ typedef enum
 {
     AREAC, CONTACT, CSCALE, DEVICE, FET, FETRESIST, HEIGHT, LAMBDA, OVERC,
     PERIMC, PLANEORDER, NOPLANEORDER, RESIST, RSCALE, SIDEHALO, SIDEOVERLAP,
-    SIDEWALL, STEP, STYLE
+    SIDEWALL, STEP, STYLE, VARIANT
 } Key;
 
 typedef struct
@@ -132,9 +132,11 @@ static keydesc keyTable[] = {
     "step",		STEP,		2,	2,
 "size(lambda)",
 
-    "style",		STYLE,		2,	2,
+    "style",		STYLE,		2,	4,
 "stylename",
 
+    "variants",		VARIANT,	2,	2,
+"style,...",
 
     0
 };
@@ -165,6 +167,9 @@ static keydesc devTable[] = {
     "subcircuit",	DEV_SUBCKT,		3,	5,
 "name dev-types [term-types [sub-types]]",
 
+    "rsubcircuit",	DEV_RSUBCKT,		4,	4,
+"name dev-types terminal-types",
+
     0
 };
 
@@ -303,7 +308,7 @@ ExtGetDevInfo(idx, devnameptr, sd_rclassptr, sub_rclassptr, subnameptr)
 	}
     }
 
-    FREE(uniquenamelist);
+    freeMagic(uniquenamelist);
     return TRUE;
 }
 
@@ -873,7 +878,7 @@ ExtTechLine(sectionName, argc, argv)
     int argc;
     char *argv[];
 {
-    int n, size, val, p1, p2, p3, nterm, pov, pshield, class;
+    int n, l, size, val, p1, p2, p3, nterm, pov, pshield, class;
     CapValue capVal, gscap, gccap;
     TileTypeBitMask types1, types2, near, far, ov, shield, subsTypes;
     char *subsName, *transName, *cp, *endptr;
@@ -904,6 +909,7 @@ ExtTechLine(sectionName, argc, argv)
     kp = &keyTable[n];
     if (argc < kp->k_minargs || argc > kp->k_maxargs)
 	goto usage;
+    else if (argc >= 2) l = strlen(argv[1]);
 
     /* If ExtCurStyle is NULL, this is a first pass, and we should	*/
     /* immediately load this style as default.  Otherwise, check if	*/
@@ -912,24 +918,71 @@ ExtTechLine(sectionName, argc, argv)
 
     if (kp->k_key == STYLE)
     {
+	if (argc != 2)
+	    if ((argc != 4) || (strncmp(argv[2], "variant", 7)))
+		goto usage;
+
 	for (newStyle = ExtAllStyles; newStyle != NULL;
 		newStyle = newStyle->exts_next)
 	{
-	    if (!strcmp(newStyle->exts_name, argv[1])) break;
+	    if (!strcmp(newStyle->exts_name, argv[1]))
+		break;
 	}
 	if (newStyle == NULL)
 	{
-	    newStyle = (ExtKeep *)mallocMagic(sizeof(ExtKeep));
-	    newStyle->exts_next = NULL;
-	    newStyle->exts_name = StrDup((char **) NULL, argv[1]);
-
-	    if (ExtAllStyles == NULL)
-	        ExtAllStyles = newStyle;
-	    else
+	    if (argc == 2)
 	    {
-	        /* Append to end of style list */
-	        for (es = ExtAllStyles; es->exts_next; es = es->exts_next);
-	        es->exts_next = newStyle;
+		newStyle = (ExtKeep *)mallocMagic(sizeof(ExtKeep));
+		newStyle->exts_next = NULL;
+		newStyle->exts_name = StrDup((char **) NULL, argv[1]);
+
+		if (ExtAllStyles == NULL)
+		    ExtAllStyles = newStyle;
+		else
+		{
+	            /* Append to end of style list */
+	            for (es = ExtAllStyles; es->exts_next; es = es->exts_next);
+	            es->exts_next = newStyle;
+		}
+	    }
+	    else	/* Handle style variants */
+	    {
+		ExtKeep *saveStyle = NULL;
+		char *tptr, *cptr;
+
+		/* 4th argument is a comma-separated list of variants.	*/
+		/* In addition to the default name recorded above,	*/
+		/* record each of the variants.				*/
+
+		tptr = argv[3];
+		while (*tptr != '\0')
+		{
+		    cptr = strchr(tptr, ',');
+		    if (cptr != NULL) *cptr = '\0';
+		    newStyle = (ExtKeep *)mallocMagic(sizeof(ExtKeep));
+		    newStyle->exts_next = NULL;
+		    newStyle->exts_name = (char *)mallocMagic(l
+				+ strlen(tptr) + 1);
+		    sprintf(newStyle->exts_name, "%s%s", argv[1], tptr);
+
+		    /* Remember the first variant as the default */
+		    if (saveStyle == NULL) saveStyle= newStyle;
+
+		    /* Append to end of style list */
+		    if (ExtAllStyles == NULL)
+			ExtAllStyles = newStyle;
+		    else
+		    {
+			for (es = ExtAllStyles; es->exts_next; es = es->exts_next);
+			es->exts_next = newStyle;
+		    }
+		    
+		    if (cptr == NULL)
+			break;
+		    else
+			tptr = cptr + 1;
+		}
+		newStyle = saveStyle;
 	    }
 	}
 
@@ -943,20 +996,95 @@ ExtTechLine(sectionName, argc, argv)
 	    ExtCurStyle->exts_name = newStyle->exts_name;
 	    ExtCurStyle->exts_status = TECH_PENDING;
 	}
-	else if (ExtCurStyle->exts_status == TECH_PENDING) /* Finished loading; stop */
+	else if ((ExtCurStyle->exts_status == TECH_PENDING) ||
+		(ExtCurStyle->exts_status == TECH_SUSPENDED))
+	    /* Finished loading; stop */
 	    ExtCurStyle->exts_status = TECH_LOADED;
 	else if (ExtCurStyle->exts_status == TECH_NOT_LOADED)
 	{
 	    if (ExtCurStyle->exts_name == NULL)
 	        return (FALSE);		/* Don't know what to load! */
-	    else if (!strcmp(argv[1], ExtCurStyle->exts_name))
-		ExtCurStyle->exts_status = TECH_PENDING; 	/* load pending */
+	    else if (argc == 2)
+	    {
+		if (!strcmp(argv[1], ExtCurStyle->exts_name))
+		     ExtCurStyle->exts_status = TECH_PENDING; 	/* load pending */
+	    }
+	    else if (argc == 4)
+	    {
+		/* Verify that the style matches one variant */
+		char *tptr, *cptr;
+		if (!strncmp(ExtCurStyle->exts_name, argv[1], l))
+		{
+		    tptr = argv[3];
+		    while (*tptr != '\0')
+		    {
+			cptr = strchr(tptr, ',');
+			if (cptr != NULL) *cptr = '\0';
+			if (!strcmp(ExtCurStyle->exts_name + l, tptr))
+			{
+			    ExtCurStyle->exts_status = TECH_PENDING;
+			    return TRUE;
+			}
+			if (cptr == NULL)
+			    return TRUE;
+			else
+			    tptr = cptr + 1;
+		    }
+		}
+	    }
 	}
 	return (TRUE);
     }
 
     /* Only continue past this point if we are loading the extraction style */
     if (ExtCurStyle == NULL) return FALSE;
+    if ((ExtCurStyle->exts_status != TECH_PENDING) &&
+		(ExtCurStyle->exts_status != TECH_SUSPENDED))
+	return TRUE;
+
+    /* Process "variant" lines next */
+
+    if (kp->k_key == VARIANT)
+    {
+	int l;
+	char *cptr, *tptr;
+
+	/* If our style variant is not one of the ones declared */
+	/* on the line, then we ignore all input until we 	*/
+	/* either reach the end of the style, the end of the	*/
+	/* section, or another "variant" line.			*/
+
+	if (argc != 2) goto usage;
+	tptr = argv[1];
+	while (*tptr != '\0')
+	{
+	    cptr = strchr(tptr, ',');
+	    if (cptr != NULL) *cptr = '\0';
+
+	    if (*tptr == '*')	/* Wildcard for "all variants" */
+	    {
+		ExtCurStyle->exts_status = TECH_PENDING;
+		return TRUE;
+	    }
+	    else
+	    {
+		l = strlen(ExtCurStyle->exts_name) - strlen(tptr);
+		if (!strcmp(tptr, ExtCurStyle->exts_name + l))
+		{
+		    ExtCurStyle->exts_status = TECH_PENDING;
+		    return TRUE;
+		}
+	    }
+	
+	    if (cptr == NULL)
+		break;
+	    else
+		tptr = cptr + 1;
+	}
+	ExtCurStyle->exts_status = TECH_SUSPENDED;
+    }
+
+    /* Anything below this line is not parsed if we're in TECH_SUSPENDED mode */
     if (ExtCurStyle->exts_status != TECH_PENDING) return TRUE;
 
     switch (kp->k_key)
@@ -1164,6 +1292,11 @@ ExtTechLine(sectionName, argc, argv)
 		    if (argc > 5)
 			DBTechNoisyNameMask(argv[5], &subsTypes); /* substrate */
 		    break;
+		case DEV_RSUBCKT:
+		    nterm = 2;
+		    DBTechNoisyNameMask(argv[4], &types2);	  /* terminals */
+		    subsTypes = DBZeroTypeBits;
+		    break;
 	    }
 
 	    TTMaskSetMask(&ExtCurStyle->exts_transMask, &types1);
@@ -1374,7 +1507,7 @@ ExtTechLine(sectionName, argc, argv)
 			goto diffplane;
 		    TTMaskSetMask(&ExtCurStyle->exts_sideOverlapOtherTypes[s][t], &ov);
 		    ExtCurStyle->exts_sideOverlapOtherPlanes[s][t] |= pov;
-		    MALLOC(EdgeCap *, cnew, sizeof (EdgeCap));
+		    cnew = (EdgeCap *) mallocMagic((unsigned) (sizeof (EdgeCap)));
 		    cnew->ec_cap = capVal;
 		    cnew->ec_far = shield; /* Really types that shield */
 		    cnew->ec_near = ov;  /* Really types we create cap with */
@@ -1434,7 +1567,7 @@ ExtTechLine(sectionName, argc, argv)
 		    if (!TTMaskHasType(&types2, t))
 			continue;
 		    TTMaskSetMask(&ExtCurStyle->exts_sideCoupleOtherEdges[s][t], &far);
-		    MALLOC(EdgeCap *, cnew, sizeof (EdgeCap));
+		    cnew = (EdgeCap *) mallocMagic((unsigned) (sizeof (EdgeCap)));
 		    cnew->ec_cap = capVal;
 		    cnew->ec_near = near;
 		    cnew->ec_far = far;
