@@ -106,6 +106,10 @@ void extTimeProc();
 void extCumInit();
 void extCumOutput();
 void extCumAdd();
+void extPaintOnly(CellDef *);
+void extHierCell(CellDef *);
+int  extCountTiles();
+
 extern int extDefInitFunc();
 
 
@@ -226,8 +230,8 @@ ExtTimes(rootUse, f)
     }
 
     /* Output the summary statistics */
-    (void) fprintf(f, "\n\nSummary statistics:\n\n");
-    (void) fprintf(f, "%s %8s %8s %8s %8s\n",
+    fprintf(f, "\n\nSummary statistics:\n\n");
+    fprintf(f, "%s %8s %8s %8s %8s\n",
 	      "               ", "min", "max", "mean", "std.dev");
     extCumOutput("fets/sec paint ", &cumFetsPerSecPaint, f);
     extCumOutput("fets/sec hier  ", &cumFetsPerSecHier, f);
@@ -246,8 +250,8 @@ ExtTimes(rootUse, f)
 	clip = 100.0 * cumClippedArea.cums_sum / cumTotalArea.cums_sum;
 	inter = 100.0 * cumInteractArea.cums_sum / cumTotalArea.cums_sum;
     }
-    (void) fprintf(f, "Mean %% clipped area = %.2f\n", clip);
-    (void) fprintf(f, "Mean %% interaction area = %.2f\n", inter);
+    fprintf(f, "Mean %% clipped area = %.2f\n", clip);
+    fprintf(f, "Mean %% interaction area = %.2f\n", inter);
 
     /* Free the hash table */
     HashKill(&cellStatsTable);
@@ -334,7 +338,6 @@ extTimesCellFunc(cs)
     extern long extSubtreeTotalArea;
     extern long extSubtreeInteractionArea;
     extern long extSubtreeClippedArea;
-    int extCountTiles(), extPaintOnly(), extHierCell();
     TransRegion *transList, *tl;
     CellDef *def = cs->cs_def;
     int pNum;
@@ -511,30 +514,30 @@ extTimesSummaryFunc(cs, f)
     }
 
     /* Cell name */
-    (void) fprintf(f, "\n%8s %8s %s\n", "", "", cs->cs_def->cd_name);
+    fprintf(f, "\n%8s %8s %s\n", "", "", cs->cs_def->cd_name);
 
     /* Sizes */
-    (void) fprintf(f, "%8d %8d (paint) fets rects\n",
+    fprintf(f, "%8d %8d (paint) fets rects\n",
 			cs->cs_fets, cs->cs_rects);
-    (void) fprintf(f, "%8d %8d (hier) fets rects\n",
+    fprintf(f, "%8d %8d (hier) fets rects\n",
 			cs->cs_hfets, cs->cs_hrects);
-    (void) fprintf(f, "%8d %8d (flat) fets rects\n",
+    fprintf(f, "%8d %8d (flat) fets rects\n",
 			cs->cs_ffets, cs->cs_frects);
 
     /* Times */
-    (void) fprintf(f, "%8.2f %8.2f Tpaint, Tcell\n", tpaint, tcell);
-    (void) fprintf(f, "%8.2f %8.2f Thier, Tincr\n", thier, tincr);
+    fprintf(f, "%8.2f %8.2f Tpaint, Tcell\n", tpaint, tcell);
+    fprintf(f, "%8.2f %8.2f Thier, Tincr\n", thier, tincr);
 
     /* Speeds */
-    (void) fprintf(f, "%8.2f %8.2f (paint) fets/sec rects/sec\n",
+    fprintf(f, "%8.2f %8.2f (paint) fets/sec rects/sec\n",
 			    fetspaint, rectspaint);
-    (void) fprintf(f, "%8.2f %8.2f (hier)  fets/sec rects/sec\n",
+    fprintf(f, "%8.2f %8.2f (hier)  fets/sec rects/sec\n",
 			    fetshier, rectshier);
-    (void) fprintf(f, "%8.2f %8.2f (flat)  fets/sec rects/sec\n",
+    fprintf(f, "%8.2f %8.2f (flat)  fets/sec rects/sec\n",
 			    fetsflat, rectsflat);
 
     /* Fraction of area in subtree processing */
-    (void) fprintf(f, "%8.2f %8.2f         clip %%  inter %%\n",
+    fprintf(f, "%8.2f %8.2f         clip %%  inter %%\n",
 			    pctclip, pctinter);
 
     /* Accumulate statistics */
@@ -914,16 +917,16 @@ extCumOutput(str, cum, f)
 	var = (cum->cums_sos / (double) cum->cums_n) - (mean * mean);
     }
 
-    (void) fprintf(f, "%s", str);
+    fprintf(f, "%s", str);
     if (cum->cums_min < INFINITY) 
-	(void) fprintf(f, " %8.2f", cum->cums_min);
+	fprintf(f, " %8.2f", cum->cums_min);
     else
-	(void) fprintf(f, "   <none>");
+	fprintf(f, "   <none>");
     if (cum->cums_max > MINFINITY) 
-	(void) fprintf(f, " %8.2f", cum->cums_max);
+	fprintf(f, " %8.2f", cum->cums_max);
     else
-	(void) fprintf(f, "   <none>");
-    (void) fprintf(f, " %8.2f %8.2f\n", mean, sqrt(var));
+	fprintf(f, "   <none>");
+    fprintf(f, " %8.2f %8.2f\n", mean, sqrt(var));
 }
 
 /*
@@ -1036,8 +1039,8 @@ ExtInterCount(rootUse, halo, f)
     (void) DBCellSrDefs(0, extDefInitFunc, (ClientData) 0);
 
     /* Output the summary statistics */
-    (void) fprintf(f, "\n\nSummary statistics:\n\n");
-    (void) fprintf(f, "%s %8s %8s %8s %8s\n",
+    fprintf(f, "\n\nSummary statistics:\n\n");
+    fprintf(f, "%s %8s %8s %8s %8s\n",
 	      "               ", "min", "max", "mean", "std.dev");
     extCumOutput("% cell interact", &cumPercentInteraction, f);
 
@@ -1045,7 +1048,7 @@ ExtInterCount(rootUse, halo, f)
     inter = 0.0;
     if (cumTotalArea.cums_sum > 0)
 	inter = 100.0 * cumInteractArea.cums_sum / cumTotalArea.cums_sum;
-    (void) fprintf(f, "Mean %% interaction area = %.2f\n", inter);
+    fprintf(f, "Mean %% interaction area = %.2f\n", inter);
 }
 
 int
@@ -1086,7 +1089,7 @@ extInterAreaFunc(use, f)
     extCumAdd(&cumTotalArea, (double) area);
     extCumAdd(&cumInteractArea, (double) interarea);
 
-    (void) fprintf(f, "%7.2f%%  %s\n", pctinter, def->cd_name);
+    fprintf(f, "%7.2f%%  %s\n", pctinter, def->cd_name);
 
     /* Visit our children */
     (void) DBCellEnum(def, extInterAreaFunc, (ClientData) f);
