@@ -24,7 +24,6 @@ static char rcsid[] = "$Header: /ufs/repository/magic/garouter/gaStem.c,v 1.2 20
 #include "misc/magic.h"
 #include "utils/geometry.h"
 #include "utils/hash.h"
-#include "utils/doubleint.h"
 #include "utils/heap.h"
 #include "tiles/tile.h"
 #include "database/database.h"
@@ -86,6 +85,10 @@ int gaStemContainingChannelFunc();
 bool gaStemAssign();
 void gaStemGridRange();
 void gaStemPaint();
+bool gaStemNetClear();
+bool gaStemInternalFunc();
+bool gaStemInternal();
+bool gaStemGrow();
 
 
 /*
