@@ -65,7 +65,7 @@ GCRNewChannel(length, width)
     lenWds = length + 2;
     widWds = width + 2;
 
-    MALLOC(GCRChannel *, ch, sizeof (GCRChannel));
+    ch = (GCRChannel *) mallocMagic((unsigned) (sizeof (GCRChannel)));
     ch->gcr_type = CHAN_NORMAL;
     ch->gcr_length = length;
     ch->gcr_width = width;
@@ -74,39 +74,39 @@ GCRNewChannel(length, width)
 
     /* Malloc storage for pin arrays and zero each */
     nBytes = lenWds * sizeof (GCRPin);
-    MALLOC(GCRPin *, ch->gcr_tPins, nBytes);
-    MALLOC(GCRPin *, ch->gcr_bPins, nBytes);
+    ch->gcr_tPins = (GCRPin *) mallocMagic((unsigned) nBytes);
+    ch->gcr_bPins = (GCRPin *) mallocMagic((unsigned) nBytes);
     bzero((char *) ch->gcr_tPins, (int) nBytes);
     bzero((char *) ch->gcr_bPins, (int) nBytes);
 
     nBytes = widWds * sizeof (GCRPin);
-    MALLOC(GCRPin *, ch->gcr_lPins, nBytes);
-    MALLOC(GCRPin *, ch->gcr_rPins, nBytes);
+    ch->gcr_lPins = (GCRPin *) mallocMagic((unsigned) nBytes);
+    ch->gcr_rPins = (GCRPin *) mallocMagic((unsigned) nBytes);
     bzero((char *) ch->gcr_lPins, (int) nBytes);
     bzero((char *) ch->gcr_rPins, (int) nBytes);
 
-    MALLOC(GCRColEl *, ch->gcr_lCol, widWds * sizeof (GCRColEl));
-    MALLOC(int *, ch->gcr_density, lenWds * sizeof (int));
+    ch->gcr_lCol = (GCRColEl *) mallocMagic((unsigned) (widWds * sizeof (GCRColEl)));
+    ch->gcr_density = (int *) mallocMagic((unsigned) (lenWds * sizeof (int)));
 
     /* Global router-specific initialization */
-    MALLOC(short *, ch->gcr_dRowsByCol, lenWds * sizeof (short));
+    ch->gcr_dRowsByCol = (short *) mallocMagic((unsigned) (lenWds * sizeof (short)));
     bzero((char *) ch->gcr_dRowsByCol, (int) lenWds * sizeof (short));
-    MALLOC(short *, ch->gcr_dColsByRow, widWds * sizeof (short));
+    ch->gcr_dColsByRow = (short *) mallocMagic ((unsigned) (widWds * sizeof (short)));
     bzero((char *) ch->gcr_dColsByRow, (int) widWds * sizeof (short));
     ch->gcr_dMaxByRow = ch->gcr_dMaxByCol = 0;
 
 #ifdef	IDENSITY
 	/* For debugging */
-    MALLOC(short *, ch->gcr_iRowsByCol, lenWds * sizeof (short));
+    ch->gcr_iRowsByCol = (short *) mallocMagic((unsigned) (lenWds * sizeof (short)));
     bzero((char *) ch->gcr_iRowsByCol, (int) lenWds * sizeof (short));
-    MALLOC(short *, ch->gcr_iColsByRow, widWds * sizeof (short));
+    ch->gcr_iColsByRow = (short *) mallocMagic((unsigned) (widWds * sizeof (short)));
     bzero((char *) ch->gcr_iColsByRow, (int) widWds * sizeof (short));
 #endif	/* IDENSITY */
 
     ch->gcr_client = (ClientData) NULL;
 
     /* Allocate the result array */
-    MALLOC(short **, ch->gcr_result, lenWds * sizeof (short *));
+    ch->gcr_result = (short **) mallocMagic ((unsigned) (lenWds * sizeof (short *)));
 
     /*
      * Fill in fields of pins that aren't zero; also allocate
@@ -115,7 +115,7 @@ GCRNewChannel(length, width)
     nBytes = widWds * sizeof (short);
     for (i = 0; i < lenWds; i++)
     {
-	MALLOC(short *, ch->gcr_result[i], nBytes);
+	ch->gcr_result[i] = (short *) mallocMagic((unsigned) nBytes);
 	bzero((char *) ch->gcr_result[i], (int) nBytes);
 
 	/* BOTTOM */
@@ -171,28 +171,28 @@ GCRFreeChannel(ch)
     GCRNet *net;
     int i;
 
-    FREE((char *) ch->gcr_tPins);
-    FREE((char *) ch->gcr_bPins);
-    FREE((char *) ch->gcr_lPins);
-    FREE((char *) ch->gcr_rPins);
+    freeMagic((char *) ch->gcr_tPins);
+    freeMagic((char *) ch->gcr_bPins);
+    freeMagic((char *) ch->gcr_lPins);
+    freeMagic((char *) ch->gcr_rPins);
     for (net = ch->gcr_nets; net; net = net->gcr_next)
-	FREE((char *) net);
-    FREE((char *) ch->gcr_lCol);
+	freeMagic((char *) net);
+    freeMagic((char *) ch->gcr_lCol);
 
-    FREE((char *) ch->gcr_dRowsByCol);
-    FREE((char *) ch->gcr_dColsByRow);
+    freeMagic((char *) ch->gcr_dRowsByCol);
+    freeMagic((char *) ch->gcr_dColsByRow);
 
 #ifdef	IDENSITY
 	/* For debugging */
-    FREE((char *) ch->gcr_iRowsByCol);
-    FREE((char *) ch->gcr_iColsByRow);
+    freeMagic((char *) ch->gcr_iRowsByCol);
+    freeMagic((char *) ch->gcr_iColsByRow);
 #endif	/* IDENSITY */
 
-    FREE((char *) ch->gcr_density);
+    freeMagic((char *) ch->gcr_density);
     for (i = 0; i <= ch->gcr_length + 1; i++)
-	FREE((char *) ch->gcr_result[i]);
-    FREE((char *) ch->gcr_result);
-    FREE((char *) ch);
+	freeMagic((char *) ch->gcr_result[i]);
+    freeMagic((char *) ch->gcr_result);
+    freeMagic((char *) ch);
 }
 
 /*
