@@ -29,7 +29,6 @@ static char rcsid[] = "$Header: /ufs/repository/magic/gcr/gcrDebug.c,v 1.2 2001/
 #include "utils/hash.h"
 #include "database/database.h"
 #include "gcr/gcr.h"
-#include "utils/doubleint.h"
 #include "utils/heap.h"
 #include "router/router.h"
 #include "utils/malloc.h"
@@ -43,6 +42,7 @@ int gcrStandalone=FALSE; /*Flag to control standalone/integrated operation*/
 void gcrDumpResult();
 void gcrStats();
 void gcrShowMap();
+bool gcrMakeChannel();
 
 
 /*
@@ -79,7 +79,7 @@ GCRRouteFromFile(fname)
 	return ((GCRChannel *) NULL);
     }
 
-    MALLOC(GCRChannel *, ch, sizeof (GCRChannel));
+    ch = (GCRChannel *) mallocMagic((unsigned) (sizeof (GCRChannel)));
     ch->gcr_type = CHAN_NORMAL;
 /*     ch->gcr_area = box; */
     ch->gcr_transform = GeoIdentityTransform;
@@ -92,12 +92,12 @@ GCRRouteFromFile(fname)
     {
 	TxError("Couldn't initialize channel routing problem\n");
 	(void) fclose(fp);
-	FREE((char *) ch);
+	freeMagic((char *) ch);
 	return ((GCRChannel *) NULL);
     }
 
     (void) fclose(fp);
-    MALLOC(GCRColEl *, ch->gcr_lCol, (ch->gcr_width+2) * sizeof (GCRColEl));
+    ch->gcr_lCol = (GCRColEl *) mallocMagic((unsigned) ((ch->gcr_width+2) * sizeof (GCRColEl)));
     times(&tbuf1);
     (void) GCRroute(ch);
     times(&tbuf2);
@@ -150,15 +150,15 @@ gcrMakeChannel(ch, fp)
 
     lenWds = ch->gcr_length + 2;
     widWds = ch->gcr_width + 2;
-    MALLOC(int *, ch->gcr_density, lenWds * sizeof (int));
+    ch->gcr_density = (int *) mallocMagic((unsigned) (lenWds * sizeof (int)));
     ch->gcr_lPins = gcrMakePinLR(fp, 0, ch->gcr_width);
-    MALLOC(GCRPin *, ch->gcr_tPins, lenWds * sizeof (GCRPin));
-    MALLOC(GCRPin *, ch->gcr_bPins, lenWds * sizeof (GCRPin));
-    MALLOC(short **, ch->gcr_result, lenWds * sizeof (short *));
+    ch->gcr_tPins = (GCRPin *) mallocMagic((unsigned) (lenWds * sizeof (GCRPin)));
+    ch->gcr_bPins = (GCRPin *) mallocMagic((unsigned) (lenWds * sizeof (GCRPin)));
+    ch->gcr_result = (short **) mallocMagic((unsigned) (lenWds * sizeof (short *)));
 
     /* Initialize end columns */
-    MALLOC(short *, ch->gcr_result[0], widWds * sizeof (short));
-    MALLOC(short *, ch->gcr_result[ch->gcr_length+1], widWds * sizeof (short));
+    ch->gcr_result[0] = (short *) mallocMagic((unsigned) (widWds * sizeof (short)));
+    ch->gcr_result[ch->gcr_length+1] = (short *) mallocMagic((unsigned) (widWds * sizeof (short)));
     for (i = 0; i < widWds; i++)
     {
 	ch->gcr_result[0][i] = 0;
@@ -169,7 +169,7 @@ gcrMakeChannel(ch, fp)
     for (i = 1; i <= ch->gcr_length; i++)
     {
 	/* Allocate the column */
-	MALLOC(short *, ch->gcr_result[i], widWds * sizeof (short));
+	ch->gcr_result[i] = (short *) mallocMagic((unsigned) (widWds * sizeof (short)));
 
 	/* Initialize the bottom pin */
 	if (fscanf(fp, "%d", &pid) != 1)
@@ -240,7 +240,7 @@ gcrMakePinLR(fp, x, size)
     GCRPin *result;
     int i;
 
-    MALLOC(GCRPin *, result, (unsigned) (size+2) * sizeof (GCRPin));
+    result = (GCRPin *) mallocMagic((unsigned) ((size+2) * sizeof (GCRPin)));
     result[0].gcr_x = result[0].gcr_y = 0;
     result[0].gcr_pId = (GCRNet *) NULL;
     result[size + 1].gcr_x = result[size + 1].gcr_y = 0;
@@ -290,20 +290,20 @@ gcrSaveChannel(ch)
     }
 
     /* Output the prologue */
-    (void) fprintf(fp, "* %d %d\n", ch->gcr_width, ch->gcr_length);
+    fprintf(fp, "* %d %d\n", ch->gcr_width, ch->gcr_length);
 
 #define	NETID(pin)	((pin).gcr_pId ? (pin).gcr_pId->gcr_Id : 0)
 
     /* Output the left pin array */
     for (j = 1; j <= ch->gcr_width; j++)
-	(void) fprintf(fp, "%d ", NETID(ch->gcr_lPins[j]));
-    (void) fprintf(fp, "\n");
+	fprintf(fp, "%d ", NETID(ch->gcr_lPins[j]));
+    fprintf(fp, "\n");
 
     /* Process main body of channel */
     for (i = 1; i <= ch->gcr_length; i++)
     {
 	/* Bottom pin */
-	(void) fprintf(fp, "%d ", NETID(ch->gcr_bPins[i]));
+	fprintf(fp, "%d ", NETID(ch->gcr_bPins[i]));
 
 	/*
 	 * Interior points (for obstacle map).
@@ -319,21 +319,21 @@ gcrSaveChannel(ch)
 	    flags = ch->gcr_result[i][j];
 	    switch (flags & (GCRBLKM|GCRBLKP))
 	    {
-		case 0:			(void) fprintf(fp, ". "); break;
-		case GCRBLKM:		(void) fprintf(fp, "m "); break;
-		case GCRBLKP:		(void) fprintf(fp, "p "); break;
-		case GCRBLKM|GCRBLKP:	(void) fprintf(fp, "x "); break;
+		case 0:			fprintf(fp, ". "); break;
+		case GCRBLKM:		fprintf(fp, "m "); break;
+		case GCRBLKP:		fprintf(fp, "p "); break;
+		case GCRBLKM|GCRBLKP:	fprintf(fp, "x "); break;
 	    }
 	}
 
 	/* Top pin */
-	(void) fprintf(fp, "%d\n", NETID(ch->gcr_tPins[i]));
+	fprintf(fp, "%d\n", NETID(ch->gcr_tPins[i]));
     }
 
     /* Output the right pin array */
     for (j = 1; j <= ch->gcr_width; j++)
-	(void) fprintf(fp, "%d ", NETID(ch->gcr_rPins[j]));
-    (void) fprintf(fp, "\n");
+	fprintf(fp, "%d ", NETID(ch->gcr_rPins[j]));
+    fprintf(fp, "\n");
     (void) fclose(fp);
 }
 
@@ -374,45 +374,45 @@ gcrPrDensity(ch, chanDensity)
     if (fp == NULL)
 	fp = stdout;
 
-    (void) fprintf(fp, "Chan width: %d\n", ch->gcr_width);
-    (void) fprintf(fp, "Chan length: %d\n", ch->gcr_length);
-    (void) fprintf(fp, "Chan area: ll=(%d,%d) ur=(%d,%d)\n",
+    fprintf(fp, "Chan width: %d\n", ch->gcr_width);
+    fprintf(fp, "Chan length: %d\n", ch->gcr_length);
+    fprintf(fp, "Chan area: ll=(%d,%d) ur=(%d,%d)\n",
 		ch->gcr_area.r_xbot, ch->gcr_area.r_ybot,
 		ch->gcr_area.r_xtop, ch->gcr_area.r_ytop);
-    (void) fprintf(fp, "Max column density (global):  %d\n", ch->gcr_dMaxByCol);
-    (void) fprintf(fp, "Max column density (channel): %d\n", chanDensity);
-    (void) fprintf(fp, "Column density by column:\n");
-    (void) fprintf(fp, "%3s %5s", "COL", "GLOB");
+    fprintf(fp, "Max column density (global):  %d\n", ch->gcr_dMaxByCol);
+    fprintf(fp, "Max column density (channel): %d\n", chanDensity);
+    fprintf(fp, "Column density by column:\n");
+    fprintf(fp, "%3s %5s", "COL", "GLOB");
 #ifdef	IDENSITY
-    (void) fprintf(fp, " %5s %5s", "INIT", "DIFF");
+    fprintf(fp, " %5s %5s", "INIT", "DIFF");
 #endif	/* IDENSITY */
-    (void) fprintf(fp, " %5s\n", "CHAN");
+    fprintf(fp, " %5s\n", "CHAN");
     for (i = 1; i <= ch->gcr_length; i++)
     {
-	(void) fprintf(fp, "%3d %5d", i, ch->gcr_dRowsByCol[i]);
+	fprintf(fp, "%3d %5d", i, ch->gcr_dRowsByCol[i]);
 	diff = ch->gcr_dRowsByCol[i];
 #ifdef	IDENSITY
 	diff -= ch->gcr_iRowsByCol[i];
-	(void) fprintf(fp, " %5d %5d", ch->gcr_iRowsByCol[i], diff);
+	fprintf(fp, " %5d %5d", ch->gcr_iRowsByCol[i], diff);
 #endif	/* IDENSITY */
-	(void) fprintf(fp, "%5d%s\n", ch->gcr_density[i],
+	fprintf(fp, "%5d%s\n", ch->gcr_density[i],
 		(diff != ch->gcr_density[i]) ? " *****" : "");
     }
-    (void) fprintf(fp, "------\n");
-    (void) fprintf(fp, "Row density by column (global only):\n");
-    (void) fprintf(fp, "%3s %5s", "ROW", "GLOB");
+    fprintf(fp, "------\n");
+    fprintf(fp, "Row density by column (global only):\n");
+    fprintf(fp, "%3s %5s", "ROW", "GLOB");
 #ifdef	IDENSITY
-    (void) fprintf(fp, " %5s %5s", "INIT", "DIFF");
+    fprintf(fp, " %5s %5s", "INIT", "DIFF");
 #endif	/* IDENSITY */
-    (void) fprintf(fp, "\n");
+    fprintf(fp, "\n");
     for (i = 1; i <= ch->gcr_width; i++)
     {
-	(void) fprintf(fp, "%3d %5d", i, ch->gcr_dColsByRow[i]);
+	fprintf(fp, "%3d %5d", i, ch->gcr_dColsByRow[i]);
 #ifdef	IDENSITY
-	(void) fprintf(fp, " %5d %5d", ch->gcr_iColsByRow[i],
+	fprintf(fp, " %5d %5d", ch->gcr_iColsByRow[i],
 			ch->gcr_dColsByRow[i] - ch->gcr_iColsByRow[i]);
 #endif	/* IDENSITY */
-	(void) fprintf(fp, "\n");
+	fprintf(fp, "\n");
     }
 
     (void) fflush(fp);
