@@ -126,7 +126,7 @@ bottom:
      * anyway.
      */
     for (net = ch->gcr_nets; net; net = net->gcr_next)
-	FREE((char *) net);
+	freeMagic((char *) net);
     ch->gcr_nets = NULL;
 
     return (gcrRouterErrors);
