@@ -10,6 +10,7 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 
 /* This file is only applicable for OpenGL-enabled graphics;  THREE_D is */
 /* defined by the "make config" process and includes all preconditions.	 */
@@ -22,12 +23,13 @@
 #include <GL/gl.h>
 #include <GL/glx.h>
 
-#include "magic/tclmagic.h"
+#include "tcltk/tclmagic.h"
 #include "misc/magic.h"
 #include "utils/geometry.h"
 #include "windows/windows.h"
 #include "tiles/tile.h"
 #include "utils/hash.h"
+#include "undo/undo.h"
 #include "database/database.h"
 #include "main/main.h"
 #include "commands/commands.h"
@@ -78,6 +80,7 @@ void W3DCIFredisplay();
 void W3Dredisplay();
 void Set3DDefaults();
 void w3drefreshFunc();
+bool W3Ddelete();
 
 /* ------------------------Low-Level Routines--------------------------------- */
 
@@ -93,11 +96,7 @@ void
 w3dUnlock(w)
     MagWindow *w;
 {
-    Window wind;
-    wind = Tk_WindowId((Tk_Window)w->w_grdata);
-
     glFlush();
-    /* glXSwapBuffers(grXdpy, (GLXDrawable)wind); */
     
     glDisable(GL_CULL_FACE);
 
@@ -264,21 +263,16 @@ w3dFillOps(trans, tile, cliprect, ztop, zbot)
 #ifdef NONMANHATTAN
     if (IsSplit(tile))
     {
-	int o1, o2, o3, dir, side;
 	Rect fullr;
+	TileType dinfo;
 
-	o1 = ((trans->t_e > 0) || (trans->t_d > 0)) ? 1 : 0;
-	o2 = ((trans->t_a > 0) || (trans->t_b > 0)) ? 1 : 0;
-	o3 = (trans->t_a != 0) ? 1 : 0;
-	dir = SplitDirection(tile) ^ o1 ^ o2;
-	side = SplitSide(tile) ^ o2 ^ (SplitDirection(tile) | o3);
+	dinfo = DBTransformDiagonal(TiGetTypeExact(tile), trans);
 
 	fullr = r;
 	if (cliprect != NULL)
 	    GeoClip(&r, cliprect);
 
-	GrClipTriangle(&fullr, &r, cliprect != NULL, dir, side,
-		p, &np);
+	GrClipTriangle(&fullr, &r, cliprect != NULL, dinfo, p, &np);
 	
 	if (np > 0)
 	{
@@ -501,6 +495,7 @@ w3dSetProjection(w)
 
     crec = (W3DclientRec *) w->w_clientData;
     wind = Tk_WindowId((Tk_Window)w->w_grdata);
+    if (wind == 0) return;	/* window was closed by window manager? */
 
     /* Should not mess with surfaceArea---we want the area to be displayed */
     /* in crec, in magic internal coordinates, set by the edit box (else-  */
@@ -576,6 +571,9 @@ w3dPaintFunc(tile, cxp)
     /* 3D window can only be interrupted by events in	*/
     /* the layout window.				*/
 
+    /* Honor display suspend status */
+    if (GrDisplayStatus == DISPLAY_SUSPEND) return 0;
+
     if (GrDisplayStatus == DISPLAY_BREAK_PENDING)
     {
 	GrDisplayStatus = DISPLAY_IN_PROGRESS;
@@ -612,6 +610,9 @@ w3dCIFPaintFunc(tile, arg)
 {
     CIFLayer *layer = (CIFLayer *)arg;
 
+    /* Honor display suspend status */
+    if (GrDisplayStatus == DISPLAY_SUSPEND) return 0;
+
     /* Allow display interrupt a la dbwPaintFunc() */
     if (GrDisplayStatus == DISPLAY_BREAK_PENDING)
     {
@@ -1669,6 +1670,7 @@ W3Dcreate(window, argc, argv)
 	Tk_GeometryRequest(tkwind, crec->width, crec->height);
 
 	wind = Tk_WindowId(tkwind);
+	if (wind == 0)
 	glXMakeCurrent(grXdpy, (GLXDrawable)wind, grXcontext);
 
 	/* execute any pending Tk events */
