@@ -154,7 +154,7 @@ grClipAgainst(startllr, clip)
 	    /* this will modify the list that we are traversing! */
 	    (void) GrDisjoint(&lr->r_r, clip, grClipAddFunc, 
 		    (ClientData) &llr);
-	    FREE( (char *) lr);
+	    freeMagic( (char *) lr);
 	}
 	else
 	    llr = &((*llr)->r_next);
@@ -174,7 +174,7 @@ bool grClipAddFunc(box, cd)
 
     lllr = (LinkedRect ***) cd;
 
-    MALLOC(LinkedRect *, lr, sizeof (LinkedRect));
+    lr = (LinkedRect *) mallocMagic((unsigned) (sizeof (LinkedRect)));
     lr->r_r = *box;
     lr->r_next = **lllr;
     **lllr = lr;
@@ -189,7 +189,7 @@ grObsBox(r)
     LinkedRect *ar;
     LinkedRect **areas;
 
-    MALLOC(LinkedRect *, ar, sizeof (LinkedRect));
+    ar = (LinkedRect *) mallocMagic((unsigned) (sizeof (LinkedRect)));
     ar->r_r = *r;
     ar->r_next = NULL;
     areas = &ar;
@@ -210,7 +210,7 @@ grObsBox(r)
 	    (*grFillRectPtr)(&((*areas)->r_r));
 	oldarea = *areas;
 	*areas = (*areas)->r_next;
-	FREE( (char *) oldarea );
+	freeMagic( (char *) oldarea );
     }
 }
 
@@ -356,7 +356,7 @@ grClipPoints(line, box, p1, p1OK, p2, p2OK)
 
 
 #define NEWAREA(lr,x1,y1,x2,y2)	{LinkedRect *tmp; \
-    MALLOC(LinkedRect *, tmp, sizeof (LinkedRect)); \
+    tmp = (LinkedRect *) mallocMagic((unsigned) (sizeof (LinkedRect))); \
     tmp->r_r.r_xbot = x1; tmp->r_r.r_xtop = x2; \
     tmp->r_r.r_ybot = y1; tmp->r_r.r_ytop = y2; tmp->r_next = lr; lr = tmp;}
 
@@ -387,7 +387,7 @@ GrClipLine(x1, y1, x2, y2)
      * left endpoint of a line, and the ur corner the right endpoint
      * of the line.
      */
-    MALLOC(LinkedRect *, areas, sizeof (LinkedRect));
+    areas = (LinkedRect *) mallocMagic((unsigned) (sizeof (LinkedRect)));
     areas->r_next = NULL;
     if (x1 < x2)
     {
@@ -470,7 +470,7 @@ GrClipLine(x1, y1, x2, y2)
 		{
 		    /* clip both sides */
 		    LinkedRect *new;
-		    MALLOC(LinkedRect *, new, sizeof (LinkedRect));
+		    new = (LinkedRect *) mallocMagic((unsigned) (sizeof (LinkedRect)));
 		    new->r_r.r_ur = l->r_ur;
 		    new->r_r.r_ll = p2;
 		    new->r_next = (*ar);
@@ -488,7 +488,7 @@ deleteit: {
 	    LinkedRect *reclaim;
 	    reclaim = (*ar);
 	    *ar = reclaim->r_next;
-	    FREE( (char *) reclaim);
+	    freeMagic( (char *) reclaim);
 	}
 
     } /* for ar */
@@ -502,7 +502,7 @@ deleteit: {
 		areas->r_r.r_xtop, areas->r_r.r_ytop);
 	oldarea = areas;
 	areas = areas->r_next;
-	FREE( (char *) oldarea );
+	freeMagic( (char *) oldarea );
     }
 }
 
@@ -825,28 +825,23 @@ GrBox(MagWindow *mw, Transform *trans, Tile *tile)
     if (IsSplit(tile))
     {
 	/* Perform matrix transformations on split tiles */
-	int o1, o2, o3, dir, side;
+	TileType dinfo;
 	Rect fullr;
 
-	o1 = ((trans->t_e > 0) || (trans->t_d > 0)) ? 1 : 0;
-	o2 = ((trans->t_a > 0) || (trans->t_b > 0)) ? 1 : 0;
-	o3 = (trans->t_a != 0) ? 1 : 0;
-	dir = SplitDirection(tile) ^ o1 ^ o2;
-	side = SplitSide(tile) ^ o2 ^ (SplitDirection(tile) | o3);
-
+	dinfo = DBTransformDiagonal(TiGetTypeExact(tile), trans);
 	clipr = fullr = r;
 
 	if (needClip)
 	    GeoClip(&clipr, &grCurClip);
 
-	GrClipTriangle(&fullr, &clipr, needClip, dir, side, polyp, &np);
+	GrClipTriangle(&fullr, &clipr, needClip, dinfo, polyp, &np);
 
 	if ((grCurFill == GR_STSOLID) || 
 	(grCurFill == GR_STSTIPPLE) || (grCurFill == GR_STGRID) )
 	{
 	    if (needObscure)
 		grObsBox(&clipr);
-	    else
+	    else if (grFillPolygonPtr)
 		(void) (*grFillPolygonPtr)(polyp, np);
 	} 
     }
@@ -1148,16 +1143,15 @@ GrFastBox(prect)
 #define yround(d) (int)(((((d % width) << 1) >= width) ? 1 : 0) + (d / width))
 
 void
-GrClipTriangle(r, c, clipped, dir, side, points, np)
+GrClipTriangle(r, c, clipped, dinfo, points, np)
     Rect *r;   /* Bounding box of triangle, in screen coords	*/
     Rect *c;		/* Clipping rectangle 				*/
     bool clipped;	/* Boolean, if bounding box is clipped		*/
-    int dir;		/* Direction of split				*/
-    int side;		/* Which triangle to draw (left or right)	*/
+    TileType dinfo;	/* Split side and direction information		*/
     Point *points;	/* Point array (up to 5 points) to fill		*/
     int *np;		/* Number of points in the clipped polygon	*/
 {
-    if (side == 0)
+    if (!(dinfo & TT_SIDE))
     {
 	points[1].p_x = r->r_xbot;
 	points[0].p_y = r->r_ytop;
@@ -1172,7 +1166,7 @@ GrClipTriangle(r, c, clipped, dir, side, points, np)
 	points[0].p_x = points[2].p_x = r->r_xbot;
     }
 
-    if (dir == 0)
+    if (!(dinfo & TT_DIRECTION))
     {
 	points[1].p_y = points[0].p_y;
 	points[2].p_x = points[1].p_x;
@@ -1194,9 +1188,9 @@ GrClipTriangle(r, c, clipped, dir, side, points, np)
         dlong width = (dlong)(r->r_xtop - r->r_xbot);
 	dlong height = (dlong)(r->r_ytop - r->r_ybot);
 
-	switch((side << 1) | dir)
+	switch(dinfo)
 	{
-	    case 0x0:	/* nw */
+	    case TT_DIAGONAL:	/* nw */
 		if (c->r_ytop < r->r_ytop)	/* clip top */
 		{
 		    delx = (dlong)(points[1].p_y - c->r_ytop) * width;
@@ -1245,7 +1239,7 @@ GrClipTriangle(r, c, clipped, dir, side, points, np)
 		    *np = 0;	/* clipped out of existance */
 		break;
 
-	    case 0x1:	/* sw */
+	    case TT_DIAGONAL | TT_DIRECTION:	/* sw */
 		if (c->r_xbot > r->r_xbot)	/* clip LEFT */
 		{
 		    dely = (dlong)(c->r_xbot - points[1].p_x) * height;
@@ -1294,7 +1288,8 @@ GrClipTriangle(r, c, clipped, dir, side, points, np)
 		    *np = 0;	/* clipped out of existance */
 		break;
 
-	    case 0x2:	/* se */	/* order: (bottom, right), (top, left) */
+	    case TT_DIAGONAL | TT_SIDE:	/* se */
+		/* order: (bottom, right), (top, left) */
 		if (c->r_ybot > r->r_ybot)	/* clip BOTTOM */
 		{
 		    delx = (dlong)(c->r_ybot - points[1].p_y) * width;
@@ -1343,7 +1338,8 @@ GrClipTriangle(r, c, clipped, dir, side, points, np)
 		    *np = 0;	/* clipped out of existance */
 		break;
 
-	    case 0x3:	/* ne */	/* order: (top, right), (bottom, left) */
+	    case TT_DIAGONAL | TT_SIDE | TT_DIRECTION:	/* ne */
+		/* order: (top, right), (bottom, left) */
 		if (c->r_xtop < r->r_xtop)	/* clip RIGHT */
 		{
 		    dely = (dlong)(points[1].p_x - c->r_xtop) * height;
@@ -1410,14 +1406,14 @@ GrClipTriangle(r, c, clipped, dir, side, points, np)
  */
 
 void
-GrDrawTriangleEdge(r, dir, side)
+GrDrawTriangleEdge(r, dinfo)
     Rect *r;   /* Bounding box of triangle, in screen coords	*/
-    int dir, side;
+    TileType dinfo;
 {
     Point tpoints[5];
     int tnum, i, j;
 
-    GrClipTriangle(r, &grCurClip, TRUE, dir, side, tpoints, &tnum);
+    GrClipTriangle(r, &grCurClip, TRUE, dinfo, tpoints, &tnum);
 
     for (i = 0; i < tnum; i++)
     { 
@@ -1451,12 +1447,11 @@ GrDrawTriangleEdge(r, dir, side)
  */
 
 void
-GrDiagonal(prect, dir, side)
+GrDiagonal(prect, dinfo)
     Rect *prect;	/* The rectangle to be drawn, given in
 				 * screen coordinates.
 			         */
-    int dir;	/* direction of split: 0 = (/), 1 = (\)  */
-    int side;	/* side of triangle to draw: 0 = left, 1 = right */
+    TileType dinfo;	/* split and direction information */
 {
     Rect *r;
     bool needClip, needObscure;
@@ -1493,7 +1488,7 @@ GrDiagonal(prect, dir, side)
     if (needClip)
 	GeoClip(&clipr, &grCurClip);
 
-    GrClipTriangle(&fullr, &clipr, needClip, dir, side, polyp, &np);
+    GrClipTriangle(&fullr, &clipr, needClip, dinfo, polyp, &np);
 
     /* do solid areas */
     if ( (grCurFill == GR_STSOLID) || 
@@ -1501,7 +1496,7 @@ GrDiagonal(prect, dir, side)
     {
 	if (needObscure)
 	    grObsBox(&clipr);
-	else
+	else if (grFillPolygonPtr)
 	    (void) (*grFillPolygonPtr)(polyp, np);
     } 
 
@@ -1524,6 +1519,25 @@ GrDiagonal(prect, dir, side)
     }
 }
 
+/*---------------------------------------------------------
+ * GrFillPolygon --
+ *	This routine is simply a call to the locally-defined
+ *	grFillPolygonPtr routine.
+ *
+ *---------------------------------------------------------
+ */
+
+void
+GrFillPolygon(polyp, np)
+    Point *polyp;		/* Array of points defining polygon */
+    int np;			/* number of points in array polyp  */
+{
+    if (grFillPolygonPtr != NULL)
+    {
+	(*grFillPolygonPtr)(polyp, np);
+    }
+}
+
 #endif /* NONMANHATTAN */
 
 /*---------------------------------------------------------
