@@ -26,7 +26,7 @@
 #include <GL/gl.h>
 #include <GL/glx.h>
 
-#include "magic/tclmagic.h"
+#include "tcltk/tclmagic.h"
 #include "misc/magic.h"
 #include "misc/magsgtty.h"
 #include "utils/geometry.h"
@@ -237,7 +237,6 @@ grtoglSetStipple (stipple)
     }
 }
 
-
 
 /*------------------------------------------------------------------------
  * GrTOGLInit:
@@ -583,36 +582,39 @@ TOGLEventProc(clientData, xevent)
 					glTransY(mw, KeyPressedEvent->y),
 					mw->w_wid);
 				TxInputRedirect = False;
-				Tcl_Eval(consoleinterp, "::tkcon::Eval .text");
+				Tcl_EvalEx(consoleinterp, "::tkcon::Eval .text",
+						19, 0);
 				TxSetPrompt('%');
 
 				Tcl_SaveResult(magicinterp, &state);
-				Tcl_Eval(magicinterp, "history event 0");
+				Tcl_EvalEx(magicinterp, "history event 0", 15, 0);
 			        MacroDefine((int)'.',
 					Tcl_GetStringResult(magicinterp),
 					FALSE);
 				Tcl_RestoreResult(magicinterp, &state);
 				break;
 			    case XK_Up:
-				Tcl_Eval(consoleinterp, "::tkcon::Event -1");
+				Tcl_EvalEx(consoleinterp, "::tkcon::Event -1",
+					17, 0);
 				break;
 			    case XK_Down:
-				Tcl_Eval(consoleinterp, "::tkcon::Event 1");
+				Tcl_EvalEx(consoleinterp, "::tkcon::Event 1",
+					16, 0);
 				break;
 			    case XK_BackSpace: case XK_Delete:
-				Tcl_Eval(consoleinterp, ".text delete insert-1c ;"
-					".text see insert");
+				Tcl_EvalEx(consoleinterp, ".text delete insert-1c ;"
+					".text see insert", 40, 0);
 				break;
 			    case XK_quotedbl: case XK_backslash: case XK_bracketleft:
 				outstr[23] = '\\';
 				outstr[24] = inChar[idx];
 				outstr[25] = '\"';
-				Tcl_Eval(consoleinterp, outstr);
+				Tcl_EvalEx(consoleinterp, outstr, 25, 0);
 				outstr[24] = '\"';
 				outstr[25] = '\0';
 			    default:
 				outstr[23] = inChar[idx];
-				Tcl_Eval(consoleinterp, outstr);
+				Tcl_EvalEx(consoleinterp, outstr, 25, 0);
 				break;
 			}
 		    }
@@ -709,7 +711,7 @@ TOGLEventProc(clientData, xevent)
 				    char *outstring = Tcl_Alloc(sl + 20);
 				    sprintf(outstring, ".text insert end \"%s\"",
 						macroDef);
-				    Tcl_Eval(consoleinterp, outstring);
+				    Tcl_EvalEx(consoleinterp, outstring, -1, 0);
 				    Tcl_Free(outstring);
 				}
 				else
@@ -748,7 +750,7 @@ TOGLEventProc(clientData, xevent)
 		XConfigureEvent *ConfigureEvent = (XConfigureEvent*) xevent;
 		Rect	screenRect;
 		int width, height;
-		Bool result;
+		bool result;
 		    
 		width = ConfigureEvent->width;
 		height = ConfigureEvent->height;
@@ -836,7 +838,7 @@ oglSetDisplay (dispType, outFileName, mouseFileName)
     WindPackageType = WIND_X_WINDOWS;  /* to be changed? */
     TxInputRedirect = False;
 
-    grCursorType = "bw";
+    grCursorType = "color";
     WindScrollBarWidth = 14;
 
     /* Set up the procedure values in the indirection table. */
@@ -996,7 +998,7 @@ GrTOGLCreate(w, name)
 
     if (tkwind != 0)
     {
-	Bool result;
+	bool result;
 
 	GrTOGLFlush();
 
