@@ -13,7 +13,7 @@ char *getenv();
 #include <GL/gl.h>
 #include <GL/glx.h>
 
-#include "magic/tclmagic.h"
+#include "tcltk/tclmagic.h"
 #include "misc/magic.h"
 #include "textio/textio.h"
 #include "utils/geometry.h"
