@@ -11,7 +11,7 @@
 #include <stdio.h>
 #include <X11/Xlib.h>
 
-#include "magic/tclmagic.h"
+#include "tcltk/tclmagic.h"
 #include "misc/magic.h"
 #include "misc/magsgtty.h"
 #include "textio/textio.h"
