@@ -18,12 +18,13 @@
 #include <X11/Xutil.h>
 #include <X11/keysym.h>
 
-#include "magic/tclmagic.h"
+#include "tcltk/tclmagic.h"
 #include "misc/magic.h"
 #include "misc/styles.h"
 #include "utils/geometry.h"
 #include "utils/hash.h"
 #include "utils/malloc.h"
+#include "utils/utils.h"
 #include "tiles/tile.h"
 #include "database/database.h"
 #include "windows/windows.h"
@@ -140,14 +141,21 @@ grTkFreeFonts()
  * ----------------------------------------------------------------------------
  */
 
+typedef struct {
+    unsigned char source[32];
+    unsigned char mask[32];
+} CursorCache;
+
 void
 grTkDefineCursor(glyphs)
     GrGlyphs *glyphs;
 {
+    char *fgname, *bgname;
     int glyphnum;
     Rect oldClip;
     int red, green, blue;
-    bool isblack;
+    Tk_Window tkwin;
+    bool fg_needs_free;
 
     if (glyphs->gr_num <= 0) return;
 
@@ -157,67 +165,98 @@ grTkDefineCursor(glyphs)
 	return;
     }
 
+    tkwin = Tk_MainWindow(magicinterp);
+
     /* expand clipping amount for off-screen access on the X */
     GrLock(GR_LOCK_SCREEN, FALSE);
     oldClip = grCurClip;
     grCurClip = GrScreenRect;
     grCurClip.r_ytop += 16;
     
-    /* what color should the cursor be?  The following makes it the
-       opposite of what the background "mostly" is.
-    */
-    GrGetColor(GrStyleTable[STYLE_TRANSPARENT].color, &red, &green, &blue);
-    isblack = (red + green + blue > 0x180);	/* (0x180 = 128 * 3) */
-
     /* enter the glyphs */
     for (glyphnum = 0; glyphnum < glyphs->gr_num; glyphnum++) {
-	int *p;
+	int i, *p, fgstyle;
+	XColor curcolor;
 	GrGlyph *g;
 	int x, y;
-	unsigned char *curs;
+	CursorCache *glyphcache;
 
 	g = glyphs->gr_glyph[glyphnum];
 	if ((g->gr_xsize != 16) || (g->gr_ysize != 16)) {
 	    TxError("Tk/OpenGL Cursors must be 16 X 16 pixels.\n");
 	    return;
 	}
-	
-	curs = (unsigned char *)mallocMagic(32 * sizeof(unsigned char));
-	g->gr_cache = (ClientData)curs;
+
+        glyphcache = (CursorCache *)mallocMagic(sizeof(CursorCache));
+	g->gr_cache = (ClientData)glyphcache;
 	g->gr_free = freeMagic;
+	
+	/* Find the foreground and background colors of the glyph */
+
+	p = &(g->gr_pixels[0]);
+	fgstyle = STYLE_TRANSPARENT;
+	fg_needs_free = FALSE;
+	for (x = 0; x < 256; x++)
+	{
+	    if (*p != STYLE_TRANSPARENT)
+	    {
+		fgstyle = *p;
+		GrGetColor(GrStyleTable[*p].color, &red, &green, &blue);
+		curcolor.red = (unsigned short)(red << 8);
+		curcolor.green = (unsigned short)(green << 8);
+		curcolor.blue = (unsigned short)(blue << 8);
+		curcolor.flags = DoRed | DoGreen | DoBlue;
+		fgname = (char *)Tk_NameOfColor(Tk_GetColorByValue(tkwin, &curcolor));
+		break;
+	    }
+	    p++;
+	}
+	if (x == 256)
+	    fgname = "black";
+
+	for (; x < 256; x++)
+	{
+	    if ((*p != STYLE_TRANSPARENT) && (*p != fgstyle))
+	    {
+		GrGetColor(GrStyleTable[*p].color, &red, &green, &blue);
+		curcolor.red = (unsigned short)(red << 8);
+		curcolor.green = (unsigned short)(green << 8);
+		curcolor.blue = (unsigned short)(blue << 8);
+		curcolor.flags = DoRed | DoGreen | DoBlue;
+		bgname = StrDup((char **)NULL, fgname);
+		fgname = bgname;
+		fg_needs_free = TRUE;
+		bgname = (char *)Tk_NameOfColor(Tk_GetColorByValue(tkwin, &curcolor));
+		break;
+	    }
+	    p++;
+	}
+	if (x >= 256)
+	    bgname = "white";
 
 	/* Perform transposition on the glyph matrix since X displays
 	 * the least significant bit on the left hand side.
 	 */
 	p = &(g->gr_pixels[0]);
-	for (y = 0; y < 32; y += 2) {
-	    int i;
-
-	    curs[i = 31 - y - 1] = 0;
+	for (y = 0; y < 32; y++) {
+	    i = (y & 1) ? (32 - y) : (30 - y);
+	    glyphcache->source[i] = glyphcache->mask[i] = 0;
 	    for (x = 0; x < 8; x++) 
 	    {
-		if (GrStyleTable[*p].color != 0)
-		{
-		     curs[i] |= 1 << x; 
-		}
-		p++;
-	    }
-	    curs[i += 1] = 0;
-	    for (x = 0; x < 8; x++) 
-	    {
-		if (GrStyleTable[*p].color != 0)
-		{
-		     curs[i] |= 1 << x; 
-		}
+		if (*p == fgstyle)
+		     glyphcache->source[i] |= (1 << x);
+		if (*p != STYLE_TRANSPARENT)
+		     glyphcache->mask[i] |= (1 << x);
 		p++;
 	    }
 	}
 	grCursors[glyphnum] = Tk_GetCursorFromData(magicinterp,
 			Tk_MainWindow(magicinterp),
-			(char *)curs, (char *)curs,
+			(char *)glyphcache->source, (char *)glyphcache->mask,
 			16, 16, g->gr_origin.p_x, (15 - g->gr_origin.p_y),
-			((isblack) ? "black" : "white"),
-			((isblack) ? "white" : "black"));
+			Tk_GetUid(fgname), Tk_GetUid(bgname));
+
+	if (fg_needs_free) freeMagic(fgname);
     }
 
     /* Restore clipping */
@@ -556,7 +595,7 @@ ImgLayerConfigureMaster(masterPtr, objc, objv, flags)
     argv[objc] = NULL;
 
     if (Tk_ConfigureWidget(masterPtr->interp, Tk_MainWindow(masterPtr->interp),
-	    configSpecs, objc, argv, (char *) masterPtr, flags)
+	    configSpecs, objc, (CONST84 char **)argv, (char *) masterPtr, flags)
 	    != TCL_OK) {
 	Tcl_Free((char *) argv);
 	return TCL_ERROR;
@@ -871,8 +910,8 @@ ImgLayerCmd(clientData, interp, objc, objv)
 	Tcl_WrongNumArgs(interp, 1, objv, "option ?arg arg ...?");
 	return TCL_ERROR;
     }
-    if (Tcl_GetIndexFromObj(interp, objv[1], layerOptions, "option", 0,
-	    &index) != TCL_OK) {
+    if (Tcl_GetIndexFromObj(interp, objv[1], (CONST84 char **)layerOptions,
+		"option", 0, &index) != TCL_OK) {
 	return TCL_ERROR;
     }
     switch (index) {
@@ -1158,8 +1197,9 @@ ImgLayerCmdDeletedProc(clientData)
 void
 RegisterTkCommands(Tcl_Interp *interp)
 {
-    Tcl_CreateCommand(interp, "magic::magiccolor", _magic_magiccolor,
-		(ClientData)NULL, (Tcl_CmdDeleteProc *)NULL);
+    Tcl_CreateCommand(interp, "magic::magiccolor",
+		(Tcl_CmdProc *)_magic_magiccolor, (ClientData)NULL,
+		(Tcl_CmdDeleteProc *)NULL);
     Tk_CreateImageType(&tkLayerImageType);
 }
 
