@@ -279,7 +279,7 @@ grx11SetSPattern (stipple, pattern)
     int x,y,pat;
 
     p = XCreatePixmap(grXdpy, XDefaultRootWindow(grXdpy), 8, 8, 1);
-    if (grGCStipple == 0) {
+    if (grGCStipple == NULL) {
 	grGCStipple = XCreateGC(grXdpy, p, 0, 0);
     }
     for (y = 0; y < 8; y++) {
@@ -704,7 +704,8 @@ GrX11Close ()
     kill(Xhelper, SIGKILL);
     WaitPid (Xhelper, 0);
 #endif
-    grGCStipple = 0;
+    XFreeGC(grXdpy, grGCStipple);
+    grGCStipple = NULL;
     XCloseDisplay(grXdpy);
 #ifdef HAVE_PTHREADS
     xloop_end();
@@ -919,7 +920,7 @@ x11SetDisplay (dispType, outFileName, mouseFileName)
 
     WindPackageType = WIND_X_WINDOWS;
     WindScrollBarWidth = 14;
-    grCursorType = "bw";
+    grCursorType = "color";
 
     /* Set up helper process */
     pipe(fildes);
