@@ -24,7 +24,6 @@
 #include "utils/geometry.h"
 #include "misc/styles.h"
 #include "utils/hash.h"
-#include "utils/doubleint.h"
 #include "utils/heap.h"
 #include "utils/utils.h"
 #include "tiles/tile.h"
@@ -56,6 +55,7 @@ TileTypeBitMask	glMaskChannel;	/* Mask of all channel types */
 int glChanSplitRiver();
 int glChanRiverBlock();
 int glChanFloodVFunc(), glChanFloodHFunc();
+bool glChanClip();
 int glChanClipFunc();
 int glChanMergeFunc();
 int glChanFeedFunc();
@@ -686,7 +686,7 @@ glChanBlockDens(ch)
 		/* Remember area of blocked tile to paint */
 		area.r_xbot = shiftedOrigin + lo * RtrGridSpacing;
 		area.r_xtop = shiftedOrigin + hi * RtrGridSpacing;
-		MALLOC(PaintArea *, pa, sizeof (PaintArea));
+		pa = (PaintArea *) mallocMagic((unsigned)(sizeof (PaintArea)));
 		pa->pa_area = area;
 		pa->pa_next = glChanPaintList;
 		pa->pa_type = CHAN_VRIVER;
@@ -715,7 +715,7 @@ glChanBlockDens(ch)
 		/* Remember area of blocked tile to paint */
 		area.r_ybot = shiftedOrigin + lo * RtrGridSpacing;
 		area.r_ytop = shiftedOrigin + hi * RtrGridSpacing;
-		MALLOC(PaintArea *, pa, sizeof (PaintArea));
+		pa = (PaintArea *) mallocMagic((unsigned)(sizeof (PaintArea)));
 		pa->pa_area = area;
 		pa->pa_next = glChanPaintList;
 		pa->pa_type = CHAN_HRIVER;
@@ -761,7 +761,7 @@ glChanBlockDens(ch)
 	for (pa = glChanPaintList, glChanPaintList = NULL; pa; pa = pa->pa_next)
 	{
 	    glChanFlood(&pa->pa_area, pa->pa_type);
-	    FREE((char *) pa);
+	    freeMagic((char *) pa);
 	}
     } while (glChanPaintList != NULL);
 }
@@ -868,7 +868,7 @@ glChanFloodVFunc(tile, area)
 {
     PaintArea *pa;
 
-    MALLOC(PaintArea *, pa, sizeof (PaintArea));
+    pa = (PaintArea *) mallocMagic((unsigned) (sizeof (PaintArea)));
     pa->pa_area.r_xbot = MAX(area->r_xbot, LEFT(tile));
     pa->pa_area.r_xtop = MIN(area->r_xtop, RIGHT(tile));
     pa->pa_area.r_ybot = BOTTOM(tile);
@@ -887,7 +887,7 @@ glChanFloodHFunc(tile, area)
 {
     PaintArea *pa;
 
-    MALLOC(PaintArea *, pa, sizeof (PaintArea));
+    pa = (PaintArea *) mallocMagic((unsigned)(sizeof (PaintArea)));
     pa->pa_area.r_ybot = MAX(area->r_ybot, BOTTOM(tile));
     pa->pa_area.r_ytop = MIN(area->r_ytop, TOP(tile));
     pa->pa_area.r_xbot = LEFT(tile);
