@@ -22,11 +22,12 @@ static char rcsid[] = "$Header: /ufs/repository/magic/grouter/grouteCrss.c,v 1.2
 #endif  /* not lint */
 
 #include <stdio.h>
+#include <string.h>
+
 #include "misc/magic.h"
 #include "utils/geometry.h"
 #include "utils/geofast.h"
 #include "utils/hash.h"
-#include "utils/doubleint.h"
 #include "utils/heap.h"
 #include "utils/malloc.h"
 #include "debug/debug.h"
