@@ -25,7 +25,6 @@ static char rcsid[] = "$Header: /ufs/repository/magic/grouter/grouteDens.c,v 1.2
 #include "misc/magic.h"
 #include "utils/geometry.h"
 #include "utils/hash.h"
-#include "utils/doubleint.h"
 #include "utils/heap.h"
 #include "tiles/tile.h"
 #include "database/database.h"
@@ -194,7 +193,7 @@ glDMAlloc(dm, top, cap)
     dm->dm_max = 0;
     dm->dm_size = top + 1;
     dm->dm_cap = cap;
-    CALLOC(short *, dm->dm_value, sizeof (short) * dm->dm_size);
+    dm->dm_value = (short *) callocMagic((unsigned) (sizeof (short) * dm->dm_size));
 }
 
 /*
@@ -246,7 +245,7 @@ void
 glDMFree(dm)
     DensMap *dm;
 {
-    FREE((char *) dm->dm_value);
+    freeMagic((char *) dm->dm_value);
 }
 
 /*
