@@ -150,7 +150,7 @@ glRouteToPoint(loc, bestCost)
 
     glNumTries++;
     if (glLogFile)
-	(void) fprintf(glLogFile, "---\t%d\t0,0\t0\t0\n", glNumTries);
+	fprintf(glLogFile, "---\t%d\t0,0\t0\t0\n", glNumTries);
 
     /* Initialization */
     if (!glPenaltiesScaled)
@@ -535,7 +535,7 @@ glFinalPropagate(inPt, loc)
     HeapAddInt(&glHeap, cost, (char *) outPt);
     if (glLogFile)
     {
-	(void) fprintf(glLogFile, "FIN\t%d\t%d,%d\t%d\n",
+	fprintf(glLogFile, "FIN\t%d\t%d,%d\t%d\n",
 		glNumTries,
 		outPt->gl_point.p_x, outPt->gl_point.p_y,
 		outPt->gl_length);
@@ -599,7 +599,7 @@ glRiverPropagate(inPt)
 	    HeapAddInt(&glHeap, cost, (char *) outPt);
 	    if (glLogFile)
 	    {
-		(void) fprintf(glLogFile, "RIV\t%d\t%d,%d\t%d\t%d\n",
+		fprintf(glLogFile, "RIV\t%d\t%d,%d\t%d\t%d\n",
 			glNumTries,
 			outPt->gl_point.p_x, outPt->gl_point.p_y,
 			outPt->gl_length, cost);
@@ -1370,7 +1370,7 @@ glPropagateFn(outCh, outPin, inPt)
     HeapAddInt(&glHeap, finalCost, (char *) outPt);
     if (glLogFile)
     {
-	(void) fprintf(glLogFile, "ADD\t%d\t%d,%d\t%d\t%d\n",
+	fprintf(glLogFile, "ADD\t%d\t%d,%d\t%d\t%d\n",
 		glNumTries,
 		outPt->gl_point.p_x, outPt->gl_point.p_y,
 		outPt->gl_length, finalCost);
@@ -1759,7 +1759,7 @@ glLogPath(inPt, cost)
 
     if (bcmp((char *)&inPt->gl_range, (char *)&glInitRect, sizeof (Rect)) == 0)
     {
-	(void) fprintf(glLogFile, "TOP\t%d\t%d,%d\t%d\t%d\n",
+	fprintf(glLogFile, "TOP\t%d\t%d,%d\t%d\t%d\n",
 		glNumTries,
 		inPt->gl_point.p_x, inPt->gl_point.p_y,
 		inPt->gl_length, cost);
