@@ -24,7 +24,6 @@ static char sccsid[] = "@(#)groutePoint.c	4.9 MAGIC (Berkeley) 12/6/85";
 #include "misc/magic.h"
 #include "utils/geometry.h"
 #include "utils/hash.h"
-#include "utils/doubleint.h"
 #include "utils/heap.h"
 #include "tiles/tile.h"
 #include "debug/debug.h"
@@ -77,7 +76,7 @@ glListAdd(list, pin, cost)
 {
     GlPoint *newPt;
 
-    MALLOC(GlPoint *, newPt, sizeof (GlPoint));
+    newPt = (GlPoint *) mallocMagic((unsigned) (sizeof (GlPoint)));
     newPt->gl_tile = (Tile *) NULL;
     newPt->gl_pin = pin;
     newPt->gl_cost = cost;
@@ -172,7 +171,7 @@ glPathCopyPerm(list)
 
     for (first = prev = NULL; list; prev = new, list = list->gl_path)
     {
-	MALLOC(GlPoint *, new, sizeof (GlPoint));
+	new = (GlPoint *) mallocMagic((unsigned) (sizeof (GlPoint)));
 	*new = *list;
 	if (first == NULL) first = new;
 	if (prev) prev->gl_path = new;
@@ -190,7 +189,7 @@ glPathFreePerm(list)
     GlPoint *p;
 
     for (p = list; p; p = p->gl_path)
-	FREE((char *) p);
+	freeMagic((char *) p);
 }
 
 /*
@@ -228,7 +227,7 @@ glPathNew(pin, cost, prev)
 	/* If out of pages, allocate a new one */
 	if (glPathCurPage == NULL)
 	{
-	    MALLOC(GlPage *, glPathCurPage, sizeof (GlPage));
+	    glPathCurPage = (GlPage *) mallocMagic((unsigned) (sizeof (GlPage)));
 	    glPathCurPage->glp_next = (GlPage *) NULL;
 	    glPathCurPage->glp_free = 0;
 	    if (glPathLastPage == NULL)
