@@ -25,7 +25,6 @@ static char rcsid[] = "$Header: /ufs/repository/magic/grouter/groutePen.c,v 1.2
 #include "misc/magic.h"
 #include "utils/geometry.h"
 #include "utils/hash.h"
-#include "utils/doubleint.h"
 #include "utils/heap.h"
 #include "tiles/tile.h"
 #include "database/database.h"
@@ -84,7 +83,7 @@ glPenSetPerChan(net)
 	    czNet = czNet->cz_next)
     {
 	gc = (GlobChan *) czNet->cz_chan->gcr_client;
-	MALLOC(CZone *, czChan, sizeof (CZone));
+	czChan = (CZone *) mallocMagic((unsigned) (sizeof (CZone)));
 	*czChan = *czNet;
 	czChan->cz_next = gc->gc_penList;
 	gc->gc_penList = czChan;
@@ -104,7 +103,7 @@ glPenClearPerChan(net)
     {
 	gc = (GlobChan *) czNet->cz_chan->gcr_client;
 	for (czChan = gc->gc_penList; czChan; czChan = czChan->cz_next)
-	    FREE((char *) czChan);
+	    freeMagic((char *) czChan);
 	gc->gc_penList = (CZone *) NULL;
     }
 }
@@ -335,7 +334,7 @@ glPenScanDens(czList, ch, dm, type)
 	    if (val[i] > dm->dm_cap)
 	    {
 		/* Beginning of congested zone */
-		MALLOC(CZone *, cz, sizeof (CZone));
+		cz = (CZone *) mallocMagic((unsigned) (sizeof (CZone)));
 		cz->cz_chan = ch;
 		cz->cz_type = type;
 		cz->cz_lo = i;
@@ -419,7 +418,7 @@ glPenAssignCosts(cz, netList)
      * ns_cost, leaving crossArray[i] pointing to the i'th smallest
      * NetSet.
      */
-    MALLOC(NetSet **, crossArray, numCross * sizeof (NetSet *));
+    crossArray = (NetSet **) mallocMagic((unsigned) (numCross * sizeof (NetSet *)));
     for (ns = crossNets, nsap = crossArray; ns; ns = ns->ns_next) *nsap++ = ns;
     qsort(crossArray, numCross, sizeof (NetSet *), glPenSortNetSet);
 
@@ -444,7 +443,7 @@ glPenAssignCosts(cz, netList)
     {
 	ns = *nsap++;
 	nc = (NetClient *) ns->ns_net->nnet_cdata;
-	MALLOC(CZone *, czNew, sizeof (CZone));
+	czNew = (CZone *) mallocMagic((unsigned) (sizeof (CZone)));
 	*czNew = *cz;
 	czNew->cz_penalty = maxCost;
 	czNew->cz_nets = (NetSet *) NULL;
@@ -457,8 +456,8 @@ glPenAssignCosts(cz, netList)
 
     /* Cleanup */
     for (ns = crossNets; ns; ns = ns->ns_next)
-	FREE((char *) ns);
-    FREE((char *) crossArray);
+	freeMagic((char *) ns);
+    freeMagic((char *) crossArray);
 }
 
 /*
@@ -568,7 +567,7 @@ glPenFindCrossingFunc(cz, srcPin, dstPin, rcc)
 {
     NetSet *ns;
 
-    MALLOC(NetSet *, ns, sizeof (NetSet));
+    ns = (NetSet *) mallocMagic((unsigned) (sizeof (NetSet)));
     ns->ns_net = rcc->rcc_net;
     ns->ns_cost = 0;
     ns->ns_next = rcc->rcc_set;
