@@ -27,7 +27,6 @@ static char rcsid[] = "$Header: /ufs/repository/magic/grouter/grouteTest.c,v 1.2
 #include "misc/magic.h"
 #include "utils/geometry.h"
 #include "utils/hash.h"
-#include "utils/doubleint.h"
 #include "utils/heap.h"
 #include "tiles/tile.h"
 #include "database/database.h"
@@ -407,7 +406,7 @@ glHistoAdd(heapPtsBefore, frontierPtsBefore, startPtsBefore)
 {
     GlNetHisto *gh;
 
-    MALLOC(GlNetHisto *, gh, sizeof (GlNetHisto));
+    gh = (GlNetHisto *) mallocMagic((unsigned) (sizeof (GlNetHisto)));
     gh->glh_heap = glCrossingsExpanded - heapPtsBefore;
     gh->glh_frontier = glCrossingsAdded - frontierPtsBefore;
     gh->glh_start = startPtsBefore;
@@ -450,16 +449,16 @@ glHistoDump()
     }
 
     /* Raw data */
-    (void) fprintf(fp, "--------- raw data ---------\n");
-    (void) fprintf(fp, "%9s %9s %9s\n", "HEAP", "FRONTIER", "START");
+    fprintf(fp, "--------- raw data ---------\n");
+    fprintf(fp, "%9s %9s %9s\n", "HEAP", "FRONTIER", "START");
     for (gh = glNetHistoList; gh; gh = gh->glh_next)
     {
-	(void) fprintf(fp, "%9d %9d %9d\n",
+	fprintf(fp, "%9d %9d %9d\n",
 		gh->glh_heap, gh->glh_frontier, gh->glh_start);
     }
 
     /* Output sorted by number of heap points */
-    (void) fprintf(fp, "--------- by heap points ---------\n");
+    fprintf(fp, "--------- by heap points ---------\n");
     HeapInit(&histoHeap, 128, FALSE, FALSE);
     for (gh = glNetHistoList; gh; gh = gh->glh_next)
 	HeapAddInt(&histoHeap, gh->glh_heap, (char *) gh);
@@ -470,7 +469,7 @@ glHistoDump()
 	if (gh->glh_heap != lastsize)
 	{
 	    if (count > 0)
-		(void) fprintf(fp, "%d: %d\n", lastsize, count);
+		fprintf(fp, "%d: %d\n", lastsize, count);
 	    lastsize = gh->glh_heap;
 	    count = 0;
 	}
@@ -478,11 +477,11 @@ glHistoDump()
     }
     HeapKill(&histoHeap, (void (*)()) NULL);
     if (count > 0)
-	(void) fprintf(fp, "%d: %d\n", lastsize, count);
-    (void) fprintf(fp, "TOTAL: %d\n", total);
+	fprintf(fp, "%d: %d\n", lastsize, count);
+    fprintf(fp, "TOTAL: %d\n", total);
 
     /* Output sorted by number of frontier points */
-    (void) fprintf(fp, "--------- by frontier points ---------\n");
+    fprintf(fp, "--------- by frontier points ---------\n");
     HeapInit(&histoHeap, 128, FALSE, FALSE);
     for (gh = glNetHistoList; gh; gh = gh->glh_next)
 	HeapAddInt(&histoHeap, gh->glh_frontier, (char *) gh);
@@ -493,7 +492,7 @@ glHistoDump()
 	if (gh->glh_frontier != lastsize)
 	{
 	    if (count > 0)
-		(void) fprintf(fp, "%d: %d\n", lastsize, count);
+		fprintf(fp, "%d: %d\n", lastsize, count);
 	    lastsize = gh->glh_frontier;
 	    count = 0;
 	}
@@ -501,12 +500,12 @@ glHistoDump()
     }
     HeapKill(&histoHeap, (void (*)()) NULL);
     if (count > 0)
-	(void) fprintf(fp, "%d: %d\n", lastsize, count);
-    (void) fprintf(fp, "TOTAL: %d\n", total);
+	fprintf(fp, "%d: %d\n", lastsize, count);
+    fprintf(fp, "TOTAL: %d\n", total);
 
     /* Free memory */
     for (gh = glNetHistoList; gh; gh = gh->glh_next)
-	FREE((char *) gh);
+	freeMagic((char *) gh);
     glNetHistoList = NULL;
 
     /* Done */
