@@ -48,8 +48,7 @@ static char rcsid[] = "$Header: /ufs/repository/magic/irouter/irCommand.c,v 1.2
 #include "misc/styles.h"
 #include "utils/malloc.h"
 #include "utils/list.h"
-#include "utils/doubleint.h"
-#include "../mzrouter/mzrouter.h"
+#include "mzrouter/mzrouter.h"
 #include "irouter/irouter.h"
 #include "irouter/irInternal.h"
 
@@ -429,7 +428,7 @@ irWzdSetPenalty(s, file)
     /* print the current penalty factor. */
     if(file)
     {
-	(void) fprintf(file,"%f",
+	fprintf(file,"%f",
 		       (double)irMazeParms->mp_penalty.rf_mantissa /
 		       (double)(1<<irMazeParms->mp_penalty.rf_nExponent));
     }
@@ -2051,13 +2050,13 @@ irSaveParametersCmd(w, cmd)
     {
 	int n;
 
-	(void) fprintf(saveFile,":iroute contact %s * ",
+	fprintf(saveFile,":iroute contact %s * ",
 		       DBTypeLongNameTbl[rC->rc_routeType.rt_tileType]);
 	for(n=0; cParms[n].cP_name; n++)
 	{
 	    (*cParms[n].cP_proc)(rC,NULL,saveFile);
 	}
-	(void) fprintf(saveFile,"\n");
+	fprintf(saveFile,"\n");
     }
 
     /* save LAYER paramters */
@@ -2065,13 +2064,13 @@ irSaveParametersCmd(w, cmd)
     {
 	int n;
 
-	(void) fprintf(saveFile,":iroute layer %s * ", 
+	fprintf(saveFile,":iroute layer %s * ", 
 	    DBTypeLongNameTbl[rL->rl_routeType.rt_tileType]);
 	for(n=0; lParms[n].lP_name; n++)
 	{
 	    (*lParms[n].lP_proc)(rL,NULL,saveFile);
 	}
-	(void) fprintf(saveFile,"\n");
+	fprintf(saveFile,"\n");
     }
 
     /* save SEARCH parameters */
@@ -2080,10 +2079,10 @@ irSaveParametersCmd(w, cmd)
 
 	for(n=0; srParms[n].srP_name; n++)
 	{
-	    (void) fprintf(saveFile,":iroute search %s ", 
+	    fprintf(saveFile,":iroute search %s ", 
 			   srParms[n].srP_name); 
 	    (*srParms[n].srP_proc)(NULL,saveFile);
-	    (void) fprintf(saveFile,"\n");
+	    fprintf(saveFile,"\n");
 	}
     }
 
@@ -2092,17 +2091,17 @@ irSaveParametersCmd(w, cmd)
 	RouteType *rT;
 	int i;
 
-	(void) fprintf(saveFile,":iroute spacings CLEAR\n");
+	fprintf(saveFile,":iroute spacings CLEAR\n");
 	for (rT=irRouteTypes; rT!= NULL; rT=rT->rt_next)
 	{
 	    for (i=0;i<TT_MAXTYPES;i++)
 		if(rT->rt_spacing[i]>=0)
-		    (void) fprintf(saveFile,":iroute spacings %s %s %d\n", 
+		    fprintf(saveFile,":iroute spacings %s %s %d\n", 
 				   DBTypeLongNameTbl[rT->rt_tileType],
 				   DBTypeLongNameTbl[i],
 				   rT->rt_spacing[i]);
 	    if(rT->rt_spacing[TT_SUBCELL]>=0)
-		(void) fprintf(saveFile,":iroute spacings %s %s %d\n", 
+		fprintf(saveFile,":iroute spacings %s %s %d\n", 
 			       DBTypeLongNameTbl[rT->rt_tileType],
 			       "SUBCELL",
 			       rT->rt_spacing[TT_SUBCELL]);
@@ -2115,10 +2114,10 @@ irSaveParametersCmd(w, cmd)
 
 	for(n=0; wzdParms[n].wzdP_name; n++)
 	{
-	    (void) fprintf(saveFile,":iroute wizard %s ", 
+	    fprintf(saveFile,":iroute wizard %s ", 
 			   wzdParms[n].wzdP_name); 
 	    (*wzdParms[n].wzdP_proc)(NULL,saveFile);
-	    (void) fprintf(saveFile,"\n");
+	    fprintf(saveFile,"\n");
 	}
     }
 
