@@ -14,17 +14,19 @@ static char rcsid[] = "$Header: /ufs/repository/magic/lef/defRead.c,v 1.0 2003/0
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <ctype.h>
 #ifdef SYSV
 #include <string.h>
 #endif
 #include <errno.h>
 #include <stdarg.h>
 
-#include "magic/tclmagic.h"
+#include "tcltk/tclmagic.h"
 #include "misc/magic.h"
 #include "utils/geometry.h"
 #include "tiles/tile.h"
 #include "utils/hash.h"
+#include "undo/undo.h"
 #include "database/database.h"
 #include "windows/windows.h"
 #include "dbwind/dbwind.h"
@@ -53,11 +55,12 @@ static char rcsid[] = "$Header: /ufs/repository/magic/lef/defRead.c,v 1.0 2003/0
  */
 
 char *
-DefAddRoutes(rootDef, f, oscale, special)
+DefAddRoutes(rootDef, f, oscale, special, defLayerMap)
     CellDef *rootDef;		/* Cell to paint */
     FILE *f;			/* Input file */
     float oscale;		/* Scale factor between LEF and magic units */
     bool special;		/* True if this section is SPECIALNETS */
+    LefMapping *defLayerMap;	/* magic-to-lef layer mapping array */
 {
     char *token;
     linkedRect *routeList, *newRoute = NULL, *routeTop = NULL;
@@ -66,8 +69,8 @@ DefAddRoutes(rootDef, f, oscale, special)
     bool initial = TRUE;
     Rect locarea;
     float x, y, w;
-    int curWidth, w1, w2;
-    TileType newLayer;
+    int routeWidth, paintWidth, saveWidth;
+    TileType routeLayer, paintLayer;
     HashEntry *he;
     lefLayer *lefl;
 
@@ -88,7 +91,7 @@ DefAddRoutes(rootDef, f, oscale, special)
 	    if (he != NULL)
 	    {
 		lefl = (lefLayer *)HashGetValue(he);
-		newLayer = lefl->type;
+		routeLayer = lefl->type;
 	    }
 	    else
 	    {
@@ -97,15 +100,16 @@ DefAddRoutes(rootDef, f, oscale, special)
 		/* of type mappings in the lef section of the techfile is */
 		/* preferred.						  */
 
-		newLayer = DBTechNameType(LefLower(token));
+		routeLayer = DBTechNameType(LefLower(token));
 		lefl = NULL;
 	    }
 
-	    if (newLayer < 0)
+	    if (routeLayer < 0)
 	    {
 		LefError("Unknown layer type \"%s\" for NEW route\n", token); 
 		continue;
 	    }
+	    paintLayer = routeLayer;
 
 	    if (special)
 	    {
@@ -117,12 +121,13 @@ DefAddRoutes(rootDef, f, oscale, special)
 		    continue;
 		}
 		if (w != 0)
-		    curWidth = (int)(w / oscale);
+		    paintWidth = (int)(w / oscale);
 		else
-		    curWidth = (lefl) ? lefl->info.route.width : DEFAULT_WIDTH;
+		    paintWidth = (lefl) ? lefl->info.route.width : DEFAULT_WIDTH;
+		saveWidth = paintWidth;
 	    }
 	    else
-		curWidth = (lefl) ? lefl->info.route.width : DEFAULT_WIDTH;
+		paintWidth = (lefl) ? lefl->info.route.width : DEFAULT_WIDTH;
 	}
 	else if (*token != '(')	/* via name */
 	{
@@ -145,36 +150,110 @@ DefAddRoutes(rootDef, f, oscale, special)
 
 		if (lefl != NULL)
 		{
-		    newLayer = lefl->type;
+		    linkedRect *viaRoute, *addRoute;
+
+		    /* If there is a linkedRect structure for the via,	*/
+		    /* add those records to the route first.		*/
+
+		    for (viaRoute = lefl->info.via.lr; viaRoute != NULL;
+				viaRoute = viaRoute->rect_next)
+		    {
+			addRoute = (linkedRect *)mallocMagic(sizeof(linkedRect));
+			addRoute->rect_next = NULL;
+			addRoute->type = viaRoute->type;
+			addRoute->area = viaRoute->area;
+
+			addRoute->area.r_xbot += refp.p_x;
+			addRoute->area.r_ybot += refp.p_y;
+			addRoute->area.r_xtop += refp.p_x;
+			addRoute->area.r_ytop += refp.p_y;
+
+			addRoute->area.r_xbot >>= 1;
+			addRoute->area.r_ybot >>= 1;
+			addRoute->area.r_xtop >>= 1;
+			addRoute->area.r_ytop >>= 1;
+	
+			if (routeTop)
+			    routeList->rect_next = addRoute;
+			else
+			    routeTop = addRoute;
+
+			routeList = addRoute;
+		    }
+			
+		    paintLayer = lefl->type;
 
 		    newRoute->area.r_xbot = refp.p_x + lefl->info.via.area.r_xbot;
 		    newRoute->area.r_ybot = refp.p_y + lefl->info.via.area.r_ybot;
 		    newRoute->area.r_xtop = refp.p_x + lefl->info.via.area.r_xtop;
 		    newRoute->area.r_ytop = refp.p_y + lefl->info.via.area.r_ytop;
+
+		    newRoute->area.r_xbot >>= 1;
+		    newRoute->area.r_ybot >>= 1;
+		    newRoute->area.r_xtop >>= 1;
+		    newRoute->area.r_ytop >>= 1;
+
 		}
-		else if ((newLayer = DBTechNameType(LefLower(token))) >= 0)
+		else if ((paintLayer = DBTechNameType(LefLower(token))) >= 0)
 		{
 		    LefError("Error: Via \"%s\" named but undefined.\n", token);
-		    w1 = curWidth >> 1;
-		    w2 = curWidth - w1;
-		    newRoute->area.r_xbot = refp.p_x - w1;
-		    newRoute->area.r_ybot = refp.p_y - w1;
-		    newRoute->area.r_xtop = refp.p_x + w2;
-		    newRoute->area.r_ytop = refp.p_y + w2;
+		    newRoute->area.r_xbot = refp.p_x - paintWidth;
+		    newRoute->area.r_ybot = refp.p_y - paintWidth;
+		    newRoute->area.r_xtop = refp.p_x + paintWidth;
+		    newRoute->area.r_ytop = refp.p_y + paintWidth;
+
+		    newRoute->area.r_xbot >>= 1;
+		    newRoute->area.r_ybot >>= 1;
+		    newRoute->area.r_xtop >>= 1;
+		    newRoute->area.r_ytop >>= 1;
 		}
 		else
 		   LefError("Via name \"%s\" unknown in route.\n", token);
+
+		/* After the via, the new route layer becomes whatever	*/
+		/* residue of the via was NOT the previous route layer.	*/
+		/* This is absolutely impossible to make consistent	*/
+		/* with the DEF	spec, but there you have it. . .	*/
+
+		if (DBIsContact(paintLayer))
+		{
+		    TileTypeBitMask *rMask = DBResidueMask(paintLayer);
+		    TileType stype;
+
+		    for (stype = TT_TECHDEPBASE; stype < DBNumUserLayers; stype++)
+			if (TTMaskHasType(rMask, stype))
+			    if (stype != routeLayer)
+			    {
+				/* Diagnostic */
+				/*
+				TxPrintf("Contact %s: In=%s Out=%s\n",
+					DBTypeLongNameTbl[paintLayer],
+					DBTypeLongNameTbl[routeLayer],
+					DBTypeLongNameTbl[stype]);
+				*/
+				routeLayer = stype;
+				lefl = defLayerMap[routeLayer].lefInfo;
+				if (special)
+				    paintWidth = saveWidth;
+				else
+				    paintWidth = (lefl) ? lefl->info.route.width
+					: DEFAULT_WIDTH;
+				break;
+			    }
+		}
 	    }
 	    else
 		LefError("Via name \"%s\" unknown in route.\n", token);
 	}
 	else
 	{
+	    /* Revert to the routing layer type, in case we painted a via */
+	    paintLayer = routeLayer;
+
 	    /* Record current reference point */
 	    locarea.r_xbot = refp.p_x;
 	    locarea.r_ybot = refp.p_y;
 
-
 	    /* Read an (X Y) point */
 	    token = LefNextToken(f, TRUE);	/* read X */
 	    if (*token == '*')
@@ -187,7 +266,7 @@ DefAddRoutes(rootDef, f, oscale, special)
 	    }
 	    else if (sscanf(token, "%f", &x) == 1)
 	    {
-		refp.p_x = (int)(x / oscale);
+		refp.p_x = (int)((2 * x) / oscale);
 	    }
 	    else
 	    {
@@ -207,7 +286,7 @@ DefAddRoutes(rootDef, f, oscale, special)
 	    }
 	    else if (sscanf(token, "%f", &y) == 1)
 	    {
-		refp.p_y = (int)(y / oscale);
+		refp.p_y = (int)((2 * y) / oscale);
 	    }
 	    else
 	    {
@@ -242,21 +321,26 @@ DefAddRoutes(rootDef, f, oscale, special)
 		newRoute = (linkedRect *)mallocMagic(sizeof(linkedRect));
 
 		/* Route coordinates become the centerline of the	*/
-		/* segment (to the nearest integer; may be 1/2 off. . .	*/
-		/* roundoff error should be fixed!			*/
-
-		w1 = curWidth >> 1;
-		w2 = curWidth - w1;
+		/* segment.  "refp" is kept in 1/2 lambda units so	*/
+		/* we should always end up with integer units.		*/
 
 		locarea.r_xtop = refp.p_x;
 		locarea.r_ytop = refp.p_y;
 
 		GeoCanonicalRect(&locarea, &newRoute->area);
 
-		newRoute->area.r_xbot -= w1;
-		newRoute->area.r_ybot -= w1;
-		newRoute->area.r_xtop += w2;
-		newRoute->area.r_ytop += w2;
+		newRoute->area.r_xbot -= paintWidth;
+		newRoute->area.r_ybot -= paintWidth;
+		newRoute->area.r_xtop += paintWidth;
+		newRoute->area.r_ytop += paintWidth;
+
+		/* If we don't have integer units here, we should	*/
+		/* rescale the magic internal grid.			*/
+
+		newRoute->area.r_xbot >>= 1;
+		newRoute->area.r_ybot >>= 1;
+		newRoute->area.r_xtop >>= 1;
+		newRoute->area.r_ytop >>= 1;
 	    }
 
 endCoord:
@@ -268,7 +352,7 @@ endCoord:
 	/* Link in the new route segment */
 	if (newRoute)
 	{
-	    newRoute->type = newLayer;
+	    newRoute->type = paintLayer;
 	    newRoute->rect_next = NULL;
 
 	    if (routeTop)
@@ -330,6 +414,7 @@ DefReadNets(f, rootDef, sname, oscale, special, total)
     char *token;
     int keyword, subkey;
     int processed = 0;
+    LefMapping *defLayerMap;
 
     static char *net_keys[] = {
 	"-",
@@ -348,6 +433,8 @@ DefReadNets(f, rootDef, sname, oscale, special, total)
 	NULL
     };
 
+    defLayerMap = defMakeInverseLayerMap(); 
+
     while ((token = LefNextToken(f, TRUE)) != NULL)
     {
 	keyword = Lookup(token, net_keys);
@@ -402,7 +489,8 @@ DefReadNets(f, rootDef, sname, oscale, special, total)
 			case DEF_NETPROP_ROUTED:
 			case DEF_NETPROP_FIXED:
 			case DEF_NETPROP_COVER:
-			    token = DefAddRoutes(rootDef, f, oscale, special);
+			    token = DefAddRoutes(rootDef, f, oscale, special,
+					defLayerMap);
 			    break;
 		    }
 		}
@@ -425,6 +513,8 @@ DefReadNets(f, rootDef, sname, oscale, special, total)
     else
 	LefError("Warning:  Number of nets read (%d) does not match "
 		"the number declared (%d).\n", processed, total);
+
+    freeMagic((char *)defLayerMap);
 }
 
 /*
@@ -627,6 +717,7 @@ DefReadVias(f, sname, oscale, total)
 		    lefl->is_via = TRUE;
 		    lefl->info.via.area = GeoNullRect;
 		    lefl->info.via.cell = (CellDef *)NULL;
+		    lefl->info.via.lr = (linkedRect *)NULL;
 		    HashSetValue(he, lefl);
 		}
 
@@ -650,7 +741,11 @@ DefReadVias(f, sname, oscale, total)
 		    {
 			case DEF_VIAS_PROP_RECT:
 			    curlayer = LefReadLayer(f, FALSE);
-			    currect = LefReadRect(f, curlayer, oscale);
+
+			    /* Divide oscale by 2 so that the rectangle	*/
+			    /* read is in units of 1/2 lambda.		*/
+
+			    currect = LefReadRect(f, curlayer, (oscale / 2));
 			    if (currect)
 			    {
 				GeoInclude(currect, &viaArea);
@@ -785,7 +880,7 @@ DefReadComponents(f, rootDef, sname, oscale, total)
 
 		/* Create a use for this celldef in the edit cell */
 		/* Don't process properties for cells we could not find */
-		if ((defUse = DBCellNewUse(defMacro, token)) == NULL)
+		if ((defUse = DBCellNewUse(defMacro, usename)) == NULL)
 		{
 		    LefEndStatement(f);
 		    break;
@@ -840,7 +935,7 @@ DefReadComponents(f, rootDef, sname, oscale, total)
 		}
 
 		/* Finish final call by placing the cell use */
-		if (defUse != NULL)
+		if ((total > 0) && (defUse != NULL))
 		    DBPlaceCell(defUse, rootDef);
 
 		break;
@@ -925,6 +1020,10 @@ DefRead(inName)
 	NULL
     };
 
+    /* Make sure we have a valid LefInfo hash table, even if it's empty */
+    if (LefInfo.ht_table == (HashEntry **) NULL)
+	LefTechInit();
+
     f = lefFileOpen(NULL, inName, ".def", "r", &filename);
 
     if (f == NULL)
@@ -976,6 +1075,8 @@ DefRead(inName)
 		LefSkipSection(f, sections[DEF_REGIONS]);
 		break;
 	    case DEF_DESIGN:
+		token = LefNextToken(f, TRUE);
+		DBCellRenameDef(rootDef, token);
 		LefEndStatement(f);
 		break;
 	    case DEF_UNITS:
