@@ -6,7 +6,10 @@
  *
  */
 
-#define _LEF
+#ifndef _LEF_H
+#define _LEF_H
+
+#include "misc/magic.h"
 
 /* Procedures for reading the technology file: */
 
@@ -17,3 +20,5 @@ extern void LefTechScale();
 /* Initialization: */
 
 extern void LefInit();
+
+#endif /* _LEF_H */
