@@ -7,7 +7,10 @@
  * rcsid $Header: /ufs/repository/magic/lef/lefInt.h,v 1.1.1.1 2000/03/29 18:35:23 rajit Exp $
  */
 
-#define _LEF
+#ifndef _LEFINT_H
+#define _LEFINT_H
+
+#include "misc/magic.h"
 
 /* Some constants for LEF and DEF files */
 
@@ -18,6 +21,22 @@
 #define DEFAULT_WIDTH 3	   /* Default metal width for routes if undefined */
 #define DEFAULT_SPACING 4  /* Default spacing between metal if undefined  */
 
+/* Structure holding the counts of regular and special nets */
+
+typedef struct
+{
+    int regular;
+    int special;
+} NetCount;
+
+/* Struture used for returning information from a route or geometry section */
+
+typedef struct _linkedRect {
+   Rect      area;
+   TileType  type;
+   struct _linkedRect *rect_next;
+} linkedRect;
+
 /* Structure used to maintain default routing information for each	*/
 /* routable layer type.							*/
 
@@ -35,8 +54,11 @@ typedef struct {
 /* with a minimum area "area" for a single contact.			*/
 
 typedef struct {
-    Rect	area;		/* area of single contact, or cell bbox	*/
-    CellDef	*cell;		/* cell for fixed via def, or NULL	*/
+    Rect	area;		/* Area of single contact, or cell bbox	*/
+				/* in units of 1/2 lambda		*/
+    CellDef	*cell;		/* Cell for fixed via def, or NULL	*/
+    linkedRect	*lr;		/* Extra information for vias with	*/
+				/* more complicated geometry.		*/
 } lefVia;
 
 /* Structure defining a route or via layer and matching it to a magic	*/
@@ -56,13 +78,13 @@ typedef struct {
     } info;
 } lefLayer;
 
-/* Struture used for returning information from a route or geometry section */
+/* Inverse mapping structure for returning the primary LEF	*/
+/* layer corresponding to a magic tile type.			*/
 
-typedef struct _linkedRect {
-   Rect      area;
-   TileType  type;
-   struct _linkedRect *rect_next;
-} linkedRect;
+typedef struct {
+    char *lefName;              /* Primary name of LEF layer */
+    lefLayer *lefInfo;          /* Pointer to information about the layer */
+} LefMapping;
 
 /* External declaration of global variables */
 extern int lefCurrentLine;
@@ -85,5 +107,9 @@ void LefEstimate();
 Rect *LefReadRect();
 TileType LefReadLayer();
 
+LefMapping *defMakeInverseLayerMap();
+
 void LefError(char *, ...);	/* Variable argument procedure requires */
 				/* parameter list.			*/
+
+#endif /* _LEFINT_H */
