@@ -19,6 +19,7 @@ static char rcsid[] = "$Header: /ufs/repository/magic/lef/lefRead.c,v 1.0 2003/0
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <ctype.h>
 #ifdef SYSV
 #include <string.h>
 #endif
@@ -26,11 +27,12 @@ static char rcsid[] = "$Header: /ufs/repository/magic/lef/lefRead.c,v 1.0 2003/0
 #include <stdarg.h>
 #include <sys/time.h>
 
-#include "magic/tclmagic.h"
+#include "tcltk/tclmagic.h"
 #include "misc/magic.h"
 #include "utils/geometry.h"
 #include "tiles/tile.h"
 #include "utils/hash.h"
+#include "undo/undo.h"
 #include "database/database.h"
 #include "windows/windows.h"
 #include "graphics/graphics.h"
@@ -294,7 +296,7 @@ LefError(char *fmt, ...)
     {
 	TxError("LEF Read, Line %d: ", lefCurrentLine);
 	va_start(args, fmt);
-	(void) Vfprintf(stderr, fmt, args);
+	Vfprintf(stderr, fmt, args);
 	va_end(args);
 	TxFlushErr();
     }
@@ -348,7 +350,7 @@ LefParseEndStatement(f, match)
 
     /* END <section_name> */
     else {
-	keyword = Lookup(token, match_name);
+	keyword = LookupFull(token, match_name);
 	if (keyword == 0)
 	    return TRUE;
 	else
@@ -959,10 +961,13 @@ enum lef_macro_keys {LEF_CLASS = 0, LEF_SIZE, LEF_ORIGIN,
 	LEF_TIMING, LEF_FOREIGN, LEF_MACRO_END};
 
 void
-LefReadMacro(f, mname, oscale)
+LefReadMacro(f, mname, oscale, importForeign)
     FILE *f;			/* LEF file being read	*/
     char *mname;		/* name of the macro 	*/
     float oscale;		/* scale factor um->magic units */
+    bool importForeign;		/* Whether we should try to read 
+				 * in a cell.
+				 */
 {
     CellDef *lefMacro;
     HashEntry *he;
@@ -970,7 +975,7 @@ LefReadMacro(f, mname, oscale)
     char *token, tsave[128];
     int keyword, pinNum;
     float x, y;
-    bool has_size;
+    bool has_size, is_imported = FALSE;
     Rect lefBBox;
 
     static char *macro_keys[] = {
@@ -1096,19 +1101,43 @@ origin_error:
 		TxPrintf("   Macro defines pin %s\n", token);
 		*/
 		sprintf(tsave, "%.127s", token);
-		LefReadPin(lefMacro, f, tsave, pinNum++, oscale);
+		if (is_imported)
+		    LefSkipSection(f, tsave);
+		else
+		    LefReadPin(lefMacro, f, tsave, pinNum++, oscale);
 		break;
 	    case LEF_OBS:
 		/* Diagnostic */
 		/*
 		TxPrintf("   Macro defines obstruction\n");
 		*/
-		LefReadGeometry(lefMacro, f, oscale, FALSE);
+		if (is_imported)
+		    LefSkipSection(f, NULL);
+		else
+		    LefReadGeometry(lefMacro, f, oscale, FALSE);
 		break;
 	    case LEF_TIMING:
 		LefSkipSection(f, macro_keys[LEF_TIMING]);
 		break;
 	    case LEF_FOREIGN:
+		if (importForeign)
+		{
+		    token = LefNextToken(f, TRUE);
+		    sprintf(tsave, "%.127s", token);
+
+		    /* Attempt to read a .mag file of this name */
+
+		    DBCellClearAvail(lefMacro);
+		    if (DBCellRead(lefMacro, tsave, TRUE))
+			is_imported = TRUE;
+		    else
+		    {
+			TxError("   Cannot find magic file %s.mag "
+				"to import.\n", tsave);
+			TxError("   Filling cell with geometry from "
+				"the LEF file.\n");
+		    }
+		}
 		LefEndStatement(f);
 		break;
 	    case LEF_MACRO_END:
@@ -1124,23 +1153,27 @@ origin_error:
 
     /* Finish up creating the cell */
 
-    DBAdjustLabelsNew(lefMacro, &TiPlaneRect, 1);
-
-    if (!has_size)
-    {
-	LefError("   Macro does not define size:  computing from geometry\n");
+    if (is_imported)
 	DBReComputeBbox(lefMacro);
-    }
     else
-	lefMacro->cd_bbox = lefBBox;
+    {
+	DBAdjustLabelsNew(lefMacro, &TiPlaneRect, 1);
 
-    /* Fix the bounding box and do not allow edits */
-    lefMacro->cd_flags |= /* CDNOEDIT | */ CDFIXEDBBOX;
+	if (!has_size)
+	{
+	    LefError("   Macro does not define size:  computing from geometry\n");
+	    DBReComputeBbox(lefMacro);
+	}
+	else
+	    lefMacro->cd_bbox = lefBBox;
 
-    DRCCheckThis(lefMacro, TT_CHECKPAINT, &lefMacro->cd_bbox);
+	/* Fix the bounding box and do not allow edits */
+	lefMacro->cd_flags |= /* CDNOEDIT | */ CDFIXEDBBOX;
+
+	DRCCheckThis(lefMacro, TT_CHECKPAINT, &lefMacro->cd_bbox);
+    }
     DBWAreaChanged(lefMacro, &lefMacro->cd_bbox, DBW_ALLWINDOWS,
 		&DBAllButSpaceBits);
-    /* DBCellSetModified(lefMacro, TRUE); */
 }
 
 /*
@@ -1260,11 +1293,30 @@ LefReadLayerSection(f, lname, mode, lefl)
 		LefEndStatement(f);
 		break;
 	    case LEF_VIA_RECT:
-		currect = LefReadRect(f, curlayer, oscale);
-		GeoInclude(currect, &viaArea);
-		if ((lefl->type < 0) || DBIsContact(curlayer))
+		/* Rectangles for vias are read in units of 1/2 lambda */
+		currect = LefReadRect(f, curlayer, (oscale / 2));
+		if (GEO_SAMERECT(lefl->info.via.area, GeoNullRect))
+		{
+		    lefl->info.via.area = *currect;
 		    lefl->type = curlayer;
-		lefl->info.via.area = viaArea;
+		}
+		else 
+		{
+		    linkedRect *viaLR = (linkedRect *)mallocMagic(sizeof(linkedRect));
+		    viaLR->rect_next = lefl->info.via.lr;
+		    lefl->info.via.lr = viaLR;
+		    viaLR->type = curlayer;
+		    viaLR->area = *currect;
+
+		    /* Make sure that the primary record is a contact type. */
+		    if (DBIsContact(curlayer) && !DBIsContact(lefl->type))
+		    {
+			viaLR->type = lefl->type;
+			lefl->type = curlayer;
+			viaLR->area = lefl->info.via.area;
+			lefl->info.via.area = *currect;
+		    }
+		}
 		LefEndStatement(f);
 		break;
 	    case LEF_VIARULE_VIA:
@@ -1308,8 +1360,9 @@ enum lef_sections {LEF_VERSION = 0, LEF_NAMESCASESENSITIVE,
 	LEF_END};
 
 void
-LefRead(inName)
+LefRead(inName, importForeign)
     char *inName;
+    bool importForeign;
 {
     FILE *f;
     char *filename;
@@ -1342,6 +1395,10 @@ LefRead(inName)
 	NULL
     };
 
+    /* Make sure we have a valid LefInfo hash table, even if it's empty */
+    if (LefInfo.ht_table == (HashEntry **) NULL)
+	LefTechInit();
+
     f = lefFileOpen(NULL, inName, ".lef", "r", &filename);
 
     if (f == NULL)
@@ -1404,9 +1461,16 @@ LefRead(inName)
 		    lefl->is_via = TRUE;
 		    lefl->info.via.area = GeoNullRect;
 		    lefl->info.via.cell = (CellDef *)NULL;
+		    lefl->info.via.lr = (linkedRect *)NULL;
 		    HashSetValue(he, lefl);
+		    LefReadLayerSection(f, tsave, keyword, lefl);
 		}
-		LefReadLayerSection(f, tsave, keyword, lefl);
+		else
+		    /* If we've already seen this via, don't reprocess. */
+		    /* This deals with VIA followed by VIARULE.  We	*/
+		    /* really ought to have special processing for the	*/
+		    /* VIARULE section. . .				*/
+		    LefSkipSection(f, tsave);
 		break;
 
 	    case LEF_SECTION_LAYER:
@@ -1502,7 +1566,7 @@ LefRead(inName)
 		TxPrintf("LEF file:  Defines new cell %s\n", token);
 		*/
 		sprintf(tsave, "%.127s", token);
-		LefReadMacro(f, tsave, oscale);
+		LefReadMacro(f, tsave, oscale, importForeign);
 		break;
 	    case LEF_END:
 		if (!LefParseEndStatement(f, "LIBRARY"))
