@@ -23,7 +23,7 @@ static char rcsid[] = "$Header: /ufs/repository/magic/lef/lefWrite.c,v 1.0 2003/
 #endif
 #include <errno.h>
 
-#include "magic/tclmagic.h"
+#include "tcltk/tclmagic.h"
 #include "misc/magic.h"
 #include "utils/geometry.h"
 #include "tiles/tile.h"
@@ -553,7 +553,8 @@ lefWriteMacro(def, f, scale)
 
     /* Restrict to routing planes only */
 
-    fprintf(f, "   OBS %s\n", def->cd_name);
+//  fprintf(f, "   OBS %s\n", def->cd_name);	/* Error Fixed; Tim 6/16/04 */
+    fprintf(f, "   OBS\n");
     for (pNum = PL_PAINTBASE; pNum < DBNumPlanes; pNum++)
 	(void) DBSrPaintArea((Tile *)NULL, def->cd_planes[pNum], 
 		&TiPlaneRect, &lc.rmask,
@@ -583,18 +584,19 @@ lefWriteMacro(def, f, scale)
  */
 
 void
-LefWriteAll(rootUse)
+LefWriteAll(rootUse, writeTopCell)
     CellUse *rootUse;
+    bool writeTopCell;
 {
-    CellDef *def;
+    CellDef *def, *rootdef;
     FILE *f;
     char *filename;
     float scale = CIFGetOutputScale(1000);	/* conversion to microns */
 
-    def = rootUse->cu_def;
+    rootdef = rootUse->cu_def;
 
     /* Make sure the entire subtree is read in */
-    DBCellReadArea(rootUse, &def->cd_bbox);
+    DBCellReadArea(rootUse, &rootdef->cd_bbox);
 
     /* Fix up bounding boxes if they've changed */
     DBFixMismatch();
@@ -608,10 +610,10 @@ LefWriteAll(rootUse)
 
     /* Open the file for output */
 
-    f = lefFileOpen(def, (char *)NULL, ".lef", "w", &filename);
+    f = lefFileOpen(rootdef, (char *)NULL, ".lef", "w", &filename);
 
     TxPrintf("Generating LEF output %s for hierarchy rooted at cell %s:\n",
-		filename, def->cd_name);
+		filename, rootdef->cd_name);
 
     if (f == NULL)
     {
@@ -627,13 +629,14 @@ LefWriteAll(rootUse)
 
     /* Now generate LEF output for all the cells we just found */
 
-    lefWriteHeader(def, f);
+    lefWriteHeader(rootdef, f);
 
     while (def = (CellDef *) StackPop(lefDefStack))
     {
 	def->cd_client = (ClientData) 0;
 	if (!SigInterruptPending)
-	    lefWriteMacro(def, f, scale);
+	    if ((writeTopCell == TRUE) || (def != rootdef))
+		lefWriteMacro(def, f, scale);
     }
 
     fclose(f);
