@@ -11,7 +11,7 @@
 #include <stdio.h>
 #include <string.h>
 
-#include "magic/tclmagic.h"
+#include "tcltk/tclmagic.h"
 #include "misc/magic.h"
 #include "utils/geometry.h"
 #include "tiles/tile.h"
