@@ -1515,7 +1515,7 @@ LispError (name,s,f)
     RETURN;
   }
   i = 0; j = 0;
-  MALLOC (char *, str, strlen(LSTR(ARG1(s)))+1);
+  str = (char *) mallocMagic((unsigned) (strlen(LSTR(ARG1(s)))+1));
   strcpy (str, LSTR(ARG1(s)));
   while (str[i]) {
     if (str[i] == '\\')
@@ -1525,7 +1525,7 @@ LispError (name,s,f)
   }
   str[j] = '\0';
   TxPrintf ("%s\n", str);
-  FREE (str);
+  freeMagic(str);
   RETURN;
 }
 
