@@ -361,7 +361,7 @@ lispprintlabel (scx, label, tpath, cdarg)
       bad++;
   if (bad) {
     int i;
-    MALLOC (char *, nm, strlen(label->lab_text)+1+bad);
+    nm = (char *) mallocMagic((unsigned) (strlen(label->lab_text)+1+bad));
     p = nm;
     for (i=0; label->lab_text[i]; i++) {
       if (label->lab_text[i] == '\"')
@@ -382,7 +382,7 @@ lispprintlabel (scx, label, tpath, cdarg)
 	   targetRect.r_xbot, targetRect.r_ybot,
 	   targetRect.r_xtop, targetRect.r_ytop);
   if (bad)
-    FREE (nm);
+    freeMagic(nm);
   l = LispParseString (buf);
   CDR(LLIST(l)) = _internal_list;
   _internal_list = l;
