@@ -73,7 +73,7 @@ bool imacro;		/* is this an interactive macro? */
     }
     if (cmacro == NULL)
     {
-	MALLOC(macrodef *, newmacro, sizeof(macrodef));
+	newmacro = (macrodef *) mallocMagic((unsigned) (sizeof(macrodef)));
 	newmacro->keyvalue = xc;
 	newmacro->nextmacro = mactable;
 	newmacro->macrodef = (char *)NULL;
@@ -145,8 +145,8 @@ int xc;			/* the extended name of the macro */
 		lmacro->nextmacro = cmacro->nextmacro;
 	    else
 		mactable = cmacro->nextmacro;
-	    FREE(cmacro->macrodef);
-	    FREE(cmacro);
+	    freeMagic(cmacro->macrodef);
+	    freeMagic(cmacro);
 	    break;
 	}
 	lmacro = cmacro;
