@@ -67,7 +67,6 @@ static char rcsid[]="$Header: /ufs/repository/magic/main/main.c,v 1.6 2001/08/14
 #include "plot/plot.h"
 #endif
 #include "sim/sim.h"
-#include "utils/doubleint.h"
 #include "utils/list.h"
 #ifdef ROUTE_MODULE
 #include "mzrouter/mzrouter.h"
@@ -418,9 +417,6 @@ mainInitBeforeArgs(argc, argv)
     TxInit();
     TxSetTerminal();
 
-    /* Initialize double-precision package. */
-    DoubleInit();
-
 #ifdef SCHEME_INTERPRETER
     /* Initialize Lisp module. (rajit@cs.caltch.edu) */
     LispInit();
@@ -490,6 +486,10 @@ mainInitAfterArgs()
     LefInit();
 #endif
 
+#ifdef OPENACCESS
+    OAInit();
+#endif
+
     /* TxPrintf("Using technology \"%s\".\n", TechDefault); */
 
     /*
