@@ -71,7 +71,6 @@ static char rcsid[] = "$Header: /ufs/repository/magic/mzrouter/mzBlock.c,v 1.2 2
 #include "utils/list.h"
 #include "debug/debug.h"
 #include "textio/textio.h"
-#include "utils/doubleint.h"
 #include "utils/heap.h"
 #include "mzrouter/mzrouter.h"
 #include "mzrouter/mzInternal.h"
@@ -971,7 +970,7 @@ mzHWalksFunc(tile, cdarg)
 
 	    if(TiGetType(tLeft)==TT_SAMENODE_BLOCK)
 	    {
-		MALLOC(Walk *, walk, sizeof(Walk));
+		walk = (Walk *) mallocMagic((unsigned) (sizeof(Walk)));
 		walk->w_rT = rT; 
 		walk->w_type = TT_LEFT_WALK;
 		walk->w_rect.r_ybot = MAX(BOTTOM(tile),BOTTOM(tLeft));
@@ -997,7 +996,7 @@ mzHWalksFunc(tile, cdarg)
 	{
 	   if(TiGetType(tRight)==TT_SAMENODE_BLOCK)
 	   {
-	       MALLOC(Walk *, walk, sizeof(Walk));
+	       walk = (Walk *) mallocMagic((unsigned) (sizeof(Walk)));
 	       walk->w_rT = rT; 
 	       walk->w_type = TT_RIGHT_WALK;
 	       walk->w_rect.r_ybot = MAX(BOTTOM(tile),BOTTOM(tRight));
@@ -1059,7 +1058,7 @@ mzVWalksFunc(tile, cdarg)
 	{
 	    if(TiGetType(tBelow)==TT_SAMENODE_BLOCK)
 	    {
-		MALLOC(Walk *, walk, sizeof(Walk));
+		walk = (Walk *) mallocMagic((unsigned) (sizeof(Walk)));
 		walk->w_rT = rT;
 		walk->w_type = TT_BOTTOM_WALK;
 		walk->w_rect.r_xbot = MAX(LEFT(tile),LEFT(tBelow));
@@ -1086,7 +1085,7 @@ mzVWalksFunc(tile, cdarg)
 	{
 	    if(TiGetType(tAbove)==TT_SAMENODE_BLOCK)
 	    {
-		MALLOC(Walk *, walk, sizeof(Walk));
+		walk = (Walk *) mallocMagic((unsigned) (sizeof(Walk)));
 		walk->w_rT = rT;
 		walk->w_type = TT_TOP_WALK;
 		walk->w_rect.r_xbot = MAX(LEFT(tile),LEFT(tAbove));;
