@@ -49,7 +49,6 @@ static char rcsid[] = "$Header: /ufs/repository/magic/mzrouter/mzMain.c,v 1.2 20
 #include "main/main.h"
 #include "utils/geofast.h"
 #include "../utils/list.h"
-#include "utils/doubleint.h"
 #include "utils/heap.h"
 #include "utils/touchingtypes.h"
 #include "mzrouter/mzrouter.h"
@@ -172,19 +171,19 @@ NumberLine mzYAlignNL;
 int mzBoundsIncrement;
 
 /* minimum estimated total cost for initial path */
-DoubleInt mzMinInitialCost;
+dlong mzMinInitialCost;
 
 /* where current path came from */
 int mzPathSource;
 
 /* Parameters controlling search */
 RouteFloat mzPenalty;
-DoubleInt mzWRate;
-DoubleInt mzBloomDeltaCost;
-DoubleInt mzWWidth;
+dlong mzWRate;
+dlong mzBloomDeltaCost;
+dlong mzWWidth;
 
 /* Statistics */
-DoubleInt mzInitialEstimate;  /* Initial estimated cost of route */
+dlong mzInitialEstimate;  /* Initial estimated cost of route */
 int mzNumBlooms;
 int mzNumOutsideBlooms;	/* num blooms from outside window */
 int mzNumComplete;	/* number of complete paths so far */
@@ -198,13 +197,13 @@ int mzReportInterval;    /* frequency that # of paths etc.
 int mzPathsTilReport;	/* counts down to next path report */
 
 /* Variables controling search */
-DoubleInt mzWInitialMinToGo;
-DoubleInt mzWInitialMaxToGo;
-DoubleInt mzBloomMaxCost;	
+dlong mzWInitialMinToGo;
+dlong mzWInitialMaxToGo;
+dlong mzBloomMaxCost;	
 
 /* Search status */
-DoubleInt mzWindowMinToGo; /* Window location */
-DoubleInt mzWindowMaxToGo;
+dlong mzWindowMinToGo; /* Window location */
+dlong mzWindowMaxToGo;
 
 /* Hash table to avoid repeated expansion from same point during search */
 HashTable mzPointHash;
@@ -286,7 +285,7 @@ MZCopyParms(oldParms)
 
     /* allocate new structure and copy MazeParameters */
     {
-	MALLOC(MazeParameters *, newParms, sizeof(MazeParameters));
+	newParms = (MazeParameters *) mallocMagic((unsigned)(sizeof(MazeParameters)));
 	*newParms = *oldParms;
     }
 
@@ -302,7 +301,7 @@ MZCopyParms(oldParms)
 	    
 	    /* allocate and equivalence new rL and its rT */
 	    {
-		MALLOC(RouteLayer *, rLNew, sizeof(RouteLayer));
+		rLNew = (RouteLayer *) mallocMagic((unsigned)(sizeof(RouteLayer)));
 		AT_EQUIV(rLOld, rLNew);
 		AT_EQUIV(&(rLOld->rl_routeType),&(rLNew->rl_routeType));
 	    }
@@ -331,7 +330,7 @@ MZCopyParms(oldParms)
 	    
 	    /* allocate and equivalence new rC and its rT */
 	    {
-		MALLOC(RouteContact *, rCNew, sizeof(RouteContact));
+		rCNew = (RouteContact *) mallocMagic((unsigned)(sizeof(RouteContact)));
 		AT_EQUIV(rCOld, rCNew);
 		AT_EQUIV(&(rCOld->rc_routeType),&(rCNew->rc_routeType));
 	    }
@@ -482,7 +481,7 @@ MZInitRoute(parms, routeUse, expansionMask)
     mzPathSource = SOURCE_INIT;
 
     /* initial estimated cost is infinity */
-    mzMinInitialCost = DIMaxInt;
+    mzMinInitialCost = DLONG_MAX;
 
     /* initialize per-route statistics */
     mzBlockGenCalls = 0;
@@ -639,7 +638,7 @@ MZAddStart(point, type)
     {
 	ColoredPoint *newTerm;
 
-	MALLOC(ColoredPoint *, newTerm, sizeof(ColoredPoint));
+	newTerm = (ColoredPoint *) mallocMagic((unsigned)(sizeof(ColoredPoint)));
 	newTerm->cp_point = *point;
 	newTerm->cp_type = type;
 	LIST_ADD(newTerm, mzStartTerms);
@@ -779,7 +778,8 @@ MZRoute()
 	/* set bloom threshold to zero, so that initial points are placed
 	 * on Max ToGo heap.
 	 */
-	mzBloomMaxCost = DIZero;
+	/* BY NP */
+	mzBloomMaxCost = 0;
 
 	for(terms=mzStartTerms; 
 	    terms!=NULL; 
@@ -803,7 +803,7 @@ MZRoute()
         mzInitialEstimate = mzMinInitialCost;
 
 	mzWInitialMinToGo = mzInitialEstimate;
-	DOUBLE_ADD(mzWInitialMaxToGo, mzWInitialMinToGo, mzWWidth);
+	mzWInitialMaxToGo = mzWInitialMinToGo + mzWWidth;
     }
 
     /* Make sure we got here without interruption */
