@@ -40,7 +40,6 @@ static char rcsid[] = "$Header: /ufs/repository/magic/mzrouter/mzNumLine.c,v 1.2
 #include "utils/list.h"
 #include "debug/debug.h"
 #include "textio/textio.h"
-#include "utils/doubleint.h"
 #include "utils/heap.h"
 #include "mzrouter/mzrouter.h"
 #include "mzrouter/mzInternal.h"
@@ -74,7 +73,7 @@ mzNLInit(nL, size)
     nL->nl_sizeAlloced = size;
     nL->nl_sizeUsed = 2;
 
-    MALLOC(int *, entries, sizeof(int)*size);
+    entries = (int *) mallocMagic((unsigned)(sizeof(int)*size));
     entries[0] = MINFINITY;
     entries[1] = INFINITY;
 
@@ -144,7 +143,7 @@ mzNLInsert(nL,x)
 
 	/* allocate new entry array */
 	newSize = nL->nl_sizeUsed*2;
-	MALLOC(int *, newEntries, newSize);
+	newEntries = (int *) mallocMagic((unsigned)(newSize));
 
 	/* copy old entries to new */
 	{
@@ -158,7 +157,7 @@ mzNLInsert(nL,x)
 	}
 
 	/* free up old array */
-	FREE(nL->nl_entries);
+	freeMagic(nL->nl_entries);
 
 	/* update numberline */
 	nL->nl_sizeAlloced = newSize;
