@@ -42,7 +42,6 @@ static char rcsid[] = "$Header: /ufs/repository/magic/mzrouter/mzSubrs.c,v 1.2 2
 #include "main/main.h"
 #include "utils/geofast.h"
 #include "utils/list.h"
-#include "utils/doubleint.h"
 #include "utils/touchingtypes.h"
 #include "utils/heap.h"
 #include "mzrouter/mzrouter.h"
@@ -306,7 +305,7 @@ mzMarkConnectedTiles(rect, type, makeDests)
      */
     {
 	ColoredRect *e;
-	MALLOC(ColoredRect *, e,sizeof(ColoredRect));
+	e = (ColoredRect *) mallocMagic((unsigned)(sizeof(ColoredRect)));
 	e->cr_type = type;
 	e->cr_rect = *rect;
 	LIST_ADD(e, expandList);
@@ -357,7 +356,7 @@ mzMarkConnectedTiles(rect, type, makeDests)
 
 	/* Done processing top element of expandList, toss it */ 
 	e = (ColoredRect *) ListPop(&expandList);
-	FREE((char *) e);
+	freeMagic((char *) e);
     }
 
 
@@ -457,7 +456,7 @@ mzConnectedTileFunc(tile, cxp)
 	    ColoredRect *e;
 
 	    /* build colored rect corresponding to tile */
-	    MALLOC(ColoredRect *, e, sizeof(ColoredRect));
+	    e = (ColoredRect *) mallocMagic((unsigned)(sizeof(ColoredRect)));
 	    e->cr_type = TiGetType(tile);
 	    e->cr_rect = r;
 
@@ -772,7 +771,7 @@ mzPaintContact(path, prev)
  * mzCopyPath --
  *
  * Copy a RoutePath from the temporary arena into permanent storage
- * allocated via MALLOC.
+ * allocated via mallocMagic.
  *
  * Results:
  *	Returns a pointer to a copy of the RoutePath passed to us
@@ -793,7 +792,7 @@ mzCopyPath(path)
     newPrev = newHead = (RoutePath *) NULL;
     for (newPrev = NULL; path; newPrev = new, path = path->rp_back)
     {
-	MALLOC(RoutePath *, new, sizeof (RoutePath));
+	new = (RoutePath *) mallocMagic((unsigned)(sizeof (RoutePath)));
 	*new = *path;
 	if (newPrev) newPrev->rp_back = new;
 	if (newHead == NULL) newHead = new;
@@ -823,7 +822,7 @@ mzFreePath(path)
     RoutePath *path;
 {
     for ( ; path; path = path->rp_back)
-	FREE((char *) path);
+	freeMagic((char *) path);
 }
 
 /*--------------- Static variables only referenced by RPath allocation
@@ -844,7 +843,8 @@ RoutePage *mzCurPage = NULL;
  *
  * Results:
  *	Returns a pointer to a newly allocated RoutePath.
- *	This RoutePath is NOT allocated directly via MALLOC/FREE,
+ *	This RoutePath is NOT allocated directly via
+ *	mallocMagic/freeMagic,
  *	but rather via our own temporary mechanism.  It goes away
  *	once mzFreeAllTemp() gets called, so callers that wish to
  *	retain the "best" RoutePath must call mzCopyPath() to
@@ -866,7 +866,7 @@ mzAllocRPath()
     /* If out of pages, allocate a new one */
     if (mzCurPage == NULL)
     {
-	MALLOC(RoutePage *, mzCurPage, sizeof (RoutePage));
+	mzCurPage = (RoutePage *) mallocMagic((unsigned)(sizeof (RoutePage)));
 	mzCurPage->rpp_next = (RoutePage *) NULL;
 	mzCurPage->rpp_free = 0;
 	if (mzLastPage == NULL)
