@@ -14,6 +14,7 @@
  */
 
 #include <stdio.h>
+#include <stdlib.h>
 #include <ctype.h>
 #ifdef	SYSV
 #include <string.h>
@@ -61,7 +62,7 @@ main(argc, argv)
 
     if (argc != 3)
     {
-	(void) fprintf(stderr, "Usage: net2ir feedfile netfile\n");
+	fprintf(stderr, "Usage: net2ir feedfile netfile\n");
 	exit (1);
     }
 
@@ -91,12 +92,12 @@ getfirst:
 	he = HashFind(&feedHash, label);
 	if (HashGetValue(he))
 	{
-	    (void) fprintf(stderr,
+	    fprintf(stderr,
 		"Warning: multiple locs for label %s; 2nd loc ignored.\n",
 		label);
 	    continue;
 	}
-	MALLOC(LabInfo *, li, sizeof (LabInfo));
+	li = (LabInfo *) mallocMagic((unsigned) (sizeof (LabInfo)));
 	GEO_EXPAND(&r, -1, &li->li_area);
 	li->li_label = StrDup((char **) NULL, label);
 	li->li_layer = StrDup((char **) NULL, layer);
@@ -126,14 +127,14 @@ getfirst:
 
 	if (nterms >= 2)
 	{
-	    (void) fprintf(stderr, "Net with >2 terms ignored: %s\n", line1);
+	    fprintf(stderr, "Net with >2 terms ignored: %s\n", line1);
 	    continue;
 	}
 
 	he = HashLookOnly(&feedHash, line1);
 	if (he == NULL || (li = (LabInfo *) HashGetValue(he)) == NULL)
 	{
-	    (void) fprintf(stderr, "No location for terminal %s\n", line1);
+	    fprintf(stderr, "No location for terminal %s\n", line1);
 	    continue;
 	}
 
