@@ -788,7 +788,7 @@ NMWriteNetlist(fileName)
 	TxError("Couldn't write file %s.\n", realName);
 	return;
     }
-    (void) fprintf(file, " Netlist File\n");
+    fprintf(file, " Netlist File\n");
     (void) NMEnumNets(nmWriteNetsFunc, (ClientData) file);
     if (strcmp(realName, nmCurrentNetlist->nl_fileName) == 0)
 	nmCurrentNetlist->nl_flags &= ~NL_MODIFIED;
@@ -802,7 +802,7 @@ nmWriteNetsFunc(name, firstInNet, file)
     FILE *file;			/* File in which to write info. */
 {
     if (firstInNet) fputs("\n", file);
-    (void) fprintf(file, "%s\n", name);
+    fprintf(file, "%s\n", name);
     return 0;
 }
 
