@@ -120,8 +120,9 @@ static int nmVCount= 0;
  */
 
 int
-nmwRipTileFunc(tile, listHead)
+nmwRipTileFunc(tile, plane, listHead)
     Tile *tile;			/* Tile that is to be deleted. */
+    int plane;			/* Plane index of the tile */
     struct nmwarea **listHead;	/* Pointer to list head pointer. */
 {
     struct nmwarea *new;
@@ -468,8 +469,9 @@ nmwNetTermFunc(scx, label, tpath, netPtr)
  */
 
 int
-nmwNetTileFunc(tile, netPtr)
+nmwNetTileFunc(tile, plane, netPtr)
     Tile *tile;			/* Tile that is connected to net. */
+    int plane;			/* Plane index of the tile */
     char **netPtr;		/* Pointer to pointer to net name. */
 {
     SearchContext scx;
@@ -704,8 +706,9 @@ nmwVerifyLabelFunc2(scx, label, tpath, cd)
  */
 
 int
-nmwVerifyTileFunc(tile, func)
+nmwVerifyTileFunc(tile, plane, func)
     Tile *tile;			/* Tile that is connected to this net. */
+    int plane;			/* Plane index of the tile. */
     int (*func)();		/* Processing function for each tile. */
 {
     SearchContext scx;
@@ -1284,7 +1287,7 @@ nmAllFunc(name, firstInNet, fp)
 	saveM=nmMArea-saveM;
 	saveP=nmPArea-saveP;
 	saveV=nmVCount-saveV;
-        (void)fprintf(fp,"Net %s total: %d;  Metal: %d;  Poly: %d;  Vias: %d\n",
+        fprintf(fp,"Net %s total: %d;  Metal: %d;  Poly: %d;  Vias: %d\n",
 		name,
 	        saveM/RtrMetalWidth +saveP/RtrPolyWidth +saveV*RtrContactWidth,
 	        saveM/RtrMetalWidth,  saveP/RtrPolyWidth, saveV);
