@@ -21,7 +21,10 @@
  * rcsid = "$Header: /ufs/repository/magic/netmenu/netmenu.h,v 1.1.1.1 2000/03/29 18:35:09 rajit Exp $";
  */
 
-#define _NETMENU
+#ifndef _NETMENU_H
+#define _NETMENU_H
+
+#include "misc/magic.h"
 
 /* data structures */
 
@@ -47,3 +50,5 @@ extern char *NMNetlistName();
 #ifdef ROUTE_MODULE
 extern void NMMeasureAll();
 #endif
+
+#endif /* _NETMENU_H */
