@@ -19,6 +19,11 @@
  * rcsid "$Header: /ufs/repository/magic/parser/parser.h,v 1.1.1.1 2000/03/29 18:35:10 rajit Exp $";
  */
 
-#define _PARSER
+#ifndef _PARSER_H
+#define _PARSER_H
+
+#include "misc/magic.h"
 
 extern bool ParsSplit();
+
+#endif /* _PARSER_H */
