@@ -22,6 +22,8 @@ static char rcsid[]="$Header: /ufs/repository/magic/plot/plotVers.c,v 1.2 2001/0
 #endif  /* not lint */
 
 #include <stdio.h>
+#include <string.h>
+
 #include "misc/magic.h"
 #include "utils/geometry.h"
 #include "utils/geofast.h"
@@ -465,8 +467,8 @@ PlotColorVersTechLine(sectionName, argc, argv)
 
 void
 plotTransToSwath(src, dst)
-    register Rect *src;		/* Rectangle in Magic root coords. */
-    register Rect *dst;		/* Rectangle to be filled in with swath
+    Rect *src;		/* Rectangle in Magic root coords. */
+    Rect *dst;		/* Rectangle to be filled in with swath
 				 * corresponding to src.
 				 */
 {
@@ -542,7 +544,7 @@ plotVersLine(area, widen)
 
 void
 plotVersRect(area, widen)
-    register Rect *area;	/* Rectangular area to draw, in root
+    Rect *area;	/* Rectangular area to draw, in root
 				 * coordinates.
 				 */
     int widen;			/* If zero, rectangular outline is drawn
@@ -609,12 +611,12 @@ plotVersRect(area, widen)
 
 int
 plotVersTile(tile, cxp)
-    register Tile *tile;	/* Tile that's of type to be output. */
+    Tile *tile;	/* Tile that's of type to be output. */
     TreeContext *cxp;		/* Describes search in progress. */
 {
     Rect tileArea, rootArea, swathArea, edge;
-    register Tile *neighbor;
-    register Transform *trans = &cxp->tc_scx->scx_trans;
+    Tile *neighbor;
+    Transform *trans = &cxp->tc_scx->scx_trans;
 
     /* Transform tile coords to root coords and then to swath coords. */
     
