@@ -71,6 +71,9 @@ int plowJogMoveFunc();
 int plowJogDragLHS();
 int plowJogTopProc();
 int plowJogBotProc();
+
+extern void plowProcessJog();
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -92,7 +95,7 @@ int plowJogBotProc();
  * ----------------------------------------------------------------------------
  */
 
-Void
+void
 plowCleanupJogs(area, changedArea)
     Rect *area;
     Rect *changedArea;
@@ -162,7 +165,7 @@ plowCleanupJogs(area, changedArea)
  * ----------------------------------------------------------------------------
  */
 
-Void
+void
 plowProcessJog(edge, area)
     Edge *edge;
     Rect *area;
@@ -254,7 +257,7 @@ plowProcessJogFunc(edge, area)
     Edge *edge;		/* Edge found by shadow search */
     Rect *area;		/* Area in which jogs are being eliminated */
 {
-    register LinkedRect *lr;
+    LinkedRect *lr;
     Rect r, lhs;
     TileTypeBitMask mask;
     Point startPoint;
@@ -389,7 +392,7 @@ plowProcessJogFunc(edge, area)
 
     /* Free the erase list we built in plowJogDragLHS */
     for (lr = plowJogEraseList; lr; lr = lr->r_next)
-	FREE((char *) lr);
+	freeMagic((char *) lr);
     plowJogEraseList = (LinkedRect *) NULL;
     return (ret);
 }
@@ -415,10 +418,10 @@ plowProcessJogFunc(edge, area)
 
 int
 plowJogTopProc(outline)
-    register Outline *outline;
+    Outline *outline;
 {
     /* Stop if we're no longer adjacent to space */
-    if (TiGetBottomType(outline->o_outside) != TT_SPACE)
+    if (TiGetTypeExact(outline->o_outside) != TT_SPACE)
 	return (1);
 
     switch (outline->o_currentDir)
@@ -462,10 +465,10 @@ plowJogTopProc(outline)
 
 int
 plowJogBotProc(outline)
-    register Outline *outline;
+    Outline *outline;
 {
     /* Stop if we're no longer adjacent to space */
-    if (TiGetTopType(outline->o_inside) != TT_SPACE)
+    if (TiGetTypeExact(outline->o_inside) != TT_SPACE)
 	return (1);
 
     switch (outline->o_currentDir)
@@ -534,7 +537,7 @@ plowJogDragLHS(edge, newx)
     Edge *edge;		/* Edge potentially on the LHS of the jog */
     int newx;		/* Move the edge to this position */
 {
-    register LinkedRect *lr;
+    LinkedRect *lr;
 
     /* Ignore edges that are not to space */
     if (edge->e_ltype != TT_SPACE) return (0);
@@ -550,7 +553,7 @@ plowJogDragLHS(edge, newx)
      * Success: nothing else (except perhaps the original RHS edge) moved.
      * Append this edge to the list of linked rectangles.
      */
-    MALLOC(LinkedRect *, lr, sizeof (LinkedRect));
+    lr = (LinkedRect *) mallocMagic((unsigned) (sizeof (LinkedRect)));
     lr->r_r = edge->e_rect;
     lr->r_next = plowJogEraseList;
     plowJogEraseList = lr;
@@ -580,9 +583,9 @@ plowJogDragLHS(edge, newx)
 
 int
 plowJogMoveFunc(edge)
-    register Edge *edge;
+    Edge *edge;
 {
-    register Edge *origEdge = jogEdge;
+    Edge *origEdge = jogEdge;
 
     if (DebugIsSet(plowDebugID, plowDebJogs))
 	plowDebugEdge(edge, (RuleTableEntry *) NULL, "plowJogMoveFunc");
