@@ -146,6 +146,9 @@ int plowMovedEdges;	/* Number of edges actually moved */
     /* Forward declarations */
 int plowInitialPaint(), plowInitialCell();
 int plowUpdatePaintTile(), plowUpdateCell();
+bool plowPastBoundary();
+bool plowPropagateSel();
+bool plowPropagateRect();
 PlowRule *plowBuildWidthRules();
 
 extern void PlowRedrawBound();
