@@ -307,7 +307,7 @@ plowQueueAdd(eadd)
 	    else
 	    {
 		/* Create a new edge, replacing eprev */
-		MALLOC(Edge *, enew, sizeof (Edge));
+		enew = (Edge *) mallocMagic((unsigned) (sizeof (Edge)));
 		*enew = *ep;
 		enew->e_ybot = addRect.r_ybot;
 		enew->e_ytop = ep->e_ybot;
@@ -329,7 +329,7 @@ plowQueueAdd(eadd)
 	     * Clip the portion of ep below addRect.
 	     * The new edge becomes eprev.
 	     */
-	    MALLOC(Edge *, enew, sizeof (Edge));
+	    enew = (Edge *) mallocMagic((unsigned) (sizeof (Edge)));
 	    *enew = *ep;
 	    enew->e_ytop = ep->e_ybot = addRect.r_ybot;
 	    enew->e_next = ep;
@@ -345,7 +345,7 @@ plowQueueAdd(eadd)
 	 */
 	if (ep->e_ytop > addRect.r_ytop)
 	{
-	    MALLOC(Edge *, enew, sizeof (Edge));
+	    enew = (Edge *) mallocMagic((unsigned) (sizeof (Edge)));
 	    *enew = *ep;
 	    enew->e_ybot = ep->e_ytop = addRect.r_ytop;
 	    enew->e_next = ep->e_next;
@@ -372,7 +372,7 @@ mergeDown:
 	{
 	    eprev->e_ytop = ep->e_ytop;
 	    eprev->e_next = ep->e_next;
-	    FREE((char *) ep);
+	    freeMagic((char *) ep);
 	    ep = eprev;
 	    plowNumEdges--;
 	}
@@ -395,7 +395,7 @@ mergeDown:
 	}
 	else
 	{
-	    MALLOC(Edge *, enew, sizeof (Edge));
+	    enew = (Edge *) mallocMagic((unsigned) (sizeof (Edge)));
 	    *enew = *eadd;
 	    enew->e_ybot = addRect.r_ybot;
 	    if (eprev) eprev->e_next = enew; else *pbin = enew;
@@ -417,7 +417,7 @@ mergeFinal:
     {
 	eprev->e_ytop = ep->e_ytop;
 	eprev->e_next = ep->e_next;
-	FREE((char *) ep);
+	freeMagic((char *) ep);
 	plowNumEdges--;
     }
     goto done;
@@ -426,7 +426,7 @@ mergeFinal:
      * Prepend eadd to the current bin.
      */
 prepend:
-    MALLOC(Edge *, enew, sizeof (Edge));
+    enew = (Edge *) mallocMagic((unsigned) (sizeof (Edge)));
     *enew = *eadd;
     enew->e_next = *pbin;
     *pbin = enew;
@@ -507,7 +507,7 @@ plowQueueLeftmost(edge)
 
     /* Fill in the caller's edge */
     *edge = *enew;
-    FREE((char *) enew);
+    freeMagic((char *) enew);
     return (TRUE);
 }
 
@@ -582,6 +582,6 @@ plowQueueRightmost(edge)
 
     /* Fill in the caller's edge */
     *edge = *enew;
-    FREE((char *) enew);
+    freeMagic((char *) enew);
     return (TRUE);
 }
