@@ -100,10 +100,10 @@ PlowDRCInit()
 	for (j = 0; j < TT_MAXTYPES; j++)
 	{
 	    for (pr = plowWidthRulesTbl[i][j]; pr; pr = pr->pr_next)
-		FREE((char *)pr);
+		freeMagic((char *)pr);
 		
 	    for (pr = plowSpacingRulesTbl[i][j]; pr; pr = pr->pr_next)
-		FREE((char *)pr);
+		freeMagic((char *)pr);
 
 	    plowWidthRulesTbl[i][j] = NULL;
 	    plowSpacingRulesTbl[i][j] = NULL;
@@ -233,7 +233,7 @@ plowWidthRule(argc, argv)
 	    {
 		if (DBTypesOnSamePlane(i, j) && TTMaskHasType(&set, j))
 		{
-		    MALLOC(PlowRule *, pr, sizeof (PlowRule));
+		    pr = (PlowRule *) mallocMagic((unsigned)(sizeof (PlowRule)));
 		    pr->pr_dist = distance;
 		    pr->pr_ltypes = setC;
 		    pr->pr_oktypes = set;
@@ -340,7 +340,7 @@ plowSpacingRule(argc, argv)
 		{
 		    if (!PlaneMaskHasPlane(planes2, pNum))
 			continue;
-		    MALLOC(PlowRule *, pr, sizeof (PlowRule));
+		    pr = (PlowRule *) mallocMagic((unsigned) (sizeof (PlowRule)));
 		    TTMaskClearMask3(&tmp1, &DBPlaneTypes[pNum], &set2);
 		    TTMaskCom2(&tmp2, &setRreverse);
 		    TTMaskAndMask3(&pr->pr_ltypes, &DBPlaneTypes[pNum], &tmp2);
@@ -363,7 +363,7 @@ plowSpacingRule(argc, argv)
 		for (pNum = PL_TECHDEPBASE; pNum < DBNumPlanes; pNum++)
 		{
 		    if (!PlaneMaskHasPlane(planes1, pNum)) continue;
-		    MALLOC(PlowRule *, pr, sizeof (PlowRule));
+		    pr = (PlowRule *) mallocMagic((unsigned)(sizeof (PlowRule)));
 		    TTMaskClearMask3(&tmp1, &DBPlaneTypes[pNum], &set1);
 		    TTMaskCom2(&tmp2, &setRreverse);
 		    TTMaskAndMask3(&pr->pr_ltypes, &DBPlaneTypes[pNum], &tmp2);
@@ -477,7 +477,7 @@ plowEdgeRule(argc, argv)
 	    {
 		if (TTMaskHasType(&set2, j))
 		{
-		    MALLOC(PlowRule *, pr, sizeof (PlowRule));
+		    pr = (PlowRule *) mallocMagic((unsigned)(sizeof (PlowRule)));
 		    pr->pr_ltypes = setLeft;
 		    pr->pr_oktypes = setM;
 		    pr->pr_dist = distance;
@@ -489,7 +489,7 @@ plowEdgeRule(argc, argv)
 
 		if (needPenumbraOnly && TTMaskHasType(&setC, j))
 		{
-		    MALLOC(PlowRule *, pr, sizeof (PlowRule));
+		    pr = (PlowRule *) mallocMagic((unsigned)(sizeof (PlowRule)));
 		    pr->pr_ltypes = setRight;
 		    pr->pr_oktypes = setM;
 		    pr->pr_dist = distance;
@@ -552,7 +552,7 @@ plowEdgeRule(argc, argv)
 	    {
 		if (TTMaskHasType(&setRight, j))
 		{
-		    MALLOC(PlowRule *, pr, sizeof (PlowRule));
+		    pr = (PlowRule *) mallocMagic((unsigned)(sizeof (PlowRule)));
 		    TTMaskSetOnlyType(&pr->pr_ltypes, i);
 		    pr->pr_oktypes = setOK;
 		    pr->pr_dist = distance;
@@ -677,7 +677,7 @@ plowTechOptimizeRule(ruleList)
 		     * Delete pCand, and resume outer loop with the
 		     * new values of pCand and pCandLast set below.
 		     */
-		    FREE((char *) pCand);
+		    freeMagic((char *) pCand);
 		    if (pCandLast)
 			pCandLast->pr_next = pCand->pr_next;
 		    else
@@ -864,12 +864,12 @@ plowTechShowTable(table, header, f)
     PlowRule *pr;
     TileType i, j;
 
-    (void) fprintf(f, "\n\n------------ %s ------------\n", header);
+    fprintf(f, "\n\n------------ %s ------------\n", header);
     for (i = 0; i < DBNumTypes; i++)
 	for (j = 0; j < DBNumTypes; j++)
 	    if (pr = table[i][j])
 	    {
-		(void) fprintf(f, "\n%s -- %s:\n",
+		fprintf(f, "\n%s -- %s:\n",
 		    DBTypeLongName(i), DBTypeLongName(j));
 		for ( ; pr; pr = pr->pr_next)
 		    plowTechPrintRule(pr, f);
@@ -880,15 +880,15 @@ plowTechPrintRule(pr, f)
     PlowRule *pr;
     FILE *f;
 {
-    (void) fprintf(f, "\tDISTANCE=%d, PLANE=%s, FLAGS=",
+    fprintf(f, "\tDISTANCE=%d, PLANE=%s, FLAGS=",
 		pr->pr_dist, DBPlaneLongName(pr->pr_pNum));
-    if (pr->pr_flags & PR_WIDTH) (void) fprintf(f, " Width");
-    if (pr->pr_flags & PR_PENUMBRAONLY) (void) fprintf(f, " PenumbraOnly");
-    if (pr->pr_flags & PR_EDGE) (void) fprintf(f, " Edge");
-    if (pr->pr_flags & PR_EDGE4WAY) (void) fprintf(f, " Edge4way");
-    if (pr->pr_flags & PR_EDGEBACK) (void) fprintf(f, " EdgeBack");
-    (void) fprintf(f, "\n");
-    (void) fprintf(f, "\tLTYPES = %s\n", maskToPrint(&pr->pr_ltypes));
-    (void) fprintf(f, "\tOKTYPES = %s\n", maskToPrint(&pr->pr_oktypes));
-    (void) fprintf(f, "\t-------------------------------\n");
+    if (pr->pr_flags & PR_WIDTH) fprintf(f, " Width");
+    if (pr->pr_flags & PR_PENUMBRAONLY) fprintf(f, " PenumbraOnly");
+    if (pr->pr_flags & PR_EDGE) fprintf(f, " Edge");
+    if (pr->pr_flags & PR_EDGE4WAY) fprintf(f, " Edge4way");
+    if (pr->pr_flags & PR_EDGEBACK) fprintf(f, " EdgeBack");
+    fprintf(f, "\n");
+    fprintf(f, "\tLTYPES = %s\n", maskToPrint(&pr->pr_ltypes));
+    fprintf(f, "\tOKTYPES = %s\n", maskToPrint(&pr->pr_oktypes));
+    fprintf(f, "\t-------------------------------\n");
 }
