@@ -22,12 +22,14 @@ static char rcsid[]="$Header: /ufs/repository/magic/plow/PlowTest.c,v 1.3 2001/0
 #endif  /* not lint */
 
 #include <stdio.h>
+#include <stdlib.h>
 #include "misc/magic.h"
 #include "misc/styles.h"
 #include "utils/utils.h"
 #include "utils/geometry.h"
 #include "tiles/tile.h"
 #include "utils/hash.h"
+#include "undo/undo.h"
 #include "database/database.h"
 #include "windows/windows.h"
 #include "graphics/graphics.h"
@@ -40,8 +42,11 @@ static char rcsid[]="$Header: /ufs/repository/magic/plow/PlowTest.c,v 1.3 2001/0
 #include "textio/textio.h"
 #include "textio/txcommands.h"
 
-int plowShowShadow();
-int plowShowOutline();
+/* Forward declarations */
+extern void plowShowShadow();
+extern int plowShowOutline();
+extern void plowTestJog();
+extern void plowDebugMore();
 
 /* Imports from PlowMain.c */
 extern CellDef *plowYankDef;
@@ -123,8 +128,7 @@ struct
     0,
 };
 
-    /*ARGSUSED*/
-Void
+void
 PlowTest(w, cmd)
     MagWindow *w;
     TxCommand *cmd;
@@ -350,7 +354,7 @@ PlowTest(w, cmd)
 	    TxPrintf("Tile 0x%x  LEFT=%d RIGHT=%d BOTTOM=%d TOP=%d\n",
 		tp, LEFT(tp), RIGHT(tp), BOTTOM(tp), TOP(tp));
 	    TxPrintf("    TRAILING=%d LEADING=%d TYPE=%s\n",
-		TRAILING(tp), LEADING(tp), DBTypeLongName(TiGetType(tp)));
+		TRAILING(tp), LEADING(tp), DBTypeLongName(TiGetTypeExact(tp)));
 	    break;
 	case PC_WHENTOP:
 	    if (cmd->tx_argc == 2)
@@ -443,6 +447,7 @@ plowGetCommand(cmd)
  * ----------------------------------------------------------------------------
  */
 
+void
 plowDebugInit()
 {
     int n;
@@ -484,6 +489,7 @@ plowDebugInit()
  * ----------------------------------------------------------------------------
  */
 
+void
 plowShowShadow(edge, def)
     Edge *edge;
     CellDef *def;
@@ -522,6 +528,7 @@ plowShowShadow(edge, def)
  * ----------------------------------------------------------------------------
  */
 
+void
 plowTestJog(def, area)
     CellDef *def;
     Rect *area;
@@ -597,6 +604,7 @@ plowTestJog(def, area)
  * ----------------------------------------------------------------------------
  */
 
+void
 plowDebugEdge(edge, rtePtr, mesg)
     Edge *edge;
     RuleTableEntry *rtePtr;
@@ -667,6 +675,7 @@ plowDebugEdge(edge, rtePtr, mesg)
  * ----------------------------------------------------------------------------
  */
 
+void
 plowDebugMore()
 {
     char answer[100];
@@ -703,6 +712,7 @@ again:
  * ----------------------------------------------------------------------------
  */
 
+int
 plowShowOutline(outline, clipArea)
     Outline *outline;
     Rect *clipArea;
@@ -721,8 +731,8 @@ plowShowOutline(outline, clipArea)
 		dirNames[outline->o_prevDir],
 		dirNames[outline->o_currentDir],
 		dirNames[outline->o_nextDir],
-		DBTypeLongName(TiGetType(outline->o_inside)),
-		DBTypeLongName(TiGetType(outline->o_outside)));
+		DBTypeLongName(TiGetTypeExact(outline->o_inside)),
+		DBTypeLongName(TiGetTypeExact(outline->o_outside)));
 
     switch (outline->o_currentDir)
     {
@@ -796,6 +806,7 @@ plowShowOutline(outline, clipArea)
  * ----------------------------------------------------------------------------
  */
 
+void
 plowDisplay(dodef)
     bool dodef;
 {
