@@ -36,6 +36,9 @@ typedef struct
 {
     Rect		area;		/* Area to process */
     TileTypeBitMask	*connectMask;	/* Connection mask for search */
+#ifdef NONMANHATTAN
+    TileType		dinfo;		/* Info about triangular search areas */
+#endif
 } conSrArea;
 
 struct conSrArg2
@@ -93,7 +96,10 @@ dbcConnectFuncDCS(tile, cx)
     ResTranTile		*thisTran;
     TileTypeBitMask	notConnectMask, *connectMask;
     Tile		*tp;
-    TileType		t2, t1, loctype, newtype;
+    TileType		t2, t1, loctype, ctype;
+#ifdef NONMANHATTAN
+    TileType		dinfo = 0;
+#endif
     SearchContext	*scx = cx->tc_scx;
     int			pNum;
     CellDef		*def;
@@ -120,7 +126,7 @@ dbcConnectFuncDCS(tile, cx)
 	     TTMaskHasType(&(ExtCurStyle->exts_transSDTypes[t2]),t1))
 	     {
     	          TiToRect(tp, &tranArea);
-	          MALLOC(ResTranTile *, thisTran, sizeof(ResTranTile));
+	          thisTran = (ResTranTile *) mallocMagic((unsigned)(sizeof(ResTranTile)));
 		  ResCalcPerimOverlap(thisTran,tp);
 	          GeoTransRect(&scx->scx_trans, &tranArea, &thisTran->area);
 	          thisTran->type = TiGetType(tp);
@@ -136,7 +142,7 @@ dbcConnectFuncDCS(tile, cx)
 	     TTMaskHasType(&(ExtCurStyle->exts_transSDTypes[t2]),t1))
 	     {
     	          TiToRect(tp, &tranArea);
-	          MALLOC(ResTranTile *, thisTran, sizeof(ResTranTile));
+	          thisTran = (ResTranTile *) mallocMagic((unsigned)(sizeof(ResTranTile)));
 	          GeoTransRect(&scx->scx_trans, &tranArea, &thisTran->area);
 	          thisTran->type = TiGetType(tp);
 	          thisTran->nextTran = TransList;
@@ -152,7 +158,7 @@ dbcConnectFuncDCS(tile, cx)
 	     TTMaskHasType(&(ExtCurStyle->exts_transSDTypes[t2]),t1))
 	     {
     	          TiToRect(tp, &tranArea);
-	          MALLOC(ResTranTile *, thisTran, sizeof(ResTranTile));
+	          thisTran = (ResTranTile *) mallocMagic((unsigned)(sizeof(ResTranTile)));
 	          GeoTransRect(&scx->scx_trans, &tranArea, &thisTran->area);
 	          thisTran->type = TiGetType(tp);
 	          thisTran->nextTran = TransList;
@@ -168,7 +174,7 @@ dbcConnectFuncDCS(tile, cx)
 	     TTMaskHasType(&(ExtCurStyle->exts_transSDTypes[t2]),t1))
 	     {
     	          TiToRect(tp, &tranArea);
-	          MALLOC(ResTranTile *, thisTran, sizeof(ResTranTile));
+	          thisTran = (ResTranTile *) mallocMagic((unsigned)(sizeof(ResTranTile)));
 	          GeoTransRect(&scx->scx_trans, &tranArea, &thisTran->area);
 	          thisTran->type = TiGetType(tp);
 	          thisTran->nextTran = TransList;
@@ -180,7 +186,7 @@ dbcConnectFuncDCS(tile, cx)
     else if TTMaskHasType(&(ExtCurStyle->exts_transMask),t1)
     {
     	          TiToRect(tile, &tranArea);
-	          MALLOC(ResTranTile *, thisTran, sizeof(ResTranTile));
+	          thisTran = (ResTranTile *) mallocMagic((unsigned)(sizeof(ResTranTile)));
 		  ResCalcPerimOverlap(thisTran,tile);
 	          GeoTransRect(&scx->scx_trans, &tranArea, &thisTran->area);
 	          thisTran->type = TiGetType(tile);
@@ -214,45 +220,37 @@ dbcConnectFuncDCS(tile, cx)
     GeoClip(&newarea, csa2->csa2_bounds);
     if (GEO_RECTNULL(&newarea)) return 0;
 
+    loctype = TiGetTypeExact(tile);
+
 #ifdef NONMANHATTAN
     /* Resolve geometric transformations on diagonally-split tiles */
 
     if (IsSplit(tile))
     {
-	int o1, o2, o3, dir, side;
-
-	o1 = ((scx->scx_trans.t_e > 0) || (scx->scx_trans.t_d > 0)) ? 1 : 0;
-	o2 = ((scx->scx_trans.t_a > 0) || (scx->scx_trans.t_b > 0)) ? 1 : 0;
-	o3 = (scx->scx_trans.t_a != 0) ? 1 : 0;
-	dir = SplitDirection(tile) ^ o1 ^ o2;
-	side = SplitSide(tile) ^ o2 ^ (SplitDirection(tile) | o3);
-
-	if (side == SplitSide(tile))
-            newtype = TT_DIAGONAL | SplitLeftType(tile) |
-                        (SplitRightType(tile) << 14);
-	else
-            newtype = TT_DIAGONAL | SplitRightType(tile) |
-                        (SplitLeftType(tile) << 14);
-
-	if (dir) newtype |= TT_DIRECTION;
-	if (side)
-	{
-	    newtype |= TT_SIDE;
-	    loctype = (newtype & TT_RIGHTMASK) >> 14;
-	}
-	else
-	    loctype = newtype & TT_LEFTMASK;
+	dinfo = DBTransformDiagonal(loctype, &scx->scx_trans);
+	loctype = (SplitSide(tile)) ? SplitRightType(tile) : SplitLeftType(tile);
     }
-    else
 #endif
-    loctype = newtype = TiGetTypeExact(tile);
 
     pNum = DBPlane(loctype);
     connectMask = &csa2->csa2_connect[loctype];
 
     if (DBIsContact(loctype))
     {
+	TileTypeBitMask *rMask = DBResidueMask(loctype);
+
 	TTMaskSetOnlyType(&notConnectMask, loctype);
+
+	/* Differenct contact types may share residues (6/18/04) */     
+	for (ctype = TT_TECHDEPBASE; ctype < DBNumUserLayers; ctype++)
+	    if (TTMaskEqual(rMask, DBResidueMask(ctype)))
+		TTMaskSetType(&notConnectMask, ctype);
+
+	/* The mask of contact types must include all stacked contacts */
+	for (ctype = DBNumUserLayers; ctype < DBNumTypes; ctype++)   
+	    if TTMaskHasType(DBResidueMask(ctype), loctype)
+		TTMaskSetType(&notConnectMask, ctype);
+
 	TTMaskCom(&notConnectMask);
     }
     else
@@ -261,13 +259,21 @@ dbcConnectFuncDCS(tile, cx)
     }
 
     def = csa2->csa2_use->cu_def;
+
+#ifdef NONMANHATTAN
+    if (DBSrPaintNMArea((Tile *) NULL, def->cd_planes[pNum],
+		dinfo, &newarea, &notConnectMask, dbcUnconnectFunc,
+		(ClientData) connectMask) == 0)
+	return 0;
+#else
     if (DBSrPaintArea((Tile *) NULL, def->cd_planes[pNum],
 	    &newarea, &notConnectMask, dbcUnconnectFunc,
 	    (ClientData) connectMask) == 0)
 	return 0;
+#endif
 
 #ifdef NONMANHATTAN
-    DBNMPaintPlane(def->cd_planes[pNum], newtype,
+    DBNMPaintPlane(def->cd_planes[pNum], dinfo,
 		&newarea, DBStdPaintTbl(loctype, pNum),
 		(PaintUndoInfo *) NULL);
 #else
@@ -278,14 +284,14 @@ dbcConnectFuncDCS(tile, cx)
 #ifdef NONMANHATTAN
     /* Only extend those sides bordering the diagonal tile */
 
-    if (newtype & TT_DIAGONAL)
+    if (dinfo & TT_DIAGONAL)
     {
-	if (newtype & TT_SIDE)               /* right */
+	if (dinfo & TT_SIDE)               /* right */
 	    newarea.r_xtop += 1;
 	else                                            /* left */
 	    newarea.r_xbot -= 1;
-	if (((newtype & TT_SIDE) >> 1)
-		== (newtype & TT_DIRECTION)) /* top */
+	if (((dinfo & TT_SIDE) >> 1)
+		== (dinfo & TT_DIRECTION)) /* top */
 	    newarea.r_ytop += 1;
 	else                                            /* bottom */
 	    newarea.r_ybot -= 1;
@@ -318,7 +324,9 @@ dbcConnectFuncDCS(tile, cx)
 	{
 	    newlist[i].area = csa2->csa2_list[i].area;
 	    newlist[i].connectMask = csa2->csa2_list[i].connectMask;
-	    
+#ifdef NONMANHATTAN
+	    newlist[i].dinfo = csa2->csa2_list[i].dinfo;
+#endif
 	}
 	freeMagic((char *)csa2->csa2_list);
 	csa2->csa2_list = newlist;
@@ -326,6 +334,9 @@ dbcConnectFuncDCS(tile, cx)
 
     csa2->csa2_list[csa2->csa2_top].area = newarea;
     csa2->csa2_list[csa2->csa2_top].connectMask = connectMask;
+#ifdef NONMANHATTAN
+    csa2->csa2_list[csa2->csa2_top].dinfo = dinfo;
+#endif
 
     return 0;
 }
@@ -435,6 +446,9 @@ DBTreeCopyConnectDCS(scx, mask, xMask, connect, area, destUse)
     TileTypeBitMask	*newmask;
     ResTranTile		*CurrentT;
     CellDef		*def = destUse->cu_def;
+#ifdef NONMANHATTAN
+    TileType		newtype;
+#endif
 
     csa2.csa2_use = destUse;
     csa2.csa2_bounds = area;
@@ -465,7 +479,16 @@ DBTreeCopyConnectDCS(scx, mask, xMask, connect, area, destUse)
     {
 	newmask = csa2.csa2_list[csa2.csa2_top].connectMask;
 	scx->scx_area = csa2.csa2_list[csa2.csa2_top].area;
+#ifdef NONMANHATTAN
+	newtype = csa2.csa2_list[csa2.csa2_top].dinfo;
+#endif
 	csa2.csa2_top--;
+#ifdef NONMANHATTAN
+	if (newtype & TT_DIAGONAL)
+	    DBTreeSrNMTiles(scx, newtype, newmask, xMask, dbcConnectFuncDCS,
+			(ClientData) &csa2);
+	else
+#endif
 	DBTreeSrTiles(scx, newmask, xMask, dbcConnectFuncDCS, (ClientData) &csa2);
     }
     freeMagic((char *)csa2.csa2_list);
@@ -524,7 +547,7 @@ resSubSearchFunc(tile,cx)
      */
      if (ExtCurStyle->exts_transSDCount[t] >1) return 0;
      TiToRect(tile, &tranArea);
-     MALLOC(ResTranTile *, thisTran, sizeof(ResTranTile));
+     thisTran = (ResTranTile *) mallocMagic((unsigned)(sizeof(ResTranTile)));
      GeoTransRect(&cx->tc_scx->scx_trans, &tranArea, &thisTran->area);
      thisTran->type = t;
      thisTran->nextTran = TransList;
