@@ -16,7 +16,6 @@ static char rcsid[] = "$Header: /ufs/repository/magic/resis/ResMain.c,v 1.2 2001
 #include "tiles/tile.h"
 #include "utils/hash.h"
 #include "database/database.h"
-#include "database/databaseInt.h"
 #include "utils/malloc.h"
 #include "textio/textio.h"
 #include "extract/extract.h"
@@ -63,6 +62,8 @@ extern HashTable	ResNodeTable;
  *
  *-------------------------------------------------------------------------
  */
+
+void
 ResInitializeConn()
 {
      TileType tran, diff;
@@ -102,6 +103,8 @@ ResInitializeConn()
  *
  * --------------------------------------------------------------------------
  */
+
+void
 ResGetReCell()
 {
     if (ResUse != NULL) return;
@@ -137,43 +140,43 @@ ResGetReCell()
 ResDissolveContacts(contacts)
 	ResContactPoint *contacts;
 {
-    TileType t,oldtype;
+    TileType t, oldtype;
     Tile *tp;
-    LayerInfo	*lp;
+    TileTypeBitMask *residues;
 
     for (; contacts != (ResContactPoint *) NULL; contacts = contacts->cp_nextcontact)
 
     {
-          oldtype=contacts->cp_type;
+        oldtype=contacts->cp_type;
 #ifdef PARANOID
-	  if (oldtype == TT_SPACE)
-	  {
-	       TxError("Error in Contact Dissolving for %s \n",ResCurrentNode);
-	  }
+	if (oldtype == TT_SPACE)
+	{
+	    TxError("Error in Contact Dissolving for %s \n",ResCurrentNode);
+	}
 #endif
-	  lp = dbLayerInfo+oldtype;
-	  if (TTMaskHasType(&ExtCurStyle->exts_transMask,lp->l_images[0])) 
-	  							   continue;
-	  if (TTMaskHasType(&ExtCurStyle->exts_transMask,lp->l_images[1])) 
-	  							   continue;
-	  if (TTMaskHasType(&ExtCurStyle->exts_transMask,lp->l_images[2])) 
-	  							   continue;
-	  DBErase(ResUse->cu_def,&(contacts->cp_rect),oldtype);
-	  DBPaint(ResUse->cu_def,&(contacts->cp_rect),lp->l_residue);	
-	  if ((t=lp->l_rbelow) != TT_SPACE)
-	       DBPaint(ResUse->cu_def,&(contacts->cp_rect),t);
-	  if ((t=lp->l_rabove) != TT_SPACE)
-	       DBPaint(ResUse->cu_def,&(contacts->cp_rect),t);
-	  tp = ResDef->cd_planes[DBPlane(contacts->cp_type)]->pl_hint;
-	  GOTOPOINT(tp,&(contacts->cp_rect.r_ll));
+	
+	residues = DBResidueMask(oldtype);
+
+	DBErase(ResUse->cu_def, &(contacts->cp_rect), oldtype);
+	for (t = TT_TECHDEPBASE; t < DBNumTypes; t++)
+	{
+	    if (TTMaskHasType(residues, t))
+	    {
+		if (TTMaskHasType(&ExtCurStyle->exts_transMask, t))
+		    continue;
+		DBPaint(ResUse->cu_def, &(contacts->cp_rect), t);	
+	    }
+	}
+
+	tp = ResDef->cd_planes[DBPlane(contacts->cp_type)]->pl_hint;
+	GOTOPOINT(tp,&(contacts->cp_rect.r_ll));
 #ifdef PARANOID
-	  if (TiGetTypeExact(tp) == contacts->cp_type)
-	  {
-	       TxError("Error in Contact Preprocess Routines\n");
-	  }
+	if (TiGetTypeExact(tp) == contacts->cp_type)
+	{
+	    TxError("Error in Contact Preprocess Routines\n");
+	}
 #endif
     }
-     
 }
 
 /*
@@ -257,11 +260,13 @@ ResAddBreakpointFunc(tile, node)
  *
  *----------------------------------------------------------------------------
  */
+
+void
 ResFindNewContactTiles(contacts)
 	ResContactPoint *contacts;
 {
      int pNum;
-     register Tile *tile;
+     Tile *tile;
      TileTypeBitMask *mask;
      
      for (; contacts != (ResContactPoint *) NULL; contacts = contacts->cp_nextcontact)
@@ -287,7 +292,7 @@ ResFindNewContactTiles(contacts)
 		       tileJunk	*j = (tileJunk *)tile->ti_client;
 		       cElement *ce;
 		       
-		       MALLOC(cElement *,ce,sizeof(cElement));
+		       ce = (cElement *) mallocMagic((unsigned) (sizeof(cElement)));
 		       contacts->cp_tile[contacts->cp_currentcontact] = tile;
 		       ce->ce_thisc = contacts;
 		       ce->ce_nextc = j->contactList;
@@ -328,8 +333,8 @@ ResProcessTiles(goodies, origin)
      Tile 		*startTile;
      int 		tilenum,merged;
      resNode		*resptr2;
-     register jElement	*workingj;
-     register cElement	*workingc;
+     jElement	*workingj;
+     cElement	*workingc;
      ResFixPoint	*fix;
      resNode		*resptr;
      int		(*tilefunc)();
@@ -371,7 +376,7 @@ ResProcessTiles(goodies, origin)
 	      {
 	           int x = fix->fp_loc.p_x;
 	           int y = fix->fp_loc.p_y;
-		   MALLOC(resNode *, resptr, sizeof(resNode));
+		   resptr = (resNode *) mallocMagic((unsigned)(sizeof(resNode)));
 		   InitializeNode(resptr, x, y, RES_NODE_ORIGIN);
 	           resptr->rn_status = TRUE;
 	           resptr->rn_noderes = 0;
@@ -513,6 +518,7 @@ ResProcessTiles(goodies, origin)
  *
  *-------------------------------------------------------------------------
  */
+
 bool
 ResExtractNet(startlist,goodies,cellname) 
     ResFixPoint		*startlist;
@@ -670,7 +676,7 @@ ResExtractNet(startlist,goodies,cellname)
     {
     	 for (fix = startlist; fix != NULL;fix=fix->fp_next)
 	 {
-     	      fix->fp_tile = ResUse->cu_def->cd_planes[DBTypePlaneTbl[fix->fp_ttype]]->pl_hint;
+     	      fix->fp_tile = ResUse->cu_def->cd_planes[DBPlane(fix->fp_ttype)]->pl_hint;
 	      GOTOPOINT(fix->fp_tile,&fix->fp_loc);
 	      if (TiGetTypeExact(fix->fp_tile) == TT_SPACE) fix->fp_tile = NULL;
 	 }
@@ -696,6 +702,8 @@ ResExtractNet(startlist,goodies,cellname)
  *
  *-------------------------------------------------------------------------
  */
+
+void
 ResCleanUpEverything()
 {
 
@@ -723,13 +731,13 @@ ResCleanUpEverything()
     {
     	 oldCon = ResContactList;
 	 ResContactList = oldCon->cp_nextcontact;
-	 FREE((char *)oldCon);
+	 freeMagic((char *)oldCon);
     }
     while (ResResList != NULL)
     {
     	 oldRes = ResResList;
 	 ResResList = ResResList->rr_nextResistor;
-	 FREE((char *)oldRes);
+	 freeMagic((char *)oldRes);
     }
     while (ResTransList != NULL)
     {
@@ -737,7 +745,7 @@ ResCleanUpEverything()
 	 ResTransList = ResTransList->rt_nextTran;
 	 if ((oldTran->rt_status & RES_TRAN_SAVE) == 0)
 	 {
-	      FREE((char *)oldTran);
+	      freeMagic((char *)oldTran);
 	 }
     }
 
@@ -774,7 +782,7 @@ FindStartTile(goodies, SourcePoint)
      
     workingPoint.p_x = goodies->rg_tranloc->p_x;
     workingPoint.p_y = goodies->rg_tranloc->p_y;
-    pnum = DBTypePlaneTbl[goodies->rg_ttype];
+    pnum = DBPlane(goodies->rg_ttype);
 
     /* for drivepoints, we don't have to find a transistor */
     if (goodies->rg_status & DRIVEONLY)
@@ -912,6 +920,7 @@ FindStartTile(goodies, SourcePoint)
  *
  *-------------------------------------------------------------------------
  */
+
 resTransistor *
 ResGetTransistor(pt)
 	Point	*pt;
