@@ -28,6 +28,10 @@ static char rcsid[] = "$Header: /ufs/repository/magic/resis/ResMakeRes.c,v 1.2 2
 #include "textio/txcommands.h"
 #include "resis/resis.h"
 
+/* Forward declarations */
+bool ResCalcNearTransistor();
+bool ResCalcNorthSouth();
+bool ResCalcEastWest();
 
 
 /*
@@ -142,7 +146,7 @@ ResCalcEastWest(tile, pendingList, doneList, resList)
     if   (p1->br_next == NULL) 
     {
 	p1->br_this->rn_float.rn_area += height * (LEFT(tile) - RIGHT(tile));
-	FREE((char *)p1);
+	freeMagic((char *)p1);
 	junk->breakList = NULL;
 	return(merged);
     }
@@ -171,7 +175,7 @@ ResCalcEastWest(tile, pendingList, doneList, resList)
 	    {
 		 currNode = NULL;
 		 p1->br_next = p2->br_next;
-		 FREE((char *)p2);
+		 freeMagic((char *)p2);
 		 p2 = p1;
 	    }
 	    else if (p2->br_this == resCurrentNode)
@@ -179,7 +183,7 @@ ResCalcEastWest(tile, pendingList, doneList, resList)
 		 currNode = p1->br_this;
 	    	 ResMergeNodes(p2->br_this,p1->br_this,pendingList,doneList);
 		 merged = TRUE;
-		 FREE((char *)p1);
+		 freeMagic((char *)p1);
 	    }
 	    else if (p1->br_this == resCurrentNode)
 	    {
@@ -187,14 +191,14 @@ ResCalcEastWest(tile, pendingList, doneList, resList)
 		 p1->br_next = p2->br_next;
 	    	 ResMergeNodes(p1->br_this,p2->br_this,pendingList,doneList);
 		 merged = TRUE;
-		 FREE((char *)p2);
+		 freeMagic((char *)p2);
 		 p2 = p1;
 	    }
 	    else
 	    {
 		 currNode = p1->br_this;
 	    	 ResMergeNodes(p2->br_this,p1->br_this,pendingList,doneList);
-		 FREE((char *)p1);
+		 freeMagic((char *)p1);
 	    }
 
 	    /* 
@@ -219,18 +223,18 @@ ResCalcEastWest(tile, pendingList, doneList, resList)
 
        else
        {
-            MALLOC(resResistor *, resistor, sizeof(resResistor));
+            resistor = (resResistor *) mallocMagic((unsigned) (sizeof(resResistor)));
             resistor->rr_nextResistor = (*resList);
             resistor->rr_lastResistor = NULL;
             if ((*resList) != NULL) (*resList)->rr_lastResistor = resistor;
             (*resList) = resistor;
             resistor->rr_connection1 = p1->br_this;
             resistor->rr_connection2 = p2->br_this;
-            MALLOC(resElement *, element, sizeof(resElement));
+            element = (resElement *) mallocMagic((unsigned) (sizeof(resElement)));
             element->re_nextEl = p1->br_this->rn_re;
             element->re_thisEl = resistor;
             p1->br_this->rn_re = element;
-            MALLOC(resElement *, element, sizeof(resElement));
+            element = (resElement *) mallocMagic((unsigned) (sizeof(resElement)));
             element->re_nextEl = p2->br_this->rn_re;
             element->re_thisEl = resistor;
             p2->br_this->rn_re = element;
@@ -268,11 +272,11 @@ ResCalcEastWest(tile, pendingList, doneList, resList)
 	    resistor->rr_connection2->rn_float.rn_area += rArea;
 	    resistor->rr_float.rr_area = 0;
 		    
-	    FREE((char *)p1);
+	    freeMagic((char *)p1);
 	}
     }
     p2->br_this->rn_float.rn_area += height * (RIGHT(tile) - p2->br_loc.p_x);
-    FREE((char *)p2);
+    freeMagic((char *)p2);
     junk->breakList = NULL;
     return(merged);
 }
@@ -317,7 +321,7 @@ ResCalcNorthSouth(tile, pendingList, doneList, resList)
     if   (p1->br_next == NULL) 
     {
 	p1->br_this->rn_float.rn_area += width * (TOP(tile) - BOTTOM(tile));
-     	FREE((char *)p1);
+     	freeMagic((char *)p1);
 	junk->breakList = NULL;
 	return(merged);
     }
@@ -345,14 +349,14 @@ ResCalcNorthSouth(tile, pendingList, doneList, resList)
 	    {
 		 currNode = NULL;
 		 p1->br_next = p2->br_next;
-		 FREE((char *)p2);
+		 freeMagic((char *)p2);
 		 p2 = p1;
 	    }
 	    else if (p2->br_this == resCurrentNode)
 	    {
 		 currNode = p1->br_this;
 	    	 ResMergeNodes(p2->br_this,p1->br_this,pendingList,doneList);
-		 FREE((char *)p1);
+		 freeMagic((char *)p1);
 		 merged = TRUE;
 	    }
 	    else if (p1->br_this == resCurrentNode)
@@ -361,14 +365,14 @@ ResCalcNorthSouth(tile, pendingList, doneList, resList)
 		 p1->br_next = p2->br_next;
 	    	 ResMergeNodes(p1->br_this,p2->br_this,pendingList,doneList);
 		 merged = TRUE;
-		 FREE((char *)p2);
+		 freeMagic((char *)p2);
 		 p2 = p1;
 	    }
 	    else
 	    {
 		 currNode = p1->br_this;
 	    	 ResMergeNodes(p2->br_this,p1->br_this,pendingList,doneList);
-		 FREE((char *)p1);
+		 freeMagic((char *)p1);
 	    }
 		    
 	    /* 
@@ -393,18 +397,18 @@ ResCalcNorthSouth(tile, pendingList, doneList, resList)
 
 	else
 	{
-	    MALLOC(resResistor *, resistor, sizeof(resResistor));
+	    resistor = (resResistor *) mallocMagic((unsigned) (sizeof(resResistor)));
 	    resistor->rr_nextResistor = (*resList);
 	    resistor->rr_lastResistor = NULL;
 	    if ((*resList) != NULL) (*resList)->rr_lastResistor = resistor;
 	    (*resList) = resistor;
 	    resistor->rr_connection1 = p1->br_this;
 	    resistor->rr_connection2 = p2->br_this;
-	    MALLOC(resElement *, element, sizeof(resElement));
+	    element = (resElement *) mallocMagic((unsigned) (sizeof(resElement)));
 	    element->re_nextEl = p1->br_this->rn_re;
 	    element->re_thisEl = resistor;
 	    p1->br_this->rn_re = element;
-	    MALLOC(resElement *, element, sizeof(resElement));
+	    element = (resElement *) mallocMagic((unsigned) (sizeof(resElement)));
 	    element->re_nextEl = p2->br_this->rn_re;
 	    element->re_thisEl = resistor;
 	    p2->br_this->rn_re = element;
@@ -439,11 +443,11 @@ ResCalcNorthSouth(tile, pendingList, doneList, resList)
 	    resistor->rr_connection1->rn_float.rn_area += rArea;
 	    resistor->rr_connection2->rn_float.rn_area += rArea;
 	    resistor->rr_float.rr_area = 0;
-	    FREE((char *)p1);
+	    freeMagic((char *)p1);
 	}
     }
     p2->br_this->rn_float.rn_area += width * (TOP(tile) - p2->br_loc.p_y);
-    FREE((char *)p2);
+    freeMagic((char *)p2);
     junk->breakList = NULL;
     return(merged);
 }
@@ -489,7 +493,7 @@ ResCalcNearTransistor(tile, pendingList, doneList, resList)
      
      if   (junk->breakList->br_next == NULL) 
      {
-     	  FREE((char *)junk->breakList);
+     	  freeMagic((char *)junk->breakList);
 	  junk->breakList = NULL;
 	  return(merged);
      }
@@ -704,13 +708,13 @@ ResCalcNearTransistor(tile, pendingList, doneList, resList)
 			     	  if (p3 == NULL)
 				  {
 				       junk->breakList = p2->br_next;
-				       FREE((char *) p2);
+				       freeMagic((char *) p2);
 				       p2 = junk->breakList;
 				  }
 				  else
 				  {
 				       p3->br_next = p2->br_next;
-				       FREE((char *) p2);
+				       freeMagic((char *) p2);
 				       p2 = p3->br_next;
 				  }
 			     }
@@ -745,13 +749,13 @@ ResCalcNearTransistor(tile, pendingList, doneList, resList)
 			     	  if (p3 == NULL)
 				  {
 				       junk->breakList = p2->br_next;
-				       FREE((char *) p2);
+				       freeMagic((char *) p2);
 				       p2 = junk->breakList;
 				  }
 				  else
 				  {
 				       p3->br_next = p2->br_next;
-				       FREE((char *) p2);
+				       freeMagic((char *) p2);
 				       p2 = p3->br_next;
 				  }
 			     }
@@ -802,11 +806,11 @@ ResDoContacts(contact, nodes, resList)
 	int x = contact->cp_center.p_x;
 	int y = contact->cp_center.p_y;
 
-	MALLOC(resNode *,resptr,sizeof(resNode));
+	resptr = (resNode *) mallocMagic((unsigned) (sizeof(resNode)));
 	InitializeNode(resptr,x,y,RES_NODE_CONTACT);
 	ResAddToQueue(resptr,nodes);
 
- 	MALLOC(cElement *, ccell, sizeof(cElement));
+ 	ccell = (cElement *) mallocMagic((unsigned) (sizeof(cElement)));
 	ccell->ce_nextc = resptr->rn_ce;
 	resptr->rn_ce = ccell;
 	ccell->ce_thisc = contact;
@@ -853,13 +857,13 @@ ResDoContacts(contact, nodes, resList)
       	    int  y = contact->cp_center.p_y;
 	    Tile *tile = contact->cp_tile[tilenum];
 
-	    MALLOC(resNode *,resptr,sizeof(resNode));
+	    resptr = (resNode *) mallocMagic((unsigned) (sizeof(resNode)));
 	    InitializeNode(resptr,x,y,RES_NODE_CONTACT);
 	    ResAddToQueue(resptr,nodes);
 	  
  	    /* add contact pointer to node  */
 
-	    MALLOC(cElement *, ccell, sizeof(cElement));
+	    ccell = (cElement *) mallocMagic((unsigned) (sizeof(cElement)));
 	    ccell->ce_nextc = resptr->rn_ce;
 	    resptr->rn_ce = ccell;
 	    ccell->ce_thisc = contact;
@@ -872,7 +876,7 @@ ResDoContacts(contact, nodes, resList)
 
 	    if (tilenum > 0)
 	    {
-	        MALLOC(resResistor *, resistor, sizeof(resResistor));
+	        resistor = (resResistor *) mallocMagic((unsigned) (sizeof(resResistor)));
 	        resistor->rr_nextResistor = (*resList);
 	        resistor->rr_lastResistor = NULL;
 	        if ((*resList) != NULL) (*resList)->rr_lastResistor = resistor;
@@ -880,11 +884,11 @@ ResDoContacts(contact, nodes, resList)
 	        resistor->rr_connection1 = contact->cp_cnode[tilenum-1];
 	        resistor->rr_connection2 = contact->cp_cnode[tilenum];
 		    
-	        MALLOC(resElement *,element,sizeof(resElement));
+	        element = (resElement *) mallocMagic((unsigned) (sizeof(resElement)));
 	        element->re_nextEl = contact->cp_cnode[tilenum-1]->rn_re;
 	        element->re_thisEl = resistor;
 	        contact->cp_cnode[tilenum-1]->rn_re = element;
-	        MALLOC(resElement *,element,sizeof(resElement));
+	        element = (resElement *) mallocMagic((unsigned)(sizeof(resElement)));
 	        element->re_nextEl = contact->cp_cnode[tilenum]->rn_re;
 	        element->re_thisEl = resistor;
 	        contact->cp_cnode[tilenum]->rn_re = element;
