@@ -16,6 +16,14 @@ static char rcsid[] = "$Header: /ufs/repository/magic/resis/ResMerge.c,v 1.2 200
 
 TileTypeBitMask	ResNoMergeMask[NT];
 
+/* Forward declarations */
+
+extern void ResMergeNodes();
+extern void ResDeleteResPointer();
+extern void ResEliminateResistor();
+extern void ResCleanNode();
+extern void ResFixBreakPoint();
+
 
 /*
  *-------------------------------------------------------------------------
@@ -30,6 +38,8 @@ TileTypeBitMask	ResNoMergeMask[NT];
  *
  *-------------------------------------------------------------------------
  */
+
+void
 ResDoneWithNode(resptr)
 	resNode *resptr;
 
@@ -129,6 +139,7 @@ ResDoneWithNode(resptr)
  *------------------------------------------------------------------------
  */
 
+void
 ResFixRes(resptr,resptr2,resptr3,elimResis,newResis)
 	resNode *resptr,*resptr2,*resptr3;
 	resResistor *elimResis, *newResis; 
@@ -177,6 +188,8 @@ ResFixRes(resptr,resptr2,resptr3,elimResis,newResis)
  *
  *------------------------------------------------------------------------
  */
+
+void
 ResFixParallel(elimResis,newResis)
 	resResistor *elimResis,*newResis;
 
@@ -215,6 +228,7 @@ ResFixParallel(elimResis,newResis)
  *-------------------------------------------------------------------------
  */
 
+int
 ResSeriesCheck(resptr)
 	resNode		*resptr;
 
@@ -396,6 +410,8 @@ ResSeriesCheck(resptr)
  *
  *-------------------------------------------------------------------------
  */
+
+int
 ResParallelCheck(resptr)
 	resNode	*resptr;
 
@@ -454,11 +470,13 @@ ResParallelCheck(resptr)
  *
  *-------------------------------------------------------------------------
  */
+
+int
 ResTriangleCheck(resptr)
 	resNode		*resptr;
 
 {
-     register resResistor	*rr1,*rr2,*rr3;
+     resResistor	*rr1,*rr2,*rr3;
      int		status=UNTOUCHED;
      float		r1,r2,r3,denom;
      resNode		*n1,*n2,*n3;
@@ -520,7 +538,7 @@ ResTriangleCheck(resptr)
 			  rr2->rr_value = 0;
 			  rr3->rr_value = 0;
 	            }
-		    MALLOC(resNode  *, n3, sizeof(resNode));
+		    n3 = (resNode  *) mallocMagic((unsigned) (sizeof(resNode)));
 	      	    /*  Where should the new node be `put'? It */
 	            /* is arbitrarily assigned to the location */
 		    /* occupied by the first node.		 */
@@ -562,15 +580,15 @@ ResTriangleCheck(resptr)
 		     	   ResDeleteResPointer(rr3->rr_connection1,rr3);
 			   rr3->rr_connection1 = n3;
 		    }
-		    MALLOC(resElement *,element, sizeof(resElement));
+		    element = (resElement *) mallocMagic((unsigned)(sizeof(resElement)));
 		    element->re_nextEl = NULL;
 		    element->re_thisEl = rr1;
 		    n3->rn_re = element;
-		    MALLOC(resElement *,element, sizeof(resElement));
+		    element = (resElement *) mallocMagic((unsigned)(sizeof(resElement)));
 		    element->re_nextEl = n3->rn_re;
 		    element->re_thisEl = rr2;
 		    n3->rn_re = element;
-		    MALLOC(resElement *,element, sizeof(resElement));
+		    element = (resElement *) mallocMagic((unsigned)(sizeof(resElement)));
 		    element->re_nextEl = n3->rn_re;
 		    element->re_thisEl = rr3;
 		    n3->rn_re = element;
@@ -616,7 +634,8 @@ ResTriangleCheck(resptr)
  *---------------------------------------------------------------------------
  */
 
- ResMergeNodes(node1,node2,pendingList,doneList)
+void
+ResMergeNodes(node1,node2,pendingList,doneList)
  	resNode *node1,*node2,**pendingList,**doneList;
 	
  {
@@ -769,10 +788,10 @@ ResTriangleCheck(resptr)
       {
       	   ResRemoveFromQueue(node2,pendingList);
       }
-      if (node2->rn_client != (ClientData) NULL)
+      if (node2->rn_client != (ClientData)NULL)
       {
-	    FREE((char *)node2->rn_client);
-	    node2->rn_client = (ClientData) NULL;
+	    freeMagic((char *)node2->rn_client);
+	    node2->rn_client = (ClientData)NULL;
       }
       {
 	    node2->rn_re = (resElement *) MINFINITY;
@@ -782,7 +801,7 @@ ResTriangleCheck(resptr)
 	    node2->rn_more = (resNode   *) MINFINITY;
 	    node2->rn_less = (resNode   *) MINFINITY;
       }
-      FREE((char *)node2);
+      freeMagic((char *)node2);
  }
 
 
@@ -798,6 +817,8 @@ ResTriangleCheck(resptr)
  *
  *-------------------------------------------------------------------------
  */
+
+void
 ResDeleteResPointer(node,resistor)
 	resNode		*node;
 	resResistor	*resistor;
@@ -824,7 +845,7 @@ ResDeleteResPointer(node,resistor)
 	       /* pointers to structure.			      */
 	       rcell2->re_thisEl = NULL;
 	       rcell2->re_nextEl = NULL;
-	       FREE((char *)rcell2);
+	       freeMagic((char *)rcell2);
 	       break;
 	  }
 	  rcell1 = rcell2;
@@ -850,6 +871,8 @@ ResDeleteResPointer(node,resistor)
  *
  *-------------------------------------------------------------------------
  */
+
+void
 ResEliminateResistor(resistor,homelist)
 	resResistor *resistor,**homelist;
 
@@ -872,7 +895,7 @@ ResEliminateResistor(resistor,homelist)
      resistor->rr_lastResistor = NULL;
      resistor->rr_connection1 = NULL;
      resistor->rr_connection2 = NULL;
-     FREE((char *)resistor);
+     freeMagic((char *)resistor);
 }
 
 
@@ -884,11 +907,14 @@ ResEliminateResistor(resistor,homelist)
  *		the node is eradicated.
  *
  * Results:
+ *	None.
  *
  * Side Effects: frees memory
  *
  *-------------------------------------------------------------------------
  */
+
+void
 ResCleanNode(resptr,junk,homelist1,homelist2)
 	resNode *resptr;
 	int 	junk;
@@ -905,33 +931,33 @@ ResCleanNode(resptr,junk,homelist1,homelist2)
      {
 	     ccell = resptr->rn_ce;
 	     resptr->rn_ce = resptr->rn_ce->ce_nextc;
-	     FREE((char *)ccell);
+	     freeMagic((char *)ccell);
      }
      while (resptr->rn_je != NULL)
      {
 	     jcell = resptr->rn_je;
 	     resptr->rn_je = resptr->rn_je->je_nextj;
-	     FREE((char *)jcell->je_thisj);
-	     FREE((char *)jcell);
+	     freeMagic((char *)jcell->je_thisj);
+	     freeMagic((char *)jcell);
      }
      if (junk == TRUE)
      {
-	  if (resptr->rn_client != (ClientData) NULL)
+	  if (resptr->rn_client != (ClientData)NULL)
 	  {
-	       FREE((char *)resptr->rn_client);
-	       resptr->rn_client = (ClientData) NULL;
+	       freeMagic((char *)resptr->rn_client);
+	       resptr->rn_client = (ClientData)NULL;
 	  }
 	  while (resptr->rn_te != NULL)
           {
 	       tcell = resptr->rn_te;
 	       resptr->rn_te = resptr->rn_te->te_nextt;
-	       FREE((char *)tcell);
+	       freeMagic((char *)tcell);
           }
           while (resptr->rn_re != NULL)
           {
 	       rcell = resptr->rn_re;
 	       resptr->rn_re = resptr->rn_re->re_nextEl;
-	       FREE((char *)rcell);
+	       freeMagic((char *)rcell);
           }
 	  if (resptr->rn_less != NULL)
 	  {
@@ -964,7 +990,7 @@ ResCleanNode(resptr,junk,homelist1,homelist2)
 	       resptr->rn_more = (resNode   *) MINFINITY;
 	       resptr->rn_less = (resNode   *) MINFINITY;
 	  }
-	  FREE((char *)resptr);
+	  freeMagic((char *)resptr);
      }
 }
 
@@ -982,6 +1008,8 @@ ResCleanNode(resptr,junk,homelist1,homelist2)
  *
  *-------------------------------------------------------------------------
  */
+
+void
 ResFixBreakPoint(sourcelist,origNode,newNode)
 	Breakpoint	**sourcelist;
 	resNode		*origNode,*newNode;
@@ -1021,7 +1049,7 @@ ResFixBreakPoint(sourcelist,origNode,newNode)
 		    {
 			 bp4->br_crect = bp3->br_crect;
 		    }
-		    FREE((char *)bp3);
+		    freeMagic((char *)bp3);
 		    continue;
 	       }else
 	       {
