@@ -333,7 +333,7 @@ ResSimTransistor(line,rpersquare,ttype)
      char		*newattr,tmpattr[MAXTOKEN];
      static int		nowarning = TRUE;
      
-     MALLOC(RTran *, transistor, sizeof(RTran));
+     transistor = (RTran *) mallocMagic((unsigned) (sizeof(RTran)));
      if ((line[RTRAN_WIDTH][0] == '\0') || (line[RTRAN_LENGTH][0] == '\0'))
      {
      	  TxError("error in input file:\n");
@@ -383,7 +383,7 @@ ResSimTransistor(line,rpersquare,ttype)
 	  }
 	  tmpattr[k++]='"';
 	  tmpattr[k++]='\0';
-	  MALLOC(char *,newattr,k);
+	  newattr = (char *) mallocMagic((unsigned) k);
 	  strncpy(newattr,tmpattr,k);
 	  switch (line[i][0])
 	  {
@@ -434,7 +434,7 @@ ResSimNewNode(line,type,transistor)
      }
      entry = HashFind(&ResNodeTable,line);
      node = ResInitializeNode(entry);
-     MALLOC(tranPtr *,tptr, sizeof(tranPtr));
+     tptr = (tranPtr *) mallocMagic((unsigned) (sizeof(tranPtr)));
      tptr->thisTran = transistor;
      tptr->nextTran = node->firstTran;
      node->firstTran = tptr;
@@ -766,7 +766,7 @@ ResSimProcessFixPoints(filename)
 	       			line[RES_EXT_ATTR_TEXT]);
 	       *label ='\0';
 	  }
-	  MALLOC(ResFixPoint *, thisfix, sizeof(ResFixPoint)+strlen(label));
+	  thisfix = (ResFixPoint *) mallocMagic((unsigned) (sizeof(ResFixPoint)+strlen(label)));
 	  thisfix->fp_next = ResFixList;
 	  ResFixList = thisfix;
 	  thisfix->fp_loc.p_x = atoi(line[RES_EXT_ATTR_X]);
@@ -841,7 +841,7 @@ ResInitializeNode(entry)
 
      if ((node = (ResSimNode *) HashGetValue(entry)) == NULL)
      {
-	  MALLOC(ResSimNode *, node, sizeof(ResSimNode));
+	  node = (ResSimNode *) mallocMagic((unsigned)(sizeof(ResSimNode)));
 	  HashSetValue(entry,(char *) node);
 	  node->nextnode = ResOriginalNodes;
 	  ResOriginalNodes = node;
