@@ -21,12 +21,12 @@ static char rcsid[] = "$Header: /ufs/repository/magic/resis/ResRex.c,v 1.3 2001/
 #else
 #include <strings.h>
 #endif
-
 #include "misc/magic.h"
 #include "utils/geometry.h"
 #include "utils/geofast.h"
 #include "tiles/tile.h"
 #include "utils/hash.h"
+#include "undo/undo.h"
 #include "database/database.h"
 #include "utils/malloc.h"
 #include "textio/textio.h"
@@ -57,9 +57,9 @@ static char rcsid[] = "$Header: /ufs/repository/magic/resis/ResRex.c,v 1.3 2001/
 HashTable 	ResNodeTable;   /* Hash table of sim file nodes   */
 RTran		*ResTranList;	/* Linked list of Sim transistors */
 ResGlobalParams	gparams;	/* Junk passed between 		  */
-					/* ResCheckSimNodes and 	  */
-					/* ResExtractNet.		  */
-int		Maxtnumber;     	/*maximum transistor number 	  */
+				/* ResCheckSimNodes and 	  */
+				/* ResExtractNet.		  */
+int		Maxtnumber;     /*maximum transistor number 	  */
 extern ResSimNode	*ResOriginalNodes;	/*Linked List of Nodes		  */
 int		resNodeNum;
 
@@ -150,9 +150,9 @@ ExtResisForDef(celldef, resisdata)
 	{
 	    oldtptr = tptr;
 	    tptr = tptr->nextTran;
-	    FREE((char *)oldtptr);
+	    freeMagic((char *)oldtptr);
 	}
-	FREE((char *) node);
+	freeMagic((char *) node);
     }
     HashKill(&ResNodeTable);
     while (ResTranList != NULL)
@@ -161,10 +161,10 @@ ExtResisForDef(celldef, resisdata)
 	ResTranList = ResTranList->nextTran;
 	if (oldTran->layout != NULL)
 	{
-	    FREE((char *)oldTran->layout);
+	    freeMagic((char *)oldTran->layout);
 	    oldTran->layout = NULL;
 	}
-	FREE((char *)oldTran);
+	freeMagic((char *)oldTran);
     }
 }
 
@@ -175,20 +175,21 @@ ExtResisForDef(celldef, resisdata)
  * CmdExtResis--  reads in sim file and layout, and produces patches to the
  *	.ext files and .sim files that include resistors.
  *
- * Results: returns 0 if it completes correctly.
+ * Results:
+ *	None.
  *
  * Side Effects: Produces .res.sim file and .res.ext file for all nets that
  *	require resistors.
  *
  *-------------------------------------------------------------------------
  */
-/*ARGSUSED*/
-int
+
+void
 CmdExtResis(win, cmd)
 	MagWindow *win;
 	TxCommand *cmd;
 {
-    int		i, j, k, option;
+    int		i, j, k, option, value;
     static int	init=1;
     static float tolerance, tdiTolerance, fhFrequency;
     CellDef	*mainDef;
@@ -198,70 +199,97 @@ CmdExtResis(win, cmd)
 
     extern int resSubcircuitFunc();	/* Forward declaration */
 
-    static char *cmdExtresisCmd[] = 
+    static char *onOff[] =
     {
-    	 "tolerance value      set ratio between resistor and transistor tol.",
-	 "all 		       extract all the nets",
-	 "simplify [on/off]    turn on/off simplification of resistor nets",
-	 "extout   [on/off]    turn on/off writing of .res.ext file",
-	 "lumped   [on/off]    turn on/off writing of updated lumped resistances",
-	 "silent   [on/off]    turn on/off printing of net statistics",
-	 "skip     mask        don't extract these types",
-	 "box      type        extract the signal under the box on layer type",
-	 "cell	   cellname    extract the network for the cell named cellname",
-	 "geometry [freq]      extract subcircuit network geometry into .fh file",
-	 "help                 print this message",
-#ifdef LAPLACE
-	 "laplace  [on/off]    solve Laplace's equation using FEM",
-#endif
-	 NULL
+	"on",
+	"off",
+	NULL
     };
 
-#define RES_AMBIG		-1
-#define RES_BAD			-2
-#define RES_TOL			0
-#define RES_ALL			1
-#define RES_SIMP		2
-#define RES_EXTOUT		3
-#define RES_LUMPED		4
-#define RES_SILENT		5
-#define RES_SKIP		6
-#define RES_BOX			7
-#define RES_CELL		8
-#define RES_GEOMETRY		9
-#define RES_HELP		10
+    static char *cmdExtresisCmd[] = 
+    {
+	"tolerance value      set ratio between resistor and transistor tol.",
+	"all 		       extract all the nets",
+	"simplify [on/off]    turn on/off simplification of resistor nets",
+	"extout   [on/off]    turn on/off writing of .res.ext file",
+	"lumped   [on/off]    turn on/off writing of updated lumped resistances",
+	"silent   [on/off]    turn on/off printing of net statistics",
+	"skip     mask        don't extract these types",
+	"box      type        extract the signal under the box on layer type",
+	"cell	   cellname    extract the network for the cell named cellname",
+	"geometry [freq]      extract subcircuit network geometry into .fh file",
+	"help                 print this message",
 #ifdef LAPLACE
-#define RES_LAPLACE		11
-#define RES_RUN			12
+	"laplace  [on/off]    solve Laplace's equation using FEM",
+#endif
+	NULL
+    };
+
+#define RES_AMBIG	-1
+#define RES_BAD		-2
+#define RES_TOL		0
+#define RES_ALL		1
+#define RES_SIMP	2
+#define RES_EXTOUT	3
+#define RES_LUMPED	4
+#define RES_SILENT	5
+#define RES_SKIP	6
+#define RES_BOX		7
+#define RES_CELL	8
+#define RES_GEOMETRY	9
+#define RES_HELP	10
+#ifdef LAPLACE
+#define RES_LAPLACE	11
+#define RES_RUN		12
 #else
-#define RES_RUN			11
+#define RES_RUN		11
 #endif
 
     if (init)
     {
-         for (i=0;i != NT;i++)
-    	 {
-              TTMaskZero(&(ResCopyMask[i]));
-	      TTMaskSetMask(&ResCopyMask[i],&DBConnectTbl[i]);
-     	 }
-	 tolerance = 1;
-	 tdiTolerance = 1;
-	 fhFrequency = 10e6;	/* 10 MHz default */
-    	 
-	 init = 0;
+	for (i = 0; i < NT; i++)
+	{
+            TTMaskZero(&(ResCopyMask[i]));
+	    TTMaskSetMask(&ResCopyMask[i], &DBConnectTbl[i]);
+     	}
+	tolerance = 1;
+	tdiTolerance = 1;
+	fhFrequency = 10e6;	/* 10 MHz default */
+	init = 0;
     }
+
     if (ToolGetBoxWindow((Rect *) NULL, (int *) NULL) == NULL)
     {
 	TxError("Sorry, the box must appear in one of the windows.\n");
 	return;
     }
 
-    option= (cmd->tx_argc >1) ? Lookup(cmd->tx_argv[1],cmdExtresisCmd):RES_RUN;
+    option = (cmd->tx_argc > 1) ? Lookup(cmd->tx_argv[1], cmdExtresisCmd)
+		: RES_RUN;
+
+    switch (option)
+    {
+	case RES_SIMP:
+	case RES_EXTOUT:
+	case RES_LUMPED:
+	case RES_SILENT:
+	    if (cmd->tx_argc > 2)
+	    {
+		Lookup(cmd->tx_argv[2], onOff);
+		if (value < 0)
+		{
+		    TxError("Value must be either \"on\" or \"off\".\n");
+		    return;
+		}
+	    }
+	    break;
+    }
+
     switch (option)
     {
 	 case RES_TOL:
 	      ResOptionsFlags |=  ResOpt_ExplicitRtol;
-	      if (cmd->tx_argc > 2) 
+	      if (cmd->tx_argc >2) 
 	      {
 		   tolerance = MagAtof(cmd->tx_argv[2]);
 		   if (tolerance <= 0)
@@ -271,14 +299,11 @@ CmdExtResis(win, cmd)
 		   }
 		   tdiTolerance = tolerance;
 	      }
-	      else
-	      {
-		   TxPrintf("Tolerance is %2.1f\n", tdiTolerance);
-	      }
 	      return;
 	 case RES_ALL:
 	      ResOptionsFlags |= ResOpt_ExtractAll;
 	      break;
+
 	 case RES_GEOMETRY:
 	      if (cmd->tx_argc == 3)
 	      {
@@ -295,125 +320,147 @@ CmdExtResis(win, cmd)
 	      ResOptionsFlags &= ~(ResOpt_DoExtFile | ResOpt_DoLumpFile
 			| ResOpt_Simplify | ResOpt_Tdi);
 	      return;
-	 case RES_SIMP:
-	      if (cmd->tx_argc == 2 || strcmp(cmd->tx_argv[2],"on")==0)
-	      	   ResOptionsFlags |= ResOpt_Simplify|ResOpt_Tdi; 
-	      else
-	      	   ResOptionsFlags &= ~(ResOpt_Simplify|ResOpt_Tdi);
-	      return;
-	 case RES_EXTOUT:
-	      if (cmd->tx_argc == 2 || strcmp(cmd->tx_argv[2],"on")==0)
-	      	   ResOptionsFlags |= ResOpt_DoExtFile;
-	      else
-	      	   ResOptionsFlags &= ~ResOpt_DoExtFile;
-	      return;
-	 case RES_LUMPED:
-	      if (cmd->tx_argc == 2 || strcmp(cmd->tx_argv[2],"on")==0)
-	      	   ResOptionsFlags |= ResOpt_DoLumpFile;
-	      else
-	      	   ResOptionsFlags &= ~ResOpt_DoLumpFile;
-	      return;
-	 case RES_SILENT:
-	      if (cmd->tx_argc == 2 || strcmp(cmd->tx_argv[2],"on")==0)
-	      	   ResOptionsFlags |= ResOpt_RunSilent;
-	      else
-	      	   ResOptionsFlags &= ~ResOpt_RunSilent;
-	      return;
-	 case RES_SKIP:
-	      if (cmd->tx_argc >2) 
-	      {
-		   j = DBTechNoisyNameType(cmd->tx_argv[2]);
-		   if (j >= 0) for (k = TT_TECHDEPBASE; k < TT_MAXTYPES; k++)
-		   {
-			TTMaskClearType(&ResCopyMask[k],j);
-		   }
-		   TTMaskZero(&(ResCopyMask[j]));
-	      }
-	      else
-	      {
-    		   for (i=0;i != NT;i++)
-    		   {
-         		TTMaskZero(&(ResCopyMask[i]));
-	 		TTMaskSetMask(&ResCopyMask[i],&DBConnectTbl[i]);
-     		   }
-	      }
-	      return;
-	 case RES_HELP:
-	      {
-		     for (i=0; cmdExtresisCmd[i] != NULL;i++)
-		     {
-		     	  TxPrintf("%s\n",cmdExtresisCmd[i]);
-		     }
-	      }
-	      return;
-	 case RES_BOX:
-	      {
-		   TileType	tt;
-		   CellDef	*def;
-		   Rect		rect;
-		   int		oldoptions;
-		   ResFixPoint	fp;
 
-	     	   if (cmd->tx_argc != 3) return;
-		   tt = DBTechNoisyNameType(cmd->tx_argv[2]);
-		   if (tt <= 0 || ToolGetBox(&def,&rect)== FALSE) return;
-		   gparams.rg_tranloc = &rect.r_ll;
-		   gparams.rg_ttype = tt;
-		   gparams.rg_status = DRIVEONLY;
-		   oldoptions = ResOptionsFlags;
-		   ResOptionsFlags = ResOpt_DoSubstrate|ResOpt_Signal|ResOpt_Box;
+	case RES_SIMP:
+	    if (cmd->tx_argc == 2)
+	    {
+		value = (ResOptionsFlags & (ResOpt_Simplify | ResOpt_Tdi)) ?
+			TRUE : FALSE;
+		TxPrintf("%s\n", onOff[value]);
+	    }
+	    else if (value)
+	      	   ResOptionsFlags |= ResOpt_Simplify | ResOpt_Tdi; 
+	    else
+	      	   ResOptionsFlags &= ~(ResOpt_Simplify | ResOpt_Tdi);
+	    return;
+	case RES_EXTOUT:
+	    if (cmd->tx_argc == 2)
+	    {
+		value = (ResOptionsFlags & ResOpt_DoExtFile) ?
+			TRUE : FALSE;
+		TxPrintf("%s\n", onOff[value]);
+	    }
+	    else if (value)
+	      	   ResOptionsFlags |= ResOpt_DoExtFile;
+	    else
+	      	   ResOptionsFlags &= ~ResOpt_DoExtFile;
+	    return;
+	case RES_LUMPED:
+	    if (cmd->tx_argc == 2)
+	    {
+		value = (ResOptionsFlags & ResOpt_DoLumpFile) ?
+			TRUE : FALSE;
+		TxPrintf("%s\n", onOff[value]);
+	    }
+	    else if (value)
+	      	   ResOptionsFlags |= ResOpt_DoLumpFile;
+	    else
+	      	   ResOptionsFlags &= ~ResOpt_DoLumpFile;
+	    return;
+	case RES_SILENT:
+	    if (cmd->tx_argc == 2)
+	    {
+		value = (ResOptionsFlags & ResOpt_RunSilent) ?
+			TRUE : FALSE;
+		TxPrintf("%s\n", onOff[value]);
+	    }
+	    else if (value)
+	      	   ResOptionsFlags |= ResOpt_RunSilent;
+	    else
+	      	   ResOptionsFlags &= ~ResOpt_RunSilent;
+	    return;
+
+	case RES_SKIP:
+	    if (cmd->tx_argc > 2) 
+	    {
+		j = DBTechNoisyNameType(cmd->tx_argv[2]);
+		if (j >= 0)
+		    for (k = TT_TECHDEPBASE; k < TT_MAXTYPES; k++)
+			TTMaskClearType(&ResCopyMask[k], j);
+		TTMaskZero(&(ResCopyMask[j]));
+	    }
+	    else
+	    {
+    		for (i = 0; i != NT; i++)
+    		{
+		    TTMaskZero(&(ResCopyMask[i]));
+		    TTMaskSetMask(&ResCopyMask[i], &DBConnectTbl[i]);
+     		}
+	    }
+	    return;
+
+	case RES_HELP:
+	    for (i = 0; cmdExtresisCmd[i] != NULL; i++)
+		TxPrintf("%s\n", cmdExtresisCmd[i]);
+	    return;
+
+	case RES_BOX:
+	    {
+		 TileType	tt;
+		 CellDef	*def;
+		 Rect		rect;
+		 int		oldoptions;
+		 ResFixPoint	fp;
+
+	     	 if (cmd->tx_argc != 3) return;
+		 tt = DBTechNoisyNameType(cmd->tx_argv[2]);
+		 if (tt <= 0 || ToolGetBox(&def,&rect)== FALSE) return;
+		 gparams.rg_tranloc = &rect.r_ll;
+		 gparams.rg_ttype = tt;
+		 gparams.rg_status = DRIVEONLY;
+		 oldoptions = ResOptionsFlags;
+		 ResOptionsFlags = ResOpt_DoSubstrate|ResOpt_Signal|ResOpt_Box;
 #ifdef LAPLACE
-		   ResOptionsFlags |= (oldoptions & (ResOpt_CacheLaplace|ResOpt_DoLaplace));
-		   LaplaceMatchCount = 0;
-		   LaplaceMissCount = 0;
+		 ResOptionsFlags |= (oldoptions & (ResOpt_CacheLaplace|ResOpt_DoLaplace));
+		 LaplaceMatchCount = 0;
+		 LaplaceMissCount = 0;
 #endif
-		   fp.fp_ttype = tt;
-		   fp.fp_loc = rect.r_ll;
-		   fp.fp_next = NULL;
-		   if (ResExtractNet(&fp, &gparams, NULL) != 0) return;
-		   ResPrintResistorList(stdout,ResResList);
-		   ResPrintTransistorList(stdout,ResTransList);
+		 fp.fp_ttype = tt;
+		 fp.fp_loc = rect.r_ll;
+		 fp.fp_next = NULL;
+		 if (ResExtractNet(&fp, &gparams, NULL) != 0) return;
+		 ResPrintResistorList(stdout,ResResList);
+		 ResPrintTransistorList(stdout,ResTransList);
 #ifdef LAPLACE
-		   if (ResOptionsFlags & ResOpt_DoLaplace)
-		   {
-		   	TxPrintf("Laplace   solved: %d matched %d\n",
+		if (ResOptionsFlags & ResOpt_DoLaplace)
+		{
+		    TxPrintf("Laplace   solved: %d matched %d\n",
 				LaplaceMissCount,LaplaceMatchCount);
-		   }
+		}
 #endif
 		   
-		   ResOptionsFlags = oldoptions;
-		   return;
-	      }
-	 case RES_CELL:
-	      selectedUse = CmdGetSelectedCell((Transform *) NULL);
-	      if (selectedUse == NULL)
-	      {
-		  TxError("No cell selected\n");
-		  return;  
-	      }
-	      mainDef = selectedUse->cu_def;
-	      ResOptionsFlags &= ~ResOpt_ExtractAll;
-	      break;
+		ResOptionsFlags = oldoptions;
+		return;
+	    }
+	case RES_CELL:
+	    selectedUse = CmdGetSelectedCell((Transform *) NULL);
+	    if (selectedUse == NULL)
+	    {
+		TxError("No cell selected\n");
+		return;  
+	    }
+	    mainDef = selectedUse->cu_def;
+	    ResOptionsFlags &= ~ResOpt_ExtractAll;
+	    break;
 
-	 case RES_RUN:
-	      ResOptionsFlags &= ~ResOpt_ExtractAll;
-	      break;
+	case RES_RUN:
+	    ResOptionsFlags &= ~ResOpt_ExtractAll;
+	    break;
 #ifdef LAPLACE
-	 case RES_LAPLACE:
-	      LaplaceParseString(cmd);
-	      return;
+	case RES_LAPLACE:
+	    LaplaceParseString(cmd);
+	    return;
 #endif
-	 case RES_AMBIG:
-  	      TxPrintf("Ambiguous option: %s\n",cmd->tx_argv[1]);
-	      TxFlushOut();
-	      return;
-	 case RES_BAD:
-  	      TxPrintf("Unknown option: %s\n",cmd->tx_argv[1]);
-	      TxFlushOut();
-	      return;
-	 default:
-	      return;
-
+	case RES_AMBIG:
+  	    TxPrintf("Ambiguous option: %s\n",cmd->tx_argv[1]);
+	    TxFlushOut();
+	    return;
+	case RES_BAD:
+  	    TxPrintf("Unknown option: %s\n",cmd->tx_argv[1]);
+	    TxFlushOut();
+	    return;
+	default:
+	    return;
     }
     
 #ifdef LAPLACE
@@ -460,7 +507,13 @@ CmdExtResis(win, cmd)
  *-------------------------------------------------------------------------
  *
  * resSubcircuitFunc --
+ *	For each encountered cell, call the resistance extractor.
  *
+ * Results:
+ *	Always return 0 to keep search alive.
+ *
+ * Side Effects:
+ *	Does resistance extraction for an entire cell.
  *	
  *-------------------------------------------------------------------------
  */
@@ -1080,7 +1133,8 @@ ResFixUpConnections(simTran, layoutTran, simNode, nodename)
  *
  *  ResFixTranName-- Moves transistor connection to new node.
  *
- * Results: returns zero if node is added correctly, one otherwise.
+ * Results:
+ *	None.
  *
  * Side Effects: May create a new node. Creates a new transistor pointer.
  *
@@ -1110,7 +1164,7 @@ ResFixTranName(line,type,transistor,layoutnode)
         entry = HashFind(&ResNodeTable,line);
         node = ResInitializeNode(entry);
     }
-    MALLOC(tranPtr *,tptr, sizeof(tranPtr));
+    tptr = (tranPtr *) mallocMagic((unsigned) (sizeof(tranPtr)));
     tptr->thisTran = transistor;
     tptr->nextTran = node->firstTran;
     node->firstTran = tptr;
