@@ -24,7 +24,6 @@ static char rcsid[] = "$Header: /ufs/repository/magic/resis/ResSimple.c,v 1.2 20
 #include "utils/geofast.h"
 #include "tiles/tile.h"
 #include "utils/hash.h"
-#include "utils/doubleint.h"
 #include "utils/heap.h"
 #include "database/database.h"
 #include "utils/malloc.h"
@@ -294,7 +293,7 @@ ResSimplifyNet(nodelist,biglist,reslist,tolerance)
 	  {
 	       resistor1->rr_connection2 = node2;
 	  }
-          MALLOC(resElement *,resisptr,sizeof(resElement));
+          resisptr = (resElement *) mallocMagic((unsigned)(sizeof(resElement)));
           resisptr->re_thisEl = resistor1;
           resisptr->re_nextEl = node2->rn_re;
           node2->rn_re = resisptr;
@@ -739,7 +738,7 @@ ResCalculateChildCapacitance(me)
      {
 	  return(-1);
      }
-     MALLOC(RCDelayStuff *, myC, sizeof(RCDelayStuff));
+     myC = (RCDelayStuff *) mallocMagic((unsigned) (sizeof(RCDelayStuff)));
      me->rn_client = (ClientData) myC;
 
      /* This following assumes that ResDistributeCapacitance has been run */
