@@ -17,7 +17,6 @@ static char rcsid[] = "$Header: /ufs/repository/magic/resis/ResUtils.c,v 1.2 200
 #include "tiles/tile.h"
 #include "utils/hash.h"
 #include "database/database.h"
-#include "database/databaseInt.h"
 #include "utils/malloc.h"
 #include "textio/textio.h"
 #include "extract/extract.h"
@@ -27,7 +26,7 @@ static char rcsid[] = "$Header: /ufs/repository/magic/resis/ResUtils.c,v 1.2 200
 #include "utils/stack.h"
 #include "tech/tech.h"
 #include "textio/txcommands.h"
-#include	"resis/resis.h"
+#include "resis/resis.h"
 
 
 /*
@@ -37,7 +36,7 @@ static char rcsid[] = "$Header: /ufs/repository/magic/resis/ResUtils.c,v 1.2 200
  *	contact structure.
  * 
  *
- * Results: Always returns null.
+ * Results: Always returns NULL (in the form of a Region pointer)
  *
  * Side effects:
  *	Memory is allocated by ResFirst.
@@ -47,13 +46,14 @@ static char rcsid[] = "$Header: /ufs/repository/magic/resis/ResUtils.c,v 1.2 200
  *
  * -------------------------------------------------------------------------
  */
+
 Region *
 ResFirst(tile, arg)
     Tile *tile;
     FindRegion *arg;
 {
-    register ResContactPoint *reg;
-    register TileType t;
+    ResContactPoint *reg;
+    TileType t;
     int i;
     
 #ifdef NONMANHATTAN
@@ -65,9 +65,9 @@ ResFirst(tile, arg)
 #endif
     t = TiGetType(tile);
 
-    if ( (DBConnPlanes[t] !=0) && TTMaskHasType(&DBContactBits,t))
+    if ((DBConnPlanes[t] !=0) && DBIsContact(t))
     {
-	    MALLOC(ResContactPoint *, reg, sizeof(ResContactPoint));
+	    reg = (ResContactPoint *) mallocMagic((unsigned) (sizeof(ResContactPoint)));
 	    reg->cp_center.p_x = (LEFT(tile)+RIGHT(tile))>>1;
 	    reg->cp_center.p_y = (TOP(tile)+BOTTOM(tile))>>1;
 	    reg->cp_status = FALSE;
@@ -89,7 +89,7 @@ ResFirst(tile, arg)
 	    reg->cp_nextcontact = (ResContactPoint *) arg->fra_region;
 	    arg->fra_region = (Region *) reg;
     }
-return((Region *) NULL);
+    return((Region *) NULL);
 }
 
 /*
@@ -106,7 +106,7 @@ return((Region *) NULL);
  *
  * -------------------------------------------------------------------------
  */
-/*ARGSUSED*/
+
 int
 ResEach(tile, pNum,arg)
     Tile *tile;
@@ -137,6 +137,7 @@ ResEach(tile, pNum,arg)
  *
  *-------------------------------------------------------------------------
  */
+
 int
 ResAddPlumbing(tile, arg)
      Tile	*tile;
@@ -168,7 +169,7 @@ ResAddPlumbing(tile, arg)
      	  junk2 = resAddField(tile);
 	  if (TTMaskHasType(&(ExtCurStyle->exts_transMask), loctype))
 	  {
-   	       MALLOC(resTransistor *, resFet,sizeof(resTransistor));
+   	       resFet = (resTransistor *) mallocMagic((unsigned)(sizeof(resTransistor)));
 	       {
 	       	    int	   i;
 		    for (i=0; i != RT_TERMCOUNT;i++)
@@ -528,7 +529,7 @@ ResAddPlumbing(tile, arg)
  *
  *-------------------------------------------------------------------------
  */
-/*ARGSUSED*/
+
 int
 ResRemovePlumbing(tile, arg)
      Tile	*tile;
@@ -538,7 +539,7 @@ ResRemovePlumbing(tile, arg)
      
      if (tile->ti_client != (ClientData) MINFINITY)
      {
-          FREE(((char *)(tile->ti_client)));
+          freeMagic(((char *)(tile->ti_client)));
 	  tile->ti_client = (ClientData) MINFINITY;
      }
      return(0);
@@ -563,89 +564,96 @@ ResRemovePlumbing(tile, arg)
  *-------------------------------------------------------------------------
  */
 
-ResPreProcessTransistors(TileList,TransistorList,Def)
-	ResTranTile		*TileList;
-	resTransistor		*TransistorList;
-	CellDef			*Def;
-
+void
+ResPreProcessTransistors(TileList, TransistorList, Def)
+    ResTranTile		*TileList;
+    resTransistor	*TransistorList;
+    CellDef		*Def;
 {
-     Tile		*tile;
-     ResTranTile	*oldTile;
-     tileJunk		*tstruct;
-     TileType		tt;
-     int		pNum;
+    Tile	*tile;
+    ResTranTile	*oldTile;
+    tileJunk	*tstruct;
+    TileType	tt, residue;
+    int		pNum;
      
-     while (TileList != (ResTranTile *) NULL)
-     {
-	  tt = TileList->type;
-	  pNum = DBPlane(tt);
-	  if (!TTMaskHasType(&ExtCurStyle->exts_transMask,tt)) 
-	  /* this layer must be a contact; find which image is transistor. */
-	  {
-	       LayerInfo	*lp = &dbLayerInfo[tt];
-	       int		origin;
+    while (TileList != (ResTranTile *) NULL)
+    {
+	tt = TileList->type;
+	if (DBIsContact(tt))
+	{
+	    /* Find which residue of the contact is a transistor type. */
+	    TileTypeBitMask *ttresidues = DBResidueMask(tt);
 	       
-	       for (origin = 0; origin != lp->l_nresidues; origin++)
-	       {
-	       	    if (lp->l_images[origin] == tt) break;
-	       }
-	       pNum = (TTMaskHasType(&ExtCurStyle->exts_transMask,
-			lp->l_images[origin+1]))?pNum+1:pNum-1;
-	  }
-	  tile = (Def->cd_planes[pNum])->pl_hint;
-	  GOTOPOINT(tile, &(TileList->area.r_ll));
+	    for (residue = TT_TECHDEPBASE; residue < DBNumTypes; residue++)
+	    {
+		if (TTMaskHasType(ttresidues, residue))
+		{
+		    if (TTMaskHasType(&ExtCurStyle->exts_transMask, residue))
+		    {
+			pNum = DBPlane(residue);
+			break;
+		    }
+		}
+	    }
+	}
+	else
+	    pNum = DBPlane(tt);		/* always correct for non-contact types */
 
-	  tt = TiGetType(tile);
-	  tstruct = (tileJunk *) tile->ti_client;
-	  if (!TTMaskHasType(&ExtCurStyle->exts_transMask, tt) ||
-	  	tstruct->transistorList == NULL)
-	  {
-	       TxError("Bad Transistor Location at %d,%d\n",
-	       				TileList->area.r_ll.p_x,
-	       				TileList->area.r_ll.p_y);
-	  }
-	  else
-	  if ((tstruct->tj_status & RES_TILE_MARK) == 0)
-	  {
-	       resTransistor	*rt = tstruct->transistorList;
+	tile = (Def->cd_planes[pNum])->pl_hint;
+	GOTOPOINT(tile, &(TileList->area.r_ll));
 
-	       tstruct->tj_status |= RES_TILE_MARK;
-	       rt->rt_perim+=TileList->perim;
-	       rt->rt_length+=TileList->overlap;
-	       rt->rt_area+=
-	       		(TileList->area.r_xtop-TileList->area.r_xbot)*
-			(TileList->area.r_ytop-TileList->area.r_ybot);
-	       rt->rt_tiles++;
-	  }
-	  oldTile = TileList;
-	  TileList = TileList->nextTran;
-	  FREE((char *)oldTile);
-     }
-     for(; TransistorList != NULL;TransistorList = TransistorList->rt_nextTran)
-     {
-     	  int width  = TransistorList->rt_perim;
-	  int length = TransistorList->rt_length;
-	  if (TransistorList->rt_tiles != 0)
-	  {
-	       if (length)
-	       {
-	            TransistorList->rt_length = (float) length/((float)((TransistorList->rt_tiles)<<1));
-	            TransistorList->rt_width = (width-length)>>1;
-	       }
-	       else
-	       {
-	       	    double perimeter = TransistorList->rt_perim;
-		    double area = TransistorList->rt_area;
+	tt = TiGetType(tile);
+	tstruct = (tileJunk *) tile->ti_client;
+
+	if (!TTMaskHasType(&ExtCurStyle->exts_transMask, tt) ||
+				tstruct->transistorList == NULL)
+	{
+	    TxError("Bad Transistor Location at %d,%d\n",
+			TileList->area.r_ll.p_x,
+			TileList->area.r_ll.p_y);
+	}
+	else if ((tstruct->tj_status & RES_TILE_MARK) == 0)
+	{
+	    resTransistor	*rt = tstruct->transistorList;
+
+	    tstruct->tj_status |= RES_TILE_MARK;
+	    rt->rt_perim += TileList->perim;
+	    rt->rt_length += TileList->overlap;
+	    rt->rt_area += (TileList->area.r_xtop - TileList->area.r_xbot)
+			* (TileList->area.r_ytop - TileList->area.r_ybot);
+	    rt->rt_tiles++;
+	}
+	oldTile = TileList;
+	TileList = TileList->nextTran;
+	freeMagic((char *)oldTile);
+    }
+
+    for(; TransistorList != NULL;TransistorList = TransistorList->rt_nextTran)
+    {
+     	int width  = TransistorList->rt_perim;
+	int length = TransistorList->rt_length;
+	if (TransistorList->rt_tiles != 0)
+	{
+	    if (length)
+	    {
+	        TransistorList->rt_length = (float) length /
+			((float)((TransistorList->rt_tiles) << 1));
+	        TransistorList->rt_width = (width-length) >> 1;
+	    }
+	    else
+	    {
+	       	double perimeter = TransistorList->rt_perim;
+		double area = TransistorList->rt_area;
 		    
-		    perimeter /= 4.0;
+		perimeter /= 4.0;
 
-		    TransistorList->rt_width=
-		    	perimeter+sqrt(perimeter*perimeter-area);
-		    TransistorList->rt_length= 
-		    	(TransistorList->rt_perim-2*TransistorList->rt_width)>>1;
-	       }
-	  }
-     }
+		TransistorList->rt_width = perimeter +
+			sqrt(perimeter * perimeter-area);
+		TransistorList->rt_length = (TransistorList->rt_perim
+			- 2 * TransistorList->rt_width) >> 1;
+	    }
+	}
+    }
 }
 
 
@@ -682,6 +690,8 @@ ResAddToQueue(node,list)
  *
  *-------------------------------------------------------------------------
  */
+
+void
 ResRemoveFromQueue(node,list)
 	resNode	*node,**list;
 
@@ -717,7 +727,7 @@ resAddField(tile)
         tileJunk *Junk;
 	if ((Junk=(tileJunk *)tile->ti_client) == (tileJunk *) MINFINITY)
 	{
-     	      MALLOC(tileJunk *,Junk,sizeof(tileJunk));
+     	      Junk = (tileJunk *) mallocMagic((unsigned) (sizeof(tileJunk)));
 	      ResJunkInit(Junk);
 	      tile->ti_client = (ClientData) Junk;
 	}
