@@ -6,12 +6,12 @@
    refer to both before and after processing.
 */
 
+#ifndef _RESIS_H
+#define _RESIS_H
+
 #define LAYERS_PER_CONTACT 3
 #define TILES_PER_JUNCTION 2
 
-#define TRUE 1
-#define FALSE 0
-
 typedef struct contactpoint
 {
      struct contactpoint	*cp_nextcontact;/* Next contact in linked */
@@ -630,7 +630,7 @@ extern void			ResSortBreaks();
 	  (node)->rn_id=0;\
 	  (node)->rn_float.rn_area = 0.0;\
 	  (node)->rn_name = NULL;\
-	  (node)->rn_client = (ClientData) NULL;\
+	  (node)->rn_client = (ClientData)NULL;\
 	  (node)->rn_noderes = RES_INFINITY;\
 	  (node)->rn_je = NULL;\
 	  (node)->rn_status = FALSE;\
@@ -656,7 +656,7 @@ extern void			ResSortBreaks();
 {\
 	Breakpoint	*bp;\
 	tileJunk *jX_ = (tileJunk *)((tile)->ti_client); \
-	MALLOC(Breakpoint *,bp,sizeof(Breakpoint)); \
+	bp = (Breakpoint *) mallocMagic((unsigned)(sizeof(Breakpoint))); \
         bp->br_next= jX_->breakList; \
 	bp->br_this = (node); \
 	bp->br_loc.p_x = px; \
@@ -669,10 +669,12 @@ extern void			ResSortBreaks();
 {\
 	resPort		*rp;\
 	tileJunk *pX_ = (tileJunk *)((tile)->ti_client); \
-	MALLOC(resPort *,rp,sizeof(resPort)); \
+	rp = (resPort *) mallocMagic((unsigned)(sizeof(resPort))); \
 	rp->rp_nextPort = pX_->portList; \
 	rp->rp_bbox = node->rs_bbox; \
 	rp->rp_loc = node->drivepoint; \
 	rp->rp_nodename = node->name; \
 	pX_->portList = rp; \
 }
+
+#endif /* _RESIS_H */
