@@ -23,9 +23,11 @@
  *	but global to all source files within the router module.
  */
 
-#ifndef _ROUTER_INT
-#define _ROUTER_INT
+#ifndef _ROUTERINT_H
+#define _ROUTERINT_H
+
 extern int	rtrTarget;			/* Via minimization, target type	*/
 extern int	rtrReplace;			/* Via minimization, replacement type	*/
 extern int	rtrDelta;			/* Change in layer width		*/
-#endif
+
+#endif /* _ROUTERINT_H */
