@@ -40,6 +40,12 @@ static char rcsid[]="$Header: /ufs/repository/magic/router/rtrHazards.c,v 1.2 20
 #include "tiles/tile.h"
 #include "gcr/gcr.h"
 #include "utils/malloc.h"
+
+/* Forward declarations */
+
+extern void rtrFindEnds();
+extern void rtrFlag();
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -76,12 +82,12 @@ static char rcsid[]="$Header: /ufs/repository/magic/router/rtrHazards.c,v 1.2 20
  * ----------------------------------------------------------------------------
  */
 
-Void
+void
 RtrHazards(ch)
     GCRChannel *ch;	/* The channel to be flagged */
 {
     short **map, **rtrHeights(), **rtrWidths(), **height, **width;
-    register short *hcol, *wcol, *mcol;
+    short *hcol, *wcol, *mcol;
     int row, col, bot, top, left, right, maxcol;
 
     height = rtrHeights(ch);
@@ -200,11 +206,11 @@ RtrHazards(ch)
     maxcol = ch->gcr_length + 1;
     for (col = 0; col <= maxcol; col++)
     {
-	FREE((char *) height[col]);
-	FREE((char *) width[col]);
+	freeMagic((char *) height[col]);
+	freeMagic((char *) width[col]);
     }
-    FREE((char *) height);
-    FREE((char *) width);
+    freeMagic((char *) height);
+    freeMagic((char *) width);
 }
 
 /*
@@ -227,8 +233,8 @@ short **
 rtrHeights(ch)
     GCRChannel * ch;	/* Channel to be processed */
 {
-    register short **heights, *obstacles, *hcol;
-    register int i, row;
+    short **heights, *obstacles, *hcol;
+    int i, row;
     int col, start, n;
     unsigned lenWds, widWds;
 
@@ -238,10 +244,10 @@ rtrHeights(ch)
      */
     lenWds = ch->gcr_length + 2;
     widWds = ch->gcr_width + 2;
-    MALLOC(short **, heights, lenWds * sizeof (short *));
+    heights = (short **) mallocMagic((unsigned) (lenWds * sizeof (short *)));
     for (col = 0; col < (int) lenWds; col++)
     {
-	MALLOC(short *, heights[col], widWds * sizeof (short));
+	heights[col] = (short *) mallocMagic((unsigned) (widWds * sizeof (short)));
 	for (row = 0; row < (int) widWds; row++)
 	    heights[col][row] = 0;
     }
@@ -299,8 +305,8 @@ short **
 rtrWidths(ch)
     GCRChannel * ch;
 {
-    register short **widths, **map;
-    register int col, i;
+    short **widths, **map;
+    int col, i;
     unsigned lenWds, widWds;
     int row, start, n;
 
@@ -310,10 +316,10 @@ rtrWidths(ch)
      */
     lenWds = ch->gcr_length + 2;
     widWds = ch->gcr_width + 2;
-    MALLOC(short **, widths, lenWds * sizeof (short *));
+    widths = (short **) mallocMagic((unsigned) (lenWds * sizeof (short *)));
     for (col = 0; col < (int) lenWds; col++)
     {
-	MALLOC(short *, widths[col], widWds * sizeof (short));
+	widths[col] =  (short *) mallocMagic((unsigned) (widWds * sizeof (short)));
 	for (row = 0; row < (int) widWds; row++)
 	    widths[col][row] = 0;
     }
@@ -361,24 +367,25 @@ rtrWidths(ch)
  * to find edges (isHoriz == FALSE).
  *
  * Results:
- *	Returns the two values in * lo and * hi.
+ *	None.
  *
  * Side effects:
- *	None.
+ *	Returns the two values in * lo and * hi.
  *
  * ----------------------------------------------------------------------------
  */
 
+void
 rtrFindEnds(ch, isHoriz, bot, top, lo, hi)
     GCRChannel *ch;
     int isHoriz;	/* 1 if to find horizontal ends */
-    register int bot;	/* Low range to be scanned */
-    register int top;	/* High range to be scanned */
+    int bot;	/* Low range to be scanned */
+    int top;	/* High range to be scanned */
     int *lo;		/* Low range of result.  Also starting col or row. */
     int *hi;		/* High range of result */
 {
-    register int col, row;
-    register short **map;
+    int col, row;
+    short **map;
 
     map = ch->gcr_result;
     if (isHoriz)
@@ -478,6 +485,8 @@ gotVL:
  *
  * ----------------------------------------------------------------------------
  */
+
+void
 rtrFlag(ch, cl, cr, rb, rt, isHoriz)
     GCRChannel * ch;	/* Channel whose flag array is to be set	*/
     int cl, cr;		/* Left and right limits of columns to be set	*/
@@ -485,7 +494,7 @@ rtrFlag(ch, cl, cr, rb, rt, isHoriz)
     bool isHoriz;	/* TRUE if left/right flags, else top/bottom	*/
 {
     int extra, limit, r, c;
-    register short ** map;
+    short ** map;
 
     map = ch->gcr_result;
     if(isHoriz)
