@@ -25,7 +25,6 @@ static char rcsid[]="$Header: /ufs/repository/magic/router/rtrPaint.c,v 1.2 2001
 #include "misc/magic.h"
 #include "utils/geometry.h"
 #include "utils/hash.h"
-#include "utils/doubleint.h"
 #include "utils/heap.h"
 #include "tiles/tile.h"
 #include "database/database.h"
@@ -49,6 +48,8 @@ extern void rtrPaintRows();
 extern void rtrPaintColumns();
 extern void rtrMaxMetal();
 
+bool rtrMetalOkay();
+bool rtrDoVia();
 
 /*
  * ----------------------------------------------------------------------------
