@@ -35,7 +35,6 @@ static char rcsid[]="$Header: /ufs/repository/magic/router/rtrStem.c,v 1.2 2001/
 #include "misc/styles.h"
 #include "router/router.h"
 #include "gcr/gcr.h"
-#include "utils/doubleint.h"
 #include "utils/heap.h"
 #include "grouter/grouter.h"
 #include "netlist/netlist.h"
@@ -77,6 +76,11 @@ extern GCRPin *rtrStemTip();
 extern GCRPin *rtrStemTryPin();
 extern void rtrStemRange();
 
+bool rtrTreeSrArea();
+bool rtrSrArea();
+bool rtrStemMask();
+bool RtrComputeJogs();
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -178,7 +182,7 @@ RtrStemProcessAll(use, netList, doWarn, func)
 
 		    /* Nuke it */
 		    if (locPrev) locPrev->nloc_next = loc->nloc_next;
-		    FREE((char *) loc);
+		    freeMagic((char *) loc);
 		    continue;
 		}
 		locPrev = loc;
@@ -525,7 +529,7 @@ rtrStemTryPin(loc, dir, p, use)
     {
 	NLTermLoc *nloc;
 
-	MALLOC(NLTermLoc *, nloc, sizeof (NLTermLoc));
+	nloc = (NLTermLoc *) mallocMagic((unsigned) (sizeof (NLTermLoc)));
 	*nloc = *loc;
 	loc->nloc_next = nloc;
 	loc = nloc;
