@@ -163,45 +163,55 @@ SelectClear()
     if (SelectRootDef == NULL) return;
 
     scx.scx_area = SelectDef->cd_bbox;
-    SelRememberForUndo(TRUE, (CellDef *) NULL, (Rect *) NULL);
 
-    /* Erase all the paint from the select cell. */
-
-    DBEraseMask(SelectDef, &TiPlaneRect, &DBAllButSpaceBits);
-
-    /* Erase all of the labels from the select cell. */
-
-    (void) DBEraseLabel(SelectDef, &TiPlaneRect, &DBAllTypeBits);
-
-    /* Erase all of the subcells from the select cell.  This is a bit tricky,
-     * because we can't erase the subcells while searching for them (it will
-     * cause problems for the database).  The code below first grabs up a
-     * few subcells, then deletes them, then grabs up a few more, then deletes
-     * them, and so on until done.
-     */
-
-    scx.scx_use = SelectUse;
-    scx.scx_trans = GeoIdentityTransform;
-    while (TRUE)
+    if (!strcmp(SelectDef->cd_name, "__SELECT_NET__"))
     {
-	int i;
-
-	selNDelete = 0;
-	(void) DBCellSrArea(&scx, selClearFunc, (ClientData) NULL);
-	for (i = 0; i < selNDelete; i += 1)
-	{
-	    DBUnLinkCell(selDeleteUses[i], SelectDef);
-	    DBDeleteCell(selDeleteUses[i]);
-	    (void) DBCellDeleteUse(selDeleteUses[i]);
-	}
-	if (selNDelete < MAXUSES) break;
+	SelNetRememberForUndo((CellDef *)NULL, (Point *)NULL,
+		TT_SPACE, FALSE, FALSE);
+	DBCellRenameDef(SelectDef, "__SELECT__");
+	DBCellClearDef(SelectDef);
     }
+    else
+    {
+	SelRememberForUndo(TRUE, (CellDef *) NULL, (Rect *) NULL);
 
-    selectLastUse = NULL;
+	/* Erase all the paint from the select cell. */
+
+	DBEraseMask(SelectDef, &TiPlaneRect, &DBAllButSpaceBits);
+
+	/* Erase all of the labels from the select cell. */
+
+	(void) DBEraseLabel(SelectDef, &TiPlaneRect, &DBAllTypeBits);
+
+	/* Erase all of the subcells from the select cell.  This is a bit tricky,
+	 * because we can't erase the subcells while searching for them (it will
+	 * cause problems for the database).  The code below first grabs up a
+	 * few subcells, then deletes them, then grabs up a few more, then deletes
+	 * them, and so on until done.
+	 */
+
+	scx.scx_use = SelectUse;
+	scx.scx_trans = GeoIdentityTransform;
+	while (TRUE)
+	{
+	    int i;
+
+	    selNDelete = 0;
+	    (void) DBCellSrArea(&scx, selClearFunc, (ClientData) NULL);
+	    for (i = 0; i < selNDelete; i += 1)
+	    {
+		DBUnLinkCell(selDeleteUses[i], SelectDef);
+		DBDeleteCell(selDeleteUses[i]);
+		(void) DBCellDeleteUse(selDeleteUses[i]);
+	    }
+	    if (selNDelete < MAXUSES) break;
+	}
+	selectLastUse = NULL;
+	SelRememberForUndo(FALSE, SelectRootDef, &scx.scx_area);
+    }
 
     /* Erase the selection from the screen. */
 
-    SelRememberForUndo(FALSE, SelectRootDef, &scx.scx_area);
     DBWHLRedraw(SelectRootDef, &scx.scx_area, TRUE);
     DBReComputeBbox(SelectDef);
     DBWAreaChanged(SelectDef, &scx.scx_area, DBW_ALLWINDOWS,
@@ -733,7 +743,12 @@ SelectRegion(scx, type, xMask, pArea, less)
     /* Display the new selection. */
 
     SelRememberForUndo(FALSE, SelectRootDef, &Select2Def->cd_bbox);
-    DBReComputeBbox(SelectDef);
+
+    /* SelectDef should not need to recompute the bounding box;	*/
+    /* only to copy it (transformed) from Select2Def.		*/
+    GeoTransRect(&scx->scx_trans, &Select2Def->cd_bbox, &SelectDef->cd_bbox);
+    /* DBReComputeBbox(SelectDef); */
+
     DBWHLRedraw(SelectRootDef, &Select2Def->cd_bbox, TRUE);
     DBWAreaChanged(SelectDef, &Select2Def->cd_bbox, DBW_ALLWINDOWS,
 	 &DBAllButSpaceBits);
@@ -776,10 +791,13 @@ SelectNet(scx, type, xMask, pArea, less)
     Rect *pArea;		/* If non-NULL, points to rectangle to be
 				 * filled in with net's bounding box.
 				 */
-    bool less;
+    bool less;			/* Whether to remove material from the
+				 * selection using SelDeleteSel2
+				 */
 {
     TileTypeBitMask mask;
     SearchContext scx2;
+    Point savePoint = scx->scx_area.r_ll;
 
     /* If the source definition is changing, clear the old selection. */
 
@@ -804,17 +822,20 @@ SelectNet(scx, type, xMask, pArea, less)
 	    &TiPlaneRect, Select2Use);
     UndoEnable();
 
+    /* Network undo method added by Nishit and Tim, July 8-10, 2004 */
+    SelNetRememberForUndo(SelectRootDef, &savePoint, type, less, TRUE);
+
     /* Now transfer what we found into the main selection cell.  Pick
      * up all the labels that correspond to the selected material.
      */
     
-    SelRememberForUndo(TRUE, (CellDef *) NULL, (Rect *) NULL);
+    UndoDisable();
     if (less)
-      {
+    {
 	(void) SelRemoveSel2();
-      }
+    }
     else
-      {
+    {
 	scx2.scx_use = Select2Use;
 	scx2.scx_area = Select2Def->cd_bbox;
 	scx2.scx_trans = GeoIdentityTransform;
@@ -822,12 +843,21 @@ SelectNet(scx, type, xMask, pArea, less)
 			   0, SelectUse);
 	DBCellCopyAllLabels(&scx2, &DBAllTypeBits, -1, SelectUse,
 			(Rect *) NULL);
-      }
+    }
+
+    /* Rename the cell to "__SELECT_NET__" so that we can treat */
+    /* it differently when it comes time to unselect it.	*/
+
+    DBCellRenameDef(SelectDef, "__SELECT_NET__");
+    UndoEnable(); 
 
     /* Display the newly-selected material. */
+    /* SelectDef should not need to recompute the bounding box;	*/
+    /* only to copy it (transformed) from Select2Def.		*/
+    /* ---Nishit, 7/6/04					*/
+    GeoTransRect(&scx->scx_trans, &Select2Def->cd_bbox, &SelectDef->cd_bbox);
+    /* DBReComputeBbox(SelectDef); */
 
-    SelRememberForUndo(FALSE, SelectRootDef, &Select2Def->cd_bbox);
-    DBReComputeBbox(SelectDef);
     DBWHLRedraw(SelectRootDef, &Select2Def->cd_bbox, TRUE);
     DBWAreaChanged(SelectDef, &Select2Def->cd_bbox, DBW_ALLWINDOWS,
 	&DBAllButSpaceBits);
