@@ -198,11 +198,10 @@ selRedisplayFunc(tile, window)
 	/* Use GrDrawTriangleEdge() so that clipping for area select     */
 	/* draws the correct line.					 */
 
-	WindSurfaceToScreen(window, &area, &screenEdge);
+	WindSurfaceToScreenNoClip(window, &area, &screenEdge);
 	if (screenEdge.r_ll.p_x != screenEdge.r_ur.p_x &&
 		screenEdge.r_ll.p_y != screenEdge.r_ur.p_y)
-	    GrDrawTriangleEdge(&screenEdge, (int)SplitDirection(tile),
-		(int)SplitSide(tile));
+	    GrDrawTriangleEdge(&screenEdge, TiGetTypeExact(tile));
 	loctype = (SplitSide(tile)) ? SplitRightType(tile) : SplitLeftType(tile);
     }
     else
