@@ -24,6 +24,8 @@ static char rcsid[]="$Header: /ufs/repository/magic/select/selEnum.c,v 1.3 2001/
 #endif  /* not lint */
 
 #include <stdio.h>
+#include <string.h>
+
 #include "misc/magic.h"
 #include "utils/geometry.h"
 #include "tiles/tile.h"
@@ -170,16 +172,49 @@ selEnumPFunc1(tile, arg)
 #endif
     arg->sea_type = TiGetType(tile);
 
-    /* If this tile is a contact's secondary image, ignore it:  the
-     * primary image will take care of everything (otherwise when
-     * the second image is processed, the contact might not be there
-     * anymore).
-     */
+    /* Only process contacts once:  only process the home plane */
+    /* image of any contact type.  Stacked contacts are treated	*/
+    /* as whichever residue (contact) is in its home plane.	*/
 
-    if (arg->sea_type >= DBNumUserLayers) return 0;
+    if (DBIsContact(arg->sea_type))
+    {
+	if (arg->sea_type < DBNumUserLayers)
+	{
+	    if (arg->sea_plane != DBPlane(arg->sea_type)) return 0;
+#ifdef NONMANHATTAN
+	    arg->sea_type = TiGetTypeExact(tile);
+#endif
+	}
+	else	/* stacked contact type */
+	{
+	    TileTypeBitMask *rmask = DBResidueMask(arg->sea_type);
+	    TileType rtype;
+	    for (rtype = TT_TECHDEPBASE; rtype < DBNumUserLayers; rtype++)
+		if (TTMaskHasType(rmask, rtype) &&
+				arg->sea_plane == DBPlane(rtype))
+		    break;
+	    if (rtype == DBNumUserLayers) return 0;
+
+	    /* Continue, with the tile type recast as rtype */
 
 #ifdef NONMANHATTAN
-    arg->sea_type = TiGetTypeExact(tile);
+	    if (IsSplit(tile))
+	    {
+		if (SplitSide(tile))
+		    arg->sea_type = (rtype << 14) | SplitLeftType(tile);
+		else
+		    arg->sea_type = rtype | (SplitRightType(tile) << 14);
+		arg->sea_type |= TT_DIAGONAL | TT_DIRECTION | TT_SIDE;
+	    }
+	    else
+#endif
+	    arg->sea_type = rtype;
+	}
+    }
+
+#ifdef NONMANHATTAN
+    else
+	arg->sea_type = TiGetTypeExact(tile);
 #endif
 
     /* If the paint doesn't have to be in the edit cell, life's pretty
@@ -208,6 +243,7 @@ selEnumPFunc1(tile, arg)
     {
 	GeoTransRect(&EditToRootTransform, &arg->sea_rectList->r_r, &rootRect);
 	GeoClip(&rootRect, &rect);
+	
 	if ((*arg->sea_func)(&rootRect, arg->sea_type, arg->sea_cdarg) != 0)
 	    return 1;
 	freeMagic((char *) arg->sea_rectList);
@@ -227,18 +263,19 @@ selEnumPFunc2(tile, arg)
     struct searg *arg;		/* Describes our search. */
 {
     LinkedRect *lr;
-    register ttype;
+    int ttype;
+    TileType loctype;
 
 #ifdef NONMANHATTAN
-    TileType loctype;
-     
     if (arg->sea_type & TT_DIAGONAL)
 	loctype = (arg->sea_type & TT_SIDE) ?
 		(arg->sea_type & TT_RIGHTMASK) >> 14 :
 		arg->sea_type & TT_LEFTMASK;
     else
-	loctype = arg->sea_type;
+#endif
+    loctype = arg->sea_type;
      
+#ifdef NONMANHATTAN
     if (IsSplit(tile))
     {
 	ttype = SplitLeftType(tile);
@@ -248,11 +285,25 @@ selEnumPFunc2(tile, arg)
     else
 	ttype = TiGetType(tile);
         
-    if (loctype != ttype)
 #else
     ttype = TiGetTypeExact(tile);
-    if (arg->sea_type != ttype)
 #endif
+
+    /* Handle stacked contact types */
+    if (ttype >= DBNumUserLayers)
+    {
+	TileTypeBitMask *rmask = DBResidueMask(ttype);
+	TileType rtype;
+	for (rtype = TT_TECHDEPBASE; rtype < DBNumUserLayers; rtype++)
+	    if (TTMaskHasType(rmask, rtype))
+		if (arg->sea_plane == DBPlane(rtype))
+		{
+		    ttype = rtype;
+		    break;
+		}
+    }
+
+    if (loctype != ttype)
     {
 	if (arg->sea_nonEdit != NULL) *(arg->sea_nonEdit) = TRUE;
 	return 0;
@@ -549,7 +600,7 @@ SelEnumLabels(layers, editOnly, foundNonEdit, func, clientData)
     int (*func)();		/* Function to call for each label found. */
     ClientData clientData;	/* Argument to pass through to func. */
 {
-    register Label *selLabel;
+    Label *selLabel;
     CellUse dummy;
     SearchContext scx;
     struct searg arg;
