@@ -20,14 +20,11 @@
  * rcsid="$Header: /ufs/repository/magic/select/selInt.h,v 1.2 2001/01/12 22:13:23 jsolomon Exp $"
  */
 
-#define _SELINT
+#ifndef _SELINT_H
+#define _SELINT_H
 
-#ifndef	_MAGIC
-int err0 = Need_to_include_misc/magic_header;
-#endif	/* _MAGIC */
-#ifndef	_DATABASE
-int err0 = Need_to_include_database/database_header;
-#endif	/* _DATABASE */
+#include "misc/magic.h"
+#include "database/database.h"
 
 /* Procedures, variables, and records that are shared between
  * files:
@@ -43,3 +40,5 @@ extern CellUse *Select2Use;
 extern CellDef *Select2Def;
 
 extern CellUse *selectLastUse;
+
+#endif /* _SELINT_H */
