@@ -19,10 +19,10 @@
  * rcsid[]="$Header: /ufs/repository/magic/signals/signals.h,v 1.3 2001/04/26 23:13:36 jsolomon Exp $";
  */
 
-#define _SIGNAL
-#ifndef	_MAGIC
-	err = Need_to_include_misc/magic_h
-#endif
+#ifndef _MAGSIGNAL_H
+#define _MAGSIGNAL_H
+
+#include "misc/magic.h"
 
 /* Some machines have signal handlers returning an int, while other machines
  * have it returning a void.  If you have a machine that requires ints put 
@@ -50,3 +50,5 @@ extern void SigWatchFile();
 extern void SigUnWatchFile();
 
 extern sigRetVal sigOnInterrupt();
+
+#endif /* _MAGSIGNAL_H */
