@@ -75,6 +75,9 @@ typedef struct
 {
     Rect		area;		/* Area to process */
     TileTypeBitMask	*connectMask;	/* Connection mask for search */
+#ifdef NONMANHATTAN
+    TileType		dinfo;		/* Info about triangular search areas */
+#endif
 } conSrArea;
 
 struct conSrArg2
@@ -92,20 +95,6 @@ struct conSrArg2
 
 #define CSA2_LIST_START_SIZE 256
 
-/* We need to maintain where in the cell hierarchy we are as we search
- * the tile database when extracting a node name.  To do this, a stack is
- * maintained of the hierarchy of cell definitions (a search context)
- * as the search progresses.  This stack is doubly linked to allow fast
- * pathname construction.
- */
-
-typedef struct scx_stack_elt
-{
-    SearchContext  se_scx;		/* current cell def we're looking at */
-    struct scx_stack_elt *se_next;
-    struct scx_stack_elt *se_prev;
-} ScxStackElt;
-
 /* Forward declarations */
 
 extern char *DBPrintUseId();
@@ -114,8 +103,6 @@ extern void SimInitScxStk();
 extern void SimPopScx();
 extern void SimMakePathname();
 
-static ScxStackElt	*stackBase = (ScxStackElt *) NULL;
-					/* stack of search contexts */
 static char 		bestName[256];
 
 
@@ -157,11 +144,14 @@ SimConnectFunc(tile, cx)
     Rect 		tileArea, *srArea, newarea;
     SearchContext	*scx = cx->tc_scx;
     TileTypeBitMask	notConnectMask, *connectMask;
-    TileType		newtype, loctype;
+    TileType		loctype, ctype;
+#ifdef NONMANHATTAN
+    TileType		dinfo = 0;
+#endif
     int 		i, pNum;
-    char		pathName[256];
     static char		nodeName[256];
     CellDef 		*def;
+    TerminalPath	*tpath = cx->tc_filter->tf_tpath;
 
     TiToRect(tile, &tileArea);
     srArea = &scx->scx_area;
@@ -189,38 +179,38 @@ SimConnectFunc(tile, cx)
     GeoClip(&newarea, csa2->csa2_bounds);
     if (GEO_RECTNULL(&newarea)) return 0;
 
+    /* Stuff unique to the nodename search follows. */
+
+    if (tpath != (TerminalPath *)NULL)
+    {
+        /* Extract the node name */
+
+	char *n = tpath->tp_next;
+	char c = *n;
+
+	SigDisableInterrupts();
+	strcpy(nodeName, SimGetNodeName(cx->tc_scx, tile, tpath->tp_first)); 
+	SigEnableInterrupts();
+
+	*n = c;
+
+	/* save the "best" name for this node */
+
+	if (bestName[0] == '\0' || efPreferredName(nodeName, bestName))
+	    strcpy(bestName, nodeName);
+    }
+
+    loctype = TiGetTypeExact(tile);
+
 #ifdef NONMANHATTAN
     /* Resolve geometric transformations on diagonally-split tiles */
    
     if (IsSplit(tile))
     {
-        int o1, o2, o3, dir, side;
-
-        o1 = ((scx->scx_trans.t_e > 0) || (scx->scx_trans.t_d > 0)) ? 1 : 0;
-        o2 = ((scx->scx_trans.t_a > 0) || (scx->scx_trans.t_b > 0)) ? 1 : 0;
-        o3 = (scx->scx_trans.t_a != 0) ? 1 : 0;
-        dir = SplitDirection(tile) ^ o1 ^ o2;
-        side = SplitSide(tile) ^ o2 ^ (SplitDirection(tile) | o3);
-
-        if (side == SplitSide(tile))
-            newtype = TT_DIAGONAL | SplitLeftType(tile) |
-                        (SplitRightType(tile) << 14);
-        else
-            newtype = TT_DIAGONAL | SplitRightType(tile) |
-                        (SplitLeftType(tile) << 14);
-
-        if (dir) newtype |= TT_DIRECTION;
-        if (side)
-	{
-	    newtype |= TT_SIDE;
-	    loctype = (newtype & TT_RIGHTMASK) >> 14;
-	}
-	else
-	    loctype = newtype & TT_LEFTMASK;
+	dinfo = DBTransformDiagonal(loctype, &scx->scx_trans);
+	loctype = (SplitSide(tile)) ? SplitRightType(tile) : SplitLeftType(tile);
     }
-    else
 #endif
-    loctype = newtype = TiGetTypeExact(tile);
 
     /* See if the destination cell contains stuff over the whole
      * current area (on its home plane) that is connected to it.
@@ -233,7 +223,20 @@ SimConnectFunc(tile, cx)
 
     if (DBIsContact(loctype))
     {
+	TileTypeBitMask *rMask = DBResidueMask(loctype);
+
 	TTMaskSetOnlyType(&notConnectMask, loctype);
+
+	/* Differenct contact types may share residues (6/18/04) */     
+	for (ctype = TT_TECHDEPBASE; ctype < DBNumUserLayers; ctype++)
+	    if (TTMaskEqual(rMask, DBResidueMask(ctype)))
+		TTMaskSetType(&notConnectMask, ctype);
+
+	/* The mask of contact types must include all stacked contacts */
+	for (ctype = DBNumUserLayers; ctype < DBNumTypes; ctype++)   
+	    if TTMaskHasType(DBResidueMask(ctype), loctype)
+		TTMaskSetType(&notConnectMask, ctype);
+
 	TTMaskCom(&notConnectMask);
     }
     else
@@ -242,36 +245,28 @@ SimConnectFunc(tile, cx)
     }
 
     def = csa2->csa2_use->cu_def;
+#ifdef NONMANHATTAN
+    if (DBSrPaintNMArea((Tile *) NULL, def->cd_planes[pNum],
+		dinfo, &newarea, &notConnectMask, dbcUnconnectFunc,
+		(ClientData) connectMask) == 0)
+	return 0;
+#else
     if (DBSrPaintArea((Tile *) NULL, def->cd_planes[pNum],
 	    &newarea, &notConnectMask, dbcUnconnectFunc,
 	    (ClientData) connectMask) == 0)
 	return 0;
+#endif
 
     /* Paint this tile into the destination cell. */
 	
 #ifdef NONMANHATTAN
-    DBNMPaintPlane(def->cd_planes[pNum], newtype, &newarea,
+    DBNMPaintPlane(def->cd_planes[pNum], dinfo, &newarea,
 		DBStdPaintTbl(loctype, pNum), (PaintUndoInfo *) NULL);
 #else
     DBPaintPlane(def->cd_planes[pNum], &newarea,
 		DBStdPaintTbl(loctype, pNum), (PaintUndoInfo *) NULL);
 #endif
 
-    /* Stuff unique to the nodename search follows. */
-
-    SimMakePathname(pathName);
-
-    /* Extract the node name */
-
-    SigDisableInterrupts();
-    strcpy(nodeName, SimGetNodeName(cx->tc_scx, tile, pathName)); 
-    SigEnableInterrupts();
-
-    /* save the "best" name for this node */
-
-    if (bestName[0] == '\0' || efPreferredName(nodeName, bestName))
-	strcpy(bestName, nodeName);
-
     /* Since the whole area of this tile hasn't been recorded,
      * we must process its area to find any other tiles that
      * connect to it.  Add each of them to the list of things
@@ -284,14 +279,14 @@ SimConnectFunc(tile, cx)
 #ifdef NONMANHATTAN
     /* Only extend those sides bordering the diagonal tile */
 
-    if (newtype & TT_DIAGONAL)
+    if (dinfo & TT_DIAGONAL)
     {
-	if (newtype & TT_SIDE)			/* right */
+	if (dinfo & TT_SIDE)			/* right */
 	    newarea.r_xtop += 1;
 	else					/* left */
 	    newarea.r_xbot -= 1;
-	if (((newtype & TT_SIDE) >> 1)
-		== (newtype & TT_DIRECTION))	/* top */
+	if (((dinfo & TT_SIDE) >> 1)
+		== (dinfo & TT_DIRECTION))	/* top */
 	    newarea.r_ytop += 1;
 	else					/* bottom */
 	    newarea.r_ybot -= 1;
@@ -307,13 +302,18 @@ SimConnectFunc(tile, cx)
     }
 #endif
 
-    /* Abort the name search if the name is global or name is in the
-     * abort name search table 
+    /* Abort the name search if the name is in the abort name search table 
+     * or if the name is global and the SimIgnoreGlobals flag is not set.
      */
 
-    i = strlen(nodeName);
-    if (SimSawAbortString  || SigInterruptPending || (nodeName[i - 1] == '!'))
+    if (SimSawAbortString  || SigInterruptPending)
 	return 1;
+    else if (SimIsGetnode && !SimIgnoreGlobals)
+    {
+	i = strlen(nodeName);
+	if (nodeName[i - 1] == '!')
+	    return 1;
+    }
 
     /* Register the area and connection mask as needing to be processed */
 
@@ -332,6 +332,9 @@ SimConnectFunc(tile, cx)
 	{
 	    newlist[i].area = csa2->csa2_list[i].area;
 	    newlist[i].connectMask = csa2->csa2_list[i].connectMask;
+#ifdef NONMANHATTAN
+	    newlist[i].dinfo = csa2->csa2_list[i].dinfo;
+#endif
 	    
 	}
 	freeMagic((char *)csa2->csa2_list);
@@ -340,6 +343,9 @@ SimConnectFunc(tile, cx)
 
     csa2->csa2_list[csa2->csa2_top].area = newarea;
     csa2->csa2_list[csa2->csa2_top].connectMask = connectMask;
+#ifdef NONMANHATTAN
+    csa2->csa2_list[csa2->csa2_top].dinfo = dinfo;
+#endif
 
     return 0;
 }
@@ -372,6 +378,8 @@ SimConnectFunc(tile, cx)
  * ----------------------------------------------------------------------------
  */
 
+#define MAXPATHNAME 256
+
 void
 SimTreeCopyConnect(scx, mask, xMask, connect, area, destUse, Node_Name)
     SearchContext *scx;			/* Describes starting area.  The
@@ -407,8 +415,13 @@ SimTreeCopyConnect(scx, mask, xMask, connect, area, destUse, Node_Name)
 					 * words in some compilers.
 					 */
 {
+    TerminalPath tpath;
+    char pathName[MAXPATHNAME];
     TileTypeBitMask *newmask;
     struct conSrArg2 csa2;
+#ifdef NONMANHATTAN
+    TileType newtype;
+#endif
 
     csa2.csa2_use = destUse;
     csa2.csa2_bounds = area;
@@ -419,16 +432,31 @@ SimTreeCopyConnect(scx, mask, xMask, connect, area, destUse, Node_Name)
 			* sizeof(conSrArea));
     csa2.csa2_top = -1;
 
-    SimInitScxStk();
+    tpath.tp_first = tpath.tp_next = pathName;
+    tpath.tp_last = pathName + MAXPATHNAME; 
+
+    pathName[0] = '\0';
     bestName[0] = '\0';
 
-    (void) SimTreeSrTiles(scx, mask, xMask, SimConnectFunc, (ClientData) &csa2);
+    (void) SimTreeSrTiles(scx, mask, xMask, &tpath, SimConnectFunc,
+		(ClientData) &csa2);
     while (csa2.csa2_top >= 0)
     {
 	newmask = csa2.csa2_list[csa2.csa2_top].connectMask;
 	scx->scx_area = csa2.csa2_list[csa2.csa2_top].area;
+#ifdef NONMANHATTAN
+	newtype = csa2.csa2_list[csa2.csa2_top].dinfo;
+#endif
 	csa2.csa2_top--;
-	SimTreeSrTiles(scx, newmask, xMask, SimConnectFunc, (ClientData) &csa2);
+
+#ifdef NONMANHATTAN
+	if (newtype & TT_DIAGONAL)
+	    SimTreeSrNMTiles(scx, newtype, newmask, xMask, &tpath,
+			SimConnectFunc, (ClientData) &csa2);
+	else
+#endif
+	SimTreeSrTiles(scx, newmask, xMask, &tpath, SimConnectFunc,
+		(ClientData) &csa2);
     }
     freeMagic((char *)csa2.csa2_list);
 
@@ -487,24 +515,27 @@ efPreferredName(name1, name2)
     --np1;
     --np2;
 
-    /* both are global names */
-    if ((*np1 == '!') && (*np2 == '!')) {
-	/* check # of pathname components */
-	if (nslashes1 < nslashes2) return (TRUE);
-	if (nslashes1 > nslashes2) return (FALSE);
+    if (!SimIgnoreGlobals)
+    {
+	/* both are global names */
+	if ((*np1 == '!') && (*np2 == '!')) {
+	    /* check # of pathname components */
+	    if (nslashes1 < nslashes2) return (TRUE);
+	    if (nslashes1 > nslashes2) return (FALSE);
 
-	/* same # of pathname components; check length */
-        if (np1 - name1 < np2 - name2) return (TRUE);
-    	if (np1 - name1 > np2 - name2) return (FALSE);
+	    /* same # of pathname components; check length */
+            if (np1 - name1 < np2 - name2) return (TRUE);
+    	    if (np1 - name1 > np2 - name2) return (FALSE);
 
-	/* same # of pathname components; same length; use lex order */
-	if (strcmp(name1, name2) > 0) 
-	    return(TRUE);
-	else
-	    return(FALSE);
+	    /* same # of pathname components; same length; use lex order */
+	    if (strcmp(name1, name2) > 0) 
+		return(TRUE);
+	    else
+		return(FALSE);
+	}
+	if (*np1 == '!') return(TRUE);
+	if (*np2 == '!') return(FALSE);
     }
-    if (*np1 == '!') return(TRUE);
-    if (*np2 == '!') return(FALSE);
 
     /* neither name is global */
     /* chose label over generated name */
@@ -636,152 +667,13 @@ SimSrConnect(def, startArea, mask, connect, bounds, func, clientData)
 }
 
 
-/*
- * ----------------------------------------------------------------------------
- * SimInitScxStk
- *
- *	This procedure initializes the doubly-linked stack of search
- *	contexts which describes the current hierarchy of the search.
- *	
- *
- * Results:
- *	None.
- *
- * Side effects:
- *	The stack is set to NULL.
- *	Space in the stack is freed if the stack is not empty.
- *	
- * ----------------------------------------------------------------------------
- */
-
-void
-SimInitScxStk()
-{
-    if (stackBase == (ScxStackElt *) NULL) {
-	MALLOC(ScxStackElt *, stackBase, sizeof(ScxStackElt));
-	stackBase->se_scx.scx_use = (CellUse *) NULL;
-	stackBase->se_next = stackBase->se_prev = stackBase;
-    }
-    else {
-	while (stackBase != stackBase->se_next) {
-	    SimPopScx();
-	}
-    }
-}
-
-/*
- * ----------------------------------------------------------------------------
- * SimPushScx
- *
- *	This procedure pushes a new search context onto the search
- *	context stack.  A search context is pushed only if it is
- *	different from the one on top of the stack.
- *
- * Results:
- *	None.
- *
- * Side effects:
- *	Search context is pushed.
- *
- * ----------------------------------------------------------------------------
- */
-
-void
-SimPushScx(newscx) 
-    SearchContext *newscx;
-{
-
-    ScxStackElt *dummy;
-
-    /* check to see if the search context is different from the
-     * one on top of the stack
-     */
-
-    if (stackBase->se_prev->se_scx.scx_use != newscx->scx_use) {
-	MALLOC(ScxStackElt *, dummy, sizeof(ScxStackElt));
-	dummy->se_scx = *newscx;
-	dummy->se_next = stackBase->se_next;
-	stackBase->se_next = dummy;
-	dummy->se_prev = stackBase;
-	dummy->se_next->se_prev = dummy;
-    }
-}
-
-/*
- * ----------------------------------------------------------------------------
- * SimPopScx
- *
- *	This procedure pops the search context on top of the stack.
- *
- * Results:
- *	None.
- *
- * Side effects:
- *	The stack is popped.
- *
- * ----------------------------------------------------------------------------
- */
-
-void
-SimPopScx()
-{
-    ScxStackElt *dummy;
-    if (stackBase == stackBase->se_next) {
-	TxPrintf("SimPopScx -- stack empty\n");
-	return;
-    }
-    else {
-	dummy = stackBase->se_next;
-	stackBase->se_next = dummy->se_next;
-	stackBase->se_next->se_prev = stackBase;
-	FREE(dummy);
-    }
-}
-
-/*
- * ----------------------------------------------------------------------------
- * SimMakePathname
- *
- * 	This procedure creates a pathname for the current hierarchy
- *	by walking the search context stack in reverse order.
- *
- * Results:
- *	None.
- *
- * Side effects:
- *	The pathname is returned in the parameter pathname.
- *
- * ----------------------------------------------------------------------------
- */
-
-void
-SimMakePathname(pathname)
-    char *pathname;
-{
-    ScxStackElt *dummy;
-    char buff[100];
-
-    *pathname = '\0';
-    if (stackBase == stackBase->se_next) {
-        return;
-    } 
-    else {
-	for (dummy = stackBase->se_prev; dummy != stackBase; 
-	     dummy = dummy->se_prev) {
-	    DBPrintUseId(&(dummy->se_scx), buff, 100);
-	    strcat(pathname, buff);
-	    strcat(pathname, "/");
-	}
-    }
-}
-
-
 /*
  *-----------------------------------------------------------------------------
  *
  * SimTreeSrTiles
  *
- * Similar to the procedure DBTreeSrTiles.
+ * Similar to the procedure DBTreeSrTiles, although having a terminal
+ * path similar to procedure DBTreeSrLabels.
  *
  * Recursively search downward from the supplied CellUse for
  * all visible paint tiles matching the supplied type mask.
@@ -794,8 +686,9 @@ SimMakePathname(pathname)
  *	{
  *	}
  *
- * The SearchContext is stored in cxp->tc_scx, and the user's arg is stored
- * in cxp->tc_filter->tf_arg.
+ * The SearchContext is stored in cxp->tc_scx, the user's arg is stored
+ * in cxp->tc_filter->tf_arg, and the terminal path is stored in
+ * cxp->tc_filter->tf_tpath.
  *
  * In the above, the scx transform is the net transform from the coordinates
  * of tile to "world" coordinates (or whatever coordinates the initial
@@ -821,7 +714,7 @@ SimMakePathname(pathname)
  */
 
 int
-SimTreeSrTiles(scx, mask, xMask, func, cdarg)
+SimTreeSrTiles(scx, mask, xMask, tpath, func, cdarg)
     SearchContext *scx;		/* Pointer to search context specifying
 				 * a cell use to search, an area in the
 				 * coordinates of the cell's def, and a
@@ -835,56 +728,78 @@ SimTreeSrTiles(scx, mask, xMask, func, cdarg)
 				 * when anded with xMask, are not
 				 * equal to xMask.
 				 */
+    TerminalPath *tpath;	/* Pointer to a structure describing a
+				 * partially filled-in terminal pathname.
+				 * Add new components as encountered.
+				 */
     int (*func)();		/* Function to apply at each qualifying tile */
     ClientData cdarg;		/* Client data for above function */
 {
     int SimCellTileSrFunc();
-    TreeContext context;
     TreeFilter filter;
-    CellUse *cellUse = scx->scx_use;
-    CellDef *def = cellUse->cu_def;
-    int pNum;
-
-    ASSERT(def != (CellDef *) NULL, "SimTreeSrTiles");
-    if (!DBIsExpand(cellUse, xMask))
-	return 0;
-
-    if ((def->cd_flags & CDAVAILABLE) == 0)
-	if (!DBCellRead(def, (char *) NULL, TRUE)) return 0;
 
     filter.tf_func = func;
     filter.tf_arg = cdarg;
     filter.tf_mask = mask;
     filter.tf_xmask = xMask;
     filter.tf_planes = DBTechTypesToPlanes(mask);
+    filter.tf_tpath = tpath;
+#ifdef NONMANHATTAN
+    filter.tf_dinfo = 0;
+#endif
 
-    context.tc_scx = scx;
-    context.tc_filter = &filter;
-
-    /*
-     * Apply the function first to any of the tiles in the planes
-     * for this CellUse's CellDef that match the mask.
-     */
-
-    for (pNum = PL_PAINTBASE; pNum < DBNumPlanes; pNum++)
-	if (PlaneMaskHasPlane(filter.tf_planes, pNum))
-	{
-	    if (DBSrPaintArea((Tile *) NULL, def->cd_planes[pNum],
-		    &scx->scx_area, mask, func, (ClientData) &context))
-		return 1;
-	}
-
-    /*
-     * Now apply ourselves recursively to each of the CellUses
-     * in our tile plane.
-     */
-
-    if (DBCellSrArea(scx, SimCellTileSrFunc, (ClientData) &filter))
-	return 1;
-    else
-	return 0;
+    return SimCellTileSrFunc(scx, &filter);
 }
 
+#ifdef NONMANHATTAN
+
+/*
+ * SimTreeSrNMTiles ---
+ *	This is a variant of the above in which the search is over
+ *	a non-Manhattan triangular area.
+ */
+
+int
+SimTreeSrNMTiles(scx, dinfo, mask, xMask, tpath, func, cdarg)
+    SearchContext *scx;		/* Pointer to search context specifying
+				 * a cell use to search, an area in the
+				 * coordinates of the cell's def, and a
+				 * transform back to "root" coordinates.
+				 */
+    TileType dinfo;		/* Type containing information about the
+				 * triangular area to search.
+				 */
+    TileTypeBitMask *mask;	/* Only tiles with a type for which
+				 * a bit in this mask is on are processed.
+				 */
+    int xMask;			/* All subcells are visited recursively
+				 * until we encounter uses whose flags,
+				 * when anded with xMask, are not
+				 * equal to xMask.
+				 */
+    TerminalPath *tpath;	/* Pointer to a structure describing a
+				 * partially filled-in terminal pathname.
+				 * Add new components as encountered.
+				 */
+    int (*func)();		/* Function to apply at each qualifying tile */
+    ClientData cdarg;		/* Client data for above function */
+{
+    int SimCellTileSrFunc();
+    TreeFilter filter;
+
+    filter.tf_func = func;
+    filter.tf_arg = cdarg;
+    filter.tf_mask = mask;
+    filter.tf_xmask = xMask;
+    filter.tf_dinfo = dinfo;
+    filter.tf_planes = DBTechTypesToPlanes(mask);
+    filter.tf_tpath = tpath;
+
+    return SimCellTileSrFunc(scx, &filter);
+}
+
+#endif	/* NONMANHATTAN */
+
 /*
  * Filter procedure applied to subcells by SimTreeSrTiles().
  */
@@ -895,8 +810,10 @@ SimCellTileSrFunc(scx, fp)
     TreeFilter *fp;
 {
     TreeContext context;
+    TerminalPath *tp;
     CellDef *def = scx->scx_use->cu_def;
-    int pNum;
+    int pNum, result;
+    char *tnext;
 
     ASSERT(def != (CellDef *) NULL, "SimCellTileSrFunc");
     if (!DBIsExpand(scx->scx_use, fp->tf_xmask))
@@ -906,21 +823,53 @@ SimCellTileSrFunc(scx, fp)
 
     context.tc_scx = scx;
     context.tc_filter = fp;
-    SimPushScx(scx);
+
+    /* Create the path prefix */
+    /* Don't prepend the "Topmost cell" ID of the top-level cell. */
+
+    if ((fp->tf_tpath != (TerminalPath *)NULL)
+		&& (scx->scx_use->cu_parent != NULL))
+    {
+	tp = fp->tf_tpath;
+	tnext = tp->tp_next;
+	tp->tp_next = DBPrintUseId(scx, tp->tp_next, tp->tp_last -
+		tp->tp_next);
+	if (tp->tp_next < tp->tp_last)
+	{
+	    *(tp->tp_next++) = '/';
+	    *(tp->tp_next) = '\0';
+	}
+    }
 
     /*
      * Apply the function first to any of the tiles in the planes
      * for this CellUse's CellDef that match the mask.
      */
 
+    result = 0;
     for (pNum = PL_PAINTBASE; pNum < DBNumPlanes; pNum++)
 	if (PlaneMaskHasPlane(fp->tf_planes, pNum))
 	{
+#ifdef NONMANHATTAN
+	    if (fp->tf_dinfo & TT_DIAGONAL)
+	    {
+		TileType dinfo = DBTransformDiagonal(fp->tf_dinfo, &scx->scx_trans);
+		if (DBSrPaintNMArea((Tile *) NULL, def->cd_planes[pNum],
+			dinfo, &scx->scx_area, fp->tf_mask,
+			fp->tf_func, (ClientData) &context))
+		{
+		    result = 1;
+		    goto cleanup;
+		}
+	    }
+	    else
+#endif
 	    if (DBSrPaintArea((Tile *) NULL, def->cd_planes[pNum],
 		    &scx->scx_area, fp->tf_mask,
-		    fp->tf_func, (ClientData) &context)) {
-		SimPopScx();
-		return 1;
+		    fp->tf_func, (ClientData) &context))
+	    {
+		result = 1;
+		goto cleanup;
 	    }
 	}
 
@@ -929,19 +878,20 @@ SimCellTileSrFunc(scx, fp)
      * in our tile plane.
      */
 
-    if (DBCellSrArea(scx, SimCellTileSrFunc, (ClientData) fp)) {
-	SimPopScx();
-	return 1;
-    }
-    else {
-	SimPopScx();
-	return 0;
+    if (DBCellSrArea(scx, SimCellTileSrFunc, (ClientData) fp))
+	result = 1;
+
+cleanup:
+    /* Remove the trailing pathname component from the TerminalPath */
+    if ((fp->tf_tpath != (TerminalPath *)NULL)
+		&& (scx->scx_use->cu_parent != NULL))
+    {
+	fp->tf_tpath->tp_next = tnext;
+	*tnext = '\0';
     }
+    return (result);
 }
 
-
-
-
 
 /*
  * ----------------------------------------------------------------------------
@@ -984,7 +934,7 @@ SimPutLabel(cellDef, rect, align, text, type)
     int len, x1, x2, y1, y2, tmp, labx, laby;
 
     len = strlen(text) + sizeof (Label) - sizeof lab->lab_text + 1;
-    MALLOC(Label *, lab, (unsigned) len);
+    lab = (Label *) mallocMagic((unsigned) len);
     strcpy(lab->lab_text, text);
 
     /* Pick a nice alignment if the caller didn't give one.  If the
