@@ -23,6 +23,7 @@
 #include <string.h>
 #include <math.h>
 
+#include "tcltk/tclmagic.h"
 #include "misc/magic.h"
 #include "utils/geometry.h"
 #include "utils/geofast.h"
@@ -126,13 +127,13 @@ SimAddDefList(newdef)
     /* add the cell def to the list */
 
     if (DefList == (DefListElt *) NULL) {
-	MALLOC(DefListElt *, DefList, sizeof(DefListElt));
+	DefList = (DefListElt *) mallocMagic((unsigned) (sizeof(DefListElt)));
 	DefList->dl_def = newdef;
 	DefList->dl_next = (DefListElt *) NULL;
 	return;
     }
     else {
-	MALLOC(DefListElt *, d, sizeof(DefListElt));
+	d = (DefListElt *) mallocMagic((unsigned) (sizeof(DefListElt)));
 	d->dl_next = DefList;
 	d->dl_def = newdef;
 	DefList= d;
@@ -168,7 +169,7 @@ SimInitDefList()
 	q = p;
 	p = p->dl_next;
 	ExtResetTiles(q->dl_def, extUnInit);
-	FREE(q);
+	freeMagic(q);
     }
     DefList = (DefListElt *) NULL;
 }
@@ -511,7 +512,7 @@ SimFindOneNode( sx, tile )
     static NodeSpec	ret;
 
 	/* Allocate a new node */
-    MALLOC(NodeRegion *, reg, sizeof(NodeRegion) );
+    reg = (NodeRegion *) mallocMagic((unsigned) (sizeof(NodeRegion) ));
     reg->nreg_labels = (LabelList *) NULL;
     reg->nreg_cap = 0;
     reg->nreg_resist = 0;
@@ -551,7 +552,7 @@ SimFindOneNode( sx, tile )
 	arg.fra_each = (int (*)()) NULL;
 	(void) ExtFindNeighbors( tile, arg.fra_pNum, &arg );
 
-	FREE( reg );
+	freeMagic( reg );
 
 	ret.nd_name = SimTxtorLabel( -1, &sx->scx_trans, &transistor );
 	ret.nd_what = ND_NAME;
@@ -693,7 +694,11 @@ SimGetNodeName(sx, tp, path)
     if (SimGetnodeAlias && SimIsGetnode) {
 	if (HashLookOnly(&SimGNAliasTbl, nodename) == (HashEntry *) NULL) {
 	    HashFind(&SimGNAliasTbl, nodename);
+#ifdef MAGIC_WRAPPER
+	    Tcl_AppendElement(magicinterp, nodename);
+#else
 	    TxPrintf("alias: %s\n", nodename);
+#endif
 	}
     }
 
@@ -714,7 +719,11 @@ SimGetNodeName(sx, tp, path)
 			if (HashLookOnly(&SimGNAliasTbl, nodename) 
 			    == (HashEntry *) NULL) {
 			    HashFind(&SimGNAliasTbl, nodename);
+#ifdef MAGIC_WRAPPER
+			    Tcl_AppendElement(magicinterp, nodename);
+#else
 			    TxPrintf("alias: %s\n", nodename);
+#endif
 			}
 		    }
 		}
