@@ -29,7 +29,7 @@ static char sccsid[] = "@(#)SimSelect.c	4.14 MAGIC (Berkeley) 10/3/85";
 #include <strings.h>
 #endif
 
-#include "magic/tclmagic.h"
+#include "tcltk/tclmagic.h"
 #include "misc/magic.h"
 #include "utils/geometry.h"
 #include "tiles/tile.h"
@@ -98,6 +98,8 @@ bool	  SimGetnodeAlias = FALSE;	/* if node aliases are to be printed */
 bool	  SimSawAbortString;	/* if saw string to abort name search */
 bool	  SimIsGetnode;		/* true if command was issued from Getnode */
 bool	  SimUseCoords;		/* true if we should use trans. position */
+bool	  SimIgnoreGlobals = TRUE;	/* If FALSE, node names ending in "!"	*/
+					/* are treated as global node names.	*/
 
 #ifndef RSIM_MODULE
 bool	  SimRsimRunning = FALSE;	/* Always false if there's no rsim module */
@@ -151,22 +153,28 @@ SimSelectNode(scx, type, xMask, buffer)
 	    &TiPlaneRect, Select2Use, buffer);
     UndoEnable();
 
-    /* Strip out path if name is global (ends with a "!") */
+    /* Strip out path if name is global (ends with a "!")	*/
+    /* and flag SimIgnoreGlobals is not set.			*/
 
-    strptr = buffer + strlen(buffer) - 1; 
-    if (*strptr == '!') {		
-	*strptr = '\0';
-	while (strptr != buffer) {
-	    if (*strptr == '/') {
-		strptr++;
-		break;
+    if (!SimIgnoreGlobals)
+    {
+	strptr = buffer + strlen(buffer) - 1; 
+	if (*strptr == '!') {		
+	    *strptr = '\0';
+	    while (strptr != buffer) {
+		if (*strptr == '/') {
+		    strptr++;
+		    break;
+		}
+		strptr--;
 	    }
-	    strptr--;
+	}
+	else {
+	    strptr = buffer;
 	}
     }
-    else {
+    else
 	strptr = buffer;
-    }
 
     return(strptr);
 }
@@ -203,8 +211,8 @@ SimFreeNodeList(List)
     while (temp != NULL) {
 	current = temp;
 	temp = temp->tl_next;
-	FREE(current->tl_nodeName);
-	FREE(current);
+	freeMagic(current->tl_nodeName);
+	freeMagic(current);
     }
     *List = (TileListElt *) NULL;
 }
@@ -220,8 +228,8 @@ simFreeNodeEntry(list, entry)
 	if( curr == entry )
 	{
 	    prev->tl_next = curr->tl_next;
-	    FREE( entry->tl_nodeName );
-	    FREE( entry );
+	    freeMagic( entry->tl_nodeName );
+	    freeMagic( entry );
 	    return( prev );		/* next element in list */
 	}
 
@@ -248,7 +256,7 @@ simFreeNodeEntry(list, entry)
  */
 
 TileListElt *
-SimSelectArea()
+SimSelectArea(Rect *rect)
 {
     int plane;
     int SimSelectFunc();
@@ -266,8 +274,8 @@ SimSelectArea()
 	for (plane = PL_TECHDEPBASE; plane < DBNumPlanes; plane++)
 	{
 	    (void) DBSrPaintArea((Tile *) NULL, SelectDef->cd_planes[plane],
-		    &TiPlaneRect, &DBAllButSpaceAndDRCBits, SimSelectFunc,
-		    (ClientData) &NodeList);
+			&TiPlaneRect, &DBAllButSpaceAndDRCBits,
+			SimSelectFunc, (ClientData) &NodeList);
 	}
 
 	HashKill(&SimAbortSeenTbl);
@@ -310,8 +318,7 @@ SimSelectArea()
 int
 SimSelectFunc(tile, pHead)
     Tile *tile;			/* Tile in SelectDef. */
-    TileListElt **pHead; /* list of node names found */
-
+    TileListElt **pHead;	/* list of node names found */
 {
     TileTypeBitMask 	mask;
     SearchContext 	scx;
@@ -323,7 +330,7 @@ SimSelectFunc(tile, pHead)
     bool		coord;
 
     window = CmdGetRootPoint((Point *) NULL, &scx.scx_area);
-    if (window == NULL) return(1);
+    if (window == NULL) return 1;
 
     /* check to see if the node has already been extracted */
 
@@ -397,8 +404,8 @@ SimSelectFunc(tile, pHead)
     {
 	if( ! coord )
 	    HashFind(&SimNodeNameTbl, nodeName);
-	MALLOC(TileListElt *, newNodeTile, sizeof (TileListElt));
-	MALLOC(char *, newNodeTile->tl_nodeName, (strlen(nodeName) + 1));
+	newNodeTile = (TileListElt *) mallocMagic((unsigned) (sizeof (TileListElt)));
+	newNodeTile->tl_nodeName = (char *) mallocMagic((unsigned) (strlen(nodeName) + 1));
 	strcpy(newNodeTile->tl_nodeName, nodeName);
 	newNodeTile->tl_nodeTile = tile;
 	newNodeTile->tl_next = *pHead;
@@ -458,7 +465,7 @@ SimSelection(cmd)
     SimUseCoords = SimHasCoords;
 
     HashInit(&SimNodeNameTbl, 60, HT_STRINGKEYS);
-    node_list = SimSelectArea();
+    node_list = SimSelectArea((Rect *)NULL);
 
     if (node_list == (TileListElt *) NULL) {
 	TxPrintf("You must select paint (rather than a cell) to use Rsim on \
@@ -519,8 +526,8 @@ the selection.\n");
 		name = replyLine;
 		if( HashLookOnly(&SimNodeNameTbl, name) == (HashEntry *) NULL)
 		{
-		    FREE(current->tl_nodeName);
-		    MALLOC(char *, current->tl_nodeName, (strlen(name) + 1));
+		    freeMagic(current->tl_nodeName);
+		    current->tl_nodeName = (char *) mallocMagic((unsigned) (strlen(name) + 1));
 		    strcpy(current->tl_nodeName, name);
 		    HashFind(&SimNodeNameTbl, current->tl_nodeName);
 		    *strPtr++ = '=';
@@ -676,14 +683,14 @@ SimRsimMouse(w)
     if (!sawcell) {
 	/* add the cell def to the list */
 	if (SimCellLabList == (SimDefListElt *) NULL) {
-	    MALLOC(SimDefListElt *, SimCellLabList, sizeof(SimDefListElt));
+	    SimCellLabList = (SimDefListElt *) mallocMagic((unsigned) (sizeof(SimDefListElt)));
 	    SimCellLabList->dl_isMarked = FALSE;
 	    SimCellLabList->dl_def = cu->cu_def;
 	    SimCellLabList->dl_next = (SimDefListElt *) NULL;
 	    dummy = SimCellLabList;
 	}
 	else {
-	    MALLOC(SimDefListElt *, dummy, sizeof(SimDefListElt));
+	    dummy = (SimDefListElt *) mallocMagic((unsigned) (sizeof(SimDefListElt)));
 	    dummy->dl_isMarked = FALSE;
 	    dummy->dl_next = SimCellLabList;
 	    dummy->dl_def = cu->cu_def;
@@ -722,11 +729,11 @@ SimGetnode()
 
     /* get the list of node names */
 
-    SimIsGetnode = FALSE;
+    SimIsGetnode = TRUE;
     SimUseCoords = FALSE;
 
     HashInit(&SimNodeNameTbl, 60, HT_STRINGKEYS);
-    current = SimSelectArea();
+    current = SimSelectArea((Rect *)NULL);
     HashKill(&SimNodeNameTbl);
 
     if (current == (TileListElt *) NULL) {
@@ -738,15 +745,13 @@ SimGetnode()
     {
 #ifdef MAGIC_WRAPPER
 	/* Return the node name as the result of the command */
-	Tcl_SetResult(magicinterp, current->tl_nodeName, NULL);
+	Tcl_AppendElement(magicinterp, current->tl_nodeName);
 #else
 	TxPrintf("node name : %s\n", current->tl_nodeName);
 #endif
     }
 }
 
-
-
 /*
  * ----------------------------------------------------------------------------
  * SimGetsnode
@@ -769,11 +774,11 @@ SimGetsnode()
 
     /* get the list of node names */
 
-    SimIsGetnode = FALSE;
+    SimIsGetnode = TRUE;
     SimUseCoords = TRUE;
 
     HashInit(&SimNodeNameTbl, 60, HT_STRINGKEYS);
-    current = SimSelectArea();
+    current = SimSelectArea((Rect *)NULL);
     HashKill(&SimNodeNameTbl);
 
     if (current == (TileListElt *) NULL) {
@@ -785,7 +790,7 @@ SimGetsnode()
     {
 #ifdef MAGIC_WRAPPER
 	/* Return the node short name as the result of the command */
-	Tcl_SetResult(magicinterp, current->tl_nodeName, NULL);
+	Tcl_AppendElement(magicinterp, current->tl_nodeName);
 #else
 	TxPrintf("short node name : %s\n", current->tl_nodeName);
 #endif
