@@ -1,3 +1,8 @@
+#ifndef _SIM_H
+#define _SIM_H
+
+#include "misc/magic.h"
+
 extern	char	*SimGetNodeCommand();
 extern	char	*SimGetNodeName();
 extern	char	*SimSelectNode();
@@ -15,9 +20,11 @@ extern 	bool	SimRsimRunning;
 extern	bool	SimIsGetnode;
 extern	bool	SimHasCoords;
 extern	bool	SimUseCoords;
+extern  bool	SimIgnoreGlobals;
 
 extern HashTable SimNodeNameTbl;
 extern HashTable SimGNAliasTbl;
 extern HashTable SimGetnodeTbl;
 extern HashTable SimAbortSeenTbl;
 
+#endif /* _SIM_H */
