@@ -63,11 +63,34 @@ void TxFlushErr();
 void TxFlushOut();
 void RegisterTkCommands();
 
+/*--------------------------------------------------------------*/
+/* Verify if a command has a tag callback.			*/
+/*--------------------------------------------------------------*/
+
+int
+TagVerify(keyword)
+    char *keyword;
+{
+    char *croot, *postcmd;
+    HashEntry *entry;
+
+    /* Skip over namespace qualifier, if any */
+
+    croot = keyword;
+    if (!strncmp(croot, "::", 2)) croot += 2;
+    if (!strncmp(croot, "magic::", 7)) croot += 7;
+
+    entry = HashLookOnly(&txTclTagTable, croot);
+    postcmd = (entry) ? (char *)HashGetValue(entry) : NULL;
+    return (postcmd) ? TRUE : FALSE;
+}
+
 /*--------------------------------------------------------------*/
 /* Find any tags associated with a command and execute them.	*/
 /*--------------------------------------------------------------*/
 
-static int TagCallback(interp, tkpath, argc, argv)
+static int
+TagCallback(interp, tkpath, argc, argv)
     Tcl_Interp *interp;
     char *tkpath;
     int argc;		/* original command's number of arguments */
@@ -159,7 +182,7 @@ static int TagCallback(interp, tkpath, argc, argv)
 		case 'R':
 		    reset = TRUE;
 		case 'r':
-		    sres = Tcl_GetStringResult(magicinterp);
+		    sres = (char *)Tcl_GetStringResult(magicinterp);
 		    newcmd = (char *)mallocMagic(strlen(substcmd)
 				+ strlen(sres) + 1);
 		    strcpy(newcmd, substcmd);
@@ -230,7 +253,8 @@ static int TagCallback(interp, tkpath, argc, argv)
 /* Add a command tag callback					*/
 /*--------------------------------------------------------------*/
 
-static int AddCommandTag(ClientData clientData,
+static int
+AddCommandTag(ClientData clientData,
         Tcl_Interp *interp, int argc, char *argv[])
 {
     HashEntry *entry;
@@ -251,7 +275,7 @@ static int AddCommandTag(ClientData clientData,
 	return TCL_OK;
     }
 
-    if (hstring != NULL) FREE(hstring);
+    if (hstring != NULL) freeMagic(hstring);
 
     if (strlen(argv[2]) == 0)
     {
@@ -270,7 +294,8 @@ static int AddCommandTag(ClientData clientData,
 /* See TxTclDispatch() in textio/txCommands.c			*/
 /*--------------------------------------------------------------*/
 
-static int _tcl_dispatch(ClientData clientData,
+static int
+_tcl_dispatch(ClientData clientData,
         Tcl_Interp *interp, int argc, char *argv[])
 {
     int result, idx;
@@ -306,7 +331,7 @@ static int _tcl_dispatch(ClientData clientData,
     if (!strncmp(argv0, "::", 2)) argv0 += 2;
 
     objv0 = Tcl_NewStringObj(argv0, strlen(argv0));
-    if (Tcl_GetIndexFromObj(interp, objv0, conflicts,
+    if (Tcl_GetIndexFromObj(interp, objv0, (CONST84 char **)conflicts,
 	"overloaded command", 0, &idx) == TCL_OK)
     {
 	int i;
@@ -364,7 +389,8 @@ static int _tcl_dispatch(ClientData clientData,
 /* to point to the window, then dispatch the command.		*/
 /*--------------------------------------------------------------*/
 
-static int _tk_dispatch(ClientData clientData,
+static int
+_tk_dispatch(ClientData clientData,
         Tcl_Interp *interp, int argc, char *argv[])
 {
     int id;
@@ -397,6 +423,7 @@ static int _tk_dispatch(ClientData clientData,
 	argc--;
 	argv++;
     }
+
     TxTclDispatch(clientData, argc, argv);
 
     /* Get pathname of window and pass to TagCallback */
@@ -410,15 +437,27 @@ static int _tk_dispatch(ClientData clientData,
 void
 MakeWindowCommand(char *wname, MagWindow *mw)
 {
-    Tcl_CreateCommand(magicinterp, wname, _tk_dispatch,
+    char *tclcmdstr;
+
+    Tcl_CreateCommand(magicinterp, wname, (Tcl_CmdProc *)_tk_dispatch,
 		(ClientData)mw, (Tcl_CmdDeleteProc *) NULL);
+
+    /* Force the window manager to use magic's "close" command to close	*/
+    /* down a window.							*/
+
+    tclcmdstr = (char *)mallocMagic(52 + 2 * strlen(wname));
+    sprintf(tclcmdstr, "wm protocol %s WM_DELETE_WINDOW "
+		"{magic::closewindow %s}", wname, wname);
+    Tcl_EvalEx(magicinterp, tclcmdstr, -1, 0);
+    freeMagic(tclcmdstr);
 }
 
 /*------------------------------------------------------*/
 /* Main startup procedure				*/
 /*------------------------------------------------------*/
  
-static int _magic_initialize(ClientData clientData,
+static int
+_magic_initialize(ClientData clientData,
         Tcl_Interp *interp, int argc, char *argv[])
 {
     int n;
@@ -462,7 +501,7 @@ static int _magic_initialize(ClientData clientData,
     for (n = 0; commandTable[n] != NULL; n++)
     {
 	sscanf(commandTable[n], "%s ", kwptr); /* get first word */
-	Tcl_CreateCommand(interp, keyword, _tcl_dispatch,
+	Tcl_CreateCommand(interp, keyword, (Tcl_CmdProc *)_tcl_dispatch,
 			(ClientData)NULL, (Tcl_CmdDeleteProc *) NULL);
     }
 
@@ -470,7 +509,7 @@ static int _magic_initialize(ClientData clientData,
     for (n = 0; commandTable[n] != NULL; n++)
     {
 	sscanf(commandTable[n], "%s ", kwptr); /* get first word */
-	Tcl_CreateCommand(interp, keyword, _tcl_dispatch,
+	Tcl_CreateCommand(interp, keyword, (Tcl_CmdProc *)_tcl_dispatch,
 			(ClientData)NULL, (Tcl_CmdDeleteProc *) NULL);
     }
 
@@ -492,7 +531,8 @@ magicfatal:
 /* script, but probably should be.				*/
 /*--------------------------------------------------------------*/
 
-static int _magic_startup(ClientData clientData,
+static int
+_magic_startup(ClientData clientData,
         Tcl_Interp *interp, int argc, char *argv[])
 {
     Tcl_ChannelType *inChannel;
@@ -551,14 +591,14 @@ TxDialog(prompt, responses, defresp)
     newstr = StrDup(NULL, prompt);
     evalstr = TxPrintString("tk_dialog .dialog \"Dialog\""
 	" \"%s\" {} %d ", newstr, defresp);
-    FREE(newstr);
+    freeMagic(newstr);
 
     for (pos = 0; responses[pos] != 0; pos++)
     {
 	newstr = StrDup(NULL, evalstr);
 	evalstr = TxPrintString("%s \"%s\" ", newstr,
 		responses[pos]);
-	FREE(newstr);
+	freeMagic(newstr);
     }
 
     Tcl_EvalEx(magicinterp, evalstr, -1, 0);
@@ -629,12 +669,12 @@ TxGetLinePfix(dest, maxChars, prefix)
         if (prefix != NULL)
 	{
 	    string = Tcl_Alloc(20 + strlen(prefix));
-	    sprintf(string, "magic::dialog \"\" \"%s\"", prefix);
+	    sprintf(string, "magic::dialog \"\" \"%s\"\n", prefix);
 	    Tcl_EvalEx(magicinterp, string, -1, 0);
 	    Tcl_Free(string);
 	}
 	else
-	    Tcl_EvalEx(magicinterp, "magic::dialog", 13, 0);
+	    Tcl_EvalEx(magicinterp, "magic::dialog", 12, 0);
     }
     else
     {
@@ -649,7 +689,6 @@ TxGetLinePfix(dest, maxChars, prefix)
     objPtr = Tcl_GetObjResult(magicinterp);
     string = Tcl_GetStringFromObj(objPtr, &length);
 
-    /* Ignore any return character at end-of-line */
     if (length > 0)
 	if (*(string + length - 1) == '\n')
 	    length--;
@@ -704,7 +743,7 @@ TxParseString(str, q, event)
 
     Tcl_EvalEx(magicinterp, str, -1, 0);
 
-    reply = Tcl_GetStringResult(magicinterp);
+    reply = (char *)Tcl_GetStringResult(magicinterp);
 
     if (strlen(reply) > 0)
 	TxPrintf("%s: %s\n", str, reply);
@@ -752,6 +791,9 @@ TxFlush()
 /* function (namely, TxError() for stderr and TxPrintf() for	*/
 /* stdout).  It changes the result to a Tcl "puts" call, which	*/
 /* can be changed inside Tcl, as, for example, by TkCon.	*/
+/*								*/
+/* 6/17/04---Routine extended to escape double-dollar-sign '$$'	*/
+/* which is used by some tools when generating via cells.	*/
 /*--------------------------------------------------------------*/
 
 int
@@ -782,9 +824,13 @@ Tcl_printf(FILE *f, char *fmt, va_list args_in)
     else if (nchars == -1) nchars = 126;
 
     for (i = 24; *(outptr + i) != '\0'; i++)
+    {
 	if (*(outptr + i) == '\"' || *(outptr + i) == '[' ||
 	    	*(outptr + i) == ']' || *(outptr + i) == '\\')
 	    escapes++;
+	else if (*(outptr + i) == '$' && (*outptr + i + 1) == '$')
+	    escapes += 2;
+    }
 
     if (escapes > 0)
     {
@@ -799,6 +845,14 @@ Tcl_printf(FILE *f, char *fmt, va_list args_in)
 	        *(finalstr + i + escapes) = '\\';
 		escapes++;
 	    }
+	    else if (*(outptr + i) == '$' && *(outptr + i + 1) == '$')
+	    {
+		*(finalstr + i + escapes) = '\\';
+		*(finalstr + i + escapes + 1) = '$';
+		*(finalstr + i + escapes + 2) = '\\';
+		escapes += 2;
+		i++;
+	    }
 	    *(finalstr + i + escapes) = *(outptr + i);
 	}
         outptr = finalstr;
@@ -820,6 +874,9 @@ Tcl_printf(FILE *f, char *fmt, va_list args_in)
 /* in which characters are escaped as necessary to make them	*/
 /* printable from Tcl.  The new string is allocated by		*/
 /* Tcl_Alloc() which needs to be free'd with Tcl_Free().	*/
+/*								*/
+/* 6/17/04---extended like Tcl_printf to escape double-dollar-	*/
+/* sign ('$$') in names.					*/
 /*--------------------------------------------------------------*/
 
 char *
@@ -837,6 +894,9 @@ Tcl_escape(instring)
 	if (*(instring + i) == '\"' || *(instring + i) == '[' ||
 	    	*(instring + i) == ']')
 	    escapes++;
+
+	else if (*(instring + i) == '$' && *(instring + i + 1) == '$')
+	    escapes += 2;
     }
 
     newstr = Tcl_Alloc(nchars + escapes + 1);
@@ -846,8 +906,16 @@ Tcl_escape(instring)
 	if (*(instring + i) == '\"' || *(instring + i) == '[' ||
 	    		*(instring + i) == ']')
 	{
-	        *(newstr + i + escapes) = '\\';
-		escapes++;
+	    *(newstr + i + escapes) = '\\';
+	    escapes++;
+	}
+	else if (*(instring + i) == '$' && *(instring + i + 1) == '$')
+	{
+	    *(newstr + i + escapes) = '\\';
+	    *(newstr + i + escapes + 1) = '$';
+	    *(newstr + i + escapes + 2) = '\\';
+	    escapes += 2;
+	    i++;
 	}
 	*(newstr + i + escapes) = *(instring + i);
     }
@@ -932,15 +1000,15 @@ Tclmagic_Init(interp)
 
 
     /* Initialization and Startup commands */
-    Tcl_CreateCommand(interp, "magic::initialize", _magic_initialize,
+    Tcl_CreateCommand(interp, "magic::initialize", (Tcl_CmdProc *)_magic_initialize,
 			(ClientData)NULL, (Tcl_CmdDeleteProc *) NULL);
-    Tcl_CreateCommand(interp, "magic::startup", _magic_startup,
+    Tcl_CreateCommand(interp, "magic::startup", (Tcl_CmdProc *)_magic_startup,
 			(ClientData)NULL, (Tcl_CmdDeleteProc *) NULL);
 
     /* Initialize the command-tag callback feature */
 
     HashInit(&txTclTagTable, 10, HT_STRINGKEYS);
-    Tcl_CreateCommand(interp, "magic::tag", AddCommandTag,
+    Tcl_CreateCommand(interp, "magic::tag", (Tcl_CmdProc *)AddCommandTag,
 			(ClientData)NULL, (Tcl_CmdDeleteProc *) NULL);
 
     /* Add the magic TCL directory to the Tcl library search path */
@@ -954,7 +1022,7 @@ Tclmagic_Init(interp)
 
     Tcl_SetVar(interp, "CAD_HOME", cadhome, TCL_GLOBAL_ONLY);
 
-    Tcl_PkgProvide(interp, "Tclmagic", "7.2");
+    Tcl_PkgProvide(interp, "Tclmagic", "7.3");
     return TCL_OK;
 }
 
