@@ -42,6 +42,24 @@ set Glyph(zoom) [image create bitmap \
 	-file ${CAD_HOME}/lib/magic/tcl/bitmaps/zoom.xbm \
 	-background gray -foreground steelblue]
 
+# Handle the kludgy macros-call-Tcl-procedures setup by creating
+# a command "gmacro" (general-purpose macro).  The idea is that
+# macros only call magic (not Tcl) commands, but that tag callbacks
+# can call anything.  So "gmacro" sets up a dummy keyword that
+# is not a command, and attaches a tag callback function to it.
+# The tag callback implements the actual macro.
+
+proc magic::gmacro {key value} {
+   global Opts
+   if {[catch {set Opts(gmacrokeys)}]} {
+      set Opts(gmacrokeys) 0
+   } else {
+      incr Opts(gmacrokeys)
+   }
+   magic::macro $key gmkey$Opts(gmacrokeys)
+   magic::tag gmkey$Opts(gmacrokeys) $value
+}
+
 # Menu button callback functions
 
 proc magic::promptload {type} {
@@ -49,15 +67,33 @@ proc magic::promptload {type} {
       cif { set Layoutfilename [ tk_getOpenFile -filetypes \
 		{{CIF {.cif {.cif}}} {"All files" {*}}}]
 	    if {$Layoutfilename != ""} {
-		magic::load [file root $Layoutfilename]
+		set cifname [file tail [file root $Layoutfilename]]
+		magic::cellname create $cifname
+		magic::load $cifname
 		magic::cif read [file root $Layoutfilename]
 	    }
 	  }
       gds { set Layoutfilename [ tk_getOpenFile -filetypes \
 		{{GDS {.gds .strm .cal {.gds .strm .cal}}} {"All files" {*}}}]
 	    if {$Layoutfilename != ""} {
-		magic::load [file root $Layoutfilename]
+		set origlist [magic::cellname list top]
 		magic::gds read [file root $Layoutfilename]
+		set newlist [magic::cellname list top]
+
+		# Find entries in newlist that are not in origlist.
+		# If there's only one, load it into the window.
+
+		set newtopcells {}
+		foreach n $newlist {
+		   if {[lsearch $origlist $n] < 0} {
+		      lappend newtopcells $n
+		   }
+		}
+		if {[llength $newtopcells] == 1} {
+		   magic::load [lindex $newtopcells 0]
+		} elseif {[llength $newtopcells] != 0} {
+		   puts stdout "Top-level cells read from GDS file: $newtopcells"
+		}
 	    }
 	  }
       magic { set Layoutfilename [ tk_getOpenFile -filetypes \
@@ -1131,9 +1167,10 @@ proc magic::openwrapper {{cell ""} {framename ""}} {
 
    magic::caption ${winname}
 
-   # Rename "open" macro by rebinding
-   magic::macro o "magic::openwrapper"
-   magic::macro O "magic::closewrapper"
+   # Rename "open" macro by rebinding using the "gmacro" procedure
+   # (see above for the gmacro definition)
+   magic::gmacro o {magic::openwrapper}
+   magic::gmacro O {magic::closewrapper}
 
    return ${winname}
 }
@@ -1146,12 +1183,14 @@ proc magic::closewrapper { framename } {
 
    # Remove this window from the target list in cellmanager
    if {$MagicBLT} {
-      if { $Opts(target) == "${framename}.magic" } {
-	 set Opts(target) "default"
-	 .cellmgr.target.list configure -text "default"
-      } 
-      set idx [.cellmgr.target.list.winmenu index $framename ]
-      .cellmgr.target.list.winmenu delete $idx
+      if {![catch {wm state .cellmgr}]} {
+         if { $Opts(target) == "${framename}.magic" } {
+	    set Opts(target) "default"
+	    .cellmgr.target.list configure -text "default"
+         } 
+         set idx [.cellmgr.target.list.winmenu index $framename ]
+         .cellmgr.target.list.winmenu delete $idx
+      }
    }
 
    ${framename}.magic magic::closewindow
@@ -1182,6 +1221,13 @@ if {[file exists ${CAD_HOME}/lib/magic/tcl/padlist.tcl]} {
    set Opts(padlist) 0
 }
 
+# Check for presence of the miscellaneous tools script and include it
+
+if {[file exists ${CAD_HOME}/lib/magic/tcl/tools.tcl]} {
+   source ${CAD_HOME}/lib/magic/tcl/tools.tcl
+   set Opts(tools) 0
+}
+
 # Remove the command entry window from the bottom of a frame.
 
 proc magic::deletecommandentry { framename } {
@@ -1193,6 +1239,8 @@ puts "Use openwrapper to create a new GUI-based layout window"
 namespace import magic::closewrapper
 puts "Use closewrapper to remove a new GUI-based layout window"
 
+namespace import magic::gmacro
+
 # Create a simple wrapper for the 3D window. . . this can be
 # greatly expanded upon.
 
