@@ -569,7 +569,7 @@ TechError(char *fmt, ...)
 
     TechPrintLine();
     va_start(args, fmt);
-    (void) Vfprintf(stderr, fmt, args);
+    Vfprintf(stderr, fmt, args);
     va_end(args);
 }
 
@@ -657,6 +657,8 @@ techGetTokens(line, size, file, argv)
     /* as a comment character, rather than requiring it to be in the first  */
     /* column.								    */
 
+    /* Code revision (RTE, Open Circuit Design):  Handle DOS-style CR/LF    */
+
 start:
      get = line;
      currspace = size;
@@ -668,10 +670,16 @@ start:
 	while(isspace(*getp)) getp++;
 	if (*getp == '#') continue;
 	for (put = get; *put != '\n'; put++) currspace -= 1;
-	if ((put != get) && (*(put-1) == '\\'))
+	if (put != get)
 	{
-	    get = put-1;
-	    continue;
+	    put--;
+	    if (*put == 0xd) put--;	/* Handle DOS-style CR/LF */
+	    if (*put == '\\')
+	    {
+		get = put;
+		continue;
+	    }
+	    put++;
 	}
 	*put= '\0';
 	break;
