@@ -18,14 +18,23 @@
  * rcsid "$Header: /ufs/repository/magic/tech/tech.h,v 1.1.1.1 2000/03/29 18:35:37 rajit Exp $"
  */
 
+#ifndef _TECH_H
+#define _TECH_H
+
+#include "misc/magic.h"
+
 typedef int SectionID;		/* Mask set by TechAddClient */
 
-/* Techfile section status:  needed for switching between extract and */
-/* cif styles when only one style is kept in memory at any given time.*/
+/* Techfile section status:  needed for switching between extract and	*/
+/* cif styles when only one style is kept in memory at any given time.	*/
+/* TECH_SUSPENDED is added so that style variants can be merged together*/
+/* in a style section, and will set TECH_SUSPEND when reading a portion	*/
+/* of the style that does not pertain to the specific variant we want.	*/
 
 #define TECH_NOT_LOADED	0
 #define TECH_LOADED	1
 #define TECH_PENDING   -1
+#define TECH_SUSPENDED -2
 
 /* The last format version which was included in the file name */
 #define TECH_FORMAT_VERSION 27
@@ -46,3 +55,5 @@ extern bool TechOverridesDefault; /* Set TRUE if technology was specified on
 extern void TechError(char *, ...);
 extern void TechAddClient();
 extern void TechAddAlias();
+
+#endif /* _TECH_H */
