@@ -415,7 +415,7 @@ TxFreeCommand(command)
     ASSERT(command != NULL, "TxFreeCommand");
 #ifdef MAGIC_WRAPPER
     /* No Tx Queue in MAGIC_WRAPPER; call is just to free memory */
-    FREE((char *)command);
+    freeMagic((char *)command);
 #else
     DQPushRear(&txFreeCommands, (ClientData) command);
 #endif
@@ -685,23 +685,23 @@ txLogCommand(cmd)
 
     if (cmd->tx_wid >= 0) {
 	/* Command has a window associated with it. */
-	(void) fprintf(txLogFile, ":setpoint %d %d %d\n", 
+	fprintf(txLogFile, ":setpoint %d %d %d\n", 
 	    cmd->tx_p.p_x, cmd->tx_p.p_y, cmd->tx_wid);
     } else {
 	/* No window associated with the command. */
-	(void) fprintf(txLogFile, ":setpoint %d %d\n", 
+	fprintf(txLogFile, ":setpoint %d %d\n", 
 	    cmd->tx_p.p_x, cmd->tx_p.p_y);
     }
 
     if (cmd->tx_argc > 0)
     {
 	int i;
-	(void) fprintf(txLogFile, ":%s", cmd->tx_argv[0]);
+	fprintf(txLogFile, ":%s", cmd->tx_argv[0]);
 	for (i = 1; i < cmd->tx_argc; i++)
 	{
-	    (void) fprintf(txLogFile, " '%s'", cmd->tx_argv[i]);
+	    fprintf(txLogFile, " '%s'", cmd->tx_argv[i]);
 	}
-	(void) fprintf(txLogFile, "\n");
+	fprintf(txLogFile, "\n");
     }
     else if (cmd->tx_button == TX_CHARACTER) {
 	/* its a no-op command */
@@ -722,11 +722,11 @@ txLogCommand(cmd)
 	    default: {ASSERT(FALSE, "txLogCommand"); break; };
 	}
 	    
-	(void) fprintf(txLogFile, ":pushbutton %s %s\n", 
+	fprintf(txLogFile, ":pushbutton %s %s\n", 
 		txButTable[but], txActTable[act]);
     }
     if (txLogUpdate)
-	(void) fprintf(txLogFile, ":updatedisplay\n");
+	fprintf(txLogFile, ":updatedisplay\n");
     (void) fflush(txLogFile);
 }
 
