@@ -17,21 +17,14 @@
  *     * of the United States of America may require an export license.    * 
  *     *********************************************************************
  *
- * Needs:
- *	magic.h
- *	geometry.h
- *
  * rcsid "$Header: /ufs/repository/magic/tiles/tile.h,v 1.5 2001/11/29 20:50:09 tim Exp $"
  */
 
-#define	_TILES
+#ifndef _TILES_H
+#define	_TILES_H
 
-#ifndef	_MAGIC
-int err1 = Need_to_include_misc/magic_header;
-#endif	/* _MAGIC */
-#ifndef	_GEOMETRY
-int err2 = Need_to_include_utils/geometry_header;
-#endif	/* _GEOMETRY */
+#include "misc/magic.h"
+#include "utils/geometry.h"
 
 /*
  * A tile is the basic unit used for representing both space and
@@ -310,3 +303,5 @@ extern Rect TiPlaneRect;	/* Rectangle large enough to force area
 				 * plane.  This is the largest rectangle
 				 * that should ever be painted in a plane.
 				 */
+
+#endif /* _TILES_H */
