@@ -242,7 +242,7 @@ UndoInit(logFileName, mode)
 
     while (undoLogHead != (internalUndoEvent *) NULL)
     {
-	FREE((char *) undoLogHead);
+	freeMagic((char *) undoLogHead);
 	undoLogHead = undoLogHead->iue_forw;
     }
 
@@ -368,11 +368,11 @@ UndoFlush()
 
     while (undoLogTail != undoLogHead)
     {
-	FREE((char *) undoLogTail);
+	freeMagic((char *) undoLogTail);
 	undoLogTail = undoLogTail->iue_back;
 	ASSERT(undoLogTail != (internalUndoEvent *) NULL, "UndoFlush");
     }
-    FREE((char *) undoLogHead);
+    freeMagic((char *) undoLogHead);
 
     undoLogHead = undoLogTail = undoLogCur = (internalUndoEvent *) NULL;
     undoNumCommands = 0;
@@ -458,7 +458,7 @@ UndoEnable()
 UndoEvent *
 UndoNewEvent(clientType, size)
     UndoType clientType;	/* Type of event to allocate */
-    unsigned size;		/* Number of bytes of client data to allocate */
+    unsigned int size;		/* Number of bytes of client data to allocate */
 {
     internalUndoEvent *iup;
     int usize;
@@ -467,7 +467,7 @@ UndoNewEvent(clientType, size)
 	return ((UndoEvent *) NULL);
 
     usize = undoSize(size);
-    MALLOC(internalUndoEvent *, iup, usize);
+    iup = (internalUndoEvent *) mallocMagic((unsigned) usize);
     ASSERT(clientType >= 0 && clientType < undoNumClients, "UndoNewEvent");
     iup->iue_type = clientType;
     if (undoState == US_APPEND)
@@ -497,7 +497,7 @@ UndoNewEvent(clientType, size)
 
     return (undoExport(iup));
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -529,7 +529,7 @@ UndoNext()
     undoNumRecentEvents = 0;
     undoNumCommands++;
     usize = undoSize(0);
-    MALLOC(internalUndoEvent *, iup, usize);
+    iup = (internalUndoEvent *) mallocMagic((unsigned) usize);
     iup->iue_type = UT_DELIM;
     iup->iue_back = undoLogTail;
     iup->iue_forw = (internalUndoEvent *) NULL;
@@ -539,7 +539,7 @@ UndoNext()
     if (undoNumCommands >= MAXCOMMANDS)
 	undoFreeHead();
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -812,14 +812,14 @@ undoFreeHead()
 	do
 	{
 	    ASSERT(undoLogHead != undoLogCur, "undoFreeHead");
-	    FREE((char *) undoLogHead);
+	    freeMagic((char *) undoLogHead);
 	    undoLogHead = undoLogHead->iue_forw;
 	    ASSERT(undoLogHead != (internalUndoEvent *) NULL, "undoFreeHead");
 	}
 	while (undoLogHead->iue_type != UT_DELIM);
 	undoNumCommands--;
     }
-    FREE((char *) undoLogHead);
+    freeMagic((char *) undoLogHead);
     undoLogHead = undoLogHead->iue_forw;
     undoLogHead->iue_back = (internalUndoEvent *) NULL;
 }
@@ -862,7 +862,7 @@ undoMemTruncate()
 	up = undoLogHead;
 	while (up != (internalUndoEvent *) NULL)
 	{
-	    FREE((char *) up);
+	    freeMagic((char *) up);
 	    up = up->iue_forw;
 	}
 	undoLogTail = undoLogHead = (internalUndoEvent *) NULL;
@@ -879,7 +879,7 @@ undoMemTruncate()
 	{
 	    if (up->iue_type == UT_DELIM)
 		undoNumCommands--;
-	    FREE((char *) up);
+	    freeMagic((char *) up);
 	    up = up->iue_forw;
 	}
 	undoLogCur->iue_forw = (internalUndoEvent *) NULL;
@@ -887,7 +887,7 @@ undoMemTruncate()
     }
 
 }
-
+
 /*
  * ============================================================================
  *
@@ -896,10 +896,31 @@ undoMemTruncate()
  * ============================================================================
  */
 
-undoPrintForw(iup)
+void
+undoPrintEvent(iup)
     internalUndoEvent *iup;
 {
-    (void) printf("head=0x%x\ttail=0x%x\tcur=0x%x\n",
+    char *client_name;
+    if (iup->iue_type < 0)
+	client_name = "(delimiter)";
+    else
+	client_name = undoClientTable[iup->iue_type].uc_name;
+
+    (void) TxPrintf("0x%x: \t%s \tf=0x%x \tb=0x%x\n",
+		iup, client_name, iup->iue_forw, iup->iue_back);
+}
+
+/* Print events forward from "iup".  If n is 0 or negative, print to	*/
+/* the end of the stack.  Otherwise, print the next n events.		*/
+
+void
+undoPrintForw(iup, n)
+    internalUndoEvent *iup;
+    int n;
+{
+    int i = 0;
+
+    (void) TxPrintf("head=0x%x\ttail=0x%x\tcur=0x%x\n",
 		undoLogHead, undoLogTail, undoLogCur);
     if (iup == (internalUndoEvent *) NULL)
 	iup = undoLogHead;
@@ -907,13 +928,22 @@ undoPrintForw(iup)
     {
 	undoPrintEvent(iup);
 	iup = iup->iue_forw;
+	i++;
+	if (i == n) break;
     }
 }
 
-undoPrintBack(iup)
+/* Print events backward from "iup".  If n is 0 or negative, print to	*/
+/* the beginning of the stack.  Otherwise, print the previous n events.	*/
+
+void
+undoPrintBack(iup, n)
     internalUndoEvent *iup;
+    int n;
 {
-    (void) printf("head=0x%x\ttail=0x%x\tcur=0x%x\n",
+    int i = 0;
+
+    (void) TxPrintf("head=0x%x\ttail=0x%x\tcur=0x%x\n",
 		undoLogHead, undoLogTail, undoLogCur);
     if (iup == (internalUndoEvent *) NULL)
 	iup = undoLogTail;
@@ -921,17 +951,22 @@ undoPrintBack(iup)
     {
 	undoPrintEvent(iup);
 	iup = iup->iue_back;
+	i++;
+	if (i == n) break;
     }
 }
 
-undoPrintEvent(iup)
-    internalUndoEvent *iup;
-{
-    int size;
-    unsigned *fakeip;
+/* Called from windUndoCmd() or windRedoCmd() with n either positive, 	*/
+/* or negative with 1 offset to differentiate between n = 0 (forward)	*/
+/* and n = 0 (backward).						*/
 
-    fakeip = (unsigned *) iup;
-    size = (fakeip[-1] & (~1)) - (unsigned) fakeip;
-    (void) printf("0x%x:\t%d\tf=0x%x\tb=0x%x\tsize=%u\n", iup, iup->iue_type,
-		iup->iue_forw, iup->iue_back, size);
+void
+UndoStackTrace(n)
+    int n;
+{
+    if (n < 0)
+       undoPrintBack(undoLogCur, -(n + 1));
+    else
+       undoPrintForw(undoLogCur, n);
 }
+
