@@ -23,6 +23,8 @@ static char rcsid[] = "$Header: /ufs/repository/magic/utils/hash.c,v 1.2 2001/01
 #endif  /* not lint */
 
 #include <stdio.h>
+#include <string.h>
+
 #include "misc/magic.h"
 #include "utils/hash.h"
 #include "utils/malloc.h"
@@ -131,7 +133,7 @@ static rebuildLimit = 3;
  *	Free a key that had been allocated with (*copyFn)().
  *	If NULL, then nothing is done.
  *
- *	Void
+ *      int	
  *	(*killFn)(key)
  *	    char *key;
  *	{
@@ -149,7 +151,7 @@ HashInit(table, nBuckets, ptrKeys)
     ASSERT(ptrKeys != HT_CLIENTKEYS, "HashInit: should use HashInitClient");
     HashInitClient(table, nBuckets, ptrKeys,
 		(int (*)()) NULL, (char *(*)()) NULL,
-		(int (*)()) NULL, (Void (*)()) NULL);
+		(int (*)()) NULL, (int (*)()) NULL);
 }
 
 void
@@ -160,7 +162,7 @@ HashInitClient(table, nBuckets, ptrKeys, compareFn, copyFn, hashFn, killFn)
     int (*compareFn)();		/* Function to compare two keys */
     char *(*copyFn)();		/* Function to copy a key */
     int (*hashFn)();		/* For hashing */
-    Void (*killFn)();		/* For hashing */
+    int (*killFn)();		/* For hashing */
 {
     HashEntry ** ptr;
     int i;
@@ -185,7 +187,7 @@ HashInitClient(table, nBuckets, ptrKeys, compareFn, copyFn, hashFn, killFn)
     }
 
     /* Allocate and initialize the buckets */
-    MALLOC(HashEntry **, table->ht_table,
+    table->ht_table = (HashEntry **) mallocMagic(
 		(unsigned) (sizeof (HashEntry *)  * table->ht_size));
     ptr = table->ht_table;
     for (i = 0; i < table->ht_size; i++)
@@ -214,11 +216,11 @@ HashInitClient(table, nBuckets, ptrKeys, compareFn, copyFn, hashFn, killFn)
 
 int
 hash(table, key)
-    register HashTable *table;
-    register char *key;
+    HashTable *table;
+    char *key;
 {
-    register unsigned *up;
-    register int i, j;
+    unsigned *up;
+    int i, j;
 
     i = 0;
     switch (table->ht_ptrKeys)
@@ -278,14 +280,14 @@ hash(table, key)
 
 HashEntry *
 HashLookOnly(table, key)
-    register HashTable *table;	/* Hash table to search. */
+    HashTable *table;	/* Hash table to search. */
     char *key;			/* Interpreted according to table->ht_ptrKeys
 				 * as described in HashInit()'s comments.
 				 */
 {
-    register HashEntry *h;
-    register unsigned *up, *kp;
-    register n;
+    HashEntry *h;
+    unsigned *up, *kp;
+    int n;
     int bucket;
 
     bucket = hash(table, key);
@@ -348,14 +350,14 @@ next:
 
 HashEntry *
 HashFind(table, key)
-    register HashTable *table;	/* Hash table to search. */
+    HashTable *table;	/* Hash table to search. */
     char *key;			/* Interpreted according to table->ht_ptrKeys
 				 * as described in HashInit()'s comments.
 				 */
 {
-    register unsigned *up, *kp;
-    register HashEntry *h;
-    register n;
+    unsigned *up, *kp;
+    HashEntry *h;
+    int n;
     int bucket;
 
     bucket = hash(table, key);
@@ -415,23 +417,23 @@ next:
     switch (table->ht_ptrKeys)
     {
 	case HT_STRINGKEYS:
-	    MALLOC(HashEntry *, h, (unsigned) sizeof(HashEntry)+strlen(key)-3);
+	    h = (HashEntry *) mallocMagic((unsigned) (sizeof(HashEntry)+strlen(key)-3));
 	    (void) strcpy(h->h_key.h_name, key);
 	    break;
 	case HT_CLIENTKEYS:
 	    if (table->ht_copyFn)
 	    {
-		MALLOC(HashEntry *, h, (unsigned) sizeof (HashEntry));
+		h = (HashEntry *) mallocMagic((unsigned) (sizeof (HashEntry)));
 		h->h_key.h_ptr = (*table->ht_copyFn)(key);
 		break;
 	    }
 	    /* Fall through to ... */
 	case HT_WORDKEYS:
-	    MALLOC(HashEntry *, h, (unsigned) sizeof (HashEntry));
+	    h = (HashEntry *) mallocMagic((unsigned) (sizeof (HashEntry)));
 	    h->h_key.h_ptr = key;
 	    break;
 	case HT_STRUCTKEYS:
-	    MALLOC(HashEntry *, h,
+	    h = (HashEntry *) mallocMagic(
 		    (unsigned) (sizeof (HashEntry) + sizeof (unsigned)));
 	    up = h->h_key.h_words;
 	    kp = (unsigned *) key;
@@ -440,7 +442,7 @@ next:
 	    break;
 	default:
 	    n = table->ht_ptrKeys;
-	    MALLOC(HashEntry *, h,
+	    h = (HashEntry *) mallocMagic(
 		    (unsigned) (sizeof(HashEntry) + (n-1) * sizeof (unsigned)));
 	    up = h->h_key.h_words;
 	    kp = (unsigned *) key;
@@ -475,7 +477,7 @@ void
 rebuild(table)
     HashTable *table;		/* Table to be enlarged. */
 {
-    register HashEntry **oldTable, **old2, *h, *next;
+    HashEntry **oldTable, **old2, *h, *next;
     int oldSize, bucket;
 
     oldTable = table->ht_table;
@@ -512,7 +514,7 @@ rebuild(table)
 	}
     }
 
-    FREE((char *) oldTable);
+    freeMagic((char *) oldTable);
 }
 
 /*---------------------------------------------------------
@@ -575,7 +577,7 @@ HashStats(table)
 
 void
 HashStartSearch(hs)
-    register HashSearch *hs;	/* Area in which to keep state about search.*/
+    HashSearch *hs;	/* Area in which to keep state about search.*/
 {
     hs->hs_nextIndex = 0;
     hs->hs_h = NIL;
@@ -602,10 +604,10 @@ HashStartSearch(hs)
 
 HashEntry *
 HashNext(table, hs)
-    register HashTable *table;	/* Table to be searched. */
-    register HashSearch *hs;	/* Area used to keep state about search. */
+    HashTable *table;	/* Table to be searched. */
+    HashSearch *hs;	/* Area used to keep state about search. */
 {
-    register HashEntry *h;
+    HashEntry *h;
 
     while (hs->hs_h == NIL)
     {
@@ -637,18 +639,18 @@ void
 HashKill(table)
     HashTable *table;	/* Hash table whose space is to be freed */
 {
-    register HashEntry *h, **hp, **hend;
+    HashEntry *h, **hp, **hend;
     int (*killFn)() = (int (*)()) NULL;
 
     if (table->ht_ptrKeys == HT_CLIENTKEYS) killFn = table->ht_killFn;
     for (hp = table->ht_table, hend = &hp[table->ht_size]; hp < hend; hp++)
 	for (h = *hp; h != NIL; h = h->h_next)
 	{
-	    FREE((char *) h);
+	    freeMagic((char *) h);
 	    if (killFn)
 		(*killFn)(h->h_key.h_ptr);
 	}
-    FREE((char *) table->ht_table);
+    freeMagic((char *) table->ht_table);
 
     /*
      * Set up the hash table to cause memory faults on any future
@@ -677,13 +679,13 @@ HashFreeKill(table)
 HashTable *table;
 {
 	HashSearch hs;
-	register HashEntry *he;
-	register void *p;
+	HashEntry *he;
+	void *p;
 
 	HashStartSearch(&hs);
 	while (he = HashNext(table, &hs)) {
 		p = HashGetValue(he);
-		FREE(p);
+		freeMagic(p);
 	}
 	HashKill(table);
 }
