@@ -49,7 +49,7 @@ ListPop(listPP)
     ClientData result = LIST_FIRST(head);
 
     (*listPP) = LIST_TAIL(head);
-    FREE((char *) head);
+    freeMagic((char *) head);
 
     return result;
 }
@@ -98,7 +98,7 @@ ListDealloc(list)
     List *list;		/* List to free */
 {
     for(;list!=NULL; list=LIST_TAIL(list))
-	FREE((char *) list);
+	freeMagic((char *) list);
 
     return;
 }
@@ -125,8 +125,8 @@ ListDeallocC(list)
 {
     for(;list!=NULL; list=LIST_TAIL(list))
     {
-        FREE((char *) LIST_FIRST(list));
-	FREE((char *) list);
+        freeMagic((char *) LIST_FIRST(list));
+	freeMagic((char *) list);
     }
 
     return;
