@@ -35,7 +35,7 @@ static char rcsid[] = "$Header: /ufs/repository/magic/utils/lookupfull.c,v 1.2 2
  * Look up a string in a table of pointers to strings.  The last
  * entry in the string table must be a NULL pointer.
  * This is much simpler than Lookup() in that it does not
- * allow abbreviations.
+ * allow abbreviations.  It does, however, ignore case.
  *
  * Results:
  *	Index of the name supplied in the table, or -1 if the name
@@ -54,8 +54,20 @@ LookupFull(name, table)
     char **tp;
 
     for (tp = table; *tp; tp++)
+    {
 	if (strcmp(name, *tp) == 0)
 	    return (tp - table);
+	else
+	{
+	    char *sptr, *tptr;
+	    for (sptr = name, tptr = *tp; ((*sptr != '\0') && (*tptr != '\0'));
+			sptr++, tptr++)
+		if (toupper(*sptr) != toupper(*tptr))
+		    break;
+	    if ((*sptr == '\0') && (*tptr == '\0'))
+		return (tp - table);
+	}
+    }
 
     return (-1);
 }
