@@ -27,7 +27,7 @@ static char rcsid[] = "$Header: /ufs/repository/magic/utils/path.c,v 1.5 2001/01
 #include <ctype.h>
 #include <sys/param.h>
 
-#include "magic/tclmagic.h"
+#include "tcltk/tclmagic.h"
 #include "misc/magic.h"
 #include "utils/hash.h"
 #include "utils/malloc.h"
@@ -166,7 +166,7 @@ PaExpand(psource, pdest, size)
 	}
 	if (string != NULL)
 	{
-	    MALLOC(char *, newEntry, strlen(string) + 1);
+	    newEntry = (char *) mallocMagic((unsigned) (strlen(string) + 1));
 	    (void) strcpy(newEntry, string);
 	    HashSetValue(h, newEntry);
 	}
@@ -214,7 +214,7 @@ PaExpand(psource, pdest, size)
 	 */
 
 #ifdef MAGIC_WRAPPER
-	string = Tcl_GetVar(magicinterp, expandName, TCL_GLOBAL_ONLY);
+	string = (char *)Tcl_GetVar(magicinterp, expandName, TCL_GLOBAL_ONLY);
 #else
 	string = getenv(expandName);
 #endif
@@ -234,7 +234,7 @@ PaExpand(psource, pdest, size)
 	    }
 	}
 
-	MALLOC(char *, newEntry, strlen(string) + 1);
+	newEntry = (char *) mallocMagic((unsigned) (strlen(string) + 1));
 	(void) strcpy(newEntry, string);
 	HashSetValue(h, newEntry);
 
