@@ -629,7 +629,7 @@ windClientInit()
 	"center [x y]		center window on the cursor or indicated coordinate",
 	windCenterCmd, FALSE);
     WindAddCommand(windClientID,
-	"closewindow		close a window",
+	"closewindow [name]	close a window",
 	windCloseCmd, FALSE);
     WindAddCommand(windClientID,
 	"cursor			return magic coordinates of the cursor",
@@ -694,7 +694,12 @@ windClientInit()
 	"undo [count]		undo commands",
 	windUndoCmd, FALSE);
     WindAddCommand(windClientID,
+#ifdef MAGIC_WRAPPER
+	"updatedisplay [suspend|resume]\n\
+			force display update, or suspend/resume updates",
+#else
 	"updatedisplay		force the display to be updated",
+#endif
 	windUpdateCmd, FALSE);
     WindAddCommand(windClientID,
 	"version			print out version info",
