@@ -21,7 +21,7 @@ static char rcsid[]="$Header: /ufs/repository/magic/windows/windDisp.c,v 1.3 200
 
 #include <stdio.h>
 
-#include "magic/tclmagic.h"
+#include "tcltk/tclmagic.h"
 #include "misc/magic.h"
 #include "utils/utils.h"
 #include "textio/textio.h"
@@ -123,7 +123,7 @@ windFreeList(llr)
     {
 	freelr = lr;
 	lr = lr->r_next;
-	FREE( (char *) freelr);
+	freeMagic( (char *) freelr);
     }
 
     *llr = (LinkedRect *) NULL;
@@ -159,7 +159,7 @@ windReClip()
 	LinkedRect *tmp;
 
 	/* add window onto windCoveredAreas */
-	MALLOC(LinkedRect *, tmp, sizeof (LinkedRect));
+	tmp = (LinkedRect *) mallocMagic((unsigned) (sizeof (LinkedRect)));
 	tmp->r_next = windCoveredAreas;
 	tmp->r_r = w1->w_frameArea;
 	windCoveredAreas = tmp;
@@ -178,7 +178,7 @@ windReClip()
 	    {
 		if ( GEO_TOUCH( &(w1->w_frameArea), &(w2->w_frameArea) ))
 		{
-		    MALLOC(LinkedRect *, tmp, sizeof (LinkedRect));
+		    tmp = (LinkedRect *) mallocMagic((unsigned) (sizeof (LinkedRect)));
 		    tmp->r_next = w1->w_clipAgainst;
 		    tmp->r_r = w2->w_frameArea;
 		    w1->w_clipAgainst = tmp;
@@ -771,6 +771,8 @@ WindUpdate()
     }
 
 #ifdef MAGIC_WRAPPER
+    /* Honor the display redraw suspension state */
+    if (GrDisplayStatus == DISPLAY_SUSPEND) return;
     GrDisplayStatus = DISPLAY_IN_PROGRESS;
     SigSetTimer(0);
 #endif
