@@ -24,6 +24,7 @@ static char rcsid[]="$Header: /ufs/repository/magic/windows/windSend.c,v 1.2 200
 #include <string.h>
 #include <ctype.h>
 
+#include "tcltk/tclmagic.h"
 #include "misc/magic.h"
 #include "textio/textio.h"
 #include "utils/geometry.h"
@@ -191,6 +192,20 @@ WindSendCommand(w, cmd)
 	}
 	if ( (windCmdNum == -2) && (clientCmdNum == -2) )
 	{
+
+#ifdef MAGIC_WRAPPER
+	    /* There's a way, perhaps clever, perhaps not, to	*/
+	    /* use non-Magic TCL commands/procedures in macros, */
+	    /* and that is to put the command into a "tag	*/
+	    /* callback" to a dummy command name.  This code	*/
+	    /* will allow such a hack to work without printing	*/
+	    /* error messages to the output.  Ultimately, magic	*/
+	    /* should allow key and button bindings to be	*/
+	    /* interpreted by Tk.				*/
+		
+	    if ((cmd->tx_argc == 1) && TagVerify(cmd->tx_argv[0]))
+		return FALSE;
+#endif
 	    /* Not a valid command.  Help the user out by telling him
 	     * what might be wrong. And also print out the command!
 	     */
