@@ -239,7 +239,6 @@ WindSurfaceToScreen(w, surface, screen)
     }
 }
 
-#ifdef NONMANHATTAN
 /*     
  * ----------------------------------------------------------------------------
  *
@@ -248,6 +247,9 @@ WindSurfaceToScreen(w, surface, screen)
  *	This procedure is like WindSurfaceToScreen except that it
  *	does not clip to the window boundary.  This is necessary for
  *	non-Manhattan tiles to make sure they are not clipped.
+ *	However, this routine is also useful for Manhattan geometry
+ *	when, for example, determining the transformation of a unit
+ *	coordinate.
  *
  * Results:
  *	None.
@@ -277,8 +279,6 @@ WindSurfaceToScreenNoClip(w, surface, screen)
     screen->r_ytop = (w->w_origin.p_y + (tmp * w->w_scale)) >> 12;
 }
 
-#endif  /* NONMANHATTAN */
-
 
 /*
  * ----------------------------------------------------------------------------
