@@ -93,7 +93,7 @@ WindUnload(surfaceID)
 
     for (mw = windTopWindow; mw != NULL; mw = mw->w_nextWindow)
 	if (mw->w_surfaceID == surfaceID)
-	    DBWloadWindow(mw, (char *)NULL);
+	    DBWloadWindow(mw, (char *)NULL, TRUE, FALSE);
 }
 
 /*
