@@ -20,17 +20,12 @@
  *
  */
 
-#define _WINDOWS
+#ifndef _WINDOWS_H
+#define _WINDOWS_H
 
-#ifndef _MAGIC
-    err = Need_to_include_misc/magic_header
-#endif
-#ifndef _GEOMETRY
-    err = Need_to_include_utils/geometry_header
-#endif
+#include "misc/magic.h"
+#include "utils/geometry.h"
 
-
-
 typedef ClientData WindClient;	/* A unique ID of a client of the
 				 * window package.  The value 'NULL' is
 				 * indicates an invalid value.
@@ -261,9 +256,7 @@ extern void WindScreenToSurface();
 extern void WindSurfaceToScreen();
 extern void WindPointToSurface();
 extern void WindPointToScreen();
-#ifdef NONMANHATTAN
 extern void WindSurfaceToScreenNoClip();
-#endif /* NONMANHATTAN */
 
 /* procs to change things or inform the window manager about changes */
 extern void WindCaption();
@@ -311,3 +304,5 @@ extern int WindScrollBarWidth;
 
 /* Global identifier for the window client structure */
 extern WindClient windClientID;
+
+#endif /* _WINDOWS_H */
