@@ -19,14 +19,11 @@
  * rcsid $Header: /ufs/repository/magic/wiring/wiring.h,v 1.2 2001/01/12 22:13:37 jsolomon Exp $
  */
 
-#define _WIRING
+#ifndef _WIRING_H
+#define _WIRING_H
 
-#ifndef	_MAGIC
-int err0 = Need_to_include_misc/magic_header;
-#endif	/* _MAGIC */
-#ifndef	_DATABASE
-int err0 = Need_to_include_database/database_header;
-#endif	/* _DATABASE */
+#include "misc/magic.h"
+#include "database/database.h"
 
 /* Table that defines the shape of contacts and the layers that they
  * connect.  This definition allows some layers to extend around the
@@ -84,3 +81,5 @@ extern void WireTechScale();
 /* Initialization: */
 
 extern void WireInit();
+
+#endif /* _WIRING_H */
